<?php
/**
  _____________________________________________________________________________
 |
 | EmbeddedHelp Widget
 |
 | @project     Chameleon
 | @revision    $Id: EmbeddedHelp.widget.php,v 1.15 2006/03/21 19:49:11 bartvde Exp $
 | @purpose     Generates documents for Chameleon widgets.
 | @author      DM Solutions Group (wbronsema@dmsolutions.ca)
 | @copyright
 | <b>Copyright (c) 2004 DM Solutions Group Inc.</b>
 | Permission is hereby granted, free of charge, to any person obtaining a
 | copy of this software and associated documentation files (the "Software"),
 | to deal in the Software without restriction, including without limitation
 | the rights to use, copy, modify, merge, publish, distribute, sublicense,
 | and/or sell copies of the Software, and to permit persons to whom the
 | Software is furnished to do so, subject to the following conditions:
 |
 | The above copyright notice and this permission notice shall be included
 | in all copies or substantial portions of the Software.
 |
 | THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 | IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 | FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 | THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 | LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 | FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 | DEALINGS IN THE SOFTWARE.
 |_____________________________________________________________________________

 **/

// include the basic widget file
include_once(dirname(__FILE__)."/../Widget.php");

/**
  _____________________________________________________________________________
 |
 | EmbeddedHelp Widget Class
 |_____________________________________________________________________________

 **/
class EmbeddedHelp extends CWCWidget
{
    // declare member variables
    var $mszOutput = '';


    /**
      _________________________________________________________________________
     |
     | Constructor: EmbeddedHelp Widget Class
     |_________________________________________________________________________

     **/
    function EmbeddedHelp()
    {
        // invoke constructor of parent
        parent::CWCWidget();

        // set the description for this widget
        $this->szWidgetDescription = 'Embeds the online help content.';

        $this->maAttributes['WIDGET'] = new StringAttribute( 'WIDGET', true );

        // set the maturity level
        $this->mnMaturityLevel = MATURITY_TECHNICALRELEASE;
    }

    /**
      _________________________________________________________________________
     |
     |  ParseURL()
     |
     |  Postcondition:  This function parses information passed on the URL and
     |                  acts on it.
     |
     |  @return boolean - True if successful, false if not.
     |  @desc Parses info passed on URL.
     |_________________________________________________________________________

     **/
    function  ParseURL()
    {
        // process the help link to see if the requested file is available
        if ( isset( $this->maParams["WIDGET"] ) )
        {
            // set help extension based on language
            $szExtension = strtolower( $_SESSION["gszCurrentLanguage"] ).
                                                                    '.doc.xml';
            // loop and look for the files
            $bFind = false;
            foreach ( $this->moApp->moUI->moWidgetManger->maszWidgetDirectories
                                                                   as $szDir )
            {
                // check for a widget in a subdirectory of the same name
                $szFileName = str_replace("\\",'/', $szDir.'/'.
                    $this->maParams['WIDGET'].'/'.$this->maParams['WIDGET'].
                    '/'.$this->maParams['WIDGET'].'.'.$szExtension);
                if ( file_exists( $szFileName ) )
                {
                    $bFind = true;
                    break;
                }

                // otherwise check the current directory
                $szFileName = str_replace("\\",'/', $szDir.'/'.
                    $this->maParams['WIDGET'].'/'.
                    $this->maParams['WIDGET'].'.'.$szExtension);
                if ( file_exists( $szFileName ) )
                {
                    $bFind = true;
                    break;
                }
            }

            // parse the help file if found
            if ( $bFind )
            {
                // open the doc
                $oXML = new XMLObject(implode("", file( $szFileName )));
                $oXML = $oXML->getNextChild('');

                // get the help value from the XML doc
                if ( !($oNode =& $oXML->getNode(
                                          'ca:widgetdoc/ca:descriptions/ca:embeddedhelp' ) ) )
                {
                    // log error and exit
                     $_SESSION['gErrorManager']->setError(ERR_WARNING,
                        $this->moMLT->get( $this->mszWidgetName, '2',
                            'Could not find help node in XML document.' ) );
                    return true;
                }

                // loop for each params
                $nCount = 0;
                $ClassCounter = 1;
                if ( isset( $oNode->children ) && is_array( $oNode->children ) )
                {
                    $nCount = count( $oNode->children );
                }
                for( $i=0; $i<$nCount; $i++ )
                {
                    // get child
                    $oChild = $oNode->children[$i];

                    // only process paras
                    if ( $oChild->name != 'ca:paras' )
                    {
                        continue;
                    }

                    // output title
                    if ( isset( $oChild->title ) &&
                         strlen( trim( $oChild->title ) ) > 0 )
                    {
                        $this->mszOutput .= '<span class="EmbeddedHelpTitle'.
                            $ClassCounter.'">'.$oChild->title.'</span>';
                        $ClassCounter++;
                    }

                    // loop through param children
                    $nChildCount = 0;
                    if ( isset( $oChild->children ) &&
                         is_array( $oChild->children ) )
                    {
                        $nChildCount = count( $oChild->children );
                    }
                    for( $j=0; $j<$nChildCount; $j++ )
                    {
                        // get child children
                        $oGrandKids = $oChild->children[$j];

                        // if is set the output
                        if ( isset( $oGrandKids->value ) &&
                             strlen( trim( $oGrandKids->value ) ) > 0 )
                        {
                            $this->mszOutput .= '<p class="EmbeddedHelpParam">'.
                                $oGrandKids->value.'</p>';
                        }
                    }
                }
            }
            else
            {
                $_SESSION['gErrorManager']->setError(ERR_WARNING,  '['.
                    $this->maParams["WIDGET"].'] - '.
                    $this->moMLT->get( $this->mszWidgetName, '0',
                    'No help available for this widget.' ) );
            }
        }

        // return success
        return true;
    }

    /**
      _________________________________________________________________________
     |
     |  DrawPublish()
     |
     |  Postcondition:  This function builds the html output for the physical
     |                  appearance of the widget.
     |
     |  @return string - Error messages.
     |  @desc Generates HTML output for the widget.
     |_________________________________________________________________________

     **/
    function DrawPublish()
    {
        // return the output
        return $this->mszOutput;
    }
}
?>
