<?php
/**
 * ExtractSR Widget Class
 *
 * @project     WIX
 * @revision    $Id: ExtractSR.widget.php,v 1.6 2005/07/26 18:52:33 wbronsema Exp $
 * @purpose     Display a dialog box to extract the contents of a SharedResource
 * @author      DM Solutions Group (pspencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");
include_once(dirname(__FILE__).'/../Popup.php');

/**
 * ExtractSR
 *
 * @desc a widget that converts data in a SharedResource into a CSV file and
 * downloads it to the user
 */
class ExtractSR extends CWCWidget
{
    var $moButton;
    var $moPopup;

    var $mszSharedResource;

    /**
     * ExtractSR
     *
     * Constructor method for the ExtractSR
     */
    function ExtractSR()
    {
        parent::CWCWidget();

        $this->moButton = new CWCButton( $this );

        $this->moPopup = new CWCPopup( $this );
        $this->moPopup->mszLink = $_SESSION['gszCoreWebPath']."/widgets/ExtractSR/ExtractSR.phtml";

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The ExtractSRWidget displays a popup dialog box that allows the user
download query results as a CSV file.
EOT;

        $this->maAttributes['SHAREDRESOURCE'] = new StringAttribute( 'SHAREDRESOURCE', true );

        $this->mnMaturityLevel = MATURITY_BETA;
    }

    function InitDefaults()
    {
        parent::InitDefaults();

        //this widget should never belong to a toolset
        $this->maParams["TOOLSET"] = "";
        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick('clickExtractSR');

        if (isset($this->maParams['SHAREDRESOURCE']))
            $this->mszSharedResource = $this->maParams['SHAREDRESOURCE'];
    }

    function GetJavascriptInitFunctions()
    {
        return $this->moButton->GetJavascriptInitFunctions();
    }

    function GetJavascriptVariables()
    {
        return $this->moButton->GetJavascriptVariables();
    }

    function GetJavascriptOnLoadFunctions()
    {
        return $this->moButton->GetJavascriptOnLoadFunctions();
    }

    function GetJavascriptIncludeFunctions()
    {
        return $this->moButton->GetJavascriptIncludeFunctions();
    }

    /**
     * GetHTMLHiddenVariables
     *
     * return the NAV_CMD varaiable.
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();

        return $aReturn;
    }

    /**
     * ParseURL
     */
    function  ParseURL()
    {
        $this->moButton->ParseURL();

        return true;
    }

    /**
     * GetJavascriptFunctions
     *
     * Build and return the array of functions needed in the
     * widget.
     */
    function GetJavascriptFunctions()
    {
        $aReturn = $this->moButton->GetJavascriptFunctions();
        $this->moPopup->mszParam = "&srname=".$this->mszSharedResource;

        $szJsFunctionName = "clickExtractSR";

        if (isset($this->maSharedResourceWidgets[$this->mszSharedResource]) &&
            count($this->maSharedResourceWidgets[$this->mszSharedResource]) > 0)
        {
            $szButtonJS = $this->moPopup->DrawPublish();
            if (is_array($this->maSharedResourceWidgets[$this->mszSharedResource]))
            {
                $_SESSION['EXTRACTSR_DATA_'.$this->mszSharedResource] = $this->maSharedResourceWidgets[$this->mszSharedResource];
            }
            else
            {
                $_SESSION['EXTRACTSR_DATA_'.$this->mszSharedResource] = $this->maSharedResourceWidgets[$this->mszSharedResource]->maszContents;
            }
        }
        else
        {
            $szButtonJS = 'alert( "no data to extract." ); ';
            $_SESSION['EXTRACTSR_DATA_'.$this->mszSharedResource] = array();
        }
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * popup a ExtractSR dialog
 */
function {$szJsFunctionName}()
{
    {$szButtonJS}
    return;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

    /**
     * DrawPublish
     *
     * Return the HTML code using the name in the map file and the
     * parameters of the CWC tag.
     */
    function DrawPublish()
    {
        if (!$this->mbVisible)
            return "<!-- ExtractSR widget hidden -->";

        $szResult = $this->moButton->DrawPublish();

        return $szResult;
    }
}
?>
