<?php
/**
* GeoservicesLabeling Widget class
*
* @project     CWC2
* @purpose     LayerLabels Widget class,only WMS layers implemented through SLD for now
* @author      Geodan, Sander Dee
* @copyright
* <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");
include_once(dirname(__FILE__)."/../Popup.php");
include_once(dirname(__FILE__)."/../LayerAttributes.php");

/**
* LayerLabels Widget
*
* @desc Provides dialog functionality to label a map layer on an item using SLD, uses LabelAnnotation layer
*/
class LayerLabels extends CWCWidget
{
    var $moButton;
    var $moPopup;

    /* default settings for the widget params */
    var $mszLayer = '';
    var $mszAttribute = '';
    var $mszFont = '';
    var $mszFontSize = '';

    /**
    * LayerLabels
    *
    * @desc Constructor method for the LayerLabels widget.

    */
    function LayerLabels()
    {
        /* invoke constructor of parent */
        parent::CWCWidget();

        /* set the description for this widget */
        $this->szWidgetDescription = <<<EOT
             The LayerLabels widget allows users to show dynamic
             labels
EOT;

        $this->moPopup = new CWCPopup( $this );
        $this->moPopup->mszLink = $_SESSION['gszCoreWebPath']."/widgets/LayerLabels/LayerLabelsPopup.phtml";

        $this->moButton = new CWCButton( $this );

        $this->mnMaturityLevel = MATURITY_BETA;
    }

    /**
    * InitDefaults
    *
    */
    function InitDefaults()
    {
        parent::InitDefaults();

        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick('clickLayerLabelsPopup');
        //this widget should never belong to a toolset
        $this->moButton->mszToolset = "";
        $this->moPopup->InitDefaults();

        $this->mszPixelGif = $_SESSION['gszCoreWebPath'].'/skins/default/images/a_pixel.gif';

    }

    function GetJavascriptOnLoadFunctions()
    {
        $aReturn = $this->moButton->GetJavascriptOnLoadFunctions();

        return $aReturn;
    }

    /**
    * DrawPublish()
    *
    * @desc draw this widget on the page
    */
    function DrawPublish()
    {
        $szResult = "";

        /* return the result */
        if (!$this->mbVisible)
        return "<!-- LayerLabels widget invisble -->";

        if ($this->mbVisible)
        {
            $szResult .= $this->moButton->DrawPublish();

        }
        return $szResult;

    }

    /**
    * ParseURL inspects incoming form variables and creates the
    * SLD for the labels, then applies the SLD
    */
    function ParseURL()
    {
        if ($this->isVarSet( 'LABEL_LAYER' ))
        {
            $oMap = $this->moMapObject->oMap;

            // check for and delete annotation layer
            $nLayerCount = $oMap->numlayers;
            for ( $i=0; $i<$nLayerCount;  $i++ )
            {
                $oTempLayer = $oMap->getlayer($i);
                if ( $oTempLayer->name == 'LabelAnnotation'  )
                {
                    $oTempLayer->set( 'status', MS_DELETE );
                    break;
                }
            }

        // selected layer (by user)
        $this->mszLayer = $this->getVar('LABEL_LAYER');

            if ( $oLayer = @$oMap->getLayerByName($this->mszLayer) )
            {

               // duplicate the layer
               $oAnnoLayer = ms_newLayerObj( $oMap, $oLayer );

               // explcitly set to ON
               $oAnnoLayer->set( 'status', MS_ON );

               // set the template to an empty string to invalidate querying
               $oAnnoLayer->set( 'template', '' );

               $oAnnoLayer->set( 'name', 'LabelAnnotation' );
               $oAnnoLayer->setmetadata( 'wms_title', 'Labels' );
               $oAnnoLayer->setmetadata( 'selected', '0' );

             // set variables
             $this->mszAttribute = $this->getVar( 'LABEL_ATTRIBUTE' );
              if ( $oLayer->getmetadata('wms_name') != "")
              $szLayerName = $oLayer->getmetadata('wms_name');
            else
            $szLayerName = $oLayer->name;
            if ( $this->isVarSet( 'LABEL_FONT' ) )
            $this->mszFont = $this->getVar( 'LABEL_FONT' );
            else $szFont = 'arial';
            if ( $this->isVarSet( 'LABEL_FONTSIZE' ) )
            $this->mszFontSize = $this->getVar( 'LABEL_FONTSIZE' );
            else $szFontSize = '10';

              $szLabelSLD = "<StyledLayerDescriptor version='1.0.0'><NamedLayer><Name>$szLayerName</Name><UserStyle><FeatureTypeStyle><Rule><TextSymbolizer><Label>$this->mszAttribute</Label><Font><CssParameter name='font-color'>#000000</CssParameter><CssParameter name='font-family'>$this->mszFont</CssParameter><CssParameter name='font-size'>$this->mszFontSize</CssParameter></Font></TextSymbolizer></Rule></FeatureTypeStyle></UserStyle></NamedLayer></StyledLayerDescriptor>";

               // if layer type is WMS: add sld_body to connection
               if ( $oAnnoLayer->connectiontype == MS_WMS )
               {
                 if ( isset( $szLabelSLD ) )
                 {
                    $szTmpConnection = $oAnnoLayer->connection . "&sld_body=".urlencode($szLabelSLD);
                    $oAnnoLayer->set( 'connection', $szTmpConnection );
                 }
               }

               else  // not WMS
               {
                 $oAnnoLayer->applysld( $szLabelSLD , $szLayerName );
               }

            } // end if

        } // end if

      return true;

    } // end function

    /**
    * GetJavascriptIncludeFunctions
    *
    * javascript include files
    */
    function GetJavascriptIncludeFunctions()
    {
        return $this->moButton->GetJavascriptIncludeFunctions();
    }

    /**
    * GetJavascriptInitFunctions
    *
    * @desc These get spit out at the bottom of the template after the page is processed
    */
    function GetJavascriptInitFunctions()
    {
        return $this->moButton->GetJavascriptInitFunctions();
    }

    /**
    * GetHTMLHiddenVariables
    *
    * return the map size  variables.
    */
    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();

        $szVariable = "LABEL_LAYER";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"".$this->mszLayer."\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "LABEL_ATTRIBUTE";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"".$this->mszAttribute."\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "LABEL_FONT";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"".$this->mszFont."\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "LABEL_FONTSIZE";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"".$this->mszFontSize."\">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    /**
    * GetJavascriptFunctions
    *
    * Build and return the array of functions needed in the
    * widget.
    */
    function GetJavascriptFunctions()
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
            $bCWCJSAPI = 1;
        else
            $bCWCJSAPI = 0;

        $aReturn = $this->moButton->GetJavascriptFunctions();

        $szJsFunctionName = "clickLayerLabelsPopup";
        $szButtonJS = $this->moPopup->DrawPublish();
        $szFunction = <<<EOT

/**
 * {$szJsFunctionName}
 * popup a labeling configuration dialog
 */
function {$szJsFunctionName}()
{
    {$szButtonJS}
    return;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "applyLabel";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * apply a maptips configuration to the page
 * szLayer - the layer to draw maptips info from
 * szAttributes -
 */
function {$szJsFunctionName}(szLayer, szAttribute, szFont, szFontSize )
{
    bShow = true;
    if (arguments.length == 5)
    {
        bShow = arguments[4];
    }
    if ($bCWCJSAPI)
    {
        AddLayerLabels(szLayer, szAttribute, szFont, szFontSize);
    }
    else
    {
        {$this->mszHTMLForm}.NAV_CMD.value='';
        {$this->mszHTMLForm}.LABEL_LAYER.value=szLayer;
        {$this->mszHTMLForm}.LABEL_ATTRIBUTE.value=szAttribute;
        {$this->mszHTMLForm}.LABEL_FONT.value=szFont;
        {$this->mszHTMLForm}.LABEL_FONTSIZE.value=szFontSize;
        {$this->mszHTMLForm}.submit();
    }

    return;
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "AddLayerLabels";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * Add layer labels
 */
function {$szJsFunctionName}(szLayer, szAttribute, szFont, szFontSize)
{

    CWCDHTML_ShowLayer("ActivityLayer");

    goCWCJSAPI.NAV_CMD = '';

    aHiddenVars = new Array(2);
    aHiddenVars[0] = new Array(2);
    aHiddenVars[0][0] =  "LABEL_LAYER";
    aHiddenVars[0][1] = szLayer;
    aHiddenVars[1] = new Array(2);
    aHiddenVars[1][0] =  "LABEL_ATTRIBUTE";
    aHiddenVars[1][1] = szAttribute;
    aHiddenVars[2] = new Array(2);
    aHiddenVars[2][0] =  "LABEL_FONT";
    aHiddenVars[2][1] = szFont;
    aHiddenVars[2][0] =  "LABEL_FONTSIZE";
    aHiddenVars[2][1] = szFontSize;

    szOnLoad = 'LayerLabelsAdded()';

    goCWCJSAPI.CallServer(szOnLoad, aHiddenVars);
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;
        $szJsFunctionName = "LayerLabelsAdded";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when a label layer is added(JSAPI)
 */
function {$szJsFunctionName}()
{
        goCWCJSAPI.TriggerEvent(MAP_NEW_LAYER_ADDED);
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        // return the array of functions
        return $aReturn;

    }

    function GetJavascriptVariables()
    {

       $aResult = array();
       $aResult = $this->moButton->GetJavascriptVariables();

    }
}

?>
