<?php
/**
* LayerOrder Widget class
*
* @project     Chameleon
* @revision    $Id: LayerOrder.widget.php,v 1.6 2007/06/20 14:29:33 nsavard Exp $
* @purpose     LayerOrderDown Widget class
* @author      DM Solutions Group (pspencer@dmsolutions.ca)
* @copyright
* <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

/*
* this code is based on code contributed by GeoDan
*/

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");
/**
* LayerOrder
*
* @desc Display a clickable image that will modify the drawing order of the
* Map image by moving the selected layer up or down in the drawing order.
*/
class LayerOrder extends CWCWidget
{
	var $mszDirection = "";

	/**
	* LayerOrder
	*
	* Constructor for the LayerOrder
	*/
	function LayerOrder()
	{
		// invoke constructor of parent
		parent::CWCWidget();
		$this->mnPriority = PRIORITY_HIGH;

		// set the description for this widget
		$this->szWidgetDescription = <<<EOT
The LayerOrder widget modifies the layer drawing order by promoting or
demoting the selected layer.  The selected layer is normally indicated by
using the SelectLayer widget in conjunction with a suitably crafted
HTML Legend Template, although it may also work with the XMLThemeLegend or
some other widgets that affect layer selection.
EOT;

		$this->moButton = new CWCButton($this);

		$this->maAttributes["DIRECTION"] = new StringAttribute( "DIRECTION", true, array( "UP", "DOWN", "HOME", "END" ) );
        $this->mnMaturityLevel = MATURITY_BETA;
	}

	function InitDefaults()
	{
		parent::InitDefaults();
		//this widget should never belong to a toolset
		$this->maParams["TOOLSET"] = "";

		if (isset($this->maParams['DIRECTION']))
		{
			$this->mszDirection = strtoupper($this->maParams['DIRECTION']);
		}

		$this->moButton->SetOnClick("clickLayerOrder", $this->mszDirection);
		$this->moButton->InitDefaults();
	}

	/**
	* GetHTMLHiddenVariables
	*
	* Return HTML hidden variables.
	*/
	function GetHTMLHiddenVariables()
	{
		$aReturn = $this->moButton->GetHTMLHiddenVariables();

		$szVariable = "LAYERORDER_MOVE".$this->mnId;
		$szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
		$aReturn[$szVariable] = $szValue;

		return $aReturn;
	}

	/**
	* GetJavascriptFunctions
	*
	* Return the Javacriptfunctions needed by the widget.
	*/
	function GetJavascriptFunctions()
	{
		$aReturn = $this->moButton->GetJavascriptFunctions();


                // add javascript API functions
                if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
                    $bCWCJSAPI = 1;
                else
                    $bCWCJSAPI = 0;

		$szJsFunctionName = "clickLayerOrder";
		$nId = $this->mnId;
		$szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the user clicks the LayerOrder icon
 */

function {$szJsFunctionName}(obj, direction)
{
    with({$this->mszHTMLForm})
    {
       	LAYERORDER_MOVE{$nId}.value = direction;
       	NAV_CMD.value = '';

        if ($bCWCJSAPI)
        {
            var sOldTool = goCWCJSAPI.NAV_CMD;

            if(direction == "UP")
            {
                goCWCJSAPI.oMap.GetLayerByName(SELECTED_LAYERS.value).Promote();
            }
            else
            {

                goCWCJSAPI.oMap.GetLayerByName(SELECTED_LAYERS.value).Demote();
            }
            goCWCJSAPI.NAV_CMD = '';
            goCWCJSAPI.RefreshMap();
            goCWCJSAPI.NAV_CMD = sOldTool;
            {$this->mszHTMLForm}.NAV_CMD.value = sOldTool;

        }
        else
        {
            submit();
        }
    }
    return;
}

EOT;
		$aReturn[$szJsFunctionName] = $szFunction;

		return $aReturn;
	}
	/**
	* return javascript functions
	*/
	function GetJavascriptIncludeFunctions()
	{
		return $this->moButton->GetJavascriptIncludeFunctions();
	}

	/**
	* return javascript variables
	*/
	function GetJavascriptVariables()
	{
		return $this->moButton->GetJavascriptVariables();
	}

	/**
	* return javascript onload functions
	*/
	function GetJavascriptOnLoadFunctions()
	{
		return $this->moButton->GetJavascriptOnLoadFunctions();
	}

	/**
	* return javascript initialization funcitons
	*/
	function GetJavascriptInitFunctions()
	{
		return $this->moButton->GetJavascriptInitFunctions();
	}

	/**
	* DrawPublish
	*
	* Return the HTML code using the name in the map file and the
	* parameters of the CWC tag.
	*/
	function DrawPublish()
	{
		if (!$this->mbVisible)
		return "<!-- LayerOrder widget hidden -->";
		return $this->moButton->DrawPublish();
	}
	/**
	* ParseURL
	*
	* Look for selected layers and move them UP
	*/
	function  ParseURL()
	{
		if ($this->isVarSet('LAYERORDER_MOVE'.$this->mnId))
		{
			$szDirection = strtoupper($this->getVar( 'LAYERORDER_MOVE'.$this->mnId ));
			$oMap = $this->moMapObject->oMap;

			$aszSelectedLayers = array();
			if ($this->isVarSet('SELECTED_LAYERS'))
			{
				$szSelectedLayers = $this->getVar( 'SELECTED_LAYERS' );
				if (strlen($szSelectedLayers) > 0)
				{
					$aszSelectedLayers = explode( ",", $szSelectedLayers );
				}
			}
			else
			{
				$nLayers = $oMap->numlayers();
				for ($i=0;$i<$nLayers;$i++)
				{
					$oLayer=$oMap->getLayer($i);
					if ($oLayer->getMetaData("selected") == "1")
					array_push($aszSelectedLayers, $oLayer->name);
				}
			}

			if (count($aszSelectedLayers) > 0)
			{
				$nLayers = $oMap->numlayers;
				for($i=0; $i < $nLayers; $i++)
				{
					$oLayer = $oMap->getLayer($i);
					if (in_array( $oLayer->name, $aszSelectedLayers ))
					{
                        if ($szDirection == "HOME" && $i < $nLayers - 1)
                        {
                            while ($oMap->moveLayerDown($i) != 0) { }
                        }
                        if ($szDirection == "END" && $i > 0)
                        {
                            while ($oMap->moveLayerUp($i) != 0) { }
                        }
						if ($szDirection == "UP" && $i < $nLayers - 1)
						{
							$oMap->moveLayerDown($i);
						}
						else if ($szDirection == "DOWN" && $i > 0)
						{
							$oMap->moveLayerUp($i);
						}
					}
				}
			}
		}
		return true;
	}
}
?>
