<?php
/**
 * Query Widget class
 *
 * @project     CWC2
 * @revision    $Id: LinkQuery.widget.php,v 1.4 2007/06/20 14:18:21 nsavard Exp $
 * @purpose     Query Widget class
 * @author      DM Solutions Group (assefa@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 
include_once(CHAMELEON_PATH."/widgets/NavTool.php");
include_once(COMMON."wrapper/map_query.php" );
include_once(CHAMELEON_PATH."/widgets/Popup.php");

/**
 * a navtool that open a popup at with the query result information in the URL
 */
class LinkQuery extends NavTool
{
    var $moPopup;
    var $mszImagePath = "images/tool_query_off.gif";
    var $mszSelectedImagePath = "images/tool_query_on.gif";
    var $mszHoverImagePath = "images/tool_query_hover.gif";
    var $mszImageTip = "Zoom In";
    var $mszResultScript = '';
    var $mszEmptyResultScript = '';
    var $mszLinkQueryLayer = '';
    var $mbCallPopup = false;
    
    /**
     * construct a new query widget
     */
    function LinkQuery()
    {
        // invoke constructor of parent
        parent::NavTool();

        $this->SetNavCommand("LINKQUERY");
        $this->moPopup = new CWCPopup($this);

        $this->maAttributes["RESULTSCRIPT"] = new StringAttribute("RESULTSCRIPT", true);
        $this->maAttributes["EMPTYRESULTSCRIPT"] = new StringAttribute( "EMPTYRESULTSCRIPT", false );
        $this->maAttributes["LINKQUERYLAYER"] = new StringAttribute("LINKQUERYLAYER", true);


        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The Query widget displays a GetFeatureInfo query result for each visible layer
at the point the user clicks.
EOT;

        $this->mnMaturityLevel = MATURITY_BETA;
    }

    /**
     * initialize default values for this widget
     */
    function InitDefaults()
    {
        parent::InitDefaults();
        if (isset($this->maParams["RESULTSCRIPT"]))
            $this->mszResultScript = $this->maParams["RESULTSCRIPT"];
        if (isset($this->maParams["EMPTYRESULTSCRIPT"]))
            $this->mszEmptyResultScript = $this->maParams["EMPTYRESULTSCRIPT"];
        if (isset($this->maParams["LINKQUERYLAYER"]))
            $this->mszLinkQueryLayer = $this->maParams["LINKQUERYLAYER"];

        if (isset($_SESSION['LINKQUERYLAYER']))
            $this->mszLinkQueryLayer = $_SESSION['LINKQUERYLAYER'];
        if (isset($_SESSION['RESULTSCRIPT']))
            $this->mszResultScript = $_SESSION['RESULTSCRIPT'];
        if (isset($_SESSION['EMPTYRESULTSCRIPT']))
            $this->mszEmptyResultScript = $_SESSION['EMPTYRESULTSCRIPT'];
    }

    /**
     * handle changes coming from the URL.  In this case, we only want
     * to do a query and then change the url of the popup with the info from
     * the query
     */
    function ParseURL()
    {
        $szCmd = "";
        $szNavInputCoord = "";

        if ($this->isVarSet( "NAV_CMD" ))
            $szCmd = $this->getVar( "NAV_CMD" );
        if ($this->isVarSet( "NAV_INPUT_COORDINATES" ))
            $szNavInputCoord = $this->getVar( "NAV_INPUT_COORDINATES" );


        if($szCmd == "LINKQUERY")
        {
            $aszMinMax = explode(";", $szNavInputCoord );
            $adMin = explode(",", $aszMinMax[0]);
            $aszLayers = array();
            $oMap = $this->moMapObject->oMap;
            $bQueryable = false;

            // get layer name
            $aszLinkQueryLayers = explode(',', $this->mszLinkQueryLayer);
            foreach($aszLinkQueryLayers as $szLinkQueryLayer)
            {
                $oLayer = $oMap->getLayerByName( $szLinkQueryLayer );
                if($oLayer)
                {
                    if ($oLayer->status == MS_ON || 
                        $oLayer->status == MS_DEFAULT)
                    {
                        if ($oLayer->template != "")
                        {
                            $bQueryable = true;
                        }
                        else
                        {
                            for( $j=0; $j < $oLayer->numclasses; $j++)
                            {
                                $oClass = $oLayer->getClass($j);
                                if ($oClass->template != "")
                                {
                                    $bQueryable = true;
                                    break;
                                }
                            }
                        }
                    }
                }

                // only add if on
                if ( $bQueryable )
                {
                    array_push( $aszLayers, $oLayer->name );
                    $szName = $oLayer->name;
                }
            }

            // Check if there's any queryable layers
            if(count($aszLayers) >= 1)
            {
                $oMapQuery = new MapQuery( $this->moMapObject );

                // commit
                $oMapQuery->setQueryLayersByName( $aszLayers );

                // execute query
                if ( is_numeric($adMin[0]) )
                    $oResultSet = $oMapQuery->executePointQuery( $adMin[0],
                                                                 $adMin[1] );
                // Check if there's any result
                if($oResultSet)
                {
                    $nCount = $oResultSet->nNumResults;

                    // process each result set
                    for($i=0; $i<$nCount; $i++)
                    {
                        $aRows = $oResultSet->getResultSet( $i );
                        if(is_array($aRows))
                        {
                            foreach ($aRows[0] as $key => $value)
                            {
                                $this->mszResultScript = str_replace(
                                    "%$key%", $value, $this->mszResultScript);
                            }

                            $this->mbCallPopup = true;
                            break;
                        }
                    }
                }
                if(!$this->mbCallPopup && $this->mszEmptyResultScript)
                {
                    $this->mszResultScript = $this->mszEmptyResultScript;
                    $this->mbCallPopup = true;
                }
            }
        }

        return parent::ParseURL();
    }

    /**
     * get javascript mousemove functions
     */
    function GetJavascriptOnMouseMoveFunctions()
    {
        $aReturn = array();
        $szJsFunctionName = "LinkQueryWMouseMove";
        $szFunction = "$szJsFunctionName(e);\n";
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }
    
    function GetJavascriptInitFunctions()
    {
        $aResult = parent::GetJavascriptInitFunctions();
        
        $aResult['QueryRegisterEvent'] = "goEventManager.registerEventID( 'ON_QUERY' );\n";

        if($this->mbCallPopup)
            $aResult['LinkQueryCallPopup'] = "LinkQueryCallPopup();";

        return $aResult;
    }
    
    /**
     * get general javascript functions
     */
    function GetJavascriptFunctions()
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
            $bCWCJSAPI = 1;
        else
            $bCWCJSAPI = 0;

        //Need to pass these variables in the session since UpdateMap.php is 
        //called via UpdateNavTools() in cwcjsapi.js when a user clicks on 
        //the map.  So there is no way to pass these variables without
        //modifying UpdateNavTools().
        if ($bCWCJSAPI)
        {
            $_SESSION['LINKQUERYLAYER'] = $this->mszLinkQueryLayer;
            $_SESSION['RESULTSCRIPT'] = $this->mszResultScript;
            $_SESSION['EMPTYRESULTSCRIPT'] = $this->mszEmptyResultScript;
        }

        $aReturn = parent::GetJavascriptFunctions();
     
        $this->moPopup->mszLink = $this->mszResultScript;
        $szPopupName = $this->moPopup->GetPopupName();
        $szPopupURL = $this->moPopup->GetPopupURL();
        $szPopupOptions = $this->moPopup->GetPopupOptions();
                
        $szJsFunctionName = "LinkQueryCallPopup";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the CallPopup is true.
 */
function {$szJsFunctionName}(szURL)
{ 
    szPopupName = "{$szPopupName}";
    szPopupOptions = "{$szPopupOptions}";

    if ({$bCWCJSAPI})
    {
        szPopupURL = szURL;

    }
    else
    {
        szPopupURL = "{$szPopupURL}";
    }

    window.open( szPopupURL, szPopupName, szPopupOptions );
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;
        
        $szJsFunctionName = "LinkQueryWMouseClick";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the mouse click for the LinkQuery widget.
 */
function {$szJsFunctionName}(e)
{
    if ({$this->mszHTMLForm}.NAV_CMD.value != "{$this->mszCmd}")
        return true;

    MapWgetMouse(e);
    if ( !MapWinsideMap() )
        return true;

    nX = {$this->mszHTMLForm}.MAP_CURSOR_POS_X.value;
    nY = {$this->mszHTMLForm}.MAP_CURSOR_POS_Y.value;
    szNav = "NAV_INPUT_COORDINATES=" + nX + "," + nY;

    goEventManager.triggerEvent( 'ON_QUERY', nX, nY );
    return true;
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "LinkQueryWMouseMove";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the mouse moves for the LinkQuery widget.
 */
function {$szJsFunctionName}(e)
{
    //make sure that the forms variables are initialized.
    //It seems like that in IE, the mouse move may be called before.
    if({$this->mszHTMLForm} == null ||
      {$this->mszHTMLForm}.NAV_CMD == null ||
      {$this->mszHTMLForm}.NAV_CMD.value != "{$this->mszCmd}")
      return true;

    //document.onmousedown = LinkQueryWMouseClick;
    document.onmouseup = LinkQueryWMouseClick;

    var x = (navigator.appName == "Netscape")? e.pageX : event.x+document.body.scrollLeft;
    var y = (navigator.appName == "Netscape")? e.pageY : event.y+document.body.scrollTop;
    if ((navigator.appName == "Netscape") && e.target!=document)
      routeEvent(e);

    return true;

}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;


        return $aReturn;
    }

}
?>

