<?php
/**
* LocateByAttribute class
*
* @project     CWC2
* @revision    $Id: LocateByAttribute.widget.php,v 1.7 2007/05/17 01:18:25 jdelfos Exp $
* @purpose     Find objects on basis of an attribute value, and zoom to the results
* @author      Jacob Delfos (Maunsell Australia; jacob.delfos@maunsell.com)
* @copyright
* <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");
include_once(dirname(__FILE__)."/../Popup.php");

/**
* LocateByAttribute Widget
*
* @desc A widget that puts an IFRAME or link to popup in the page, which has 
*       input elements that allow a user to find the objects with a specific 
*       value for a selected attribute.
*
*       TODO: 
*       - when projection is changed on-the-fly without submitting, the 
*         zoom-to results will have invalid coordinates.
*       - add support for other formats (postgis, tiles)
*/
/**
* LocateByAttribute Widget
*
* @desc A widget that puts an IFRAME or link to popup in the page, which has input
*       elements that allow a user to find the objects with a specific 
*       value for a selected attribute.
*/
class LocateByAttribute extends CWCWidget
{

   function LocateByAttribute ()
   {
   parent::CWCWidget();

   $this->maAttributes["SEARCHALL"] = new StringAttribute( "SEARCHALL", false );
   $this->maAttributes["ZOOMBUFFER"] = new StringAttribute( "ZOOMBUFFER", false );
   $this->maAttributes["POPUP"] = new StringAttribute( "POPUP", false );
   $this->maAttributes["ALLOWDOWNLOAD"] = new StringAttribute( "ALLOWDOWNLOAD", false );
   $this->maAttributes["HIGHLIGHT"] = new StringAttribute( "HIGHLIGHT", false );
   $this->maAttributes["MAXNUMRESULTS"] = new StringAttribute( "MAXNUMRESULTS", false );
   $this->mnMaturityLevel = MATURITY_BETA;
   

        $this->moButton = new CWCButton( $this );
        $this->moPopup = new CWCPopup( $this );

   }



   function InitDefaults()
   {
   parent::InitDefaults();
   $this->maParams["TOOLSET"] = "";
   $this->moButton->InitDefaults();
   $this->moButton->SetOnClick("clickLocateByAttributePopup");
   
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;   
        
        /* build URL  */

           $this->searchPageURL = $_SESSION['gszCoreWebPath']."/widgets/LocateByAttribute/search.phtml?".SID;
              
           if (isset($this->maParams["SEARCHALL"])&& (strcasecmp($this->maParams["SEARCHALL"],"true") == 0))
           {
           $this->searchPageURL .= "&searchall=true";
           }
           
           if (isset($this->maParams["ZOOMBUFFER"]))
           {
           $this->searchPageURL .= "&pointzoombuffer=".$this->maParams["ZOOMBUFFER"];
           } 
           
           if ((isset($this->maParams["POPUP"])) && (strcasecmp($this->maParams["POPUP"],"true") == 0))
           {
           $this->searchPageURL .= "&launchpopup=true";
           }
                      
           if ((isset($this->maParams["ALLOWDOWNLOAD"])) && (strcasecmp($this->maParams["ALLOWDOWNLOAD"],"true") == 0))
           {
           $this->searchPageURL .= "&allowdownload=true";
           } 

           if ((isset($this->maParams["HIGHLIGHT"])) && (strcasecmp($this->maParams["HIGHLIGHT"],"true") == 0))
           {
           $this->searchPageURL .= "&highlight=true";
           }

           if ((isset($this->maParams["MAXNUMRESULTS"])) && ($this->maParams["MAXNUMRESULTS"]) > 0)
           {
           $this->searchPageURL .= "&maxnumresults=".$this->maParams["MAXNUMRESULTS"];
           }  
           
           if ($bCWCJSAPI==1)
           {
           $this->searchPageURL .= "&usejavascript=true";
           }

           $this->moPopup->mszLink = $this->searchPageURL;   
   
   }

    function GetJavascriptVariables()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptVariables();
        else return array();
    }
    

    function GetJavascriptInitFunctions()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptInitFunctions();
        else return array();
    }

    function GetJavascriptIncludeFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptIncludeFunctions();
        else
            $aReturn = array();
        return $aReturn;
    }
            
    function GetJavascriptFunctions()
    {

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;


        $szJsFunctionName = "applyBoundingBox";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * This function has been copied from the BoundingBox widget
 */
function {$szJsFunctionName}(nMinX, nMinY, nMaxX, nMaxY)
{
    szCoords = "" + nMinX + "," + nMinY;
    if (nMaxX != "" && nMaxY != "")
    {
        szType = "RECTANGLE";
        szCoords = szCoords + "," + nMaxX + "," + nMaxY;
    }
    else
    {
        szType = "POINT";
    }
    {$this->mszHTMLForm}.NAV_CMD.value="ZOOM_BBOX";
    {$this->mszHTMLForm}.NAV_INPUT_TYPE.value=szType;
    {$this->mszHTMLForm}.NAV_INPUT_COORDINATES.value=szCoords;
    
    if (arguments.length == 5)
    {
        {$this->mszHTMLForm}.NAV_INPUT_SRS.value=arguments[4];    
    }

    if ({$bCWCJSAPI})
    {
        goCWCJSAPI.NAV_CMD="ZOOM_BBOX";
        goCWCJSAPI.NAV_INPUT_TYPE = szType;
        goCWCJSAPI.NAV_INPUT_COORDINATES=szCoords;
        if (arguments.length == 5)
        {
            goCWCJSAPI.NAV_INPUT_SRS=arguments[4];
        }
        goCWCJSAPI.UpdateNavTools();
    }
    else
    {
        {$this->mszHTMLForm}.submit();
    }
     return;
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;
        
        
        $szJsFunctionName = "clickLocateByAttributePopup";
        
        $szButtonJS = $this->moPopup->DrawPublish();
        $szFunction = <<<EOT
      /**
       * {$szJsFunctionName}
       * popup a LocateByAttribute dialog
       */
      function {$szJsFunctionName}()
      {
          {$szButtonJS}
          return;
      }
EOT;
      $aReturn[$szJsFunctionName] = $szFunction;
      


        $szJsFunctionName = "goToRectangle";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * This function goes to bounding coordinates, but keeps the navigation info correct
 */
function {$szJsFunctionName} (minx, miny, maxx, maxy)
{
    if (gCWCJSAPI)
    {
    temp = goCWCJSAPI.NAV_CMD;
    }

   if ((minx==maxx) || (miny == maxy))
   {
   applyBoundingBox(minx, miny, "","");
   }
   else
   {
   applyBoundingBox(minx, miny, maxx, maxy);
   }
   
    if (gCWCJSAPI)
    {
    goCWCJSAPI.NAV_CMD = temp; 
    document.forms[0].NAV_CMD.value = temp;  
    }
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;



        $szJsFunctionName = "zoomToPoint";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * This function recenters the map on a point
 */
function {$szJsFunctionName} (x, y)
{
    if (gCWCJSAPI)
    {
    temp = goCWCJSAPI.NAV_CMD;
    }

applyBoundingBox(x, y, "","");

    if (gCWCJSAPI)
    {
    goCWCJSAPI.NAV_CMD = temp; 
    document.forms[0].NAV_CMD.value = temp;  
    }
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;


        return $aReturn;
    }




    /**
     * DrawPublish
     *
     * Return the HTML code using the name in the map file and the
     * parameters of the CWC tag.
     */
    function DrawPublish()
    {
   
        if (!$this->mbVisible)
            return "<!-- LocateByAttribute widget hidden -->";
        else
        $objectheight = $this->maParams["POPUPHEIGHT"];
        $objectwidth = $this->maParams["POPUPWIDTH"];
                                

           // now determine whether to search from IFRAME or POPUP
           if ((isset($this->maParams["POPUP"])) && (strcasecmp($this->maParams["POPUP"],"true") == 0))
           {
           $widgetCode=$this->moButton->DrawPublish();
           } 
           else
           {
           $widgetCode = "<IFRAME ID='searchByLocation' BORDER=0 HEIGHT=".$objectheight." WIDTH=".$objectwidth." FRAMEBORDER=0 SRC='".$this->searchPageURL."'></IFRAME>";
           }

        
        return $widgetCode;
    }

}
?>
