<?php
/**
 * MapSize Class
 *
 * @project     CWC2
 * @revision    $Id: MapSize.widget.php,v 1.18 2007/04/05 18:08:33 nsavard Exp $
 * @purpose     Display a dialog box to type in a new map size
 * @author      DM Solutions Group (spencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");
include_once(dirname(__FILE__)."/../Popup.php");
include_once(dirname(__FILE__)."/../Popin.php");
include_once(dirname(__FILE__)."/../Label.php");

/**
 * MapSize is an embeddable widget that can be used to modify the
 * current map size (context window)
 */
class MapSize extends CWCWidget
{
    var $moButton;
    var $moPopin;
    var $moLabel;
    var $mszWidgetClass = '';
    var $bRequiresPopin = false;

    /**
     * constructor
     */
    function MapSize()
    {
        // invoke constructor of parent
        parent::CWCWidget();
        $this->moPopin = new CWCPopin($this);
        $this->moButton = new CWCButton($this);
        $this->moLabel = new CWCLabel($this);

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The MapSize widget displays a widget that allows the user to
specify new dimensions for the map image.  Note that the MapDHTML widget must
have ALLOWRESIZE set to true for this to have any effect.
EOT;

        $this->maAttributes["WIDGETCLASS"] = new StringAttribute("WIDGETCLASS", false);
        $this->maAttributes['EMBEDDED'] = new BooleanAttribute( 'EMBEDDED', false );

        $this->mnMaturityLevel = MATURITY_TECHNICALRELEASE;
    }

    function InitDefaults()
    {
        parent::InitDefaults();
        //this widget should never belong to a toolset
        $this->maParams["TOOLSET"] = "";
        if (isset($this->maParams["EMBEDDED"]) &&
            strcasecmp(trim($this->maParams["EMBEDDED"]), "FALSE") == 0)
            $this->mbEmbedded = false;
        else
            $this->mbEmbedded = true;

        // set the embedded attribute
        $this->maAttributes['EMBEDDED'] = $this->mbEmbedded;

        // default the width and height of the popup (can be overriden in the tag
        // if necessary)
        $this->moPopin->mnWidth = isset($this->maParams["POPUPWIDTH"])?
                                            $this->maParams["POPUPWIDTH"]:480;
        $this->moPopin->mnHeight = isset($this->maParams["POPUPHEIGHT"])?
                                            $this->maParams["POPUPHEIGHT"]:200;

        // set the icon and title
        $this->moPopin->mszTitle = $this->moMLT->get( 'MapSizePopup', '0',
                                                                'Resize Map' );
        $this->moPopin->mszIcon = $_SESSION['gszCoreWebPath'].
                                                      '/images/icon_mapsize.png';
        $this->moPopin->mnIconWidth = 19;
        $this->moPopin->mnIconHeight = 18;

        $this->moPopin->InitDefaults();


        if (!$this->mbEmbedded)
        {
            $this->moButton->SetOnClick( 'launchPopin' );

            $this->moButton->InitDefaults();
        }

        $this->moPopin->mszLink = $_SESSION['gszCoreWebPath']."/widgets/MapSize/MapSizePopup.phtml";
        $this->moLabel->InitDefaults();

        if(isset($this->maParams["WIDGETCLASS"]) && $this->maParams["WIDGETCLASS"] != "")
          $this->mszWidgetClass = $this->maParams["WIDGETCLASS"];
    }

    /**
     * inspect URL parameters for map size changes
     */
    function ParseURL()
    {
        if (!$this->isVarSet( "SET_MAP_HEIGHT" ) &&
            !$this->isVarSet( "SET_MAP_WIDTH" ))
            return true;

        $oMap = $this->moMapObject->oMap;


/* ==================================================================== */
/*      only update the width,height id SET_MAP_HEIGHT &&               */
/*      SET_MAP_WIDTH are defined. Other wise the width and height      */
/*      management would pass through the mapdhtml widget. (Note : There is*/
/*      a duplication of code between this widget and the mapdhtml      */
/*      widget that should be removed at one point)                     */
/* ==================================================================== */
         if ($this->isVarSet( "SET_MAP_HEIGHT" ) &&
            $this->getVar( "SET_MAP_HEIGHT" ) != "" &&
            $this->getVar( "SET_MAP_HEIGHT" ) > 0 &&
            $this->isVarSet( "SET_MAP_WIDTH" ) &&
            $this->getVar( "SET_MAP_WIDTH" ) != "" &&
            $this->getVar( "SET_MAP_WIDTH" ) > 0)
          {
              $nHeight = intval($this->getVar( "SET_MAP_HEIGHT" ));

              $nWidth = intval($this->getVar( "SET_MAP_WIDTH" ));

              $this->moMapObject->oMap->set( "width", $nWidth );
              $this->moMapObject->oMap->set( "height", $nHeight );
          }

          return true;
    }

    /**
     * GetHTMLHiddenVariables
     *
     * return the map size  variables.
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();

        $szVariable = "SET_MAP_WIDTH";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "SET_MAP_HEIGHT";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    function GetJavascriptVariables()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptVariables();
        else return array();
    }

    function GetJavascriptInitFunctions()
    {
        if ($this->mbVisible)
        {
            $aReturn = $this->moButton->GetJavascriptInitFunctions();
        }
        else
        {
            $aReturn = array();
        }
        return $aReturn;
    }

    function GetJavascriptOnLoadFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptOnLoadFunctions();
        else
            $aReturn = array();
        return $aReturn;
    }

    function GetJavascriptIncludeFunctions()
    {
        if ($this->mbVisible)
        {
            $aReturn = $this->moButton->GetJavascriptIncludeFunctions();
            $aReturn = array_merge( $aReturn,
                            $this->moPopin->GetJavascriptIncludeFunctions() );
        }
        else
            $aReturn = array();
        return $aReturn;
    }


    /**
     * GetJavascriptFunctions
     *
     * Build and return the array of functions needed in the
     * widget.
     */
    function GetJavascriptFunctions()
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        // get popin functions
        $aReturn = $this->moPopin->GetJavascriptFunctions();


        $szBaseLink = $this->moPopin->mszLink."?";
        if (isset( $this->maszContents["MAPSIZE"] ))
        {
            $szQueryString = "MAPSIZE=";
            $szSep = "";
            foreach( $this->maszContents["MAPSIZE"] as $aMapSize )
            {
                if (isset($aMapSize["LABEL"]) && isset($aMapSize["WIDTH"]) &&
                    isset($aMapSize["HEIGHT"]))
                {
                    if (((int)$aMapSize["WIDTH"] > 0) && ((int)$aMapSize["HEIGHT"] > 0))
                    {
                        $szQueryString .= $szSep.$aMapSize["LABEL"].",".$aMapSize["WIDTH"].",".$aMapSize["HEIGHT"];
                        $szSep = "|";
                    }
                }
                else
                {
                    $_SESSION['gErrorManager']->setError(ERR_WARNING,
                      $this->moMLT->get( 'MapSize', '0', 'ERROR: Invalid MAPSIZE tag definition in MapSize Tag' ));
                }
            }
            $szBaseLink .= $szQueryString;
        }

        $this->moPopin->mszLink = $szBaseLink;
        $szJsFunctionName = "clickMapSizePopup";

        $szJsFunctionName = "applyMapSize";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * apply a new map size.
 */
function {$szJsFunctionName}(nWidth, nHeight)
{
      if ({$bCWCJSAPI})
      {
          SetMapSize(nWidth, nHeight);
      }
      else
      {
          {$this->mszHTMLForm}.NAV_CMD.value = "";
          {$this->mszHTMLForm}.SET_MAP_WIDTH.value=nWidth;
          {$this->mszHTMLForm}.SET_MAP_HEIGHT.value=nHeight;
          {$this->mszHTMLForm}.submit();
      }

     return;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "cancelMapSize";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * cancel the user changing the map size.
 */
function {$szJsFunctionName}()
{
     obj = {$this->mszHTMLForm}.MAP_SIZE;
     aMapSize = getMapSize();
     szMapSize = aMapSize[0] + "," + aMapSize[1];
     for (i=0; i<obj.options.length; i++ )
     {
        val = obj.options[i].value;
        if (val == szMapSize)
        {
           obj.selectedIndex = i;
            break;
        }
     }
     return;
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        $nWidth = $this->moMapObject->oMap->width;
        $nHeight = $this->moMapObject->oMap->height;

        $szJsFunctionName = "getMapSize";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * return the current extents as an array (width,height)
 */
function {$szJsFunctionName}()
{
    if ({$bCWCJSAPI})
    {
        return new Array( goCWCJSAPI.oMap.width, goCWCJSAPI.oMap.height );
    }
    else
    {
        return new Array( {$nWidth}, {$nHeight} );
    }
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "changeMapSize";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the mapsize value is changed.
 */
function {$szJsFunctionName}(obj)
{
    mapsize=obj.options[obj.selectedIndex].value;
    pos = mapsize.indexOf(",");
    width = mapsize.substring( 0, pos );
    height = mapsize.substring( pos + 1, mapsize.length );
    if (width == "custom" || height == "custom")
        if ({$bCWCJSAPI})
        {
            SetMapSize(width, height);
        }
        else
        {
            clickMapSizePopup();
        }
    else
    {
        curMapSize = getMapSize();
        if (curMapSize[0] != width || curMapSize[1] != height )
        {
            if ({$bCWCJSAPI})
            {
                SetMapSize(width, height);
            }
            else
            {
                {$this->mszHTMLForm}.NAV_CMD.value = "";
                {$this->mszHTMLForm}.SET_MAP_WIDTH.value = width;
                {$this->mszHTMLForm}.SET_MAP_HEIGHT.value = height;
                {$this->mszHTMLForm}.submit();
            }
        }
    }
    return;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;
        $szJsFunctionName = "SetMapSize";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * Change the map size
 */
function {$szJsFunctionName}(nNewWidth, nNewHeight)
{
    CWCDHTML_ShowLayer("ActivityLayer");

    var sOldTool = goCWCJSAPI.NAV_CMD;
    goCWCJSAPI.NAV_CMD = "";
    {$this->mszHTMLForm}.NAV_CMD.value = "";
    goCWCJSAPI.oMap.width = nNewWidth;
    goCWCJSAPI.oMap.height = nNewHeight;

    aHiddenVars = new Array(2);
    aHiddenVars[0] = new Array(2);
    aHiddenVars[0][0] =  "SET_MAP_WIDTH";
    aHiddenVars[0][1] = nNewWidth;
    aHiddenVars[1] = new Array(2);
    aHiddenVars[1][0] =  "SET_MAP_HEIGHT";
    aHiddenVars[1][1] = nNewHeight;

    szOnLoad = 'MapSizeChanged()';

    goCWCJSAPI.CallServer(szOnLoad, aHiddenVars);

    goCWCJSAPI.NAV_CMD = sOldTool;
    {$this->mszHTMLForm}.NAV_CMD.value = sOldTool;

}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;
        $szJsFunctionName = "MapSizeChanged";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the size of the map is changed(JSAPI)
 */
function {$szJsFunctionName}()
{
        goCWCJSAPI.TriggerEvent(MAP_SIZE_CHANGED);
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

    /**
     * Draw the widget
     */
    function DrawPublish()
    {
        if (!$this->mbVisible)
            return "<!-- MapSize widget hidden -->";

        $nHeight = $this->moMapObject->oMap->height;
        $nWidth = $this->moMapObject->oMap->width;

        $szWidgetClass = '';
        if ($this->mszWidgetClass != '')
        {
            $szWidgetClass = " class='".$this->mszWidgetClass."'";
        }

        if ($this->mbEmbedded)
        {
            $szResult = "<SELECT$szWidgetClass NAME=\"MAP_SIZE\" onchange=\"changeMapSize(this)\">\n";

            if (isset( $this->maszContents["MAPSIZE"] ))
            {
                $bCustom = true;
                foreach( $this->maszContents["MAPSIZE"] as $aMapSize )
                {
                    if (isset($aMapSize["LABEL"]) && isset($aMapSize["WIDTH"]) &&
                        isset($aMapSize["HEIGHT"]))
                    {
                        $szMapSize = $aMapSize["WIDTH"].",".$aMapSize["HEIGHT"];
                        $szResult .= "<OPTION VALUE=\"".$szMapSize."\"";
                        if ($nHeight == $aMapSize["HEIGHT"] && $nWidth == $aMapSize["WIDTH"])
                        {
                            $szResult .= " SELECTED";
                            $bCustom = false;
                        }
                        $szResult .= ">".$aMapSize["LABEL"]."</OPTION>\n";
                    }
                    else
                    {
                        $_SESSION['gErrorManager']->setError(ERR_WARNING,
                          $this->moMLT->get( 'MapSize', '0', 'ERROR: Invalid MAPSIZE tag definition in MapSize Tag' ));
                    }
                }
                if ($bCustom)
                {
                    $szResult .= "<OPTION VALUE=\"".$nWidth.",".$nHeight;
                    $szResult .= "\" SELECTED>".$nWidth."x".$nHeight."</OPTION>\n";
                }
            }
            $szResult .= "</SELECT>\n";
            $szResult = $this->moLabel->DrawPublish( $szResult );
        }
        else
        {
            $szResult = $this->moButton->DrawPublish();
        }
        return $szResult;
    }
}
?>
