<?php
/**
 * Nav tool base class
 *
 * @project     CWC2
 * @revision    $Id: NavTool.php,v 1.46 2007/09/06 14:14:18 jlacroix Exp $
 * @purpose     LegendPopup Widget class
 * @author      DM Solutions Group (assefa@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * javascript events:
 *
 * NAV_TOOL_CHANGED
 * useage:
 *
 * function myOnLoad()
 * {
 *    CWC2OnLoadFunction();
 *    goEventManager.registerForEvent( 'NAV_TOOL_CHANGED', 'myNavToolChanged' );
 * }
 * 
 * function myNavToolChanged( sTool )
 * {
 *     alert( 'nav tool selected: ' + sTool );
 * } 
 */
include_once("Widget.php");
include_once("Button.php");
include_once( COMMON."wrapper/map_navigator.php" );

/**
 * class NavTool is a common base class for widgets that want to navigate
 * and might be part of a toolset
 */
class NavTool extends CWCWidget
{
    var $moButton;
    var $mszCmd = "";
    var $moMapNavigator;
    var $mnAllowRectangle = 0;
    var $mbSubmitOnClick = 0;
    /* control submission of the page after navigation.  By default,
     * this is true.  Only widgets that need to override this should
     * do so (for instance QUERY).
     */
    var $mbNavSubmit = true;
    var $mszMapId = "DEFAULT_MAP";
    var $mszCursorType = "AUTO";

    /**
     * construct a new navtool
     */
    function NavTool()
    {
        // invoke constructor of parent
        parent::CWCWidget();
        
        // set the description for this widget
        $this->szWidgetDescription = "No description available.  Please set ".
                        "this description in the constructor of this class.";

        $this->moButton = new CWCButton($this);
        $this->maAttributes["WIDGETSTYLE"] = new StringAttribute( "WIDGETSTYLE", false );
        $this->maAttributes["MAPID"] = new StringAttribute( "MAPID", false );
        $this->maAttributes["CURSORTYPE"] = new StringAttribute( "CURSORTYPE", false );
        $this->maAttributes['SUBMITONCLICK'] = new BooleanAttribute( 'SUBMITONCLICK', false );

        $this->mnPriority = PRIORITY_LOW;
    }

    /**
     * initialize default values from tag
     */
    function InitDefaults()
    {
        parent::InitDefaults();

        if ($this->mbVisible)
        {
            $this->moButton->InitDefaults();
        }
        else
            $this->maParams["IMAGE"] = "DUMMY";

        if (isset($this->maParams["MAPID"]))
            $this->maszMapId = $this->maParams["MAPID"];

        if (isset($this->maParams["CURSORTYPE"]))
            $this->mszCursorType = $this->maParams["CURSORTYPE"];
        
        if (isset($this->maParams['SUBMITONCLICK']))
        {
            $this->mbSubmitOnClick = strcasecmp($this->maParams['SUBMITONCLICK'], 'true') == 0?true:false;
        }
        
        if ($this->mbSubmitOnClick)
            $this->moButton->mszToolset = '';
    }


    /**
     * set the command this tool uses to navigate
     */
    function SetNavCommand($szNavCmd)
    {
        $this->mszCmd = $szNavCmd;
    }


    /**
     * set the map navigator for this nav tool
     */
    function SetMap(&$oMapSession)
    {
        $this->moMapObject =& $oMapSession;

        $this->moMapNavigator = new MapNavigator( $oMapSession );
    }


    /**
     * return javascript functions required for this tool
     */
    function GetJavascriptFunctions()
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptFunctions();
        else
            $aReturn = array();

        $szJsFunctionName = "NavSetActiveTool";
        $szFunction = <<<EOT
/**
 * set the active navigation tool
 */
function {$szJsFunctionName}( oButton, aVals )
{
    var nTool = aVals[0];
    var sTool = aVals[1];
    var bAllowRectangle = (aVals[2] == 0 || aVals[2] == "0") ? false : true;
    var bAutoSubmit = (aVals[3] == 0 || aVals[3] == "0") ? false : true;
    var szCursorType = aVals[4];
    var nNavSubmit = (aVals[5] != undefined) ? aVals[5] : 1;

    // Keep the old information for the jsapi mode.
    if({$bCWCJSAPI})
    {
        var sOldTool = goCWCJSAPI.NAV_CMD;
        var bOldAllowRectangle ={$this->mszHTMLForm}.NAV_ALLOW_RECTANGLE.value;
        var bOldAutoSubmit = {$this->mszHTMLForm}.NAV_SUBMIT.value;
    }

    {$this->mszHTMLForm}.NAV_CMD.value = sTool;
    {$this->mszHTMLForm}.NAV_ALLOW_RECTANGLE.value=bAllowRectangle;
    {$this->mszHTMLForm}.NAV_SUBMIT.value=nNavSubmit;

    if ({$bCWCJSAPI})
    {
        goCWCJSAPI.NAV_CMD = sTool;
    }

    if (bAutoSubmit)
    {
        if ({$bCWCJSAPI})
        {
            goCWCJSAPI.UpdateNavTools();

            goCWCJSAPI.NAV_CMD = sOldTool;
            {$this->mszHTMLForm}.NAV_CMD.value = sOldTool;
            {$this->mszHTMLForm}.NAV_ALLOW_RECTANGLE.value=bOldAllowRectangle;
            {$this->mszHTMLForm}.NAV_SUBMIT.value=bOldAutoSubmit;
        }
        else
        {
            {$this->mszHTMLForm}.submit();
        }    
    }

    if ({$this->mszHTMLForm}.CURSOR_TYPE != null)
    {
      {$this->mszHTMLForm}.CURSOR_TYPE.value = szCursorType;
    }
    
    goEventManager.triggerEvent( 'NAV_TOOL_CHANGED', sTool );
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

    function GetJavascriptVariables()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptVariables();
        else return array();
    }

    function GetJavascriptInitFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
        {        
            $aResult = $this->moButton->GetJavascriptInitFunctions();
        }
        
        $aResult['NavToolRegisterEvent'] = "goEventManager.registerEventID( 'NAV_TOOL_CHANGED' );\n";
        
        return $aResult;
    }

    function GetJavascriptOnLoadFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptOnLoadFunctions();
        else
            $aReturn = array();
        
        return $aReturn;
    }
    
    function GetJavascriptIncludeFunctions()
    {
        $aReturn = $this->moButton->GetJavascriptIncludeFunctions();
        
        $szVar = "cwc_events.js";
        $aReturn[$szVar] = '<script src="'.$_SESSION['gszCoreWebPath'].
                            '/widgets/js/cwc_events.js" type="text/javascript"></script>';
        
        return $aReturn;
    }

    /**
     * handle the user using this tool as part of a toolset
     */
    function  ParseURL()
    {
        $aVals = array();
        array_push( $aVals, $this->mnId );
        array_push( $aVals, $this->mszCmd );
        array_push( $aVals, $this->mnAllowRectangle );
        array_push( $aVals, $this->mbSubmitOnClick ? 1 : 0);
        array_push( $aVals, $this->mszCursorType );
        array_push( $aVals, $this->mbNavSubmit ? 1 : 0 );

        $this->moButton->SetOnClick('NavSetActiveTool', $aVals);


        if ($this->isVarSet( strtoupper($this->mszMapId)."_X" ) &&
            $this->isVarSet( strtoupper($this->mszMapId)."_Y" ))
        {
            $this->setVar( "NAV_INPUT_COORDINATES" , 
                           $this->getVar( strtoupper($this->mszMapId)."_X" ).",".
                           $this->getVar( strtoupper($this->mszMapId)."_Y" ) );
        }
        
        $this->moButton->ParseURL();

        return true;
    }

    /**
     * return hidden variables required for this tool
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();
        
        $szVariable = "NAV_CMD";
        $szTool = '';
        if ($this->isVarSet( 'NAV_CMD' ))
            $szTool = $this->getVar( 'NAV_CMD' );

        $szValue = "<INPUT TYPE='HIDDEN' NAME='$szVariable' VALUE='$szTool'>";
        $aReturn[$szVariable] = $szValue;

        $szAllowRect = "";
         if ($this->isVarSet( "NAV_ALLOW_RECTANGLE" ))
          $szAllowRect = $this->getVar( "NAV_ALLOW_RECTANGLE" );

        $szVariable = "NAV_ALLOW_RECTANGLE";
        $szValue = "<INPUT TYPE='HIDDEN' NAME='$szVariable' VALUE='$szAllowRect'>";
        $aReturn[$szVariable] = $szValue;

        $szNavSubmit = "1";
        if ($this->isVarSet( "NAV_SUBMIT" ))
          $szNavSubmit = $this->getVar( "NAV_SUBMIT" );
          
        $szVariable = "NAV_SUBMIT";
        $szValue = "<INPUT TYPE='HIDDEN' NAME='$szVariable' VALUE='$szNavSubmit'>";
        $aReturn[$szVariable] = $szValue;
        
        return $aReturn;
    }


    /**
     * draw this widget on the page
     */
    function DrawPublish()
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        if (!$this->mbVisible)
            return "<!-- NavTool hidden -->";

        $szReturn = $this->moButton->DrawPublish();

        return $szReturn;
    }
}
