<?php
/**
  _____________________________________________________________________________
 |
 | Popin Class
 |
 | @project     Chameleon
 | @revision    $Id: Popin.php,v 1.2 2005/12/15 15:44:48 wbronsema Exp $
 | @purpose     Helper class to manage Popin window and attributes.
 | @author      William A. Bronsema, C.E.T. (wbronsema@dmsolutions.ca)
 | @copyright
 | <b>Copyright (c) 2005 DM Solutions Group Inc.</b>
 | Permission is hereby granted, free of charge, to any person obtaining a
 | copy of this software and associated documentation files (the "Software"),
 | to deal in the Software without restriction, including without limitation
 | the rights to use, copy, modify, merge, publish, distribute, sublicense,
 | and/or sell copies of the Software, and to permit persons to whom the
 | Software is furnished to do so, subject to the following conditions:
 |
 | The above copyright notice and this permission notice shall be included
 | in all copies or substantial portions of the Software.
 |
 | THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 | IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 | FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 | THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 | LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 | FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 | DEALINGS IN THE SOFTWARE.
 |_____________________________________________________________________________

 **/

class CWCPopin
{
    // define member vars
    var $moParent;
    var $mnWidth = 500;
    var $mnHeight = 500;
    var $mnLeft;
    var $mnTop;
    var $mszTitle = '';
    var $mszIcon = '';
    var $mnIconWidth = 20;
    var $mnIconHeight = 20;
    var $mszLink;
    var $mszLayerName;
    var $mszFrameName;

   /**
      _________________________________________________________________________
     |
     |  CWCPopin()
     |
     |  Postcondition:  This is the CWCPopin class constructor function.  It
     |                  will construct a new CWCPopin instance and initialize
     |                  it.
     |
     |  @param $oParent object - Parent object to work with.
     |  @desc CWCPopin class constructor.
     |  @return void.
     |_________________________________________________________________________

     **/
    function CWCPopin( &$oParent )
    {

        // parent object
        $this->moParent =& $oParent;

        // width of window
        $oParent->maAttributes['POPINWIDTH'] =
            new IntegerAttribute( 'POPINWIDTH', false, 0 );

        // height of window
        $oParent->maAttributes['POPINHEIGHT'] =
            new IntegerAttribute( 'POPINHEIGHT', false, 0 );

        // left position of window
        $oParent->maAttributes['POPINLEFT'] =
            new IntegerAttribute( 'POPINLEFT', false, 0 );

        // top position of window
        $oParent->maAttributes['POPINTOP'] =
            new IntegerAttribute( 'POPINTOP', false, 0 );

        // window title
        $oParent->maAttributes['POPINTITLE'] =
            new StringAttribute( 'POPINTITLE', false );

        // window icon
        $oParent->maAttributes['POPINICON'] =
            new StringAttribute( 'POPINICON', false );
        $oParent->maAttributes['POPINICONWIDTH'] =
            new IntegerAttribute( 'POPINICONWIDTH', false );
        $oParent->maAttributes['POPINICONHEIGHT'] =
            new IntegerAttribute( 'POPINICONHEIGHT', false );

        // style resource
        $oParent->maAttributes['POPINSTYLERESOURCE'] =
            new StringAttribute( 'POPINSTYLERESOURCE', false );

    // end CWCPopin() function
    }

    /**
      _________________________________________________________________________
     |
     |  InitDefaults()
     |
     |  Postcondition:  This function intializes all defaults.
     |
     |  @desc Initialize member variables.
     |  @return void.
     |_________________________________________________________________________

     **/
    function InitDefaults()
    {
        // initalize member vars
        $this->mszLink = '';

        // setup the style resource
        //$this->SetStyleResource();

        // set window size
        $this->mnWidth = isset($this->moParent->maParams['POPINWIDTH'])?
            $this->moParent->maParams['POPINWIDTH']:$this->mnWidth;
        $this->mnHeight  = isset($this->moParent->maParams['POPINHEIGHT'])?
            $this->moParent->maParams['POPINHEIGHT']:$this->mnHeight;

        // set window positon
        $this->mnLeft = isset($this->moParent->maParams['POPINLEFT'])?
            $this->moParent->maParams['POPINLEFT']:50;
        $this->mnTop = isset($this->moParent->maParams['POPINTOP'])?
            $this->moParent->maParams['POPINTOP']:50;

        // set window title and icon
        $this->mszTitle = isset($this->moParent->maParams['POPINTITLE'])?
            $this->moParent->maParams['POPINTITLE']:$this->mszTitle;
        $this->mnIconWidth = isset($this->moParent->maParams['POPINICONWIDTH'])?
            $this->moParent->maParams['POPINICONWIDTH']:$this->mnIconWidth;
        $this->mnIconHeight = isset($this->moParent->maParams['POPINICONHEIGHT'])?
            $this->moParent->maParams['POPINICONHEIGHT']:$this->mnIconHeight;
        $this->mszIcon = isset($this->moParent->maParams['POPINICON'])?
            $this->moParent->maParams['POPINICON']:$this->mszIcon;



        // set window style
        //$this->mszPopinStyleResource =
            //isset($this->moParent->maParams['POPINSTYLERESOURCE'])?
            //$this->moParent->maParams['POPINSTYLERESOURCE']:'';

        // set the frame and layer names
        $szPopinName = $this->GetPopinName();
        $this->mszLayerName = 'div'.$szPopinName;
        $this->mszFrameName = $szPopinName;

    // end InitDefaults() function
    }

    /**
      _________________________________________________________________________
     |
     |  GetPopinpName()
     |
     |  Postcondition:  This function returns a unique name for the popin.
     |
     |  @desc Get a unique name for the popin.
     |  @return string - Unique name.
     |_________________________________________________________________________

     **/
    function GetPopinName()
    {
        // start the name
        $szName = 'popin';

        // add the type if set
        $szName .= isset($this->moParent->maParams['TYPE'])?
            $this->moParent->maParams['TYPE']:'';

        // return the name
        return  $szName.'_'.$this->moParent->mnId.'_'.session_id();

    // end GetPopinName() function
    }

    /**
      _________________________________________________________________________
     |
     |  GetJavascriptIncludeFunctions()
     |
     |  Postcondition:  This function defines the javascript include statements.
     |
     |  @return Array - Array of include statements.
     |  @desc Defines javascript include statements.
     |_________________________________________________________________________

     **/
    function GetJavascriptIncludeFunctions()
    {
        // init vars
        $aReturn = array();

        // include necessary dhtml to manipulate popin
        $szVar = 'cwc_dhtml.js';
        $aReturn[$szVar] = '<script language="JavaScript" src="'.
            $_SESSION['gszCoreWebPath'].
            '/widgets/js/cwc_dhtml.js" type="text/javascript"></script>';

        $szVar = 'popin.css';
        $aReturn[$szVar] = '<link rel="stylesheet" type="text/css" href="'.
            $_SESSION['gszCoreWebPath'].'/widgets/css/'.$szVar.'" />';

        // return
        return $aReturn;

    // end GetJavascriptIncludeFunctions() function
    }
    /**
      _________________________________________________________________________
     |
     |  GetJavascriptFunctions()
     |
     |  Postcondition:  This function defines javascript functions and adds
     |                  them to the array.
     |
     |  @return Array - Array of javascript functions.
     |  @desc Defines javascript functions.
     |_________________________________________________________________________

     **/
    function GetJavascriptFunctions()
    {
        // init vars
        $aReturn = array();
        $szSID = SID;
        $szCloseIcon = $_SESSION['gszCoreWebPath'].'/images/icon_close.png';
        $szJoin = strstr( $this->mszLink, '?' ) === false?'?':'&';
        $szLink = $this->mszLink.$szJoin.SID;
        $szTitle = $this->mszTitle;
        $szIcon = strlen( $this->mszIcon ) > 0?'<img class="titleBarIcon" src="'.
            $this->mszIcon.'" width="'.$this->mnIconWidth.'" height="'.
            $this->mnIconHeight.'">':'';


        // add popin show function
        $szJsFunctionName = 'launchPopin';
        $szFunction = <<<EOT
/**
  _____________________________________________________________________________
 |
 |  {$szJsFunctionName}
 |  Displays popin window.
 |_____________________________________________________________________________

 **/
function {$szJsFunctionName}()
{
    // get the current highest zIndex
    var maxZIndex = PopinGetHighestZ();

    // lock application
    CWCDHTML_ShowMaskLayer( maxZIndex + 1 );

    // store div name
    szCurrentDiv = '{$this->mszLayerName}';

    // create the layer
    var newDiv = document.createElement('div');
    newDiv.id = '{$this->mszLayerName}';
    var divContents = '<div class="titleBar"><img class="titleBarClose" src="{$szCloseIcon}" width="20" height="20" onclick="closePopin()">{$szIcon}<span class="titleBarText">{$szTitle}</span></div><iframe class="PopinFrame" name="{$this->mszFrameName}" id="{$this->mszFrameName}" src="$szLink" frameborder="0" scrolling="auto" style="width:{$this->mnWidth}px; height:{$this->mnHeight}px;" allowtransparency="true"></iframe>';
    newDiv.innerHTML = divContents;
    newDiv.className = 'popinChrome';
    newDiv.style.width = '{$this->mnWidth}px';
    newDiv.style.position = 'absolute';
    newDiv.style.top = '{$this->mnTop}px';
    newDiv.style.left = '{$this->mnLeft}px';
    newDiv.style.zIndex = maxZIndex + 2;
    document.body.appendChild( newDiv );

}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        // add popin hide function
        $szJsFunctionName = 'closePopin';
        $szFunction = <<<EOT
/**
  _____________________________________________________________________________
 |
 |  {$szJsFunctionName}
 |  Hides popin window.
 |_____________________________________________________________________________

 **/
function {$szJsFunctionName}()
{
    // only close if current div is set
    if ( szCurrentDiv == '' )
    {
        return;
    }

    // destroy popin layer
    var oDiv = document.getElementById( szCurrentDiv );
    document.body.removeChild( oDiv );

    // clear current div
    szCurrentDiv = '';

    // hide mask
    CWCDHTML_HideMaskLayer();

}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        // add getHighestZ function
        $szJsFunctionName = 'PopinGetHighestZ';
        $szFunction = <<<EOT
/**
  _____________________________________________________________________________
 |
 |  {$szJsFunctionName}
 |  gets highest Z index
 |_____________________________________________________________________________

 **/
function {$szJsFunctionName}()
{
    var documentDivs = new Array();
    documentDivs = document.getElementsByTagName("DIV")
    var highestZ = 0;
    for (var i = 0; i < documentDivs.length; i++)
    {
        var zIndex = documentDivs[i].style.zIndex;
        highestZ = (zIndex > highestZ) ? zIndex : highestZ;
    }

    return highestZ;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;


        // return
        return $aReturn;

    // end GetJavascriptFunctions() function
    }

    /**
      _________________________________________________________________________
     |
     |  GetJavascriptVariables()
     |
     |  OPTIONAL - Use this function to define any javascript variables that
     |  are required by this widget.
     |
     |  Postcondition:  This function defines javascript variables and adds
     |                  them to the array.
     |
     |  Example:
     |
     |  $aReturn['MyJSVar'] = 'var szMyJSVar = "Default Value";'."\n";
     |
     |  @return Array - Array of javascript variables.
     |  @desc Defines javascript variables.
     |_________________________________________________________________________

     **/

    function GetJavascriptVariables()
    {
        // init vars
        $aReturn['szCurrentDiv'] = 'var szCurrentDiv = "Default Value";'."\n";

        // return
        return $aReturn;

    // end GetJavascriptVariables() function
    }

}
?>
