<?php
/**
 * Query Widget class
 *
 * @project     CWC2
 * @revision    $Id: Query.widget.php,v 1.21 2007/08/13 13:53:49 nsavard Exp $
 * @purpose     Query Widget class
 * @author      DM Solutions Group (assefa@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * javascript events:
 *
 * NAV_TOOL_CHANGED : see NavTool.php
 *
 * ON_QUERY
 * useage:
 *
 * function myOnLoad()
 * {
 *    CWC2OnLoadFunction();
 *    goEventManager.registerForEvent( 'ON_QUERY', 'myOnQuery' );
 * }
 *
 * function myOnQuery( nX, nY)
 * {
 *     alert( 'myOnQuery: ' + nX + ',' + nY );
 * }
 */

include_once(dirname(__FILE__)."/../NavTool.php");
include_once( COMMON."wrapper/map_query.php" );
include_once( dirname(__FILE__)."/../Popup.php");

/**
 * a navtool that displays query results at the clicked location
 */
class Query extends NavTool
{
    var $moPopup;
    var $mnRadius = 1;
    var $mnMaxFeatures = "";
    var $mszImagePath = "images/tool_query_off.gif";
    var $mszSelectedImagePath = "images/tool_query_on.gif";
    var $mszHoverImagePath = "images/tool_query_hover.gif";
    var $mszImageTip = "Query";
    var $mszTableLayout = 'auto';
    var $maszTableHeader = '';
    var $mbIncludeEmptyResults = false;
    var $mbHighlightResults = false;
    var $mbPopupResults = true;
    var $mbPersistentResults = false;
    var $mbSubmitPage = false;
    var $mszIFrameName = '';
    var $mszDefaultIFrameSrc = "default_info.html";
    var $mszDetailIFrameName = '';
    var $mszDefaultDetailIFrameSrc = "default_info2.html";
    var $mszResultFieldsMetadata = "RESULT_FIELDS_LIMITED";
    var $mszMinScaleZoom = "";

    /**
     * construct a new query widget
     */
    function Query()
    {
        // invoke constructor of parent
        parent::NavTool();

        $this->SetNavCommand("QUERY");
        $this->maParams['POPUPWIDTH'] = 400;
        $this->maParams['POPUPHEIGHT'] = 600;
        $this->moPopup = new CWCPopup($this);
        $this->moPopup->mszLink = $_SESSION['gszCoreWebPath']."widgets/Query/QueryResults.phtml";
        $this->maAttributes["RADIUS"] = new IntegerAttribute( "RADIUS", false, 1 );
        $this->maAttributes["MAXFEATURES"] = new IntegerAttribute( "MAXFEATURES", false, 1 );
        $this->maAttributes["INCLUDEEMPTYRESULTS"] = new BooleanAttribute( "INCLUDEEMPTYRESULTS", false );
        $this->maAttributes['ALLOWRECTANGLE'] = new BooleanAttribute( 'ALLOWRECTANGLE', false );
        $this->maAttributes['HIGHLIGHTRESULTS'] = new BooleanAttribute( 'HIGHLIGHTRESULTS', false );
        $this->maAttributes['POPUPRESULTS'] = new BooleanAttribute( 'POPUPRESULTS', false );
        $this->maAttributes['PERSISTENTRESULTS'] = new BooleanAttribute( 'PERSISTENTRESULTS', false );
        $this->maAttributes['SUBMITPAGE'] = new BooleanAttribute( 'SUBMITPAGE', false );
        $this->maAttributes["IFRAMENAME"] = new StringAttribute("IFRAMENAME", false);
        $this->maAttributes["DEFAULTIFRAMESRC"] = new StringAttribute("DEFAULTIFRAMESRC", false);
        $this->maAttributes["DETAILIFRAMENAME"] = new StringAttribute("DETAILIFRAMENAME", false);
        $this->maAttributes["DEFAULTDETAILIFRAMESRC"] = new StringAttribute("DEFAULTDETAILIFRAMESRC", false);
        $this->maAttributes['LAYOUT'] = new StringAttribute('LAYOUT', false, array('AUTO', 'VERTICAL', 'HORIZONTAL'));
        $this->maAttributes['RESULTFIELDSMETADATA'] = new StringAttribute('RESULTFIELDSMETADATA', false);
        $this->maAttributes['MINSCALEZOOM'] = new StringAttribute('MINSCALEZOOM', false);



        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The Query widget displays a GetFeatureInfo query result for each visible layer
at the point the user clicks.
EOT;

        $this->mnMaturityLevel = MATURITY_TECHNICALRELEASE;
    }

    /**
     * initialize default values for this widget
     */
    function InitDefaults()
    {
        parent::InitDefaults();
        if (isset($this->maParams["RADIUS"]))
            $this->mnRadius = strtoupper($this->maParams["RADIUS"]);
        if (isset($this->maParams["MAXFEATURES"]))
            $this->mnMaxFeatures = strtoupper($this->maParams["MAXFEATURES"]);
        if (isset($this->maParams["MAXFEATURES"]))
            $this->mnMaxFeatures = strtoupper($this->maParams["MAXFEATURES"]);
        if (isset($this->maParams["INCLUDEEMPTYRESULTS"]))
        {
            $this->mbIncludeEmptyResults = strcasecmp($this->maParams["INCLUDEEMPTYRESULTS"], "true") == 0 ? true : false;
        }
        $this->moPopup->InitDefaults();
        if (isset($this->maParams['ALLOWRECTANGLE']))
        {
            $this->mnAllowRectangle = strcasecmp($this->maParams["ALLOWRECTANGLE"], "true") == 0 ? true : false;
        }
        if (isset($this->maParams['HIGHLIGHTRESULTS']))
        {
            $this->mbHighlightResults = strcasecmp($this->maParams["HIGHLIGHTRESULTS"], "true") == 0 ? true : false;
        }
        if (isset($this->maParams['POPUPRESULTS']))
        {
            $this->mbPopupResults = strcasecmp($this->maParams["POPUPRESULTS"], "true") == 0 ? true : false;
        }
        if (isset($this->maParams['PERSISTENTRESULTS']))
        {
            $this->mbPersistentResults = strcasecmp($this->maParams["PERSISTENTRESULTS"], "true") == 0 ? true : false;
        }
        if (isset($this->maParams['SUBMITPAGE']))
        {
            $this->mbSubmitPage = strcasecmp($this->maParams["SUBMITPAGE"], "true") == 0 ? true : false;
        }
        if (isset($this->maParams["IFRAMENAME"]) && $this->maParams["IFRAMENAME"] != "")
        {
            $this->mszIFrameName =  $this->maParams["IFRAMENAME"];
        }
        if (isset($this->maParams["DEFAULTIFRAMESRC"]) && $this->maParams["DEFAULTIFRAMESRC"] != "")
        {
            $this->mszDefaultIFrameSrc =  $this->maParams["DEFAULTIFRAMESRC"];
        }
        if (isset($this->maParams["DETAILIFRAMENAME"]) && $this->maParams["DETAILIFRAMENAME"] != "")
        {
            $this->mszDetailIFrameName =  $this->maParams["DETAILIFRAMENAME"];
        }
        if (isset($this->maParams["DEFAULTDETAILIFRAMESRC"]) && $this->maParams["DEFAULTDETAILIFRAMESRC"] != "")
        {
            $this->mszDefaultDetailIFrameSrc =  $this->maParams["DEFAULTDETAILIFRAMESRC"];
        }
        if (isset($this->maParams['LAYOUT']))
        {
            $this->mszTableLayout = strtolower($this->maParams["LAYOUT"]);
        }
        if (isset($this->maParams['RESULTFIELDSMETADATA']))
        {
            $this->mszResultFieldsMetadata = strtolower($this->maParams["RESULTFIELDSMETADATA"]);
        }
        if (isset($this->maParams['MINSCALEZOOM']))
        {
            $this->mszMinScaleZoom = strtolower($this->maParams["MINSCALEZOOM"]);
        }
        if (isset($this->maszContents['TABLEHEADER']))
        {
            foreach($this->maszContents['TABLEHEADER'] as $aszHeader)
            {
                if(isset($aszHeader['LAYER']) && isset($aszHeader["HEADER"]))
                    $this->maszTableHeader[] =& $aszHeader;
            }
        }
        $this->mbNavSubmit = false;

    }

    /**
     * get javascript mousemove functions
     */
    function GetJavascriptOnMouseMoveFunctions()
    {
        $aReturn = array();
        $szJsFunctionName = "QueryWMouseMove";
        $szFunction = "$szJsFunctionName(e);\n";
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

    function GetJavascriptInitFunctions()
    {
        $aResult = parent::GetJavascriptInitFunctions();

        $aResult['QueryRegisterEvent'] = "goEventManager.registerEventID( 'ON_QUERY' );\n";

        $aResult['QueryShowShapeDetailRegisterEvent'] = "goEventManager.registerEventID( 'ON_QUERYSHOWSHAPEDETAIL' );\n";

        return $aResult;
    }

    /**
     * GetHTMLHiddenVariables
     *
     * Return HTML hidden variables.
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = array();
        $szVariable = "ShowShapeDetail_LayerName";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "ShowShapeDetail_ShapeId";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "ShowShapeDetail_TileIndex";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "ShowShapeDetail_bZoomShape";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }


    /**
     * get general javascript functions
     */
    function GetJavascriptFunctions()
    {
        $aReturn = parent::GetJavascriptFunctions();

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
            $bCWCJSAPI = 1;
        else
            $bCWCJSAPI = 0;

        $szRadius = $this->mnRadius;
        if ($szRadius != "")
            $szRadius = "&RADIUS=".$szRadius;

        $szMaxFeatures = $this->mnMaxFeatures;
        if ($szMaxFeatures != "")
            $szMaxFeatures = "&FEATURE_COUNT=".$szMaxFeatures;

        if ($this->mbIncludeEmptyResults)
            $szIncludeEmpty = "&include_empty=true";
        else
            $szIncludeEmpty = "&include_empty=false";

        $this->moPopup->mszParam = $szRadius.$szMaxFeatures.$szIncludeEmpty;
        $this->moPopup->mszParam.= "&LAYOUT={$this->mszTableLayout}";
        $this->moPopup->mszParam.= "&RESULTFIELDSMETADATA={$this->mszResultFieldsMetadata}";
        $this->moPopup->mszParam.= "&MINSCALEZOOM={$this->mszMinScaleZoom}";

        if($this->maszTableHeader != "")
        {
            foreach($this->maszTableHeader as $aszHeader)
            {
                $oLayer = @$this->moMapObject->oMap->getLayerByName(
                    $aszHeader["LAYER"] );
                if($oLayer)
                    $oLayer->set("header", $aszHeader["HEADER"]);
            }
        }

        $szPopupName = $this->moPopup->GetPopupName();
        $szPopupURL = $this->moPopup->GetPopupURL();
        $szPopupOptions = $this->moPopup->GetPopupOptions();

        $szPopupResults = $this->mbPopupResults ? 'true' : 'false';
        $szHighlightResults = $this->mbHighlightResults ? 'true' : 'false';
        $szPersistentResults = $this->mbPersistentResults ? 'true' : 'false';
        $szSubmitPage = $this->mbSubmitPage ? 'true' : 'false';

        $szJsFunctionName = "QueryWMouseClick";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the mouse moves for the Query widget.
 */
function {$szJsFunctionName}(e)
{
    var szCoord = "";

    if ({$this->mszHTMLForm}.NAV_CMD.value != "{$this->mszCmd}")
        return true;

    MapWgetMouse(e);
    if ( !MapWinsideMap() )
        return true;

    if ({$this->mnAllowRectangle} && !gMapWdragging && MapWinsideMap()) {
        gMapWx1 = 0;
        gMapWy1 = 0;
        gMapWx2 = 0;
        gMapWy2 = 0;
        MapWstartZoomBox(e);
        gMapWx2--;
        gMapWy2--;
        return false;
    } 
    else if ({$this->mnAllowRectangle} && gMapWdragging) 
    {
        gMapWmouseX = Math.min(Math.max(gMapWmouseX, 0), gMapWiWidth);
        gMapWmouseY = Math.min(Math.max(gMapWmouseY, 0), gMapWiHeight);
        MapWgetMouse(e);
        TrackMouseXY(e);
        
        gMapWdragging = false;
        gMapWsecondx = gMapWx2;
        gMapWsecondy = gMapWy2;
        //TODO : values for the smallest rectangle availablee
        if (gMapWfirstx == gMapWsecondx && gMapWfirsty == gMapWsecondy)
        {
            szCoord = ""+ gMapWfirstx + "," + gMapWfirsty;
            szCoord += ";"+szCoord;
            gMapDHTMLForm.NAV_INPUT_COORDINATES.value = szCoord;
            gMapDHTMLForm.NAV_INPUT_TYPE.value = "POINT";

            //Without this condition the query only works once
            if({$bCWCJSAPI} != 1)
            {
                document.onmousemove = null;
                document.onmouseup = null;
                document.onmousedown = null;

            }
            CWCDHTML_HideLayer("zBoxTop");
            CWCDHTML_HideLayer("zBoxLeft");
            CWCDHTML_HideLayer("zBoxRight");
            CWCDHTML_HideLayer("zBoxBottom");
        } 
        else 
        {
            // Zoom to box
            var tx1 = Math.min(gMapWfirstx, gMapWsecondx);
            var tx2 = Math.max(gMapWfirstx, gMapWsecondx);
            var ty1 = Math.min(gMapWfirsty, gMapWsecondy);
            var ty2 = Math.max(gMapWfirsty, gMapWsecondy);
        
            szCoord = ""+ tx1 + "," + ty1 + ";" + tx2 + "," + ty2;

            gMapDHTMLForm.NAV_INPUT_COORDINATES.value = szCoord;
            gMapDHTMLForm.NAV_INPUT_TYPE.value = "RECTANGLE";
            if({$bCWCJSAPI} != 1)
            {
                document.onmousemove = null;
                document.onmouseup = null;
                document.onmousedown = null;
            }
            CWCDHTML_HideLayer("zBoxTop");
            CWCDHTML_HideLayer("zBoxLeft");
            CWCDHTML_HideLayer("zBoxRight");
            CWCDHTML_HideLayer("zBoxBottom");
        }
    }
    else if(!{$this->mnAllowRectangle})
    {
        szCoord = {$this->mszHTMLForm}.MAP_CURSOR_POS_X.value+','+
                  {$this->mszHTMLForm}.MAP_CURSOR_POS_Y.value;
    }
    if(szCoord == "")
        return false;

    var nX = {$this->mszHTMLForm}.MAP_CURSOR_POS_X.value;
    var nY = {$this->mszHTMLForm}.MAP_CURSOR_POS_Y.value;
    szNav = "NAV_INPUT_COORDINATES=" + szCoord;

    if ({$szPopupResults})
    {
        szPopupURL = "{$szPopupURL}";
        szPopupName = "{$szPopupName}";
        szPopupOptions = "{$szPopupOptions}";
        szPopupURL = szPopupURL + "&" + szNav;

        window.open( szPopupURL, szPopupName, szPopupOptions );
    }
    if ({$szHighlightResults})
    {
        var img = CWCDHTML_GetImage( 'MapLayerDivImg' );
        img.src = gMapDHTMLURL + "&run_query=1&gszQueryCoords=" + szCoord + "&gszPersistentQuery={$szPersistentResults}";
        window.status = img.src;
    }
    if ("{$this->mszIFrameName}" != "" )
    {
        szPopupURL = "{$szPopupURL}";
        szPopupURL = szPopupURL + "&" + szNav;
        if ("{$this->mszDetailIFrameName}" != "" )
        {
            //Reset the iframe to its default
            window.frames['{$this->mszDetailIFrameName}'].location.href = "{$this->mszDefaultDetailIFrameSrc}";

            szPopupURL = szPopupURL + "&nFrames=2";
        }
        else
        {
            szPopupURL = szPopupURL + "&nFrames=1";
        }
        window.frames['{$this->mszIFrameName}'].location.href = szPopupURL + "&run_query=1";
    }
    if ({$szSubmitPage})
    {
        {$this->mszHTMLForm}.submit();
    }
    goEventManager.triggerEvent( 'ON_QUERY', nX, nY );
    return true;
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "QueryWMouseMove";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the mouse moves for the Query widget.
 */
function {$szJsFunctionName}(e)
{

    //make sure that the forms varaibales are initialized.
    //It seems like that in IE, the mouse move may be called before.
    if({$this->mszHTMLForm} == null ||
      {$this->mszHTMLForm}.NAV_CMD == null ||
      {$this->mszHTMLForm}.NAV_CMD.value != "QUERY")
      return true;

    if({$this->mnAllowRectangle})
        document.onmousedown = QueryWMouseClick;
    document.onmouseup = QueryWMouseClick;

    if ({$this->mnAllowRectangle} && gMapWdragging) 
    {
        gMapWx2 = gMapWmouseX = Math.min(Math.max(gMapWmouseX, 0), gMapWiWidth);
        gMapWy2 = gMapWmouseY = Math.min(Math.max(gMapWmouseY, 0), gMapWiHeight);
        MapWsetClip();
        return false;
    } 

    var x = (navigator.appName == "Netscape")? e.pageX : event.x+document.body.scrollLeft;
    var y = (navigator.appName == "Netscape")? e.pageY : event.y+document.body.scrollTop;
    if ((navigator.appName == "Netscape") && e.target!=document)
      routeEvent(e);

    return true;

}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;


        $szJsFunctionName = "ShowShapeDetail";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * Sumbit page so that it shows detail on a specific shape
 */
function {$szJsFunctionName}(layername, shapeid, shapename, tileindex, bzoomshape)
{
  {$this->mszHTMLForm}.ShowShapeDetail_LayerName.value = layername;
  {$this->mszHTMLForm}.ShowShapeDetail_ShapeId.value = shapeid;
  {$this->mszHTMLForm}.ShowShapeDetail_TileIndex.value = tileindex;
  {$this->mszHTMLForm}.ShowShapeDetail_bZoomShape.value = bzoomshape;
    
  if ({$bCWCJSAPI})
  {
      goEventManager.triggerEvent( 'ON_QUERYSHOWSHAPEDETAIL', shapename );
  }
  ShowShapeDetailPopup();
  
}
EOT;
        if ($this->mszDetailIFrameName != "")
            $aReturn[$szJsFunctionName] = $szFunction;



        //We need this variable in JSAPI mode
        $oMap = &$this->moMapObject->oMap;
        $oLayer = '';
        $szLayerName = '';
        if(isset($this->moURLArray["ShowShapeDetail_LayerName"]))
            $szLayerName =  $this->moURLArray["ShowShapeDetail_LayerName"];
        if($szLayerName)
            $oLayer = @$oMap->getlayerbyname( $szLayerName );
        if ($this->mszDetailIFrameName != "")
        {
            $_SESSION['DetailIFrameName'] = $this->mszDetailIFrameName;

            $szJsFunctionName = "ShowShapeDetailPopup";

            $szURL = $this->moPopup->GetPopupURL();


            $bIframe= 1;
            $szIframeName= $this->mszDetailIFrameName;
            $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * Open a window if displaying dbf fields values
 */

function {$szJsFunctionName}()
{
    var bValid = 1;
    var bIframe = 1;
    var szLayerName = {$this->mszHTMLForm}.ShowShapeDetail_LayerName.value;
    var nShapeId = {$this->mszHTMLForm}.ShowShapeDetail_ShapeId.value;
    var nTileIndex = {$this->mszHTMLForm}.ShowShapeDetail_TileIndex.value;
    var bZoomShape = {$this->mszHTMLForm}.ShowShapeDetail_bZoomShape.value;

    if (bZoomShape == "true")
        bZoomShape = 1;
    else
        bZoomShape = 0;

    var szURLDefault = '{$szURL}&LAYERNAME='+szLayerName+
         '&SHAPEID='+nShapeId+'&TILEINDEX='+nTileIndex+'&nFrames=2'+
         '&bZOOMSHAPE='+bZoomShape;
     szURL = szURLDefault;

     if (document.all)
          var xMax = screen.width, yMax = screen.height;
     else if (document.layers)
       var xMax = window.outerWidth, yMax = window.outerHeight;
     else
       var xMax = 640, yMax=480;

     var xOffset = (xMax - 510)/2, yOffset = (yMax - 390)/2;
     if (bValid)
     {
         if (bIframe)
         {
             frames['{$szIframeName}'].location.href = szURL;
             {$this->mszHTMLForm}.{$szIframeName}.value = szURL;
             {$this->mszHTMLForm}.LASTQUERYTOOL.value = "4";
             showInfoTab();
         }
         else
         {
             window.open(szURL, 'dbfquery', 'scrollbars=yes,resizable=yes,location=no,menubar=np,width=510,height=390,screenX='+xOffset+',screenY='+yOffset+',top='+yOffset+',left='+xOffset+'');
         }

     }
}

EOT;
            $aReturn[$szJsFunctionName] = $szFunction;
        }

        return $aReturn;
    }

}
?>
