<?php
/**
 * QueryByShape Widget class
 *
 * @project     CWC2
 * @revision    $Id: QueryByShape.widget.php,v 1.1 2007/07/20 14:49:34 jlacroix Exp $
 * @purpose     QueryByShape Widget class
 * @author      Mapgears (jlacroix@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2007, Mapgears Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once(dirname(__FILE__)."/../NavTool.php");
include_once(dirname(__FILE__)."/../geomath.php");
include_once( dirname(__FILE__)."/../Popup.php");

/**
 * QueryByShape
 *
 * @desc QueryByShape widget.
 */
class QueryByShape extends NavTool
{
    var $moPopup = "";
    var $mszImageTip = "Query by shape";
    var $mdfZoomMin = 4;
    var $mszLayerName = "";
    var $manColor = array(255, 0, 0);
    var $mbShowPopup = false;
    var $maszResultIds = array();
    var $mszIdCol = 'ID';
    var $mnBuffer = 0;

    /**
     * QueryByShape
     *
     * Constructor. Set the default values.
     */
    function QueryByShape()
    {
        // invoke constructor of parent
        parent::NavTool();

        $this->SetNavCommand("QUERY_SHAPE");
        $this->mnAllowRectangle = 1;
        $this->maAttributes["MINIMUMZOOMRECTANGLE"] = new IntegerAttribute( "MINIMUMZOOMRECTANGLE", false, 1 );
        $this->maAttributes["BUFFER"] = new IntegerAttribute( "BUFFER", false, 1 );
        $this->maAttributes["LAYERNAME"] = new StringAttribute( "LAYERNAME", false );
        $this->maAttributes["IDCOLUMN"] = new StringAttribute( "IDCOLUMN", false );
        $this->maAttributes["COLOR"] = new RGBColorAttribute( "COLOR", false );
        $this->maAttributes["SHOWPOPUP"] = new BooleanAttribute( "SHOWPOPUP", false );

        $this->maParams['POPUPWIDTH'] = 400;
        $this->maParams['POPUPHEIGHT'] = 600;
        $this->moPopup = new CWCPopup($this);
        $this->moPopup->mszLink = $_SESSION['gszCoreWebPath']."widgets/QueryByShape/QueryByShapeResults.phtml";

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The QueryByShape widget is a navigation tool that allows the user to query
layers on the map image based on a previously selected shape 
using either a single point click or by dragging a
zooming box on the map.
EOT;
        $this->mnMaturityLevel = MATURITY_BETA;

    }

    function InitDefaults()
    {
        parent::InitDefaults();
        if (isset( $this->maParams["MINIMUMZOOMRECTANGLE"] ))
            $this->mdfZoomMin = $this->maParams["MINIMUMZOOMRECTANGLE"];
        if (isset( $this->maParams["LAYERNAME"] ))
            $this->mszLayerName = $this->maParams["LAYERNAME"];
        if (isset( $this->maParams["IDCOLUMN"] ))
            $this->mszIdCol = $this->maParams["IDCOLUMN"];
        if (isset( $this->maParams["COLOR"] ))
            $this->manColor = explode(" ", $this->maParams["COLOR"]);
        if (isset( $this->maParams["BUFFER"] ))
            $this->mnBuffer = $this->maParams["BUFFER"];
        $this->moPopup->InitDefaults();

    }
    /**
     * ParseURL
     *
     * Look for the ZOOM_IN command and if founddo the zooming
     * according to the other NAV paramaters.
     */
    function  ParseURL()
    {
        parent::ParseURL();

        $szCmd = "";
        if ($this->isVarSet( "NAV_CMD" ))
            $szCmd = $this->getVar( "NAV_CMD" );

        if ($szCmd == "QUERY_SHAPE")
        {
            $szInputCoords = "";
            $szInputCoords = $this->getVar( "NAV_INPUT_COORDINATES" );
            $szInputType = $this->getVar( "NAV_INPUT_TYPE" );

            if(strlen($szInputCoords) <= 0)
            {
                //The nav tool could be set and the page submitted
                //using other tools so this message is invalid.

                return true;
            }
            if (strlen($szInputType) <= 0)
            {
                $_SESSION['gErrorManager']->setError(ERR_WARNING,
                    $this->moMLT->get( $this->mszWidgetName, '0', 'ERROR: No input type specified in ' )."QueryByShape.php.");
                return false;
            }

            $oMap = $this->moMapObject->oMap;
            if ($szInputType == "POINT")
            {
                $aPixPos = explode(",", $szInputCoords);

                // Get the Geo coords of the click and create a PointObj withit
                $nX = Pix2Geo($aPixPos[0], 0, $oMap->width, 
                              $oMap->extent->minx, $oMap->extent->maxx);
                $nY = Pix2Geo($aPixPos[1], 0, $oMap->height, 
                              $oMap->extent->miny, $oMap->extent->maxy, 1);
                $oPoint = ms_newPointObj();
                $oPoint->setXY($nX, $nY);

                // Extract the layer to query
                $oLayer = @$oMap->getLayerByName($this->mszLayerName);

                // Query by point
                @$oLayer->queryByPoint($oPoint, MS_MULTIPLE, 0);
            }
            else if ($szInputType == "RECTANGLE")
            {
                $aPixPos = explode(";", $szInputCoords);
                $aPixMin = explode(",", $aPixPos[0]);
                $aPixMax = explode(",", $aPixPos[1]);
/* -------------------------------------------------------------------- */
/*      check if the rectangle is bigger than the minmum size allowed.  */
/* -------------------------------------------------------------------- */

                if (abs($aPixMax[0] - $aPixMin[0]) < $this->mdfZoomMin &&
                    abs($aPixMax[1] - $aPixMin[1]) < $this->mdfZoomMin)
                {
                    // Get the Geo coords of the click and create a PointObj
                    $nX = Pix2Geo($aPixMin[0], 0, $oMap->width, 
                                  $oMap->extent->minx, $oMap->extent->maxx);
                    $nY = Pix2Geo($aPixMin[1], 0, $oMap->height, 
                                  $oMap->extent->miny, $oMap->extent->maxy, 1);
                    $oPoint = ms_newPointObj();
                    $oPoint->setXY($nX, $nY);

                    // Extract the layer to query
                    $oLayer = @$oMap->getLayerByName($this->mszLayerName);

                    // Query by point
                    @$oLayer->queryByPoint($oPoint, MS_MULTIPLE, 0);
                }
                else
                {
                    // Get the Geo coords of the click and create a PointObj
                    $nMinX = Pix2Geo($aPixMin[0], 0, $oMap->width, 
                                  $oMap->extent->minx, $oMap->extent->maxx);
                    $nMinY = Pix2Geo($aPixMin[1], 0, $oMap->height, 
                                  $oMap->extent->miny, $oMap->extent->maxy, 1);
                    $nMaxX = Pix2Geo($aPixMax[0], 0, $oMap->width, 
                                  $oMap->extent->minx, $oMap->extent->maxx);
                    $nMaxY = Pix2Geo($aPixMax[1], 0, $oMap->height, 
                                  $oMap->extent->miny, $oMap->extent->maxy, 1);
                    $oRect = ms_newRectObj();
                    $oRect->setExtent($nMinX, $nMinY, $nMaxX, $nMaxY);

                    // Extract the layer to query
                    $oLayer = $oMap->getLayerByName($this->mszLayerName);

                    // Query by point
                    $oLayer->queryByRect($oRect);
                }
            }
            else
            {
                $_SESSION['gErrorManager']->setError(ERR_WARNING,
                                                     $this->moMLT->get( $this->mszWidgetName, '1', 'ERROR: Invalid input type specified in ' )."QueryByShape.php.");
                return false;
            }

            // Extract the result IDs and put that in a hidden and session
            $nNumResults = $oLayer->getNumResults();
            if($nNumResults == 0)
                return true;
            $oLayer->open();
            $_SESSION['QUERYBYSHAPE_TILEIDX'] = array();
            $_SESSION['QUERYBYSHAPE_SHPIDX'] = array();
            for($i=0; $i<$nNumResults; $i++)
            {
                $oResult = $oLayer->getResult($i);
                $oShape = $oLayer->getShape($oResult->tileindex,
                                            $oResult->shapeindex);
                $this->maszResultIds[] = $oShape->values[$this->mszIdCol];
                $_SESSION['QUERYBYSHAPE_TILEIDX'][] = $oResult->tileindex;
                $_SESSION['QUERYBYSHAPE_SHPIDX'][] = $oResult->shapeindex;
            }
            $_SESSION["QUERYBYSHAPE_RESULTIDS"] = $this->maszResultIds;
            $_SESSION["QUERYBYSHAPE_LAYERNAME"] = $this->mszLayerName;

            // Create a temporary layer to highlight the selected shape
            // If already exist, just modify it
            $oHighlight=@$oMap->getLayerByName($oLayer->name."_highlight");
            if(!$oHighlight)
            {
                // Create the layer
                $oHighlight = ms_newLayerObj($oMap, $oLayer);
                $oHighlight->set("name", $oLayer->name."_highlight");
                $oHighlight->set("status", MS_ON);
                $oHighlight->set("template", "dummy.html");
                if(isset($oHighlight->transparency))
                    $oHighlight->set("transparency", 75);
                else
                    $oHighlight->set("opacity", 75);
                $oHighlight->set("classitem", $this->mszIdCol);

                // remove all the classes in the base layer
                // we just want to highlight few shapes in it
                for($i=0; $i<$oHighlight->numclasses; $i++)
                {
                    $oClass = $oHighlight->getClass($i);
                    $oClass->set("status", MS_DELETE);
                }
                $oClass = ms_newClassObj($oHighlight);
                $oStyle = ms_newStyleObj($oClass);
                $oStyle->color->setRGB($this->manColor[0], 
                                       $this->manColor[1],
                                       $this->manColor[2]);
            }
            else
                $oClass = $oHighlight->getClass(0);
            $oClass->setExpression("/".implode("|", $this->maszResultIds).
                                   "/");

            // Open a popup to query layers in that shape
            if(!isset($this->maParams['SHOWPOPUP']) || 
               $this->maParams['SHOWPOPUP'] != 'false')
                $this->mbShowPopup = true;
        }

        // return success
        return true;
    }

    function GetJavascriptInitFunctions()
    {
        $aResult = parent::GetJavascriptInitFunctions();

        if($this->mbShowPopup)
            $aResult['QueryByShapeWShowPopup'] = "QueryByShapeWShowPopup();\n";

        return $aResult;
    }

    /**
     * GetHTMLHiddenVariables
     *
     * Return HTML hidden variables.
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = array();
        $szVariable = "QueryByShape_ResultIds";
        $szValue = implode(",", $this->maszResultIds);
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"$szValue\">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }


    /**
     * get general javascript functions
     */
    function GetJavascriptFunctions()
    {
        $aReturn = parent::GetJavascriptFunctions();

        $this->moPopup->mszParam.= "&ResultIds=".
            implode(",", $this->maszResultIds);
        $this->moPopup->mszParam.= "&Buffer=".$this->mnBuffer;

        $szPopupName = $this->moPopup->GetPopupName();
        $szPopupURL = $this->moPopup->GetPopupURL();
        $szPopupOptions = $this->moPopup->GetPopupOptions();

        $szJsFunctionName = "QueryByShapeWShowPopup";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the mouse moves for the Query widget.
 */
function {$szJsFunctionName}()
{
    szPopupURL = "{$szPopupURL}";
    szPopupName = "{$szPopupName}";
    szPopupOptions = "{$szPopupOptions}";

    var nX = {$this->mszHTMLForm}.MAP_CURSOR_POS_X.value;
    var nY = {$this->mszHTMLForm}.MAP_CURSOR_POS_Y.value;

    window.open( szPopupURL, szPopupName, szPopupOptions );

    goEventManager.triggerEvent( 'ON_QUERY', nX, nY );
    return true;
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

}
?>
