<?php
/**
 * ROIArea Widget class
 *
 * @project     CWC2
 * @revision    $Id: ROIArea.widget.php,v 1.1 2006/10/30 01:52:18 jdelfos Exp $
 * @purpose     Displaying the area of the last drawn ROI in a label
 *
 * @author      Maunsell Australia Pty Ltd (Jacob Delfos)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 * 
 *
 *   
 *  example definition:
 *  <cwc2 type="ROIArea" visible="true" label="Area (m):" UNITS="METERS" labelclass="label" widgetclass="inputBox"/>
 * 
 *  Note: this widget depends on the fix made to cwc_roi.js on 30 Oct 2006
 *  
 *
 *    
 */

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");

/**
 * ROIArea
 *
 * @desc A widget to draw a rectangular Region of Interest
 */
class ROIArea extends CWCWidget
{
    var $moLabel;
    var $mnDefaultTextFieldSize = 20;
    var $mszStyle = "";
    
    
    function ROIArea()
    {
        // invoke constructor of parent
        parent::CWCWidget();
        
        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The ROIArea Widgets displays the area of the last drawn ROI in a label
EOT;
        $this->moLabel = new CWCLabel( $this );
        $this->maAttributes["WIDGETCLASS"] = new StringAttribute( "WIDGETCLASS", false );
        $this->maAttributes["WIDGETSTYLE"] = new StringAttribute( "WIDGETSTYLE", false );
        $this->maAttributes["TEXTFIELDSIZE"] = new IntegerAttribute( "TEXTFIELDSIZE", false, 0 );
        $this->maAttributes["UNITS"] = new StringAttribute( "UNITS", false,
            array( "INCHES", "FEET", "MILES", "METERS", "KILOMETERS", "DEGREES", "PIXELS" ) );        
        $this->mnMaturityLevel = MATURITY_BETA;
    }
    
    
    /**
     * convert a measurement from a unit to meters
     */
    function ConvertToMeter($szInUnit, $dfValue)
    {
        //assuming the unit is valid
        //if ($this->IsUnitValid($szInUnit))
        $dfResult = -1.0;

        $aMeterPerUnit = array(); //how many meters are there in the unit
        $aMeterPerUnit[0] =  0.0254; //inches
        $aMeterPerUnit[1] =  0.3048; //feet
        $aMeterPerUnit[2] =  1609.3445; //Miles
        $aMeterPerUnit[3] =  1.0; //Meters
        $aMeterPerUnit[4] =  1000.0; //Kilometers
        $aMeterPerUnit[5] =  110570; //using value a 0 latitude 1degree = 110.57km


        $szInUnit = trim(strtoupper($szInUnit));
        switch ($szInUnit)
        {
            case "INCHES":
              $dfResult = $dfValue * $aMeterPerUnit[0];
              break;

            case "FEET":
               $dfResult = $dfValue * $aMeterPerUnit[1];
              break;

            case "MILES":
              $dfResult = $dfValue * $aMeterPerUnit[2];
              break;

            case "METERS":
              $dfResult = $dfValue * $aMeterPerUnit[3];
              break;

            case "KILOMETERS":
              $dfResult = $dfValue * $aMeterPerUnit[4];
              break;

            case "DEGREES":
              $dfResult = $dfValue * $aMeterPerUnit[5];
              break;

            default:
              break;
        }

        return $dfResult;

    }

    /**
     * convert a measurement from meters to a unit
     */
    function ConvertFromMeter($szInUnit, $dfValue)
    {
        //assuming the unit is valid
        //if ($this->IsUnitValid($szInUnit))
        $dfResult = -1.0;

        $aUnitPerMeter = array(); //how many units in one meters
        $aUnitPerMeter[0] =  39.37; //inches
        $aUnitPerMeter[1] =  3.2808; //feet
        $aUnitPerMeter[2] =  0.00062137; //Miles
        $aUnitPerMeter[3] =  1.0; //Meters
        $aUnitPerMeter[4] =  0.001; //Kilometers
        $aUnitPerMeter[5] =  0.000009044; //using value a 0 latitude 1degree = 110.57km

        $szInUnit = trim(strtoupper($szInUnit));
        switch ($szInUnit)
        {
            case "INCHES":
              $dfResult = $dfValue * $aUnitPerMeter[0];
              break;

            case "FEET":
               $dfResult = $dfValue * $aUnitPerMeter[1];
              break;

            case "MILES":
              $dfResult = $dfValue * $aUnitPerMeter[2];
              break;

            case "METERS":
              $dfResult = $dfValue * $aUnitPerMeter[3];
              break;

            case "KILOMETERS":
              $dfResult = $dfValue * $aUnitPerMeter[4];
              break;

            case "DEGREES":
              $dfResult = $dfValue * $aUnitPerMeter[5];
              break;

            default:
              break;
        }

        return $dfResult;

    }

    /**
     * arbitrarily convert a measurement from one unit to another
     */
    function ConvertUnit($szInUnit, $szOutUnit, $dfValue)
    {
        $dfValue = $this->ConvertToMeter($szInUnit, $dfValue);
        //echo "after convert to meter = $dfValue <br>";
        $dfValue = $this->ConvertFromMeter($szOutUnit, $dfValue);
        //echo "after convert from meter = $dfValue <br>";

        return $dfValue;
    }    
    
    
    
    
    /**
     * initialize the widget, only need to initialize the button in this case.
     */
    function InitDefaults()
    {
        parent::InitDefaults();
        if (isset($this->maParams["WIDGETCLASS"]))
            $this->mszClass = $this->maParams["WIDGETCLASS"];
        if (isset($this->maParams["WIDGETSTYLE"]))
            $this->mszStyle = $this->maParams["WIDGETSTYLE"];
        if (isset($this->maParams["TEXTFIELDSIZE"]))
            $this->mnDefaultTextFieldSize = strtoupper($this->maParams["TEXTFIELDSIZE"]);    
    }
    
    function GetJavascriptVariables()
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;
              
    $szUserUnit = "";
       if (isset($this->maParams["UNITS"]))
       {
       $szUserUnit = strtoupper($this->maParams["UNITS"]);
       }
       
    $szVariable = "gAreaUserUnit";
    $szValue = " var $szVariable = \"$szUserUnit\";\n";
    $aReturn[$szVariable] = $szValue;


      if ($bCWCJSAPI==0) // if no JSAPI, predefine the ratio to convert to user units
      {
           if (isset($this->moMapObject))
           {
           $poMap = $this->moMapObject->oMap;
           $nUnits = $this->moMapObject->oMap->units;
           }
           else
           {
           define("LOAD_MAPSESSION", "1");
           include_once("../session.inc.php");

           $oMap =& $oMapSession->oMap;
           $nUnits = $oMap->units;
           }
        
           //echo "orig pix value = $dfPixelSize <br>";
           $szUserUnit = "";
           if (isset($this->maParams["UNITS"]))
               $szUserUnit = strtoupper($this->maParams["UNITS"]);

           $szMapUnits = "";
           switch ($nUnits)
           {
              case MS_INCHES:
               $szMapUnits = "INCHES";
               break;

              case MS_FEET:
               $szMapUnits = "FEET";
               break;

             case MS_MILES:
              $szMapUnits = "MILES";
              break;

             case MS_METERS:
              $szMapUnits = "METERS";
              break;

             case MS_KILOMETERS:
              $szMapUnits = "MS_KILOMETERS";
              break;

             case MS_DD:
              $szMapUnits = "DEGREES";
              break;

             case MS_PIXELS:
              $szMapUnits = "PIXELS";
              break;
           }
        
        
           if ($szMapUnits != "" && $szUserUnit != "" && (strcasecmp($szUserUnit,$szMapUnits) != 0))
           {
            //echo "convert from $szMapUnits to $szUserUnit <br>";
            $ratio =  $this->ConvertUnit($szMapUnits, $szUserUnit, 1);
           }
           else
           {
           $ratio=1;
           }
        
          $szVariable = "gAreaUnitRatio";
          $szValue = " var $szVariable = \"$ratio\";\n";
          $aReturn[$szVariable] = $szValue;
       }
    
    return $aReturn;
    }
    
    function GetJavascriptFunctions()
    {

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;


            $szJsFunctionName = "ROIAreaRegisterForEvent";
            $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 */
function {$szJsFunctionName}()
{
    goCWCROIManager.RegisterEvent(ROI_CHANGED, "updateArea");
}
EOT;

       $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "updateArea";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * 
 */
function {$szJsFunctionName}()
{
   if ((goCWCROIManager.aROI.length > 0) && (goCWCROIManager.aROI[goCWCROIManager.aROI.length-1]))
   {
      if ({$bCWCJSAPI})
      {
      szMapUnit = goCWCJSAPI.oMap.GetUnitString(goCWCJSAPI.oMap.units);
      }
      areaObject=goCWCROIManager.aROI[goCWCROIManager.aROI.length-1]
      
   geometryType = areaObject.type;

      switch (geometryType)
      {
      case "rectangle":
      var geoL = areaObject.geoMinX;
      var geoT = areaObject.geoMaxY;
      var geoR = areaObject.geoMaxX;
      var geoB = areaObject.geoMinY;
  
      area=(geoR-geoL) * (geoT-geoB);
      break;

      case "polygon":
      area=0;
      polygonCoords=areaObject.geoCoords.split(",");
   
         for (v=0;v<(polygonCoords.length-3);v+=2)
         {
         area+=(
         (polygonCoords[0]*polygonCoords[v+1])
          -
         (polygonCoords[1]*polygonCoords[v])
         +
         (polygonCoords[1]*polygonCoords[v+2])
         -
         (polygonCoords[0]*polygonCoords[v+3])
         +
         (polygonCoords[v]*polygonCoords[v+3])
          -
         (polygonCoords[v+2]*polygonCoords[v+1])
         );
         }
      area=Math.abs(area/2);   
      break;  
   
      case "circle":
      theRadius = areaObject.geoRadius
      area= Math.PI*theRadius*theRadius;
      break;     
      }
      
      if ({$bCWCJSAPI})
      {
      ratio = goCWCJSAPI.oMap.ConvertUnit(szMapUnit, gAreaUserUnit, 1);   
      area = area*(ratio*ratio);
      }
      else
      {
      area = area*(gAreaUnitRatio*gAreaUnitRatio);
      }
      
   document.forms[0].AREA_RESULT.value=(Math.round(area*1000)/1000);
   }
   else
   {
   document.forms[0].AREA_RESULT.value="";
   }
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }
    
    

    /**
     * return an array of javascript functions needed by ROIArea widget
     * and called when the page is loaded.
     * @return array of name = function values
     */
    function GetJavascriptOnLoadFunctions()
    {
        $aReturn = array();

        $aReturn = parent::GetJavascriptOnLoadFunctions();

        $szJsFunctionName = "ROIAreaRegisterForEvent";
        $szFunction = "$szJsFunctionName();\n";
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }    

    /**
     * draw this widget on the page
     */
    function DrawPublish()
    {
        $nSize = $this->mnDefaultTextFieldSize;
        if (isset($this->maParams["TextFieldSize"]))
            $nSize = strtoupper($this->maParams["TextFieldSize"]);

        $szClass = "";
        if ( strlen($this->mszClass) > 0 )
            $szClass = " CLASS=\"$this->mszClass\"";
        $szStyle = "";
        if ( strlen($this->mszStyle) > 0 )
            $szStyle = " STYLE=\"$this->mszStyle\"";

        $szContents="<input$szClass$szStyle type=\"text\" name=\"AREA_RESULT\" value=\"\"  size=\"$nSize\" readonly>";

        return $this->moLabel->DrawPublish( $szContents );

    }
}
?>
