<?php
/**
 * ROILineTool Widget class
 *
 * @project     CWC2
 * @revision    $Id: ROILineTool.widget.php,v 1.1 2007/01/05 21:12:58 jlacroix Exp $
 * @purpose     ROI Line Tool Widget class.  Allows the user to select
 * a circular region of interest on the map.
 * @author      DM Solutions Group (bronsema@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002-2003, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");

/**
 * ROILineTool
 *
 * @desc A widget to draw a polagonal Region of Interest
 */
class ROILineTool extends CWCWidget
{
    /* the button that is the interface for this widget */
    var $moButton;
    var $mnNumPoints = 12;  // default number of points
    var $mnPointSpacing = 10; // default the point spacing;

    /**
     * construct a new ROI Line Tool widget
     */
    function ROILineTool()
    {
        // invoke constructor of parent
        parent::CWCWidget();

        // set the attributes
        $this->maAttributes["NUMBEROFPOINTS"] = new IntegerAttribute( "NUMBEROFPOINTS", false, 0 );
        $this->maAttributes["POINTSPACING"] = new IntegerAttribute( "POINTSPACING", false, 0 );

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The ROILineTool widget allows the user to draw a circular region
of interest on the map interface.
EOT;
        $this->moButton = new CWCButton( $this );
        $this->mnMaturityLevel = MATURITY_BETA;
    }

    /**
     *
     */
    function InitDefaults()
    {
        parent::InitDefaults();

        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick('ROILineTool'.$this->mnId);
    }

    /**
     * pass URL to button
     */
    function  ParseURL()
    {
        return $this->moButton->ParseURL();
    }

    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();

        $szCmd = '';
        if ($this->isVarSet('NAV_CMD'))
            $szCmd = $this->getVar('NAV_CMD');
        $szVariable = "NAV_CMD";
        $szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "ROI_TOOL_LOCKED";
        $szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"0\">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    /**
     * GetJavascriptInitFunctions
     *
     * Functions to be called at the end of the load.
     */
    function GetJavascriptInitFunctions()
    {
        $aReturn = $this->moButton->GetJavascriptInitFunctions();

        $szJsFunctionName = "ROILineToolInit";
        $szFunction = "$szJsFunctionName();\n";
        $aReturn[$szJsFunctionName] = $szFunction;

        $aReturn['InitCWCROIManagerForm'] = 'goCWCROIManager.SetFormObject('.$this->mszHTMLForm.');';

        return $aReturn;
    }

    /**
     * javascript include files
     */
    function GetJavascriptIncludeFunctions()
    {
        $aReturn = parent::GetJavascriptIncludeFunctions();

        $szVar = "cwc_roi.js";
        $aReturn[$szVar] = '<script src="'.$_SESSION['gszCoreWebPath'].
                            '/widgets/js/cwc_roi.js" type="text/javascript"></script>';

        $szVar = "cwc_line.js";
        $aReturn[$szVar] = '<script src="'.$_SESSION['gszCoreWebPath'].
                            '/widgets/js/cwc_line.js" type="text/javascript"></script>';

        $szVar = "cwc_dhtml.js";
        $aReturn[$szVar] = '<script src="'.$_SESSION['gszCoreWebPath'].
                            '/widgets/js/cwc_dhtml.js" type="text/javascript"></script>';
        return $aReturn;
    }

    /**
     * @desc javascript mousemove functions
     */
    function GetJavascriptOnMouseMoveFunctions()
    {
        $aReturn = parent::GetJavascriptOnMouseMoveFunctions();

        $aReturn['ROILineTool'] = 'ROILineToolMouseMove(e);';

        return $aReturn;
    }

    /**
     * @desc general javascript functions
     */
    function GetJavascriptFunctions()
    {
        $aReturn = parent::GetJavascriptFunctions();

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        // get the chameleon images url
        $szImagesURL = trim(str_replace("\\", '/', $_SESSION['gszCoreWebPath']));
        $szImagesURL = substr( $szImagesURL, -1 ) == '/' ? $szImagesURL.'images/' :
                                                            $szImagesURL.'/images/';

        // determine the number of points to use
        if ( isset($this->maParams["NUMBEROFPOINTS"] ) )
            $nPoints = intval( $this->maParams["NUMBEROFPOINTS"] );
        else
            $nPoints = $this->mnNumPoints;

        // add JSAPI mode functions
        if ($bCWCJSAPI)
        {
            $szJsFunctionName = "ROILineToolWRegisterForEvent";
            $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called to register events when the layers change (JSAPI)
 */
function {$szJsFunctionName}()
{
    goCWCJSAPI.RegisterEvent(MAP_EXTENT_CHANGED, "ROILineToolWExtentChanged");
    goCWCJSAPI.RegisterEvent(MAP_PROJECTION_CHANGED, "ROILineToolWExtentChanged");
}
EOT;

           $aReturn[$szJsFunctionName] = $szFunction;
           $szJsFunctionName = "ROILineToolWExtentChanged";
           $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the extent is changed (JSAPI)
 */
function {$szJsFunctionName}()
{
    // re-draw the ROI
    goCWCROIManager.SetGeoExtents( goCWCJSAPI.oMap.minx, goCWCJSAPI.oMap.miny,
                                    goCWCJSAPI.oMap.maxx, goCWCJSAPI.oMap.maxy );
    goCWCROIManager.Redraw();
}
EOT;
            $aReturn[$szJsFunctionName] = $szFunction;

        }

        $nId = $this->mnId;
        $szName = 'ROILineTool'.$nId;
        $szFunction = <<<EOT
        function {$szName}()
        {
            goCWCROIManager.SetROITool('{$nId}');
        }
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolInit';
        $szFunction = <<<EOT
function {$szName}()
{
    // create new layers to use with line drawing
    var sId = "";
    gaszLineDivPointNames = new Array();
    gaszLineDivNodeNames = new Array();
    var content = "<img  src='{$szImagesURL}ruler_pix.gif' width='6' height='6'><br>";
    var content2 = "<img  src='{$szImagesURL}ruler_node.gif' width='7' height='7'><br>";
    for (var i=1; i < {$nPoints}; i++)
    {
        // create the point layers
        sId = 'linePointLayer'+i;
        CWCDHTML_CreateLayer(sId, 10, 10, 10, 10, false, content);
        CWCDHTML_SetLayerZOrder( sId, 20 );
        gaszLineDivPointNames.push( sId );

        // create the node layers
        sId = 'lineNodeLayer'+i;
        CWCDHTML_CreateLayer(sId, 10, 10, 10, 10, false, content2);
        CWCDHTML_SetLayerZOrder( sId, 20 );
        gaszLineDivNodeNames.push( sId );
    }

    // init line object
    goROILineObj = new CWCLineROI({$nPoints});
    goROILineObj.pointImageWidth = 6;
    goROILineObj.nodeImageWidth = 7;
    goROILineObj.left = gMapWhspc;
    goROILineObj.top = gMapWvspc;
    goROILineObj.right = gMapWhspc + gMapWiWidth;
    goROILineObj.bottom = gMapWvspc + gMapWiHeight;
    goROILineObj.edgeColor = goCWCROIManager.szEdgeColor;
    goROILineObj.edgeWidth = goCWCROIManager.nEdgeWidth;
    goROILineObj.fillLayer = goCWCROIManager.bUseFill;
    goROILineObj.fillColor = goCWCROIManager.szFillColor;
    goROILineObj.fillOpacity = goCWCROIManager.nFillOpacity;

    // show the line
    goROILineObj.Show();

}
EOT;
        $aReturn[$szName] = $szFunction;

        $szObjectJS = "    var jsObj;\n";

        /**
         * TODO: this does not reconstitute the points associated with the
         * ROI tool because this is complicated ;)  In reality, the points may
         * not be at valid locations.  The only reason to reconstitute the points
         * would be to enable editing, which can't happen
         */
        if (isset($_SESSION['ROIRENDERER']))
        {
            $nROI = count($_SESSION['ROIRENDERER']);
            for ($i=0; $i<$nROI; $i++)
            {
                $aROI = $_SESSION['ROIRENDERER'][$i];
                if ($aROI['type'] == 'line')
                {
                    $szObjectJS .= "    goCWCROIManager.SetMode(".
                                   $aROI['mode'] . ");\n";
                    $szObjectJS .= "    jsObj = new CWCLineROI();\n";
                    $szObjectJS .= "    jsObj.edgeColor = \"".$aROI['edgecolor']."\";\n";
                    $szObjectJS .= "    jsObj.edgeWidth = ".$aROI['edgewidth'].";\n";
                    $szObjectJS .= "    jsObj.fillLayer = goCWCROIManager.bUseFill;\n";
                    $szObjectJS .= "    jsObj.fillColor = \"".$aROI['fillcolor']."\";\n";
                    $szObjectJS .= "    jsObj.fillOpacity = ".$aROI['fillopacity'].";\n";
                    $szObjectJS .= "    goCWCROIManager.Insert( jsObj, $i );\n";
                }
            }
            if (strlen( $szObjectJS ) > 0)
            {
                $szObjectJS .= "    goCWCROIManager.UpdateLayerVisibility();\n";
            }
        }


        $szName = 'ROILineToolInitObjects';
        $szFunction = <<<EOT
function {$szName}()
{
{$szObjectJS}
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolDrawFloater';
        $szFunction = <<<EOT
function {$szName}()
{
        // set or default the start points
        var nTmpStartX = 0;
        var nTmpStartY = 0;
        nTmpNodeSpacing = 10;
        if ( goROILineObj.aNodeCoords[0] )
        {
            nTmpStartX = goROILineObj.aNodeCoords[goROILineObj.aNodeCoords.length-1][0];
            nTmpStartY = goROILineObj.aNodeCoords[goROILineObj.aNodeCoords.length-1][1];
            nTmpNodeSpacing = gnROILinePointSpacing;
        }

        // delete old floater
        goFloaterLine.deleteLine();

        // create and setup the new floater line
        goFloaterLine = new Line( nTmpStartX, nTmpStartY,
            gROIToolmouseX, gROIToolmouseY, false );
        goFloaterLine.pointSpacing = nTmpNodeSpacing;
        goFloaterLine.layerNodeList = gaszLineDivNodeNames;
        goFloaterLine.layerPointList = gaszLineDivPointNames;

        // limit points to 3
        goFloaterLine.numPoints = 3

        // draw the line
        goFloaterLine.drawLine();
}
EOT;
        $aReturn[$szName] = $szFunction;

        // get translated text for this section
        $szTmpNotice = $this->moMLT->get( $this->mszWidgetName, '2', 'Press `esc` to cancel or any other key to close the line.' );

        $szName = 'ROILineToolMouseDown';
        $szFunction = <<<EOT
function {$szName}(e)
{
    // skip if not on map or locked
    if ( !MapWinsideMap() || {$this->mszHTMLForm}.ROI_TOOL_LOCKED.value == 1 )
    {
        return true;
    }

    ROILineToolGetMousePosition(e);

    if (gROIToolmouseX >=0 && gROIToolmouseY >=0)
    {
        gROILineToolx1 = gROIToolmouseX;
        gROILineTooly1 = gROIToolmouseY;

        // flag that the line is open
        gROILineOpen = true;

        // draw next node
        ROILineDrawNode();
    }

    // set status message
    //window.status='{$szTmpNotice}';

    return false;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolMouseUp';
        $szFunction = <<<EOT
function {$szName}(e)
{
    return null;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolGetMousePosition';
        $szFunction = <<<EOT
function {$szName}(e)
{
    if (CWCIsNav4 || CWCIsNav6) // Netscape
    {
        gROIToolmouseX = e.pageX;
        gROIToolmouseY = e.pageY;
    }
    else if (CWCIsIE) // IE
    {
        gROIToolmouseX = event.clientX + document.body.scrollLeft;
        gROIToolmouseY = event.clientY + document.body.scrollTop;
    }
    else
    {                              // Don't know
        gROIToolmouseX = gROIToolmouseY = 0;
    }
    var oImage = CWCDHTML_GetImage('mapimage');

    if (oImage == null)
    {
        gROIToolmouseX = -1;
        gROIToolmouseY = -1
        return false;
    }
    var left = CWCDHTML_FindObjectPosX( oImage );
    var top = CWCDHTML_FindObjectPosY( oImage );

    var width = oImage.width;
    var height = oImage.height;

    // subtract offsets from page left and top
    var nROIToolmouseX = gROIToolmouseX - left;
    var nROIToolmouseY = gROIToolmouseY - top;

    if (nROIToolmouseX < 0 || nROIToolmouseX > width ||
        nROIToolmouseY < 0 || nROIToolmouseY > height)
    {
        gROIToolmouseX = -1;
        gROIToolmouseY = -1
    }
    return true;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolMouseMove';
        $szFunction = <<<EOT
function {$szName}(e)
{
    ROILineToolGetMousePosition(e);

    if (gROIToolmouseX >=0 && gROIToolmouseY >=0 &&
        {$this->mszHTMLForm}.NAV_CMD != null &&
        {$this->mszHTMLForm}.NAV_CMD.value == "ROI_TOOL" &&
        {$this->mnId} == goCWCROIManager.GetROITool() )
    {
        // capture events
        if ( CWCIsNav4 || CWCIsNav6 )
        {
            document.captureEvents(Event.MOUSEDOWN);
            document.captureEvents(Event.MOUSEMOVE);
            document.captureEvents(Event.RESIZE);
        }
        if( CWCIsNav4 )
        {
            document.onmousedown = ROILineToolMouseDown;
            document.onmouseup = ROILineToolMouseUp;
            document.onKeyPress = ROILineToolKeyPress;
        }
        else if ( CWCIsIE || CWCIsNav6 )
        {
            var layer  = document.getElementById("MapLayerDiv");
            document.onmousedown = ROILineToolMouseDown;
            document.onmouseup = ROILineToolMouseUp;
            document.onkeypress = ROILineToolKeyPress;

            if ( CWCIsNav6 )
              document.addEventListener("keypress", ROILineToolKeyPress, true);
        }

        if ( gROILineOpen )
        {
            // update end point of current line
            gROILineToolx2 = gROIToolmouseX;
            gROILineTooly2 = gROIToolmouseY;

            // continuously update the floater line
            ROILineToolDrawFloater();
        }
    }

    return false;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineDrawNode';
        $szFunction = <<<EOT
function {$szName}()
{
    // calculate coordinates
    var nPreviousX = gROILineToolx1;
    var nPreviousY = gROILineTooly1;
    var bDefaulted = 1;
    var nNumNodes = 0;
    if ( goROILineObj.aNodeCoords )
        nNumNodes = goROILineObj.aNodeCoords.length;

    if ( nNumNodes > 0 )
    {
        nPreviousX = goROILineObj.aNodeCoords[goROILineObj.aNodeCoords.length-1][0];
        nPreviousY = goROILineObj.aNodeCoords[goROILineObj.aNodeCoords.length-1][1];
        bDefaulted = 0;
    }

    // skip if coordinates match (i.e. double event triggered)
    if ( nPreviousX == gROILineToolx1 && nPreviousY == gROILineTooly1 && bDefaulted == 0 )
        return null;

    // delete floater
    goFloaterLine.deleteLine();

    // draw line
    ROILineToolDrawLine( nPreviousX, nPreviousY, gROILineToolx1, gROILineTooly1 );

    // return
    return null;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolUnlockTool';
        $szFunction = <<<EOT
function {$szName}()
{
    // reset the lock
    {$this->mszHTMLForm}.ROI_TOOL_LOCKED.value = 0;
    return null;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolKeyPress';
        $szFunction = <<<EOT
function {$szName}(e)
{
    // only process if line tool is selected
    if ( {$this->mszHTMLForm}.NAV_CMD != null &&
        {$this->mszHTMLForm}.NAV_CMD.value == "ROI_TOOL" &&
        {$this->mnId} == goCWCROIManager.GetROITool() &&
        {$this->mszHTMLForm}.ROI_TOOL_LOCKED.value != 1 )
    {
        // delete floater
        goFloaterLine.deleteLine();

        // check which key was pressed
        var nKey;

        // capture based on browser type
        if ( CWCIsIE )
            nKey = event.keyCode;
        else
            nKey = e.keyCode;

        switch ( nKey )
        {
            case 27:
                // esc was pressed so abort
                ROILineToolClearLine();
                break;

            default:
                // close line
                if (goROILineObj.aNodeCoords.length > 0)
                {
                ROILineToolCloseLine();
                }
        }
    }

    // return
    return null;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolCloseLine';
        $szFunction = <<<EOT
function {$szName}()
{
    // close line
    gROILineOpen = false;

    // add aoi
    goCWCROIManager.Add( goROILineObj );

    // hide the points
    ROILineToolClearLine();

    // lock the tool on a timer
    {$this->mszHTMLForm}.ROI_TOOL_LOCKED.value = 1;
    setTimeout( "ROILineToolUnlockTool()", 500 );

    // return
    return true;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolDrawLine';
        $szFunction = <<<EOT
function {$szName}( x1, y1, x2, y2, bDrawStartNode )
{
    // draw a new line if necessary
    var nLineCount = 0;
    if ( goROILineObj.aNodeCoords )
        nLineCount = goROILineObj.aNodeCoords.length;

    if ( nLineCount == 0 )
    {
        var oLine = new Line( x1, y1, x2, y2, bDrawStartNode );
        oLine.layerNodeList = gaszLineDivNodeNames;
        oLine.layerPointList = gaszLineDivPointNames;

        // ensure that no nodes are drawn
        oLine.numPoints = 0;

        // draw the line
        oLine.drawLine();

        // add to array of line objects
        gaROILineObj.push( oLine );
    }
    else if ( nLineCount > 0 )
    {
        var oLine = new Line( x1, y1, x2, y2, bDrawStartNode );
        oLine.pointSpacing = gnROILinePointSpacing;
        oLine.layerNodeList = gaszLineDivNodeNames;
        oLine.layerPointList = gaszLineDivPointNames;

        // draw the line
        oLine.drawLine();

        // add to array of line objects
        gaROILineObj.push( oLine );
    }

    // check if no more nodes then redraw all lines using double spacing
    if ( gaszLineDivPointNames.length == 0 )
    {
        // update global point spacing
        gnROILinePointSpacing = gnROILinePointSpacing * 2;

        // loop through the list lines, delete, and recreate
        nLineCount = gaROILineObj.length;
        for ( var i=0; i<nLineCount; i++ )
        {
            gaROILineObj[i].deleteLine();
            gaROILineObj[i].pointSpacing = gnROILinePointSpacing;
            gaROILineObj[i].drawLine();
        }
    }

    // record node click if line is open
    if ( gROILineOpen )
    {
        var aTmpArray = new Array( gROILineToolx1, gROILineTooly1 );
        goROILineObj.aNodeCoords.push( aTmpArray );
    }
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolClearLine';
        $szFunction = <<<EOT
function {$szName}()
{
    // delete all lines
    var nLineCount = gaROILineObj.length;
    for ( var i=0; i<nLineCount; i++ )
        gaROILineObj[i].deleteLine();
    // create new object, not to overwrite the old one
    goROILineObj = new CWCLineROI({$nPoints});
    goROILineObj.pointImageWidth = 6;
    goROILineObj.nodeImageWidth = 7;
    goROILineObj.left = gMapWhspc;
    goROILineObj.top = gMapWvspc;
    goROILineObj.right = gMapWhspc + gMapWiWidth;
    goROILineObj.bottom = gMapWvspc + gMapWiHeight;
    goROILineObj.edgeColor = goCWCROIManager.szEdgeColor;
    goROILineObj.edgeWidth = goCWCROIManager.nEdgeWidth;
    goROILineObj.fillLayer = goCWCROIManager.bUseFill;
    goROILineObj.fillColor = goCWCROIManager.szFillColor;
    goROILineObj.fillOpacity = goCWCROIManager.nFillOpacity;  
    // show the line
    goROILineObj.Show();  
    // reset
    goROILineObj.aNodeCoords = new Array();
    gaROILineObj = new Array();
    gROILineOpen = false;
    //window.status='';
    gnROILinePointSpacing = gnInitialROILinePointSpacing;
    return true;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROILineToolUnlockTool';
        $szFunction = <<<EOT
function {$szName}()
{
    // reset the lock
    {$this->mszHTMLForm}.ROI_TOOL_LOCKED.value = 0;
    return null;
}
EOT;
        $aReturn[$szName] = $szFunction;

        return $aReturn;
    }

    /**
     * GetJavascriptVariables
     *
     * Return JS global variables and global code.
     */
    function GetJavascriptVariables()
    {
        $aReturn = array();

        $szVariable = "gROIToolmouseX";
        $szValue = " var $szVariable = 0;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gROIToolmouseY";
        $szValue = " var $szVariable = 0;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gROILineToolx1";
        $szValue = " var $szVariable = 0;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gROILineTooly1";
        $szValue = " var $szVariable = 0;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gROILineToolx2";
        $szValue = " var $szVariable = 0;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gROILineTooly2";
        $szValue = " var $szVariable = 0;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gROILineOpen";
        $szValue = " var $szVariable = false;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "goROILineObj";
        $szValue = " var $szVariable = null;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gaROILineObj";
        $szValue = " var $szVariable = new Array();\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gaszLineDivPointNames";
        $szValue = " var $szVariable;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gaszLineDivNodeNames";
        $szValue = " var $szVariable;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "goFloaterLine";
        $szValue = " var $szVariable = new Line( 0, 0, 0, 0, false ); ;\n";
        $aReturn[$szVariable] = $szValue;

        // determine point spacing
        if ( isset($this->maParams["POINTSPACING"] ) )
            $nPointSpacing = intval( $this->maParams["POINTSPACING"] );
        else
            $nPointSpacing = $this->mnPointSpacing;
        $szVariable = "gnROILinePointSpacing";
        $szValue = " var $szVariable = $nPointSpacing;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gnInitialROILinePointSpacing";
        $szValue = " var $szVariable = $nPointSpacing;\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    /**
     * return an array of javascript functions needed by the Ruler widget
     * and called when the page is loaded.
     * @return array of name = function values
     */
    function GetJavascriptOnLoadFunctions()
    {
        $aReturn = parent::GetJavascriptOnLoadFunctions();

        $aReturn["ROILineToolInitObjects"] = "    ROILineToolInitObjects();\n";
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
        {
            $szJsFunctionName = "ROILineToolWRegisterForEvent";
            $szFunction = "$szJsFunctionName();\n";
            $aReturn[$szJsFunctionName] = $szFunction;
        }

        return $aReturn;
    }

    /**
     * draw this widget on the page
     */
    function DrawPublish($szPostEvent = "")
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        if (!$this->mbVisible)
            return "<!-- ROILineTool hidden -->";

        $szReturn = $this->moButton->DrawPublish();

        return $szReturn;
    }
}
?>
