<?php
/**
 * Render one or more ROIs onto a single image for display over the map.
 *
 * @project     CWC2
 * @revision    $Id: ROIRenderer.php,v 1.15 2007/08/22 14:56:52 wbronsema Exp $
 * @purpose     render ROIs
 * @author      DM Solutions Group (spencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

// notices are messing up the renderer TODO: fix the notices
ini_set( 'error_reporting','E_ALL & ~E_NOTICE' );

// give the renderer extra time
set_time_limit( 600 );

// include the session handling file
define("LOAD_MAPSESSION", 1);
$bInvalidSessionDontFail = true;
include("../session.inc.php");
define( 'ROIRENDERER_QUERY', getSessionSavePath()."/roirenderer.qry");
if (!extension_loaded( "gd" ))
{
    $szGDMod2 = "php_gd2".( (PHP_OS=="WINNT" || PHP_OS=="WIN32") ? ".dll" : ".so" );
    dl( $szGDMod2 );
}

$width = $_GET['width'];
$height = $_GET['height'];

$oImage = imagecreate( $width, $height );
$nTransparent = imagecolorallocate( $oImage, 255, 255, 255 );
//$nTransparent = imagecolorresolvealpha( $oImage, 255, 255, 255, 0 );
$nBlack = imagecolorallocate( $oImage, 0, 0, 0 );

//imagealphablending( $oImage, false );
imagecolortransparent( $oImage, $nTransparent );
imagefill( $oImage, 0, 0, $nTransparent );
//imagealphablending( $oImage, true );

if (!isset($GLOBALS['bSessionOK']) || !($GLOBALS['bSessionOK']))
{
    renderText("session expired!");
}
else
{
    $geoMinX = $oMapSession->oMap->extent->minx;
    $geoMinY = $oMapSession->oMap->extent->miny;
    $geoMaxX = $oMapSession->oMap->extent->maxx;
    $geoMaxY = $oMapSession->oMap->extent->maxy;
    //renderText( $geoMinX." ".$geoMinY." ".$geoMaxX." ".$geoMaxY );
    //mode settings:
    //mode = -2 : remove all ROIs
    //mode = -1 : remove the last ROI
    //mode = 0 : replace any existing ROIs with the one passed
    //mode = 1 : replace any existing ROIs with the one passed
    //mode = 2 : add the ROI
    //mode = 3 : subtract the ROI
    //
    if (!isset($_SESSION["ROIRENDERER_COUNT"]))
    {
        $_SESSION["ROIRENDERER_COUNT"] = 1;
    }
    else
    {
        $_SESSION["ROIRENDERER_COUNT"] = $_SESSION["ROIRENDERER_COUNT"] + 1;
    }
    $mode = isset($_GET['mode']) ? $_GET['mode'] : 0;
    //renderText( "count is " .$_SESSION["ROIRENDERER_COUNT"]. ", mode is " . $mode );
    if (!isset($_SESSION['ROIRENDERER']) || $mode == 1)
    {
        //clean up after feature ROIs ...
        if (isset($_SESSION['ROIRENDERER']))
        {
            $nROI = count($_SESSION['ROIRENDERER']);

            for($i=0; $i<$nROI; $i++)
            {
                cleanUpROI( $_SESSION['ROIRENDERER'][$i] );
            }
        }
        $_SESSION['ROIRENDERER'] = array();
    }
    if ( $mode > 0 )
    {
        $roi = array();
        $roi['mode'] = $mode;
        $roi['type'] = isset($_GET['type']) ? $_GET['type'] : 'rectangle';
        $roi['aGeoCoords'] = isset($_GET['coords']) ? explode( ",", $_GET['coords']) : array();
        $roi['edgecolor'] = isset($_GET['edgecolor']) ? $_GET['edgecolor'] : '#000000';
        $roi['edgewidth'] = isset($_GET['edgewidth']) ? $_GET['edgewidth'] : 1;
        $roi['fillcolor'] = isset($_GET['fillcolor']) ? $_GET['fillcolor'] : '#000000';
        $roi['fillopacity'] = isset($_GET['fillopacity']) ? $_GET['fillopacity'] : 0;
        if (isset($_GET['selectedLayer']))
        {
            $roi['selectedLayer'] = $_GET['selectedLayer'];
        }
        array_push( $_SESSION['ROIRENDERER'], $roi );
    }
    elseif ($mode == -1)
    {
        $aROI = array_pop($_SESSION['ROIRENDERER']);
        //clean up after feature ROIs ...
        cleanUpROI( $aROI );
    }
    elseif($mode == -2)
    {
        $nROI = count($_SESSION['ROIRENDERER']);

        //clean up after feature ROIs ...
        for($i=0; $i<$nROI; $i++)
        {
           cleanUpROI($_SESSION['ROIRENDERER'][$i]);
        }
        //delete all ROIs ...
        $_SESSION['ROIRENDERER'] = array();
    }

    if (isset($_SESSION['ROIRENDERER']))
    {
        $nROI = count($_SESSION['ROIRENDERER']);
        for($i=0; $i<$nROI; $i++)
        {
            drawROI( $_SESSION['ROIRENDERER'][$i] );
        }
    }
}
header( 'Content-type: image/png' );
//imagetruecolortopalette($oImage, false, 256);
imagepng($oImage);
exit;

/*
 * draw a single ROI on the output image.  The intent is for this function to handle
 * any type of ROI.
 */
function drawROI( &$aROI )
{
    // debug
    //renderText(count($_SESSION['ROIRENDERER']));

    if ($aROI['mode'] == 3)
    {
        //subtract, draw a transparent ROI
        $nFillColor = $GLOBALS['nTransparent'];
    }
    else
    {
        //normal or add
        $nFillColor = allocateImageFromHexValue( $GLOBALS['oImage'], $aROI['fillcolor'], $aROI['fillopacity']);
    }
    $aPoints = array();
    if ($aROI['type'] == 'rectangle')
    {
        //for rectangles, we assume that there are two coordinates, top left and bottom right.
        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][0], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] ) );
        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][1], 0, $GLOBALS['height'],
                    $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 ) );

        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][2], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] ) );
        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][1], 0, $GLOBALS['height'],
                    $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 ) );

        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][2], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] ) );
        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][3], 0, $GLOBALS['height'],
                    $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 ) );

        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][0], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] ) );
        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][3], 0, $GLOBALS['height'],
                    $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 ) );

        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][0], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] ) );
        array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][1], 0, $GLOBALS['height'],
                    $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 ) );
        $nPoints = 5;

        // flag for poly
        $bPoly = true;
    }
    elseif ( $aROI['type'] == 'circle' )
    {
        // convert center and radius to pixel
        $dCenterX = Geo2Pix($aROI['aGeoCoords'][0], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] );
        $dCenterY = Geo2Pix($aROI['aGeoCoords'][1], 0, $GLOBALS['height'],
                    $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 );
        $dRadiusX = Geo2Pix($aROI['aGeoCoords'][2], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX']);

        $dRadius = abs( $dRadiusX - $dCenterX );

        // flag for non-poly
        $bPoly = false;
    }
    elseif ( $aROI['type'] == 'point' )
    {
        // convert center and radius to pixel
        $dCenterX = Geo2Pix($aROI['aGeoCoords'][0], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] );
        $dCenterY = Geo2Pix($aROI['aGeoCoords'][1], 0, $GLOBALS['height'],
                    $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 );
        $dRadius = 5;

        // flag for non-poly
        $bPoly = false;
    }
    elseif ( $aROI['type'] == 'polygon' ||
             $aROI['type'] == 'line' )
    {
        // loop and set coordinates
        $nCount = count( $aROI['aGeoCoords'] );
        for( $i=0;$i<$nCount; $i++ )
        {
            // even is x, odd is y
            if ( fmod( $i, 2 ) == 0 )
            {
                array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][$i], 0, $GLOBALS['width'],
                    $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] ) );
            }
            else
            {
                array_push( $aPoints, Geo2Pix($aROI['aGeoCoords'][$i], 0, $GLOBALS['height'],
                    $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 ) );
            }
        }

        $nPoints = round( $nCount/2 );

        // flag for poly
        if ( $aROI['type'] == 'polygon' )
            $bPoly = true;
        else
            $bPoly = false;
    }
    elseif ($aROI['type'] == 'feature')
    {
        // get mapfile handle
        $oNewMap = $GLOBALS['oMapSession']->oMap;

        // init
        $nPoints = 0;
        $bBuffered = false;

        $nLayers = $oNewMap->numlayers;
        for ($i=0; $i<$nLayers; $i++ )
        {
            $oLayer  = $oNewMap->getLayer( $i );
            $oLayer->set( 'status', MS_OFF );
        }
        //$oCloneLayer = @$oNewMap->getLayerByName( $aROI['selectedLayer'] );
        $oLayer = @$oNewMap->getLayerByName( $aROI['selectedLayer'] );
        //$oLayer = ms_newLayerObj( $oNewMap, $oCloneLayer );

        // process the layer if found
        if ( $oLayer )
        {
            // init
            $nResults = 0;
            $szCurrentTemplate = '';
            $nCurrentStatus = -1;
            $nCurrentTolerance = 0;
            $nCurrentToleranceUnits = 0;

            //create the query template that turns polygons into something useful.
            $szTemplatePath = getSessionSavePath()."roirenderer_template.html";
            if (!file_exists( $szTemplatePath ) )
            {
                $szTemplate = '[shpxy xf="," yf="," sf="|" proj="image"]';
                $hTemplate = fopen( $szTemplatePath, "w+" );
                fwrite( $hTemplate, $szTemplate );
                fclose( $hTemplate );
            }

            // put the template on the layer
            $szCurrentTemplate = $oLayer->template;
            $oLayer->set( "template", $szTemplatePath );

            // force the layer on
            $nCurrentStatus = $oLayer->status;
            $oLayer->set( "status", MS_ON );

            // set tolerance according to layer type
            $nCurrentTolerance = $oLayer->tolerance;
            switch ( $oLayer->type )
            {
                case MS_LAYER_POLYGON:
                    $oLayer->set( 'tolerance', 0 );
                    break;

                case MS_LAYER_LINE:
                    $oLayer->set( 'tolerance', 10 );
                    break;

                default:
                    $oLayer->set( 'tolerance', 5 );
                    break;
            }

            // set the units
            $nCurrentToleranceUnits = $oLayer->toleranceunits;
            $oLayer->set( 'toleranceunits', MS_PIXELS );

            // save query to file on first run
            //if ( count( $aROI['aGeoCoords'] ) == 2 )
            if ( !isset( $aROI['query_file'] ) )
            {
                // record coords and clear var
                $aROI['aOrigCoords'] = $aROI['aGeoCoords'];

                // reset the geocoords array
                //unset( $aROI['aGeoCoords'] );

                // create new point object to query
                $oPoint = ms_newPointObj( );
                $oPoint->setXY( $aROI['aOrigCoords'][0], $aROI['aOrigCoords'][1] );

                // run query
                @$oLayer->queryByPoint( $oPoint, MS_SINGLE, 0 );

                // check the number of results
                $nResults = $oLayer->getNumResults();

                // save the results to a file
                if ($nResults > 0)
                {
                    // save the query results for future passes
                    $aROI['query_file'] = getSessionSavePath()."/".uniqid("").".qry";
                    $oNewMap->saveQuery( $aROI['query_file'] );
                }
                else
                {
                    // give error message
                    //renderText('Nothing found at the selected location.');
                }

                // save the buffer amount
                $aROI['buffer'] = isset($_REQUEST['featurebuffer'])?$_REQUEST['featurebuffer']:0;
            }
            else if (isset($aROI['query_file']) && file_exists( $aROI['query_file'] ))
            {
                // load the query and find a layer with query results
                $oNewMap->loadQuery( $aROI['query_file'] );
                $nResults = $oLayer->getNumResults();
            }

            // process query results
            if ( $nResults > 0 )
            {
                // check for buffering
                if ( isset( $aROI['buffer'] ) && $aROI['buffer'] > 0 )
                {
                    $bBuffered = true;
                }

                // process buffered and unbuffered separately
                if ( !$bBuffered )
                {
                    // process the query through the template
                    $szCode = $oNewMap->processQueryTemplate( array(), MS_FALSE );

                    // multiple queries could be run so take only the last one
                    $aszCode = explode( '|', $szCode );
                    $szCode = substr($aszCode[0], 0, -1 );

                    // convert the text string of coords into an array
                    $szCode = '$'.'aPoints = array('.$szCode.');';
                    eval( $szCode );
                }
                else
                {
                    // loop through the query results and buffer
                    $oLayer->open();

                    // add element to results array for each result row
                    for ( $i=0; $i < $nResults; $i++ )
                    {
                        // get next shape row
                        $oResult = $oLayer->getResult($i);
                        $oShape = $oLayer->getShape($oResult->tileindex,
                                                                    $oResult->shapeindex);

                        // check if a buffer has been set in the session
                        $oShape = $oShape->buffer( $aROI['buffer'] );

                        // loop through lines of the shape
                        for( $j=0; $j<$oShape->numlines; $j++)
                        {
                            // get the next line
                            $oLine = $oShape->line( $j );

                            // loop through each point on the line (for a polygon, the first and last point are the same so skip)
                            $nNumPoints = $oShape->type == MS_SHAPE_POLYGON?$oLine->numpoints - 1:$oLine->numpoints;
                            for( $k=0; $k<$nNumPoints; $k++ )
                            {
                                // get the next point
                                $oPoint = $oLine->point( $k );

                                // convert the points to pixel and store in array
                                array_push( $aPoints, Geo2Pix($oPoint->x, 0, $GLOBALS['width'],
                                            $GLOBALS['geoMinX'], $GLOBALS['geoMaxX'] ) );
                                array_push( $aPoints, Geo2Pix($oPoint->y, 0, $GLOBALS['height'],
                                            $GLOBALS['geoMinY'], $GLOBALS['geoMaxY'], 1 ) );
                            }
                        }

                    }

                    // close the layer
                    $oLayer->close();
                }

                // record the number of points (half the number of entries)
                $nPoints = count($aPoints)/2;

                // set flag according to layer type
                if ( $oLayer->type == MS_LAYER_POLYGON || $bBuffered )
                {
                    $bPoly = true;
                }
                else if ( $oLayer->type == MS_LAYER_LINE )
                {
                    $bLine = true;
                }
            }
            else
            {
                renderText('Results not found - '.time());
            }

            //$oLayer->set( 'status', MS_DELETE );

            // undo everything that was set
            $oLayer->set( 'status', $nCurrentStatus );
            $oLayer->set( 'template', $szCurrentTemplate );
            $oLayer->set( 'tolerance', $nCurrentTolerance );
            $oLayer->set( 'toleranceunits', $nCurrentToleranceUnits );

        }

        // clear the map object
        $oNewMap = '';
    }

    // draw poly or ellipse or feature
    if ( $bPoly && $nPoints > 0)
    {
        if ( !imagefilledpolygon( $GLOBALS['oImage'], $aPoints, $nPoints, $nFillColor) )
        {
            renderText( 'Error drawing polygon' );
        }
    }
    else if ($aROI['type'] == 'circle')
    {
        //renderText( $dCenterX.','.$dCenterY.' radius='.$dRadius );
        $nEdgeColor = allocateImageFromHexValue( $GLOBALS['oImage'], $aROI['edgecolor']);

        imagefilledellipse ( $GLOBALS['oImage'], $dCenterX,
            $dCenterY,  $dRadius * 2+$aROI['edgewidth'], $dRadius * 2+$aROI['edgewidth'],
            $nEdgeColor );

        imagefilledellipse ( $GLOBALS['oImage'], $dCenterX,
            $dCenterY,  $dRadius * 2, $dRadius * 2,
            $nFillColor );
    }
    else if ( ( $aROI['type'] == 'line' || $bLine ) && $nPoints > 0 )
    {
        for ( $i=2 ; $i < $nPoints*2 ; $i+=2 )
        {
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2], $aPoints[$i-1],
                        $aPoints[$i], $aPoints[$i+1], $nFillColor );
            // Draw 9 lines to simulate thick lines
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2]+1, $aPoints[$i-1],
                        $aPoints[$i]+1, $aPoints[$i+1], $nFillColor );
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2]+1, $aPoints[$i-1]+1,
                        $aPoints[$i]+1, $aPoints[$i+1]+1, $nFillColor );
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2], $aPoints[$i-1]+1,
                        $aPoints[$i], $aPoints[$i+1]+1, $nFillColor );
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2]-1, $aPoints[$i-1],
                        $aPoints[$i]-1, $aPoints[$i+1], $nFillColor );
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2], $aPoints[$i-1]-1,
                        $aPoints[$i], $aPoints[$i+1]-1, $nFillColor );
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2]-1, $aPoints[$i-1]-1,
                        $aPoints[$i]-1, $aPoints[$i+1]-1, $nFillColor );
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2]+1, $aPoints[$i-1]-1,
                        $aPoints[$i]+1, $aPoints[$i+1]-1, $nFillColor );
            imageline ( $GLOBALS['oImage'], $aPoints[$i-2]-1, $aPoints[$i-1]+1,
                        $aPoints[$i]-1, $aPoints[$i+1]+1, $nFillColor );
        }
    }
    else if ( $aROI['type'] == 'point' )
    {
        imagefilledellipse ( $GLOBALS['oImage'], $dCenterX,
            $dCenterY,  $dRadius * 2, $dRadius * 2,
            $nFillColor );
    }
    else
    {
        //something went wrong, can't do anything here.
    }
}

// clean up after an ROI, specifically Feature ROIs ...
function cleanUpROI( $aROI )
{
    if ($aROI['type'] == 'feature' && file_exists($aROI['query_file']))
    {
        unlink( $aROI['query_file'] );
        if (file_exists($aROI['query_file'].".txt"))
            unlink($aROI['query_file'].".txt");
    }
}

//use this to put text into the output image (for debugging :))
function renderText( $szText )
{
    $x = 10;
    if (!isset($GLOBALS['y']))
        $GLOBALS['y'] = 10;
    else
        $GLOBALS['y'] += 20;
    imagestring( $GLOBALS['oImage'], 2, $x, $GLOBALS['y'], $szText, $GLOBALS['nBlack'] );

}
/*
 * return a color index from the given image for a given hex color.  If opacity
 * is not false, then assume that it is either a percent or a number from 0-100 and
 * massage accordingly into a value between 0 (transparent) and 127 (opaque).
 */
function allocateImageFromHexValue( $im, $szHexColor, $opacity = false )
{
    if (substr($szHexColor, 0, 1) == "#")
        $szHexColor = substr($szHexColor, 1 );
    $nRed = intval(substr($szHexColor,0,2),16);
    $nGreen = intval(substr($szHexColor,2,2),16);
    $nBlue = intval(substr($szHexColor,4,2),16);

    if ($opacity && $opacity >= 0)
    {
        if ($opacity < 1)
            $opacity = $opacity * 100;
        $opacity = round(127 - ($opacity / 100 * 127));
        //IE doesn't support transparency like this in PNG files
        //$nColor = imagecolorallocatealpha( $im, $nRed, $nGreen, $nBlue, $opacity );
        $nColor = imagecolorresolvealpha( $im, $nRed, $nGreen, $nBlue, $opacity );
     }
    else
        $nColor = imagecolorallocate( $im, $nRed, $nGreen, $nBlue );
    return $nColor;
}

/**
 * convert a geocoded position to pixel coord
 *
 * @param nGeoPos double Geocoded position
 * @param dfPixMin double minimum map pixel value
 * @param dfPixMax double maximum map pixel value
 * @param dfGeoMin double minimum map geocoded value
 * @param dfGeoMax double maximum map geocoded value
 * @param nInverseGeo integer optional flag to inverse , set to 1 for
 *                            Y pixel coordinates where UL > LR
 * @return double geocoded position
 */
function Geo2Pix ($nGeoPos, $dfPixMin, $dfPixMax, $dfGeoMin,
                       $dfGeoMax, $nInverseGeo = "")
    {
        //echo "$nGeoPos, $dfPixMin, $dfPixMax, $dfGeoMin, $dfGeoMax";
        //renderText( "$nGeoPos, $dfPixMin, $dfPixMax, $dfGeoMin, $dfGeoMax" );
        // calculate the geocoded & pixel width
        $dfWidthGeo = abs($dfGeoMax - $dfGeoMin);
        $dfWidthPix = abs($dfPixMax - $dfPixMin);

        // get ratio
        $dfGeoToPix = $dfWidthPix / $dfWidthGeo;

        // get difference
        if (!$nInverseGeo)
            $dfDeltaGeo = $nGeoPos - $dfGeoMin;
        else
            $dfDeltaGeo = $dfGeoMax - $nGeoPos;

        // calculate
        $dfDeltaPix = $dfDeltaGeo * $dfGeoToPix;
        $dfPosPix = $dfPixMin + $dfDeltaPix;

        //echo ": ".round($dfPosPix)."<BR>";
        //renderText( "$nGeoPos, $dfPixMin, $dfPixMax, $dfGeoMin, $dfGeoMax, ".round($dfPosPix) );
        // return value
        return round ($dfPosPix);

        // end pixel_to_geo function
    }

?>
