<?php
/**
 * ROIPGDigitize Widget Class
 *
 * @project     Chameleon
 * @revision    $Id: ROIPGDigitize.widget.php,v 1.1 2007/07/20 16:07:09 jlacroix Exp $
 * @purpose     Use a ROI array to add GIS info
 * @author      Julien-Samuel Lacroix (jlacroix@mapgears.com)
 * @copyright
 * <b>Copyright (c) 2006, Mapgears Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once(CHAMELEON_PATH."/widgets/Widget.php");
include_once(CHAMELEON_PATH."/widgets/Button.php");
include_once(CHAMELEON_PATH."/widgets/SQLQuery/SQLQuery.widget.php");

/**
 * ROIPGDigitize
 */
class ROIPGDigitize extends SQLQuery
{
    var $moButton;
    var $mszTable = '';
    var $mszPolygonTable = '';
    var $mszLineTable = '';
    var $mszPointTable = '';
    var $mszLayer = '';
    var $mszGeometryColumn = 'the_geom';
    var $maszMoreColumns = array();
    var $maszErrorMsg = array();
    var $mnPGProjection = 4326;

    function ROIPGDigitize()
    {
        // We will fetch the database information from the layer connection
        $this->mszSQLQuery = "FROMLAYER";
        $this->mszServer   = "FROMLAYER";
        $this->mszDatabase = "FROMLAYER";
        $this->mnPort = 'FROMLAYER';
        $this->mszUserName = "FROMLAYER";
        $this->mszPassword = "FROMLAYER";

        parent::SQLQuery();

        $this->mszSQLQuery = "";
        $this->mszServer   = "";
        $this->mszDatabase = "";
        $this->mnPort = '';
        $this->mszUserName = "";
        $this->mszPassword = "";

        $this->maAttributes["TABLE"] = new StringAttribute( "TABLE", true );
        $this->maAttributes["POLYGONTABLE"] = new StringAttribute( "POLYGONTABLE", false );
        $this->maAttributes["LINETABLE"] = new StringAttribute( "LINETABLE", false );
        $this->maAttributes["POINTTABLE"] = new StringAttribute( "POINTTABLE", false );
        $this->maAttributes["LAYER"] = new StringAttribute( "LAYER", true );
        $this->maAttributes["GEOMETRYCOLUMN"] = new StringAttribute( "GEOMETRYCOLUMN", false );

        $this->maAttributes["PGPROJECTION"] = new IntegerAttribute( "PGPROJECTION", false, 0 );
        $this->moButton = new CWCButton($this);

        $this->mnPriority = PRIORITY_MEDIUM;
    }

    function InitDefaults()
    {
        parent::InitDefaults();

        $this->mszDBType = "PGSQL";
        $this->mszSQLQuery = '';

        if(isset($this->maParams["SHAREDRESOURCENAME"]))
            $this->mszSharedResourceName=$this->maParams["SHAREDRESOURCENAME"];
        else
            $this->mszSharedResourceName = "ROIPGDigitizeSR";
        if(isset($this->maParams["TABLE"]))
            $this->mszTable = $this->maParams["TABLE"];
        if(isset($this->maParams["POLYGONTABLE"]))
            $this->mszPolygonTable = $this->maParams["POLYGONTABLE"];
        if(isset($this->maParams["LINETABLE"]))
            $this->mszLineTable = $this->maParams["LINETABLE"];
        if(isset($this->maParams["POINTTABLE"]))
            $this->mszPointTable = $this->maParams["POINTTABLE"];
        if(isset($this->maParams["LAYER"]))
            $this->mszLayer = $this->maParams["LAYER"];
        if(isset($this->maParams["PGPROJECTION"]))
            $this->mnPGProjection = $this->maParams["PGPROJECTION"];

        // Extract the DB connection from the layer connection
        $oLayer = $this->moMapObject->oMap->getLayerByName($this->mszLayer);
        if($oLayer->connectiontype == MS_POSTGIS)
        {
            // Extract host name
            if(preg_match('/host=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mszServer = $aMatch[1];
            // Extract port
            if(preg_match('/port=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mnPort = $aMatch[1];
            // Extract user name
            if(preg_match('/user=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mszUserName = $aMatch[1];
            // Extract password
            if(preg_match('/password=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mszPassword = $aMatch[1];
            // Extract database name
            if(preg_match('/dbname=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mszDatabase = $aMatch[1];
        }

        if(isset($this->maParams["GEOMETRYCOLUMN"]))
            $this->mszGeometryColumn = $this->maParams["GEOMETRYCOLUMN"];

        if(isset($this->maszContents["MORECOLUMNS"]))
        {

            foreach($this->maszContents["MORECOLUMNS"] as $aszMoreColumn)
            {
                if(isset($aszMoreColumn['FIELD']) && 
                   $aszMoreColumn['FIELD'] != '' &&
                   ((isset($aszMoreColumn['VALUE']) && 
                     $aszMoreColumn['VALUE'] != '') || 
                    (isset($aszMoreColumn['URL']) && 
                     $aszMoreColumn['URL'] != '')))
                {
                    $this->maszMoreColumns[] = $aszMoreColumn;
                }
            }
        }

        //this widget should never belong to a toolset
        $this->maParams["TOOLSET"] = "";
        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick('ROIPGDigitizeClick');
    }

    function ParseURL()
    {
        if(!$this->moButton->ParseURL())
            return false;

        $szCmd = "";
        if ($this->isVarSet( "NAV_CMD" ))
            $szCmd = $this->getVar( "NAV_CMD" );

        if ($szCmd == "ROIPG_DIGITIZE")
        {
            $this->addGeometryToDB();
        }

        return true;
    }

    function addGeometryToDB()
    {

        // Build a SQL Query template
        // Once we have it, we will add to it the Geometry statement to include
        // in the database for each ROI.
        //
        // To add the polygon to the DB we will build a PostGreSQL INSERT 
        // request to add it to the geometry table.
        $szSQL = "INSERT INTO __TABLE__";

        // Define the columns that we will need. There's obviously the 
        // geometry column, but also all those passed in the MoreColumn content
        // attribute in the widget
        $szSQL .= " (".$this->mszGeometryColumn;

        // Add the other fields via the widget parameters
        if(count($this->maszMoreColumns))
            foreach($this->maszMoreColumns as $aszColumn)
                $szSQL .= ','.$aszColumn['FIELD'];
        $szSQL .= ') ';

        // Finally add the values to the SQL INSERT query
        // First build the Geometry Values.
        $szGeometryFromText = '__PUT_GEOMETRY_HERE__';
        
        $szSQL .= " VALUES($szGeometryFromText";

        // Add the other fields via the widget parameters
        if(count($this->maszMoreColumns))
        {
            foreach($this->maszMoreColumns as $aszColumn)
            {

                if(isset($aszColumn['VALUE']))
                    $szSQL .= ',\''.$aszColumn['VALUE'].'\'';
                else if(isset($aszColumn['URL']) && 
                        $this->getVar($aszColumn['URL']))
                    $szSQL .= ',\''.$this->getVar($aszColumn['URL']).'\'';
                else if(isset($aszColumn['URL']))
                {
                    $this->maszErrorMsg[] = "Missing URL attribute: ".$aszColumn['URL'];
                    return;
                }
                else
                    $szSQL .= ',""';
            }
        }
        $szSQL .= ");";

        // For each ROI, we need to extract the coordinates and
        // add them to the table.
        $nROICount = count($_SESSION['ROIRENDERER']);

        for($i=0; $i<$nROICount; $i++)
        {
            $szTmpSQL = $szSQL;
            if($this->mszPolygonTable && 
               ($_SESSION['ROIRENDERER'][$i]['type'] == 'polygon' || 
                $_SESSION['ROIRENDERER'][$i]['type'] == 'rectangle'))
            {
                $szTmpSQL = str_replace( '__TABLE__', $this->mszPolygonTable, 
                                         $szTmpSQL);
            }
            else if($this->mszLineTable && 
                    $_SESSION['ROIRENDERER'][$i]['type'] == 'line')
            {
                $szTmpSQL = str_replace( '__TABLE__', $this->mszLineTable, 
                                         $szTmpSQL);
            }
            else if($this->mszPointTable && 
                    $_SESSION['ROIRENDERER'][$i]['type'] == 'point')
            {
                $szTmpSQL = str_replace( '__TABLE__', $this->mszPointTable, 
                                         $szTmpSQL);
            }
            else
            {
                $szTmpSQL = str_replace('__TABLE__',$this->mszTable,$szTmpSQL);
            }

            $szReturn = $this->getSQLGeometry($_SESSION['ROIRENDERER'][$i]);

            if($szReturn)
            {
                $this->mszSQLQuery .= str_replace( '__PUT_GEOMETRY_HERE__', 
                                                   $szReturn, $szTmpSQL);
            }
            else
            {
                $this->mszSQLQuery = '';
                break;
            }
        }
        $_SESSION['ROIRENDERER'] = array();

        if($this->mszSQLQuery)
        {
            parent::ParseURL();
        }
    }

    function getSQLGeometry($aROI)
    {
        // Extract the Geometries
        $aCoords = array();
        switch($aROI['type'])
        {
          case 'rectangle':
            // Only mode 1 and 2 (add) is supported for now.
            if($aROI['mode'] != 1 and $aROI['mode'] != 2)
            {
                $this->maszErrorMsg[] = "Invalid ROI Mode. ".
                    "Only addition mode is supported";
            }

            // For rectangle we assume we have 4 coordinates and 
            // build a rectangle with that. The 4 coords are the bbox
            $aCoords[] = array($aROI['aGeoCoords'][0], 
                               $aROI['aGeoCoords'][1]);
            $aCoords[] = array($aROI['aGeoCoords'][0], 
                               $aROI['aGeoCoords'][3]);
            $aCoords[] = array($aROI['aGeoCoords'][2], 
                               $aROI['aGeoCoords'][3]);
            $aCoords[] = array($aROI['aGeoCoords'][2], 
                               $aROI['aGeoCoords'][1]);
            $aCoords[] = array($aROI['aGeoCoords'][0], 
                               $aROI['aGeoCoords'][1]);
            break;
          case 'polygon':
            // Only mode 1 and 2 (add) is supported for now.
            if($aROI['mode'] != 1 and $aROI['mode'] != 2)
            {
                $this->maszErrorMsg[] = "Invalid ROI Mode. ".
                    "Only addition mode is supported";
            }

            // For a polygon, we have a pair of value for each vertex.
            $nVertex = count($aROI['aGeoCoords']);
            for($j=0; $j<$nVertex; $j++)
            {
                $aCoords[] = array($aROI['aGeoCoords'][$j], 
                                   $aROI['aGeoCoords'][++$j]);
            }
            // To close polygons, we must end with the first vertex
            $aCoords[] = array($aROI['aGeoCoords'][0], 
                               $aROI['aGeoCoords'][1]);
            break;
          case 'line':
            // Only mode 1 and 2 (add) is supported for now.
            if($aROI['mode'] != 1 and $aROI['mode'] != 2)
            {
                $this->maszErrorMsg[] = "Invalid ROI Mode. ".
                    "Only addition mode is supported";
            }

            // For a line, we have a pair of value for each vertex.
            $nVertex = count($aROI['aGeoCoords']);
            for($j=0; $j<$nVertex; $j++)
            {
                $aCoords[] = array($aROI['aGeoCoords'][$j], 
                                   $aROI['aGeoCoords'][++$j]);
            }
            break;
          case 'point':
            // Only mode 1 and 2 (add) is supported for now.
            if($aROI['mode'] != 1 and $aROI['mode'] != 2)
            {
                $this->maszErrorMsg[] = "Invalid ROI Mode. ".
                    "Only addition mode is supported";
            }

            // For rectangle we assume we have 4 coordinates and 
            // build a rectangle with that. The 4 coords are the bbox
            $aCoords[] = array($aROI['aGeoCoords'][0], 
                               $aROI['aGeoCoords'][1]);
            break;
          default:
            $aCoords = array(); 
            $this->maszErrorMsg[] = "Invalid ROI Type. ".
                "Only rectangle, polygon, line and point geometries are ".
                "supported for now.";
            break;
        }

        if(count($aCoords) == 0)
            return '';

        // Finally add the values to the SQL INSERT query
        // First build the Geometry Values.
        $szGeometryFromText = '';
        foreach($aCoords as $aVertex)
        {
            if($szGeometryFromText != '')
                $szGeometryFromText .= ',';
            $szGeometryFromText .= implode(' ', $aVertex);
        }

        $szTempMapProj = $this->moMapObject->oMap->getprojection();
        $aSRS = explode(":", $szTempMapProj);
        if($szGeometryFromText)
        {
            if($aROI['type'] == 'polygon' || $aROI['type'] == 'rectangle')
            {
                $szGeometryFromText = "GeometryFromText('MULTIPOLYGON(((".
                    $szGeometryFromText.")))', $aSRS[1])";
            }
            else if($aROI['type'] == 'line')
            {
                $szGeometryFromText = "GeometryFromText('MULTILINESTRING((".
                    $szGeometryFromText."))', $aSRS[1])";
            }
            else if($aROI['type'] == 'point')
            {
                $szGeometryFromText = "GeometryFromText('POINT(".
                    $szGeometryFromText.")', $aSRS[1])";
            }
        }

        // Reproject the data if necessary

        if ($aSRS[1] != $this->mnPGProjection && $szGeometryFromText)
            $szGeometryFromText = "Transform($szGeometryFromText,$this->mnPGProjection)";

        if(!$szGeometryFromText)
            $szGeometryFromText = '""';

        return $szGeometryFromText;
    }

    function GetJavascriptFunctions()
    {
        $aReturn = array();
        $aReturn = $this->moButton->GetJavascriptFunctions();

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
            $bCWCJSAPI = 1;
        else
            $bCWCJSAPI = 0;

        if($bCWCJSAPI)
        {
            $_SESSION['ROIPG_DIGITIZE_MORECOLUMNS'] = $this->maszMoreColumns;

            $szJSMoreColumns = "new Array(";
            //Need to pass the URL values
            $nCount = count($this->maszMoreColumns);
            $bFirstElement = true;

            if($nCount)
            {
                foreach($this->maszMoreColumns as $aszColumn)
                {
                    if(isset($aszColumn['URL']))
                    { 
                        if(!$bFirstElement)
                        {
                            $szJSMoreColumns .= ",";
                        }
                        $szJSMoreColumns .= '"'.$aszColumn['URL'].'"';
                        $bFirstElement = false;
                    }
                }
                $szJSMoreColumns .= ")";

            }
        }
        else
        {
            $szJSMoreColumns = "new Array();";
        }


        $szJsFunctionName = "ROIPGDigitizeError";
        $szErrorMsg = implode("\n", $this->maszErrorMsg);
        $szFunction = <<<EOT
function {$szJsFunctionName}()
{
    alert('{$szErrorMsg}');
    return true;
}
EOT;
        if($szErrorMsg)
            $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "ROIPGDigitizeClick";
        $szFunction = <<<EOT
function {$szJsFunctionName}()
{
    {$this->mszHTMLForm}.NAV_CMD.value = "ROIPG_DIGITIZE";

    if({$bCWCJSAPI})
    {
        var aMoreColumns = {$szJSMoreColumns};
        nCount = aMoreColumns.length + 7;

        aHiddenVars = new Array(nCount);
        aHiddenVars[0] = new Array(2);
        aHiddenVars[0][0] = {$this->mszHTMLForm}.NAV_CMD.name;
        aHiddenVars[0][1] = {$this->mszHTMLForm}.NAV_CMD.value;
        aHiddenVars[1] = new Array(2);
        aHiddenVars[1][0] = "ROIPG_DIGITIZE_TABLE";
        aHiddenVars[1][1] = "{$this->mszTable}";
        aHiddenVars[2] = new Array(2);
        aHiddenVars[2][0] = "ROIPG_DIGITIZE_POLYGONTABLE";
        aHiddenVars[2][1] = "{$this->mszPolygonTable}";
        aHiddenVars[3] = new Array(2);
        aHiddenVars[3][0] = "ROIPG_DIGITIZE_LINETABLE";
        aHiddenVars[3][1] = "{$this->mszLineTable}";
        aHiddenVars[4] = new Array(2);
        aHiddenVars[4][0] = "ROIPG_DIGITIZE_POINTTABLE";
        aHiddenVars[4][1] = "{$this->mszPointTable}";
        aHiddenVars[5] = new Array(2);
        aHiddenVars[5][0] = "ROIPG_DIGITIZE_LAYER";
        aHiddenVars[5][1] = "{$this->mszLayer}";
        aHiddenVars[6] = new Array(2);
        aHiddenVars[6][0] = "ROIPG_DIGITIZE_PGPROJECTION";
        aHiddenVars[6][1] = "$this->mnPGProjection";


        for(var i = 0; i < aMoreColumns.length; i++)
        {

            if({$this->mszHTMLForm}[aMoreColumns[i]])
            {
                nElement = 7 + i;
                aHiddenVars[nElement] = new Array(1);
                aHiddenVars[nElement][0] = aMoreColumns[i];
                aHiddenVars[7 + i][1] = {$this->mszHTMLForm}[aMoreColumns[i]].value;

            }

        }

        szOnLoad = "ROIPGDigitizeExtentsChanged()";
        goCWCJSAPI.CallServer(szOnLoad, aHiddenVars);

    }
    else
    {
        {$this->mszHTMLForm}.submit();
    }
    return true;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        if($bCWCJSAPI)
        {
            $szJsFunctionName = "ROIPGDigitizeExtentsChanged";
            $szFunction = <<<EOT
function {$szJsFunctionName}()
{
    goCWCJSAPI.TriggerEvent(MAP_EXTENT_CHANGED);

}
EOT;

            $aReturn[$szJsFunctionName] = $szFunction;
        }//End of JSAPI if

        return $aReturn;
    }

    function GetJavascriptVariables()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptVariables();
        else return array();
    }

    function GetJavascriptInitFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
        {        
            $aResult = $this->moButton->GetJavascriptInitFunctions();
        }
        
        return $aResult;
    }

    function GetJavascriptOnLoadFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptOnLoadFunctions();
        else
            $aReturn = array();
        if(count($this->maszErrorMsg) > 0)
            $aReturn["ROIPGDigitizeError"] = "ROIPGDigitizeError();";
        
        return $aReturn;
    }
    
    function GetJavascriptIncludeFunctions()
    {
        $aReturn = $this->moButton->GetJavascriptIncludeFunctions();

        return $aReturn;
    }

    /**
     * return hidden variables required for this tool
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();
        
        return $aReturn;
    }

    function DrawPublish()
    {
        if(!$this->mbVisible)
            return "<!-- ROIPGDigitize widget is here -->";

        return $this->moButton->DrawPublish();
    }
}
?>
