<?php
/**
 * ROIPGManager Widget Class
 *
 * @project     Chameleon
 * @revision    $Id: ROIPGManager.widget.php,v 1.1 2007/07/20 16:07:09 jlacroix Exp $
 * @purpose     Use a ROI array to add GIS info
 * @author      Julien-Samuel Lacroix (jlacroix@mapgears.com)
 * @copyright
 * <b>Copyright (c) 2006, Mapgears Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once(CHAMELEON_PATH."/widgets/Widget.php");
include_once(CHAMELEON_PATH."/widgets/Button.php");
include_once(CHAMELEON_PATH."/widgets/NavTool.php");

/**
 * ROIPGDigitize
 */
class ROIPGManager extends NavTool
{
    var $mszLayer = '';
    var $szLayerId = '';
    var $mszHighlightColor = '';
    var $maszMoreColumns = array();
    var $maszErrorMsg = array();

    function ROIPGManager()
    {
        parent::NavTool();

        $this->SetNavCommand("ROIPGMANAGER");
        $this->mnAllowRectangle = 0;
        $this->maAttributes["LAYER"] = new StringAttribute( "LAYER", true );
        $this->maAttributes["LAYERID"] = new StringAttribute( "LAYERID", true );
        $this->maAttributes["HIGHLIGHTCOLOR"] = new RGBColorAttribute( "HIGHLIGHTCOLOR", false );

        $this->mnMaturityLevel = MATURITY_BETA;
    }

    function InitDefaults()
    {
        parent::InitDefaults();

        $this->mszDBType = "PGSQL";
        if(isset($this->maParams["LAYER"]))
            $this->mszLayer = $this->maParams["LAYER"];
        if(isset($this->maParams["LAYERID"]))
            $this->mszLayerId = $this->maParams["LAYERID"];
        if(isset($this->maParams["HIGHLIGHTCOLOR"]))
            $this->mszHighlightColor = $this->maParams["HIGHLIGHTCOLOR"];

        $this->mbNavSubmit = true;
    }

    function ParseURL()
    {
        parent::ParseURL();

        $szCmd = "";
        if ($this->isVarSet( "NAV_CMD" ))
            $szCmd = $this->getVar( "NAV_CMD" );

        if ($szCmd == "ROIPGMANAGER")
        {
            $szInputCoords = "";
            $szInputCoords = $this->getVar( "NAV_INPUT_COORDINATES" );
            $szInputType = $this->getVar( "NAV_INPUT_TYPE" );

            if(strlen($szInputCoords) <= 0)
            {
                $_SESSION['gErrorManager']->setError(ERR_WARNING,
                    'ERROR: Can\'t select a ROI without the input coordinates' );
                return false;
            }
            if (strlen($szInputType) <= 0)
            {
                $_SESSION['gErrorManager']->setError(ERR_WARNING,
                    $this->moMLT->get( $this->mszWidgetName, '0', 'ERROR: No input type specified in ' )."ZoomIn.php.");
                return false;
            }

            $oResult = '';
            $oMapQuery = new MapQuery($this->moMapObject);
            $aszLayerList = explode(',', $this->mszLayer);
            $oMapQuery->setQueryLayersByName( $aszLayerList );
            if ($szInputType == "POINT")
            {
                $aPixPos = explode(",", $szInputCoords);
                $oResult = $oMapQuery->executePointQuery($aPixPos[0], 
                                                         $aPixPos[1]);
            }
            else if ($szInputType == "RECTANGLE")
            {
                $aPixPos = explode(";", $szInputCoords);
                $aPixMin = explode(",", $aPixPos[0]);
                $aPixMax = explode(",", $aPixPos[1]);
/* -------------------------------------------------------------------- */
/*      check if the rectangle is bigger than the minmum size allowed.  */
/* -------------------------------------------------------------------- */

                if (abs($aPixMax[0] - $aPixMin[0]) < 4 &&
                    abs($aPixMax[1] - $aPixMin[1]) < 4)
                {
                    $oResult = $oMapQuery->executePointQuery($aPixPos[0], 
                                                             $aPixPos[1]);
                }
                else
                  $oResult = $oMapQuery->executeRectQuery($aPixMin[0], 
                                                          $aPixMin[1],
                                                          $aPixMax[0], 
                                                          $aPixMax[1]);
            }

            // If we clicked on a ROI, highlight it with the good color
            // We will create a class called "__ROIPGMANAGER__" and put
            // an expression in it to only filter the ROI we clicked on.
            $_SESSION['ROIPGSELECTION'] = array();
            for($i=0; $i<count($oResult->axResultSets); $i++)
            {
                if($oResult && $oResult->nNumResults > 0 && 
                   is_array($oResult->axResultSets[$i]) && 
                   isset($oResult->axResultSets[$i][0][$this->mszLayerId]) &&
                   count($_SESSION['ROIPGSELECTION']) == 0)
                {
                    // Put the selection in the session
                    $_SESSION['ROIPGSELECTION'] = array();
                    $_SESSION['ROIPGSELECTION'][] = 
                        $oResult->axResultSets[$i][0];

                    // Build the expression string
                    $nId = $oResult->axResultSets[$i][0][$this->mszLayerId];
                    $szExpression = "(\"[$this->mszLayerId]\" != \"$nId\")";

                    // First check if the class does not already exist
                    $oLayer = $this->moMapObject->oMap->getLayer( 
                        $oResult->axResultSets[$i][0]['LAYER_INDEX'] );
                    $oClass = $oLayer->getClass(0);

                    if($oClass->name == "__ROIPGMANAGER__")
                    {
                        // If the class exist, change the expression in it.
                        $oClass->setExpression($szExpression);
                    }
                    else
                    {
                        // If the class does not exist, create a new class and 
                        // change the color
                        // Copy the class 0 at 1
                        $oNewClass = ms_newClassObj($oLayer, $oClass);
                        $oClass->setExpression($szExpression);
                        $oClass->set('name', "__ROIPGMANAGER__");
                        $oStyle = $oNewClass->getStyle(0);
                        // Change the color
                        $aColor = explode(' ', $this->mszHighlightColor);
                        $oStyle->color->setRGB($aColor[0], 
                                               $aColor[1], 
                                               $aColor[2]);
                    }
                }
                else
                {
                    $oLayer = $this->moMapObject->oMap->getLayerByName(
                        $aszLayerList[$i] );
                    if($oLayer->numclasses > 1)
                    {
                        $oClass = $oLayer->getClass(0);
                        $oNewClass = $oLayer->getClass(1);

                        if($oClass->name == "__ROIPGMANAGER__")
                        {
                            // If the class exist, delete it.
                            $oClass->setExpression('');
                            $oClass->set('name', $oNewClass->name);
                            $oNewClass->set('status', MS_DELETE);
                        }
                    }
                }
            }
        }

        return true;
    }
}
?>
