<?php
/**
 * ROIPGRemove Widget Class
 *
 * @project     Chameleon
 * @revision    $Id: ROIPGRemove.widget.php,v 1.1 2007/07/20 16:07:09 jlacroix Exp $
 * @purpose     Use a ROI array to add GIS info
 * @author      Julien-Samuel Lacroix (jlacroix@mapgears.com)
 * @copyright
 * <b>Copyright (c) 2006, Mapgears Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once(CHAMELEON_PATH."/widgets/Widget.php");
include_once(CHAMELEON_PATH."/widgets/Button.php");
include_once(CHAMELEON_PATH."/widgets/SQLQuery/SQLQuery.widget.php");

/**
 * ROIPGRemove
 */
class ROIPGRemove extends SQLQuery
{
    var $moButton;
    var $mszTable = '';
    var $mszPolygonTable = '';
    var $mszLineTable = '';
    var $mszPointTable = '';
    var $mszLayer = '';
    var $mszColor = '0 255 255';
    var $mnTransparency = 50;
    var $mbROISelected = 0;
    var $maszMoreColumns = array();
    var $maszErrorMsg = array();

    function ROIPGRemove()
    {
        // We will fetch the database information from the layer connection
        $this->mszSQLQuery = "FROMLAYER";
        $this->mszServer   = "FROMLAYER";
        $this->mszDatabase = "FROMLAYER";
        $this->mnPort = 'FROMLAYER';
        $this->mszUserName = "FROMLAYER";
        $this->mszPassword = "FROMLAYER";

        parent::SQLQuery();

        $this->mszSQLQuery = "";
        $this->mszServer   = "";
        $this->mszDatabase = "";
        $this->mnPort = '';
        $this->mszUserName = "";
        $this->mszPassword = "";

        $this->maAttributes["TABLE"] = new StringAttribute( "TABLE", true );
        $this->maAttributes["POLYGONTABLE"] = new StringAttribute( "POLYGONTABLE", false );
        $this->maAttributes["LINETABLE"] = new StringAttribute( "LINETABLE", false );
        $this->maAttributes["POINTTABLE"] = new StringAttribute( "POINTTABLE", false );
        $this->maAttributes["LAYER"] = new StringAttribute( "LAYER", true );
        $this->maAttributes["LAYERID"] = new StringAttribute( "LAYERID", true );

        $this->moButton = new CWCButton($this);

        $this->mnPriority = PRIORITY_LOW;
    }

    function InitDefaults()
    {
        parent::InitDefaults();

        $this->mszDBType = "PGSQL";
        $this->mszSQLQuery = '';

        if(isset($this->maParams["SHAREDRESOURCENAME"]))
            $this->mszSharedResourceName=$this->maParams["SHAREDRESOURCENAME"];
        else
            $this->mszSharedResourceName = "ROIPGRemoveSR";
        if(isset($this->maParams["TABLE"]))
            $this->mszTable = $this->maParams["TABLE"];
        if(isset($this->maParams["POLYGONTABLE"]))
            $this->mszPolygonTable = $this->maParams["POLYGONTABLE"];
        if(isset($this->maParams["LINETABLE"]))
            $this->mszLineTable = $this->maParams["LINETABLE"];
        if(isset($this->maParams["POINTTABLE"]))
            $this->mszPointTable = $this->maParams["POINTTABLE"];
        if(isset($this->maParams["LAYER"]))
            $this->mszLayer = $this->maParams["LAYER"];
        if(isset($this->maParams["LAYERID"]))
            $this->mszLayerId = $this->maParams["LAYERID"];

        // Extract the DB connection from the layer connection
        $oLayer = $this->moMapObject->oMap->getLayerByName($this->mszLayer);
        if($oLayer->connectiontype == MS_POSTGIS)
        {
            // Extract host name
            if(preg_match('/host=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mszServer = $aMatch[1];
            // Extract port
            if(preg_match('/port=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mnPort = $aMatch[1];
            // Extract user name
            if(preg_match('/user=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mszUserName = $aMatch[1];
            // Extract password
            if(preg_match('/password=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mszPassword = $aMatch[1];
            // Extract database name
            if(preg_match('/dbname=([^\s]+)/', $oLayer->connection, $aMatch))
                if(isset($aMatch[1]) && $aMatch[1] != '')
                    $this->mszDatabase = $aMatch[1];
        }

        //this widget should never belong to a toolset
        $this->maParams["TOOLSET"] = "";
        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick('ROIPGRemoveClick');
    }

    function ParseURL()
    {
        if(!$this->moButton->ParseURL())
            return false;

        $szCmd = "";
        if ($this->isVarSet( "NAV_CMD" ))
            $szCmd = $this->getVar( "NAV_CMD" );

        if(isset($_SESSION["ROIPGSELECTION"]) && 
           count($_SESSION["ROIPGSELECTION"]) > 0 && 
           isset($_SESSION["ROIPGSELECTION"][0][$this->mszLayerId]))
        {
            $this->mbROISelected = true;
        }

        if(!$this->mbROISelected && isset($_SESSION["bCWCJSAPI"]))
        {
            return false;
        }
        if ($szCmd == "ROIPG_REMOVE" && $this->mbROISelected)
        {
            $this->removeGeometryFromDB(
                $_SESSION["ROIPGSELECTION"][0][$this->mszLayerId]);

            // Once erased, empty the selection
            $_SESSION["ROIPGSELECTION"] = array();
            $this->mbROISelected = 0;
        }

        return true;
    }

    function removeGeometryFromDB($nId)
    {
        // Build a SQL Query to delete that particular row in the table

        // Select the right table based on the type of the layer
        $szTable = $this->mszTable;
        $oLayer = $this->moMapObject->oMap->getLayer($_SESSION["ROIPGSELECTION"][0]['LAYER_INDEX']);
        if($this->mszPolygonTable &&  $oLayer->type == MS_LAYER_POLYGON)
        {
            $szTable = $this->mszPolygonTable;
        }
        else if($this->mszLineTable && $oLayer->type == MS_LAYER_LINE)
        {
            $szTable = $this->mszLineTable;
        }
        else if($this->mszPointTable && $oLayer->type == MS_LAYER_POINT)
        {
            $szTable = $this->mszPointTable;
        }


        $szSQL = "DELETE FROM ".$szTable;
        $szSQL .= " WHERE ".$this->mszLayerId." = '".$nId."'";

        $this->mszSQLQuery = $szSQL;
        if($this->mszSQLQuery)
        {
            parent::ParseURL();
        }
    }

    function GetJavascriptFunctions()
    {
        $aReturn = array();
        $aReturn = $this->moButton->GetJavascriptFunctions();

        $bCWCJSAPI = (isset($this->maSharedResourceWidgets["CWCJSAPI"])) ? 1 : 0;
        if($bCWCJSAPI)
            $_SESSION['bCWCJSAPI'] = $bCWCJSAPI;

        $szJsFunctionName = "ROIPGRemoveError";
        $szErrorMsg = implode("\n", $this->maszErrorMsg);
        $szFunction = <<<EOT
function {$szJsFunctionName}()
{
    alert('{$szErrorMsg}');
    return true;
}
EOT;
        if($szErrorMsg)
            $aReturn[$szJsFunctionName] = $szFunction;

        if ($bCWCJSAPI)
        {

            $szJsFunctionName = "ROIPGRemoveMsg";
            $szFunction = <<<EOT
function {$szJsFunctionName}(passedArgsObj)
{
     /* defaults */
     args = { szMsg:"" };
 
     /* override the defaults if necessary */
     for (var argName in passedArgsObj) {
         args[argName] = passedArgsObj[argName];
     }
 
     szErrorMsg = args.szMsg;
 
     if(szErrorMsg)
     {
         alert(szErrorMsg);
     }
     return true;
}
EOT;
            $aReturn[$szJsFunctionName] = $szFunction;
        $szJsFunctionName = "ROIPG_REMOVE_EXTENTSCHANGED";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the size of the map is changed(JSAPI)
 */
function {$szJsFunctionName}()
{
        goCWCJSAPI.TriggerEvent(MAP_EXTENT_CHANGED);
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;

        }//End of if JSAPI

        $szJsFunctionName = "ROIPGRemoveClick";
        $szFunction = <<<EOT
function {$szJsFunctionName}()
{
    {$this->mszHTMLForm}.NAV_CMD.value = "ROIPG_REMOVE";

        if ({$bCWCJSAPI})
        {
            {$this->mszHTMLForm}.EXTENTHISTORY_BNAV.value = "true";

            aHiddenVars = new Array(6);
            aHiddenVars[0] = new Array(2);
            aHiddenVars[0][0] = {$this->mszHTMLForm}.NAV_CMD.name;
            aHiddenVars[0][1] = {$this->mszHTMLForm}.NAV_CMD.value;
            aHiddenVars[1] = new Array(2);
            aHiddenVars[1][0] = "ROIPG_REMOVE_TABLE";
            aHiddenVars[1][1] = "{$this->mszTable}";
            aHiddenVars[2] = new Array(2);
            aHiddenVars[2][0] = "ROIPG_REMOVE_POLYGONTABLE";
            aHiddenVars[2][1] = "{$this->mszPolygonTable}";
            aHiddenVars[3] = new Array(2);
            aHiddenVars[3][0] = "ROIPG_REMOVE_LINETABLE";
            aHiddenVars[3][1] = "{$this->mszLineTable}";
            aHiddenVars[4] = new Array(2);
            aHiddenVars[4][0] = "ROIPG_REMOVE_POINTTABLE";
            aHiddenVars[4][1] = "{$this->mszPointTable}";
            aHiddenVars[5] = new Array(2);
            aHiddenVars[5][0] = "ROIPG_REMOVE_LAYER";
            aHiddenVars[5][1] = "{$this->mszLayer}";
            aHiddenVars[6] = new Array(2);
            aHiddenVars[6][0] = "ROIPG_REMOVE_LAYERID";
            aHiddenVars[6][1] = "{$this->mszLayerId}";

            szOnLoad = 'ROIPG_REMOVE_EXTENTSCHANGED()';

            goCWCJSAPI.CallServer(szOnLoad, aHiddenVars);
        }
        else
        {

            if({$this->mbROISelected})
            {
                {$this->mszHTMLForm}.submit();
            }
            else
            {
                alert("You must select a ROI before deleting it.");
            }

        }

    return true;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

    function GetJavascriptVariables()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptVariables();
        else return array();
    }

    function GetJavascriptInitFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
        {        
            $aResult = $this->moButton->GetJavascriptInitFunctions();
        }
        
        return $aResult;
    }

    function GetJavascriptOnLoadFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptOnLoadFunctions();
        else
            $aReturn = array();
        if(count($this->maszErrorMsg) > 0)
            $aReturn["ROIPGRemoveError"] = "ROIPGRemoveError();";
        
        return $aReturn;
    }
    
    function GetJavascriptIncludeFunctions()
    {
        $aReturn = $this->moButton->GetJavascriptIncludeFunctions();

        return $aReturn;
    }

    /**
     * return hidden variables required for this tool
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();
        
        return $aReturn;
    }

    function DrawPublish()
    {
        if(!$this->mbVisible)
            return "<!-- ROIPGRemove widget is here -->";

        return $this->moButton->DrawPublish();
    }
}
?>
