<?php
/**
 * ROIPointTool Widget class
 *
 * @project     CWC2
 * @revision    $Id: ROIPointTool.widget.php,v 1.1 2007/01/05 21:12:58 jlacroix Exp $
 * @purpose     ROI Point Tool Widget class.  Allows the user to select
 * a point region of interest on the map.
 * @author      DM Solutions Group (spencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");

/**
 * ROIPointTool
 *
 * @desc A widget to draw a point Region of Interest
 */
class ROIPointTool extends CWCWidget
{


    /* the button that is the interface for this widget */
    var $moButton;

    var $mszCursorType = "AUTO";

    /**
     * construct a new ROI Point Tool widget
     */
    function ROIPointTool()
    {
        // invoke constructor of parent
        parent::CWCWidget();

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The ROIPointTool widget allows the user to draw a point region
of interest on the map interface.
EOT;
        $this->moButton = new CWCButton( $this );

        $this->mnMaturityLevel = MATURITY_BETA;

        $this->maAttributes["CURSORTYPE"] = new StringAttribute( "CURSORTYPE", false);

    }

    /**
     *
     */
    function InitDefaults()
    {
        parent::InitDefaults();

        if (isset($this->maParams["CURSORTYPE"]))
          $this->mszCursorType = $this->maParams["CURSORTYPE"];

        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick('ROIPointTool'.$this->mnId);

    }

    /**
     * pass URL to button
     */
    function  ParseURL()
    {
        return $this->moButton->ParseURL();
    }

    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();

        $szCmd = '';
        if ($this->isVarSet('NAV_CMD'))
            $szCmd = $this->getVar('NAV_CMD');
        $szVariable = "NAV_CMD";
        $szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    /**
     * @desc javascript mousemove functions
     */
    function GetJavascriptOnMouseMoveFunctions()
    {
        $aReturn = parent::GetJavascriptOnMouseMoveFunctions();

        $aReturn['ROIPointTool'] = 'ROIPointToolMouseMove(e);';

        return $aReturn;
    }

    /**
     * GetJavascriptInitFunctions
     *
     * Functions to be called at the end of the load.
     */
    function GetJavascriptInitFunctions()
    {
        $aReturn = $this->moButton->GetJavascriptInitFunctions();

        $aReturn['InitCWCROIManagerForm'] = 'goCWCROIManager.SetFormObject('.$this->mszHTMLForm.');';

        return $aReturn;
    }

    /**
     * javascript include files
     */
    function GetJavascriptIncludeFunctions()
    {
        $aReturn = parent::GetJavascriptIncludeFunctions();

        $szVar = "cwc_roi.js";
        $aReturn[$szVar] = '<script src="'.$_SESSION['gszCoreWebPath'].
                            '/widgets/js/cwc_roi.js" type="text/javascript"></script>';

        $szVar = "cwc_dhtml.js";
        $aReturn[$szVar] = '<script src="'.$_SESSION['gszCoreWebPath'].
                            '/widgets/js/cwc_dhtml.js" type="text/javascript"></script>';
        return $aReturn;
    }

    /**
     * @desc general javascript functions
     */
    function GetJavascriptFunctions()
    {
        $aReturn = parent::GetJavascriptFunctions();

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        // add JSAPI mode functions
        if ($bCWCJSAPI)
        {
            $szJsFunctionName = "ROIPointToolWRegisterForEvent";
            $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called to register events when the layers change (JSAPI)
 */
function {$szJsFunctionName}()
{
    goCWCJSAPI.RegisterEvent(MAP_EXTENT_CHANGED, "ROIPointToolWExtentChanged");
    goCWCJSAPI.RegisterEvent(MAP_PROJECTION_CHANGED, "ROIPointToolWExtentChanged");
}
EOT;

           $aReturn[$szJsFunctionName] = $szFunction;
           $szJsFunctionName = "ROIPointToolWExtentChanged";
           $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the extent is changed (JSAPI)
 */
function {$szJsFunctionName}()
{
    // re-draw the ROI
    goCWCROIManager.SetGeoExtents( goCWCJSAPI.oMap.minx, goCWCJSAPI.oMap.miny,
                                    goCWCJSAPI.oMap.maxx, goCWCJSAPI.oMap.maxy );
    goCWCROIManager.Redraw();
}
EOT;
            $aReturn[$szJsFunctionName] = $szFunction;

        }

        $nId = $this->mnId;
        $szName = 'ROIPointTool'.$nId;
        $szFunction = <<<EOT
function {$szName}()
{
    goCWCROIManager.SetROITool('{$nId}');
    if ({$this->mszHTMLForm}.CURSOR_TYPE != null)
	{
	  {$this->mszHTMLForm}.CURSOR_TYPE.value = '{$this->mszCursorType}';
    }
}
EOT;
        $aReturn[$szName] = $szFunction;



        $szObjectJS = "    var jsObj;\n";

        /**
         * TODO: this does not reconstitute the points associated with the
         * ROI tool because this is complicated ;)  In reality, the points may
         * not be at valid locations.  The only reason to reconstitute the points
         * would be to enable editing, which can't happen
         */

        if (isset($_SESSION['ROIRENDERER']))
        {
            $nROI = count($_SESSION['ROIRENDERER']);
            for ($i=0; $i<$nROI; $i++)
            {
                $aROI = $_SESSION['ROIRENDERER'][$i];
                if ($aROI['type'] == 'point')
                {
                    $szObjectJS .= "    goCWCROIManager.SetMode(".
                                   $aROI['mode'] . ");\n";
                    $szObjectJS .= "    jsObj = new CWCPointROI();\n";
                    $szObjectJS .= "    jsObj.edgeColor = \"".$aROI['edgecolor']."\";\n";
                    $szObjectJS .= "    jsObj.edgeWidth = ".$aROI['edgewidth'].";\n";
                    $szObjectJS .= "    jsObj.fillLayer = goCWCROIManager.bUseFill;\n";
                    $szObjectJS .= "    jsObj.fillColor = \"".$aROI['fillcolor']."\";\n";
                    $szObjectJS .= "    jsObj.fillOpacity = ".$aROI['fillopacity'].";\n";
                    $szObjectJS .= "    jsObj.X = gROIPointX;\n";
                    $szObjectJS .= "    jsObj.Y = gROIPointY;\n";
                    $szObjectJS .= "    goCWCROIManager.Insert( jsObj, $i );\n";
                }
            }
            if (strlen( $szObjectJS ) > 0)
            {
                $szObjectJS .= "    goCWCROIManager.UpdateLayerVisibility();\n";
            }
        }


        $szName = 'ROIPointToolInitObjects';
        $szFunction = <<<EOT
function {$szName}()
{
{$szObjectJS}
}
EOT;
        $aReturn[$szName] = $szFunction;


        $szName = 'ROIPointToolMouseDown';
        $szFunction = <<<EOT
function {$szName}(e)
{
    ROIPointToolGetMousePosition(e);

    if (gROIToolmouseX >=0 && gROIToolmouseY >=0)
    {
        goROIPointObj = new CWCPointROI();
        goROIPointObj.edgeColor = goCWCROIManager.szEdgeColor;
        goROIPointObj.edgeWidth = goCWCROIManager.nEdgeWidth;
        goROIPointObj.fillLayer = goCWCROIManager.bUseFill;
        goROIPointObj.fillColor = goCWCROIManager.szFillColor;
        goROIPointObj.fillOpacity = goCWCROIManager.nFillOpacity;
        goROIPointObj.X = gROIPointX;
        goROIPointObj.Y = gROIPointY;

        goROIPointObj.Show();

        goROIPointObj.Set( gROIToolmouseX, gROIToolmouseY );
        goCWCROIManager.Add( goROIPointObj );
        goROIPointObj.Hide();
        goROIPointObj = null;
    }
    return false;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROIPointToolGetMousePosition';
        $szFunction = <<<EOT
function {$szName}(e)
{
    if (CWCIsNav4 || CWCIsNav6) // Netscape
    {
        gROIToolmouseX = e.pageX;
        gROIToolmouseY = e.pageY;
    }
    else if (CWCIsIE) // IE
    {
        gROIToolmouseX = event.clientX + document.body.scrollLeft;
        gROIToolmouseY = event.clientY + document.body.scrollTop;
    }
    else
    {                              // Don't know
        gROIToolmouseX = gROIToolmouseY = 0;
    }
    var oImage = CWCDHTML_GetImage('mapimage');

    if (oImage == null)
    {
        gROIToolmouseX = -1;
        gROIToolmouseY = -1
        return false;
    }
    var left = CWCDHTML_FindObjectPosX( oImage );
    var top = CWCDHTML_FindObjectPosY( oImage );

    var width = oImage.width;
    var height = oImage.height;

    // subtract offsets from page left and top
    gROIToolmouseX = gROIToolmouseX - left;
    gROIToolmouseY = gROIToolmouseY - top;

    if (gROIToolmouseX < 0 || gROIToolmouseX > width ||
        gROIToolmouseY < 0 || gROIToolmouseY > height)
    {
        gROIToolmouseX = -1;
        gROIToolmouseY = -1
    }
    return true;
}
EOT;
        $aReturn[$szName] = $szFunction;

        $szName = 'ROIPointToolMouseMove';
        $szFunction = <<<EOT
function {$szName}(e)
{
    ROIPointToolGetMousePosition(e);

    if (gROIToolmouseX >=0 && gROIToolmouseY >=0 &&
        {$this->mszHTMLForm}.NAV_CMD != null &&
        {$this->mszHTMLForm}.NAV_CMD.value == "ROI_TOOL" &&
        {$this->mnId} == goCWCROIManager.GetROITool())
    {
        document.onmousedown = ROIPointToolMouseDown;
    }
    else
    {
    }
    return false;
}
EOT;
        $aReturn[$szName] = $szFunction;

        return $aReturn;
    }

    /**
     * GetJavascriptVariables
     *
     * Return JS global variables and global code.
     */
    function GetJavascriptVariables()
    {
        $aReturn = array();

        $szVariable = "gROIToolmouseX";
        $szValue = " var $szVariable = 0;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gROIToolmouseY";
        $szValue = " var $szVariable = 0;\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "goROIPointObj";
        $szValue = " var $szVariable = null;\n";
        $aReturn[$szVariable] = $szValue;


        $szVariable = "gROIPointX";
        $szValue = " var $szVariable = \"gROIPointX\";\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "gROIPointY";
        $szValue = " var $szVariable = \"gROIPointY\";\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    /**
     * return an array of javascript functions needed by the Ruler widget
     * and called when the page is loaded.
     * @return array of name = function values
     */
    function GetJavascriptOnLoadFunctions()
    {
        $aReturn = parent::GetJavascriptOnLoadFunctions();

        $aReturn["ROIPointToolInitObjects"] = "    ROIPointToolInitObjects();\n";
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
        {
            $szJsFunctionName = "ROIPointToolWRegisterForEvent";
            $szFunction = "$szJsFunctionName();\n";
            $aReturn[$szJsFunctionName] = $szFunction;
        }

        return $aReturn;
    }

    /**
     * draw this widget on the page
     */
    function DrawPublish($szPostEvent = "")
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        if (!$this->mbVisible)
            return "<!-- ROIPointTool hidden -->";

        $szReturn = $this->moButton->DrawPublish();

        return $szReturn;
    }
}
?>
