<?php
/**
 * ROIRemoveAll Widget class
 *
 * @project     CWC2
 * @revision    $Id: ROIRemoveAll.widget.php,v 1.4 2004/10/14 18:19:18 pspencer Exp $
 * @purpose     ROI Removal Tool Widget class.  Allows the user to remove all
 *              previously selected ROIs.
 *
 * @author      DM Solutions Group (spencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");

/**
 * ROIRemoveAll
 *
 * @desc A widget to draw a rectangular Region of Interest
 */
class ROIRemoveAll extends CWCWidget
{
    /* the button that is the interface for this widget */
    var $moButton;
    
    /**
     * construct a new ROI Removal Tool widget
     */
    function ROIRemoveAll()
    {
        // invoke constructor of parent
        parent::CWCWidget();
        
        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The ROIRemoveAll widget allows the user to remove all
ROIs from the ROI manager.
EOT;
        $this->moButton = new CWCButton( $this );
        $this->mnMaturityLevel = MATURITY_BETA;
    }
    
    /**
     * initialize the widget, only need to initialize the button in this case.
     */
    function InitDefaults()
    {
        parent::InitDefaults();
        
        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick("ROIRemoveAll");
    }
    
    function GetJavascriptVariables()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptVariables();
        else return array();
    }

    function GetJavascriptFunctions()
    {
        $aReturn = array();
        
        $aReturn['ROIRemoveAll'] = <<<EOT
function ROIRemoveAll()
{
    goCWCROIManager.RemoveAll();
}
EOT;
        
        return $aReturn;
    }
    
    function GetJavascriptInitFunctions()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptInitFunctions();
        else return array();
    }

    function GetJavascriptOnLoadFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptOnLoadFunctions();
        else
            $aReturn = array();
        return $aReturn;
    }

    function GetJavascriptIncludeFunctions()
    {
        return $this->moButton->GetJavascriptIncludeFunctions();
    }

    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();
        
        return $aReturn;
    }

    /**
     * draw this widget on the page
     */
    function DrawPublish()
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        if (!$this->mbVisible)
            return "<!-- ROIRemoveAll hidden -->";

       $szReturn = $this->moButton->DrawPublish();

        return $szReturn;
    }
}
?>
