<?php
/**
* RemoveLayer Widget class
*
* @project     CWC2
* @revision    $Id: RemoveLayer.widget.php,v 1.6 2006/05/05 07:53:19 bartvde Exp $
* @purpose     RemoveLayer Widget class
* @author      DM Solutions Group (pspencer@dmsolutions.ca)
* @copyright
* <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

/* based on Code provided by GeoDan */

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Button.php");

/**
* RemoveLayer
*
* @desc Display a clickable image that will update the map
*/
class RemoveLayer extends CWCWidget
{

	/**
	* RemoveLayer
	*
	* Constructor for the RemoveLayer
	*/
	function RemoveLayer()
	{
        // set the language file
        $this->mszLanguageResource = dirname(__FILE__)."/RemoveLayer";

		// invoke constructor of parent
		parent::CWCWidget();
		$this->mnPriority = PRIORITY_HIGH;

		// set the description for this widget
		$this->szWidgetDescription = <<<EOT
The RemoveLayer widget removes the selected layer(s) from the map.  This widget uses
the SelectLayer widget which works best with the HTML LegendTemplate widget.
EOT;

		$this->moButton = new CWCButton($this);
		$this->moButton->SetOnClick( "clickRemoveLayer" );
        $this->mnMaturityLevel = MATURITY_BETA;
	}

	function InitDefaults()
	{
		parent::InitDefaults();
		$this->moButton->InitDefaults();
	}

	/**
	* GetHTMLHiddenVariables
	*
	* Return HTML hidden variables.
	*/
	function GetHTMLHiddenVariables()
	{
		$szVariable = "REMOVE_LAYER".$this->mnId;
		$szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
		$aReturn[$szVariable] = $szValue;
		return $aReturn;
	}

	/**
	* GetJavascriptFunctions
	*
	* Return the Javacriptfunctions needed by the widget.
	*/
	function GetJavascriptFunctions()
	{
        $szMessage = $this->moMLT->get( $this->mszWidgetName, '0', 'There is no active map layer' );

		$aReturn = $this->moButton->GetJavascriptFunctions();

		$szJsFunctionName = "clickRemoveLayer";
		$nId = $this->mnId;
		$szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the user clicks the RemoveLayer icon
 */

function {$szJsFunctionName}()
{
    if ({$this->mszHTMLForm}.SELECTED_LAYERS == null ||
        {$this->mszHTMLForm}.SELECTED_LAYERS.value == null ||
        {$this->mszHTMLForm}.SELECTED_LAYERS.value == "" )
    {
        alert( "{$szMessage}" );
        return;
    }
    with({$this->mszHTMLForm})
    {
       // reset the NAV_CMD
       NAV_CMD.value = "";
       REMOVE_LAYER{$nId}.value = "1";
       submit();
    }
    return;
}

EOT;
		$aReturn[$szJsFunctionName] = $szFunction;

		return $aReturn;
	}
	/**
	* return javascript functions
	*/
	function GetJavascriptIncludeFunctions()
	{
		return $this->moButton->GetJavascriptIncludeFunctions();
	}

	/**
	* return javascript variables
	*/
	function GetJavascriptVariables()
	{
		return $this->moButton->GetJavascriptVariables();
	}

	/**
	* return javascript onload functions
	*/
	function GetJavascriptOnLoadFunctions()
	{
		return $this->moButton->GetJavascriptOnLoadFunctions();
	}

	/**
	* return javascript initialization funcitons
	*/
	function GetJavascriptInitFunctions()
	{
		return $this->moButton->GetJavascriptInitFunctions();
	}

	/**
	* DrawPublish
	*
	* Return the HTML code using the name in the map file and the
	* parameters of the CWC tag.
	*/
	function DrawPublish()
	{
		if (!$this->mbVisible)
			return "<!-- RemoveLayer widget hidden -->";
		return $this->moButton->DrawPublish();
	}
	/**
	* ParseURL
	*
	* Look for selected layers and move them UP
	*/
	function  ParseURL()
	{
		if ($this->isVarSet('REMOVE_LAYER'.$this->mnId) &&
		$this->getVar('REMOVE_LAYER'.$this->mnId) == "1")
		{
			$oMap = $this->moMapObject->oMap;

			$aszSelectedLayers = array();
			if ($this->isVarSet('SELECTED_LAYERS'))
			{
				$szSelectedLayers = $this->getVar( 'SELECTED_LAYERS' );
				if (strlen($szSelectedLayers) > 0)
				{
					$aszSelectedLayers = explode( ",", $szSelectedLayers );
				}
			}
			else
			{
				$nLayers = $oMap->numlayers();
				for ($i=0;$i<$nLayers;$i++)
				{
					$oLayer=$oMap->getLayer($i);
					if ($oLayer->getMetaData("selected") == "1")
					array_push($aszSelectedLayers, $oLayer->name);
				}
			}

			if (count($aszSelectedLayers) > 0)
			{
				$nLayers = $oMap->numlayers;
				for($i=0; $i < $nLayers; $i++)
				{
					$oLayer = $oMap->getLayer($i);
					if (in_array( $oLayer->name, $aszSelectedLayers ))
					{
						$oLayer->set( "status", MS_DELETE );
					}
				}
			}
			$this->setVar('SELECTED_LAYERS', "");
		}
		return true;
	}
}
?>
