<?php
/**
 * SQRCLocalisation Widget class
 *
 * @project     CWC2
 * @revision    $Id:
 * @purpose     SQRCLocalisation Popup Widget class
 * @author      DM Solutions Group (sfournier@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
include_once(CHAMELEON_PATH."/widgets/Widget.php");
include_once(CHAMELEON_PATH."/widgets/Button.php");
include_once( COMMON."wrapper/map_navigator.php" );
include_once('SQRCLookup.php');

/**
 * a simple widget to link to another page
 */
class SQRCLocalisation extends CWCWidget
{
    var $moButton;
    var $maszProjections = array();
    var $maszScales = array();
    var $mbSubmit = false;
    var $mbHiddenVar = false;
    var $mszSQRC = '';
    var $mszMap = '';
    var $maszErrors = array();

    /**
     * construct a new link widget
     */
    function SQRCLocalisation()
    {
        // invoke constructor of parent
        parent::CWCWidget();

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The SQRCLocalisation widget allows the template to display a button that will open an
arbitrary URL such as a help page for an application.
EOT;

        $this->moButton = new CWCButton($this);
        $this->maAttributes["INCLUDESUBMIT"] = new BooleanAttribute("INCLUDESUBMIT", false );
        $this->maAttributes["INCLUDEHIDDEN"] = new BooleanAttribute("INCLUDEHIDDEN", false );
        $this->mnMaturityLevel = MATURITY_BETA;
    }

    /**
     * initialize default values
     */
    function InitDefaults()
    {
        parent::InitDefaults();
        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick( 'SQRCLocalisation' );

        if (isset($this->maParams['INCLUDESUBMIT']))
        {
            $this->mbSubmit = strcasecmp($this->maParams['INCLUDESUBMIT'], 'true') == 0 ? true : false;
        }
        if (isset($this->maParams['INCLUDEHIDDEN']))
        {
            $this->mbHiddenVar = strcasecmp($this->maParams['INCLUDEHIDDEN'], 'true') == 0 ? true : false;
        }

     }
    
    function ParseURL()
    {
        if ($this->mbSubmit && $this->mbVisible)
            $this->moButton->ParseURL();

        $this->mszSQRC = $this->getVar('SQRCLOCALISATION_SQRC');
        $this->mszMap = $this->getVar('SQRCLOCALISATION_MAP');

        if($this->getVar('SQRCLOCALISATION') != 1)
        {
            return true;
        }

        // Get SQRC coordinates
        $oSQRCLookup = new SQRCLookup();
        $aBBox = $oSQRCLookup->Sqrc2geo($this->mszSQRC, 2);
        if(!is_array($aBBox))
        {
            $this->maszErrors[] = 'Nom de feuillet introuvable: '.
                $this->mszSQRC;
            return true;
        }

        // create new rectangle object
        $oRect = ms_newrectobj();
        
        // get current map extents
        $dMapMinX = $aBBox[0][1]*-1; // In montreal, west of greenwich
        $dMapMinY = $aBBox[0][0];
        $dMapMaxX = $aBBox[2][1]*-1; // In montreal, west of greenwich
        $dMapMaxY = $aBBox[2][0];
        // set the extents of the rectangle
        $oRect->setextent($dMapMinX, $dMapMinY, $dMapMaxX, $dMapMaxY);

        // reproject rectangle in map projection
        $oIn = ms_newProjectionObj("init=epsg:4269");
        $oOut = ms_newProjectionObj($this->moMapObject->oMap->getProjection());
        $oRect->project($oIn, $oOut);

        // Change map extent to zoom on this SQRC
        $this->moMapObject->oMap->setExtent($oRect->minx, $oRect->miny,
                                            $oRect->maxx, $oRect->maxy);

        // If needed, draw the SQRC above the map
        if($this->mszMap && $this->mszMap != 'false')
        {
            $oMap = $this->moMapObject->oMap;
            // Delete old SQRC layer
            $oLayer = @$oMap->getLayerByName('Feuillet SQRC');
            if($oLayer)
            {
                //$oLayer->set('status', MS_DELETE);
            }

            // Create new SQRC layer
            $oLayer = ms_newLayerObj($oMap);
            $oLayer->set('name', 'Feuillet SQRC');
            $oLayer->set('status', MS_ON);
            $oLayer->set('type', MS_LAYER_POLYGON);
            $oLayer->set('transparency', '35');
            $oLayer->set('sizeunits', MS_DD);
            $oLayer->setProjection('init=epsg:4269');
            $oClass = ms_newClassObj($oLayer);
            $oStyle = ms_newStyleObj($oClass);
            $oStyle->color->setRGB( 0, 0, 255 );
            $oStyle->outlinecolor->setRGB( 0, 0, 0 );

            // Add the feature in layer
            $oLine = ms_newLineObj();
            $oShape = ms_newShapeObj(MS_SHAPE_POLYGON);

            foreach($aBBox as $aCoords)
            {
                $oLine->addXY($aCoords[1]*-1, $aCoords[0]);
            }
            $oShape->add($oLine);

            $oLayer->addFeature($oShape);
        }

        return true;
    }
    
    /**
     * return javascript functions
     */
    function GetJavascriptFunctions()
    {
        $aReturn = array();
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptFunctions();

        $szJsFunctionName = "SQRCLocalisation";
        $szFunction = <<<EOT
function $szJsFunctionName()
{
    {$this->mszHTMLForm}.SQRCLOCALISATION = 1;
    {$this->mszHTMLForm}.submit();
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "SQRCLocalisationGetArguments";
        $szFunction = <<<EOT
function $szJsFunctionName()
{
    var aszArgs = Array();
    aszArgs[0] = Array('SQRCLOCALISATION', 1);
    aszArgs[1] = Array('SQRCLOCALISATION_SQRC', 
                       {$this->mszHTMLForm}.SQRCLOCALISATION_SQRC.value);
    aszArgs[2] = Array('SQRCLOCALISATION_MAP', 
                       {$this->mszHTMLForm}.SQRCLOCALISATION_MAP.checked);

    return aszArgs;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "SQRCLocalisationValid";
        $szFunction = <<<EOT
function $szJsFunctionName()
{
    var szSQRC = {$this->mszHTMLForm}.SQRCLOCALISATION_SQRC.value;
    if(!szSQRC)
    {
        alert("Veuillez entrer un nom de feuillet SQRC valide.\\n"+
              "Trois formes sont disponible:\\n"+
              '99A99-999-9999, 99A99-999-999999 et 99A99');
        return false;
    }

    if(szSQRC.length == 5)
    {
        var postalPattern =/^\d\d[a-zA-Z]\d\d$/;

    }
    else if(szSQRC.length == 14)
    {
        var postalPattern =/^\d\d[a-zA-Z]\d\d-\d\d\d-\d\d\d\d$/;

    }
    else if(szSQRC.length == 16)
    {
        var postalPattern =/^\d\d[a-zA-Z]\d\d-\d\d\d-\d\d\d\d\d\d$/;

    }
    else if(szSQRC.length == 14)
    {
        alert('Veuillez entrer un nom de feuillet SQRC valide.\\n'+
              'Trois formes sont disponible:\\n'+
              '99A99-999-9999, 99A99-999-999999 et 99A99');
        return false;
    }

    var postalRegExp = new RegExp(postalPattern);
    if (postalRegExp.test(szSQRC) == false )
    {
        alert('Veuillez entrer un nom de feuillet SQRC valide.\\n'+
              'Trois formes sont disponible:\\n'+
              '99A99-999-9999, 99A99-999-999999 et 99A99');
        return false;
    }

    return true;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

    /**
     * return javascript functions
     */
    function GetJavascriptIncludeFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
            $aResult = $this->moButton->GetJavascriptIncludeFunctions();
        return $aResult;
    }

    /**
     * return javascript variables
     */
    function GetJavascriptVariables()
    {
        $aResult = array();
        if ($this->mbVisible)
            $aResult =  $this->moButton->GetJavascriptVariables();
        return $aResult;
    }

    /**
     * return javascript onload functions
     */
    function GetJavascriptOnLoadFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
            $aResult = $this->moButton->GetJavascriptOnLoadFunctions();
        return $aResult;
    }
    
    /**
     * return javascript initialization funcitons
     */
    function GetJavascriptInitFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
            $aResult =  $this->moButton->GetJavascriptInitFunctions();

        $szJSErrors = '';
        foreach($this->maszErrors as $szError)
        {
            $szJSErrors .= 'alert("'.$szError.'");';
        }
        if($szJSErrors)
            $aResult['SQRCErrors'] = $szJSErrors;

        return $aResult;
    }

    function GetHTMLHiddenVariables()
    {
        $aReturn = array();
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetHTMLHiddenVariables();

        $szVariable = "SQRCLOCALISATION";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
        $aReturn[$szVariable] = $szValue;

        if($this->mbHiddenVar)
        {
            $szVariable = "SQRCLOCALISATION_SQRC";
            $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
            $aReturn[$szVariable] = $szValue;
            $szVariable = "SQRCLOCALISATION_MAP";
            $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
            $aReturn[$szVariable] = $szValue;
        }

        return $aReturn;
    }

    /**
     * draw the link widget
     */
    function DrawPublish()
    {
        if (!$this->mbVisible)
            return "<!-- SQRCLocalisationWidget is hidden -->";

        $szReturn = '';
        if($this->mbSubmit)
            $szReturn = $this->moButton->DrawPublish();

        $szSQRC = $this->mszSQRC;
        $szMap = $this->mszMap;

        $szReturn = <<<EOT
        <table><tr>
          <td><img src="icon_find_ca_place.gif" width="22" height="22"> </td>
          <td><span class="label"> Code de feuillet SQRC </span></td></tr>
          <tr><td colspan="2"><input class="inputBox" type="text" name ="SQRCLOCALISATION_SQRC" value="$szSQRC" size="20"></td>
          </tr>
            <tr><td colspan="2"><span class="label"><input class="inputBox" type="checkbox" name ="SQRCLOCALISATION_MAP" value="$szMap" size="20"> Ajouter sur la carte </span></td>
          </tr>
          </table>
EOT;

        return $szReturn;
    }
}
?>
