<?php
/**
 * TableRenderer Class
 *
 * @project     CWC2
 * @revision    $Id: TableRenderer.php,v 1.2 2006/03/13 21:50:21 jlacroix Exp $
 * @purpose     Helper class to quickly build a HTML table from an array
 * @author      DM Solutions Group (jlacroix@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/**
 * This was provided by Pirmin Kalberer <pka@sourcepole.ch>
 */

/**
 *Renderer base class with factory
 */
class TableRenderer 
{
    /**
     *Factory to create a special Renderer
     */
    function factory($sLayout)
    {
        $sLayout = strtolower($sLayout);
        $classname = "TableRenderer_{$sLayout}";
        
        if (@include_once 'TableRenderer/' . $sLayout . '.class.php')
        {
            return new $classname;
        } 
        else 
        {        
            trigger_error($GLOBALS['oMLT']->get("16", '<b>Unknown layout!</b><br>Could not found a Renderer for the specified layout.'), E_USER_ERROR);
        }
    } // End of Method: factory()
           
    /**
     *wraps array_change_key_case to use with array walk
     *
     *@access private
     *@param $aRow array
     */
    function _array_change_key_case(&$aRow)
    {
        if (!is_array($aRow))
            return;
                        
        $aRow = array_change_key_case($aRow, CASE_LOWER);
                                
    } // End of method: _array_change_key_case()
           
} // End of factoryclass: TableRenderer


class TableRenderer_horizontal extends TableRenderer 
{
        /**
        *Displays a column for each recordset
        *Used when the "layout" attribute is set to "horizontal"
        *
        *@return string
        *@author Reto Zahner 
        *@param array $aRows
        *@param string $sTableTitle
        *@param array $aHeaderMap
        */
        function render($aRows, $sTableTitle, $aHeaderMap)
        {
            $sTableHTML = null;
            
            // build title rows
            $sTableHTML .= '<table class="layoutTable" width="100%" border="0" cellspacing="1" cellpadding="2">';
            $sTableHTML .= '<tr>';
            $sTableHTML .= '<td colspan='.count($aRows[0]).' class="titleArea">';
            $sTableHTML .= '<p class="text"><u><b>'.$sTableTitle.'</b></u></p>';
                   $sTableHTML .= '</td>';
            $sTableHTML .= '</tr>';
        
            // Change all keys to lowercase
            array_walk($aRows, array(&$this, '_array_change_key_case')); 
            
            // loop and add a cell for each result
            foreach ( $aHeaderMap as $key => $value ) 
            {
                    $sTableHTML .= '<tr>';
                    
                    // Add a header for each row
                    $sTableHTML .= '<td class="subContentArea2"><p class="text">'.$value.'</p></td>';
                    
                    // loop through the results and format each row
                    foreach ( $aRows as $aRow )
                    {
                        $sTableHTML .= '<td class="subContentArea1"><p class="text">'.$aRow[strtolower($key)].'</p></td>';
                    }        
                        $sTableHTML .= '</tr>';
            }
        
            // close the table
            $sTableHTML .= '</table>';
                
            return $sTableHTML;
                
        } // End of method: render
}



class TableRenderer_vertical extends TableRenderer  
{
        /**
        *Displays a column for each recordset
        *Used when the "layout" attribute is set to "horizontal"
        *
        *@return string
        *@author Reto Zahner 
        *@param array $aRows
        *@param string $sTableTitle
        *@param array $aHeaderMap
        */
        function render($aRows, $sTableTitle, $aHeaderMap)
        {
            $sTableHTML = null;
            
            // build title rows
            $sTableHTML .= '<table class="layoutTable" width="100%" border="0" cellspacing="1" cellpadding="2">';
            $sTableHTML .= '<tr>';
            $sTableHTML .= '<td colspan='.count($aRows[0]).' class="titleArea">';
            $sTableHTML .= '<p class="text"><u><b>'.$sTableTitle.'</b></u></p>';
            $sTableHTML .= '</td>';
            $sTableHTML .= '</tr>';
            $sTableHTML .= '<tr>';
            
            // Change all keys to lowercase
            array_walk($aRows, array(&$this, '_array_change_key_case')); 
            
            foreach ($aHeaderMap as $key => $value) 
            { 
                $sTableHTML .= '<td class="subContentArea2"><p class="text">'.$value.'</p></td>';
            }
            
            $sTableHTML .= '</tr>';
            
            // loop through the results and format each row
            foreach ( $aRows as $aRow )
            {
                // add a row for each
                $sTableHTML .= '<tr>';
        
                // loop and add a cell for each result
                foreach ( $aHeaderMap as $key => $value ) 
                {
                    $sTableHTML .= '<td class="subContentArea1"><p class="text">'.$aRow[strtolower($key)].'</p></td>';
                }
        
                $sTableHTML .= '</tr>';
            }
        
            // close the table
            $sTableHTML .= '</table>';

            return $sTableHTML;
                
        } // End of Method: render
        
} // End of Class: TableRenderer_vertical

?>
