<?php
/**
  _____________________________________________________________________________
 |
 | WFSSearch.widget.php
 |
 | @project     Chameleon
 | @purpose     Widget class for search functionality based on a Web Feature Service (WFS)
 | @author      Bart van den Eijnden (bartvde@xs4all.nl)
 | @updated     November 24, 2005 - William Bronsema (wbronsema@dmsolutions.ca)
 | @copyright
 | <b>Copyright (c) 2002-2005 DM Solutions Group Inc.</b>
 | Permission is hereby granted, free of charge, to any person obtaining a
 | copy of this software and associated documentation files (the "Software"),
 | to deal in the Software without restriction, including without limitation
 | the rights to use, copy, modify, merge, publish, distribute, sublicense,
 | and/or sell copies of the Software, and to permit persons to whom the
 | Software is furnished to do so, subject to the following conditions:
 |
 | The above copyright notice and this permission notice shall be included
 | in all copies or substantial portions of the Software.
 |
 | THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 | IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 | FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 | THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 | LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 | FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 | DEALINGS IN THE SOFTWARE.
 |_____________________________________________________________________________

 **/

include_once(CHAMELEON_PATH."/widgets/Widget.php");
include_once(CHAMELEON_PATH."/widgets/Button.php");
include_once(CHAMELEON_PATH."/widgets/Popup.php");
include_once ( COMMON."wrapper/map_navigator.php" );

/**
  _____________________________________________________________________________
 |
 | "WFSSearch" Widget Class
 |_____________________________________________________________________________

 **/
class WFSSearch extends CWCWidget
{
    // define member variables
    var $moButton;
    var $moPopup;
    var $mszZoomScale;
    var $mszOnClick;
    var $mszTargetFrame;

    /**
      _________________________________________________________________________
     |
     | Constructor: "WFSSearch" Widget Class
     |_________________________________________________________________________

     **/
    function WFSSearch()
    {
        // set the language resource
        $this->mszLanguageResource = str_replace("\\","/",dirname(__FILE__)).
                                                            "/WFSSearch.dbf";
        // invoke constructor of parent
        parent::CWCWidget();

        $this->mnPriority = PRIORITY_HIGH;

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The WFSSearch widget provides a dialog box that can be used to search for
any number of attributes on a WFS typename.
EOT;

        $this->moButton = new CWCButton($this);
        $this->moPopup = new CWCPopup ($this);
        $this->moPopup->mszLink = $_SESSION['gszCoreWebPath']."/widgets/WFSSearch/WFSSearch.phtml";
        $this->moPopup->mszParam = "&szCallbackFunc=WFSSearchCB";
        $this->mnMaturityLevel = MATURITY_BETA;


        // define widget attributes
        $this->maAttributes["ZOOMSCALE"] = new StringAttribute( "ZOOMSCALE", true );
        $this->maAttributes["TARGETFRAME"] = new StringAttribute( "TARGETFRAME", false );
    }

    /**
      _________________________________________________________________________
     |
     |  InitDefaults()
     |
     |  Use this section to set any predetermined variables or default vaules
     |
     |  Postcondition:  Initialize defaults.
     |
     |  @return n/a
     |  @desc Initialize defaults.
     |_________________________________________________________________________

     **/
    function InitDefaults()
    {
        // initialize the parent defaults
        parent::InitDefaults();

        // initialize the search option(s)
        $_SESSION['wfs_search_widget_options'] = array();
        if ( isset( $this->maszContents['SEARCHOPTION'] ) &&
             is_array( $this->maszContents['SEARCHOPTION'] ) )
        {
            // loop and process each search option into the session
            foreach( $this->maszContents['SEARCHOPTION'] as $aszSearchOption )
            {
                array_push( $_SESSION['wfs_search_widget_options'],
                                                              $aszSearchOption );

            }
        }

        // init the target frame
        if (isset($this->maParams["TARGETFRAME"]))
            $this->mszTargetFrame = $this->maParams["TARGETFRAME"];
        if ($this->mszTargetFrame != '')
        {
          $this->mszHTMLForm = 'opener.document.forms[0]';
        }
        else
        {
          $this->moPopup->InitDefaults();
        }

        // init the zoom scale
        if (isset($this->maParams["ZOOMSCALE"]))
            $this->mszZoomScale = $this->maParams["ZOOMSCALE"];
        $this->moPopup->mszParam .= "&szZoomScale=".$this->mszZoomScale;

        // init the button defaults
        $this->moButton->InitDefaults();

        // set the onclick event
        $this->mszOnClick = "onClickWFSSearch";
        $this->moButton->SetOnClick( $this->mszOnClick );
    }


    /**
      _________________________________________________________________________
     |
     |  ParseURL()
     |
     |  Use this function to process any information or changes passed in the
     |  URL.  Set any appropriate member variables.  This section should NOT
     |  be used to output any HTML to the screen.
     |
     |  Postcondition:  This function parses information passed on the URL and
     |                  acts on it.
     |
     |  @return boolean - True if successful, false if not.
     |  @desc Parses info passed on URL.
     |_________________________________________________________________________

     **/
    function ParseURL ()
    {
        // zoom into the search results as necessary
        $szCoordinates = "";
        $oMap = $this->moMapObject->oMap;
        if ($this->isVarSet( "WFSSEARCH_COORDINATES" ))
            $szCoordinates = trim( $this->getVar( "WFSSEARCH_COORDINATES" ));

        if ($szCoordinates != "")
        {
            $arCoordinates = split(" ", $szCoordinates);
            $arLowerLeft = split(",", $arCoordinates[0]);
            $arUpperRight = split(",", $arCoordinates[1]);
            // test if we have a point geometry
            if (($arUpperRight[0] == $arLowerLeft[0]) && ($arUpperRight[1] == $arLowerLeft[1]))
            {
                $centerX = ($arUpperRight[0]+$arLowerLeft[0])/2;
                $centerY = ($arUpperRight[1]+$arLowerLeft[1])/2;
                $inchesPerUnit = array(1, 12, 63360.0, 39.3701, 39370.1, 4374754);
                $widthX = ($this->mszZoomScale*$oMap->width)/($oMap->resolution*$inchesPerUnit[$oMap->units]);
                $widthY = ($this->mszZoomScale*$oMap->height)/($oMap->resolution*$inchesPerUnit[$oMap->units]);
                $oMap->setextent( $centerX-($widthX/2), $centerY-($widthY/2), $centerX+($widthX/2), $centerY+($widthY/2));
            }
            else
            {
                $oMap->setextent( $arLowerLeft[0], $arLowerLeft[1], $arUpperRight[0], $arUpperRight[1]);
            }
        }
      return true;
    }

    /**
      _________________________________________________________________________
     |
     |  GetJavascriptFunctions()
     |
     |  Postcondition:  This function defines javascript functions and adds
     |                  them to the array.
     |
     |  Example:
     |
     |    $szJsFunctionName = "myJSFunction";
     |    $szFunction = <<<EOT
     |    function {$szJsFunctionName}()
     |    {
     |        alert('myJSFunction');
     |        return true;
     |    }
     |    EOT;
     |    $aReturn[$szJsFunctionName] = $szFunction;
     |
     |
     |  @return Array - Array of javascript functions.
     |  @desc Defines javascript functions.
     |_________________________________________________________________________

     **/
    function GetJavascriptFunctions ()
    {
        // set JSAPI mode flag
        //$bCWCJSAPI = isset($this->maSharedResourceWidgets["CWCJSAPI"])?1:0;

        // init return var
        $aReturn = array();

        // create the onclick javascript function
        $szJsFunctionName = $this->mszOnClick;
        if ($this->mszTargetFrame != "")
        {
          $szFunction = "function $szJsFunctionName() \n".
          "{\ndocument.getElementById('".$this->mszTargetFrame."').src = '".$this->moPopup->GetPopupURL()."'; \n"."}\n";
        }
        else
        {
          $szFunction = "function $szJsFunctionName() \n".
            "{\n".$this->moPopup->DrawPublish ()."\n"."return;\n"."}\n";
        }

        $aReturn[$szJsFunctionName] = $szFunction;

        // define call back function
        $szJsFunctionName = "WFSSearchCB";
        $szFunction = <<<EOF
        function {$szJsFunctionName}(actionID, wh)
        {
            if (actionID == 0)
            {
                // check for selected index
                if( wh.document.forms[0].results.selectedIndex == -1 )
                {
                    return;
                }

                value = wh.document.forms[0].results[wh.document.forms[0].results.selectedIndex].value;
                var coordinates = value;
                {$this->mszHTMLForm}.WFSSEARCH_ZOOMSCALE.value = $this->mszZoomScale;
                {$this->mszHTMLForm}.WFSSEARCH_COORDINATES.value = coordinates;
                {$this->mszHTMLForm}.NAV_CMD.value = '';
                {$this->mszHTMLForm}.submit();
                wh.close();
                if ('{$this->mszTargetFrame}' != '')
                {
                  self.close();
                }
            }
            else if (actionID == 2) // close window
                wh.close ();

            return;
        }

EOF;
        $aReturn[$szJsFunctionName] = $szFunction;


        return $aReturn;
    }

    /**
      _________________________________________________________________________
     |
     |  GetJavascriptVariables()
     |
     |  Postcondition:  This function defines javascript variables and adds
     |                  them to the array.
     |
     |  Example:
     |
     |  $aReturn['MyJSVar'] = 'var szMyJSVar = "Default Value";'."\n";
     |
     |  @return Array - Array of javascript variables.
     |  @desc Defines javascript variables.
     |_________________________________________________________________________

     **/
    function GetJavascriptVariables()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptVariables();
        else return array();
    }

    /**
      _________________________________________________________________________
     |
     |  GetJavascriptInitFunctions()
     |
     |  Postcondition:  This function defines the list javascript functions
     |                  and adds them to the array.
     |
     |  Example:
     |
     |  $aReturn['MyInitFunction'] = "MyInitFunction();\n";
     |
     |  @return Array - Array of javascript functions.
     |  @desc Defines javascript functions.
     |_________________________________________________________________________

     **/
    function GetJavascriptInitFunctions()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptInitFunctions();
        else return array();
    }

    /**
      _________________________________________________________________________
     |
     |  GetJavascriptOnLoadFunctions()
     |
     |  Postcondition:  This function defines javascript functions and adds
     |                  them to the array.
     |
     |  Example:
     |
     |  $aReturn['MyOnloadFunction'] = "MyOnloadFunction();\n";
     |
     |  @return Array - Array of javascript functions.
     |  @desc Defines javascript functions.
     |_________________________________________________________________________

     **/
    function GetJavascriptOnLoadFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptOnLoadFunctions();
        else
            $aReturn = array();
        return $aReturn;
    }

    /**
      _________________________________________________________________________
     |
     |  GetJavascriptIncludeFunctions()
     |
     |  Postcondition:  This function defines the javascript include statements.
     |
     |  Example:
     |
     |  $szVar = "myjavascript.js";
     |  $aReturn[$szVar] = '<script src="http://myjavascript.js"
     |                                     type="text/javascript"></script>';
     |
     |
     |  @return Array - Array of include statements.
     |  @desc Defines javascript include statements.
     |_________________________________________________________________________

     **/
    function GetJavascriptIncludeFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptIncludeFunctions();
        else
            $aReturn = array();
        return $aReturn;
    }

    /**
      _________________________________________________________________________
     |
     |  GetHTMLHiddenVariables()
     |
     |  Postcondition:  This function defines html form variables and adds
     |                  them to the array.
     |
     |  Example:
     |
     |  $szVariable = "MY_HIDDEN_VAR";
     |  $szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"0\">";
     |  $aReturn[$szVariable] = $szValue;
     |
     |  @return Array - Array of html form variables.
     |  @desc Defines html form variables.
     |_________________________________________________________________________

     **/
    function GetHTMLHiddenVariables ()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();

        $szVariable = "WFSSEARCH_ZOOMSCALE";
        $szVal = "";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"$szVal\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "WFSSEARCH_COORDINATES";
        $szVal = "";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"$szVal\">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    /**
      _________________________________________________________________________
     |
     |  DrawPublish()
     |
     |  Postcondition:  This function builds the html output for the physical
     |                  appearance of the widget.
     |
     |  @return string - Output
     |  @desc Generates HTML output for the widget.
     |_________________________________________________________________________

     **/
    function DrawPublish ()
    {

        if (!$this->mbVisible)
            return "<!-- WFSSearch widget hidden -->";
        else
        {
          $szReturn = $this->moButton->DrawPublish();
          return $szReturn;
        }
    }

}
?>
