<?php
/**
  _____________________________________________________________________________
 |
 | searchoptionapi.php
 |
 | @project     Chameleon
 | @purpose     API script to process search option requests
 | @revision    $Id: searchoptionapi.php,v 1.4 2007/01/02 15:23:15 bartvde Exp $
 | @author      William Bronsema (wbronsema@dmsolutions.ca)
 | @copyright
 | <b>Copyright (c) 2005 DM Solutions Group Inc.</b>
 | Permission is hereby granted, free of charge, to any person obtaining a
 | copy of this software and associated documentation files (the "Software"),
 | to deal in the Software without restriction, including without limitation
 | the rights to use, copy, modify, merge, publish, distribute, sublicense,
 | and/or sell copies of the Software, and to permit persons to whom the
 | Software is furnished to do so, subject to the following conditions:
 |
 | The above copyright notice and this permission notice shall be included
 | in all copies or substantial portions of the Software.
 |
 | THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 | IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 | FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 | THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 | LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 | FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 | DEALINGS IN THE SOFTWARE.
 |_____________________________________________________________________________

 **/

// set max timeout
@set_time_limit( 300 );

// init vars
$szResults = '';

// respond based on request type
switch ( $_REQUEST['request'] )
{
    case 'getlist':
        $szResults .= getList();
        break;
    case 'search':
        $szResults .= performSearch();
        break;

}

// return results
echo $szResults;

function getList()
{
    // add a blank if the list required
    $szBlank = $_REQUEST['required'] == 0?'|-----':'';

    //set the language resource file
    $szLanguageResource = str_replace("\\","/",dirname(__FILE__))."/WFSSearch";

    // include the supporting php functions
    include_once("../session.inc.php");

    // check if the seach string and online resource have been passed
    if ( !isset( $_SESSION[ 'online_res_'.$_REQUEST['id'] ] ) ||
         !isset( $_SESSION[ 'typename_'.$_REQUEST['id'] ] ) ||
         !isset( $_SESSION[ 'namespace_'.$_REQUEST['id'] ] ) )
    {
        return 'Error';
    }

    // Include geocoder API
    include_once(COMMON."phpgeocoder/geocoder.php" );

    // List of search services to use
    include_once(COMMON."phpgeocoder/services/web_feature_service.php");

    // create an new geocoder instance
    $oGeoCoder = new PHPGeoCoder();
    $oGeoCoder->oErrorManager =& $_SESSION['gErrorManager'];

    // clear previous error messages
    $oGeoCoder->oErrorManager->reset();

    // add the search param
    $oGeoCoder->addSearchAttribute( new SearchAttribute(
        $_REQUEST['propertyname'], '*', true ) );

    // add a search service - check if host is provided
    $szOnlineResource = $_SESSION[ 'online_res_'.$_REQUEST['id'] ];
    $szTypeName = $_REQUEST['normalize'] == 1 ?$_SESSION[ 'typename_'.
        $_REQUEST['id'] ].'_'.$_REQUEST['propertyname']:$_SESSION[ 'typename_'.
        $_REQUEST['id'] ];
    $szNameSpace = $_SESSION[ 'namespace_'.$_REQUEST['id'] ];
    $szHost = substr( strtolower( $szOnlineResource ), 0, 4) != 'http'?
                                            'http://'.$_SERVER['HTTP_HOST']:'';
    $oGeoCoder->addSearchService(
        new WebFeatureService( $oGeoCoder, $szHost.$szOnlineResource.
        '&service=WFS&version=1.0.0', $szTypeName, strtoupper( $szNameSpace.':'.
        $_REQUEST['propertyname']) , "", "250" ) );

    // perform the search
    $oGeoCoder->searchServices( '', true );

    // get the search results
    $aoResult = $oGeoCoder->getResult();
    $nResult = count( $aoResult );

    // return empty if no results
    if ( $nResult <= 0 )
    {
        return 'No Results';
    }

    // must have results so process
    $aszResults = array();
    foreach( array_keys( $aoResult ) as $szKey )
    {
        // add the result to the list
        array_push( $aszResults,  $aoResult[$szKey]->getAttributeValueByName(
            strtoupper( $szNameSpace.':'.$_REQUEST['propertyname'])) );
    }

    // parse to a string and return
    return $_REQUEST['id'].'|'.$_REQUEST['propertyname'].$szBlank.'|'.
                                                implode( '|', $aszResults );
}

function performSearch()
{
    //set the language resource file
    $szLanguageResource = str_replace("\\","/",dirname(__FILE__))."/WFSSearch";

    // include the supporting php functions
    include_once("../session.inc.php");

    // check if the seach string and online resource have been passed
    if ( !isset( $_REQUEST['searchstring'] ) ||
         !isset( $_SESSION[ 'online_res_'.$_REQUEST['id'] ] ) ||
         !isset( $_SESSION[ 'typename_'.$_REQUEST['id'] ] ) ||
         !isset( $_SESSION[ 'namespace_'.$_REQUEST['id'] ] ) )
    {
        return 'Error';
    }

    // Include geocoder API
    include_once(COMMON."phpgeocoder/geocoder.php" );

    // List of search services to use
    include_once(COMMON."phpgeocoder/services/web_feature_service.php");

    // create an new geocoder instance
    $oGeoCoder = new PHPGeoCoder();
    $oGeoCoder->oErrorManager =& $_SESSION['gErrorManager'];

    // clear previous error messages
    $oGeoCoder->oErrorManager->reset();

    // split the list of search properties
    $aszSearchParams = explode( '||', $_REQUEST['searchstring'] );

    // loop and add a search attribute for each param
    $aszAttributes = array();
    foreach ( $aszSearchParams as $szSearchParam )
    {
        // separate values
        $aszParamValues = explode( '|', $szSearchParam );

        // add the param
        $oGeoCoder->addSearchAttribute( new SearchAttribute(
            $aszParamValues[0], utf8_encode($aszParamValues[2]),
            $aszParamValues[1] == 'propertyislike' ) );

        // keep a running list of namespace:attributes
        array_push( $aszAttributes,$_SESSION[ 'namespace_'.$_REQUEST['id'] ].':'.$aszParamValues[0] );
    }

    // add a search service - check if host is provided
    $szOnlineResource = $_SESSION[ 'online_res_'.$_REQUEST['id'] ];
    $szTypeName = $_SESSION[ 'typename_'.$_REQUEST['id'] ];
    $szNameSpace = $_SESSION[ 'namespace_'.$_REQUEST['id'] ];
    $szHost = substr( strtolower( $szOnlineResource ), 0, 4) != 'http'?
                                            'http://'.$_SERVER['HTTP_HOST']:'';
    $oGeoCoder->addSearchService(
        new WebFeatureService( $oGeoCoder, $szHost.$szOnlineResource.
        '&service=WFS&version=1.0.0', $szTypeName,
        implode( '|',$aszAttributes ) , "", "250" ) );

    // perform the search
    $oGeoCoder->searchServices( '', true );

    // get the search results
    $aoResult = $oGeoCoder->getResult();
    $nResult = count( $aoResult );

    // return empty if no results
    if ( $nResult <= 0 )
    {
        return '0';
    }

    // must have results so process
    $aszResults = array();
    foreach( array_keys( $aoResult ) as $szKey )
    {
        // loop and add a header for each item
        $szDisplayValue = '';
        foreach ($aszAttributes as $szPropertyName)
        {
          $szDisplayValue .= $aoResult[$szKey]->getAttributeValueByName(strtoupper($szPropertyName)).' ';
        }

        // add the result to the list
        array_push( $aszResults,  $szDisplayValue.'|'.
            $aoResult[$szKey]->getAttributeValueByName("GML:COORDINATES") );
    }

    // parse to a string and return
    return implode( '||', $aszResults );
}
?>