<?php
/**
 * Base Widget class
 *
 * @project     CWC2
 * @revision    $Id: Widget.php,v 1.64 2006/03/23 14:04:09 jlacroix Exp $
 * @purpose     Base Widget class
 * @author      DM Solutions Group (assefa@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002-2004, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * Priorities for widgets:
 *
 * every widget is processed through three stages
 * 1. initialization
 * 2. ParseURL
 * 3. DrawPublish (and associated GetXXX functions)
 *
 * For stages 2 and 3, the widgets are processed in priority order
 *
 * By default, widgets are PRIORITY_MEDIUM.  A widget developer can
 * explicitly set the widget priority in the constructor using one of
 * these defined values.  The order of processing is normally unimportant
 * but in some cases it is absolutely critical that something happen before
 * or after other normal processing.
 *
 * It is important to use the defined values because priorites are validated
 * by the chameleon framework.
 *
 * Processing within a priority is done by order of appearance in the page.
 * For two widgets at the same priority, the first one in the page will be
 * processed first.
 */
define("PRIORITY_LAST",0);
define("PRIORITY_LOW",1);
define("PRIORITY_MEDIUM",2);
define("PRIORITY_HIGH",3);
define("PRIORITY_SUPER",4);

define( "PRIORITY_MINIMUM", PRIORITY_LAST );
define( "PRIORITY_MAXIMUM", PRIORITY_SUPER );

/*
 * widget events need to be documented
 */
define( "ONEVENT_UNDEFINED", -1);
define( "ONEVENT_FATAL", 0);
define( "ONEVENT_WARNING", 1);

/*
 * what is this?
 */
define("AUTO_PROJECTION_UNIT",9001);

include_once( COMMON.'/xml_utils/XMLObject.php' );

/// Base widget class
/**
 *  class  CWCWidget
 *
 * Base widget class to be used as the base
 * class for every widget.  CWCWidget provides essential
 * framework member variables and methods used by the
 * chameleon framework, as well as common functionality
 * shared by all widgets (such as parsing contents of
 * tags).
 */
class CWCWidget
{
    /// Store the Map object. Via the MapSession class.
    var $moMapObject;

    /// DEPRACATED: use moFormVars instead
    /** DEPRECATED: use moFormVars instead
     * an array of values that came from either GET or POST (or both).
     */
    var $moURLArray;

    /// An object holding values that came from either GET or POST (or both).
    /** a reference to an HttpFormVars instance maintained by the application
     * and shared amongst all widgets.  It provides case-insensitive access
     * to form variables from the last page submit.
     */
    var $moFormVars;

    /// String of the widget tag
    /* a string containing the tag that was used to instantiate this widget */
    var $mszCWCTag="";

    /// Number of parameter in the widget declaration
    /** the number of parameters passed to this widget as attributes of the
     * cwc2 tag that instantiated this widget
     */
    var $mnParams = 0;

    /// Parameters from widget tag
    /** an associative array of attributes (name="value") pairs parsed from the
     * cwc2 tag that instantiated this widget
     */
    var $maParams = array();
    /// Widget unique Id
    var $mnId = 0;
    /// Container of JavaScript to access the working form
    var $mszHTMLForm = "document.forms[0]";

    /// Priority of the widget among all widget
    /**
     * Priorities for widgets:
     *
     * every widget is processed through three stages
     * - 1. initialization
     * - 2. ParseURL
     * - 3. DrawPublish (and associated GetXXX functions)
     *
     * For stages 2 and 3, the widgets are processed in priority order
     *
     * By default, widgets are PRIORITY_MEDIUM.  A widget developer can
     * explicitly set the widget priority in the constructor using one of
     * these defined values.  The order of processing is normally unimportant
     * but in some cases it is absolutely critical that something happen before
     * or after other normal processing.
     *
     * It is important to use the defined values because priorites are
     * validated by the chameleon framework.
     *
     * Processing within a priority is done by order of appearance in the page.
     * For two widgets at the same priority, the first one in the page will be
     * processed first.
     */
    var $mnPriority = PRIORITY_MEDIUM;

    /// Contents between > and </CWC2> in raw form
    var $mszContent = "";
    /// Contents of mszContent parsed as per ParseContents
    var $maszContents = array();
    /// Array of attributes available for the widget declaration
    /**
     * List of all attributes available in a widget.
     * All attributes must be added in the widget constructor and are
     * case-insensitive.
     *
     * Each element of this array are an object from one of this class:
     * StringAttribute, IntegerAttribute, FloatAttribute, HexColorAttribute,
     * RGBColorAttribute or BooleanAttribute.
     *
     * The key of the array is the Attribute name in uppercase.
     */
    var $maAttributes = array();
    /// Set the visibility of the widget. If false, the widget is still processed, just not showed on the screen.
    var $mbVisible = true;
    /// Enable a widget
    var $mbEnabled = true;
    /// True when the InitDefault function have been called
    var $mbInitDefaults = false;

    /// Maturity level of the widget
    /**
     * Code maturity indicates the stability and completeness of a widget.
     * It is to be maintained by the widget developer in conjunction with the
     * QA and documentation team.  Every widget, by default, will
     * inherit an UNKNOWN maturity level.  Widgets that are undergoing
     * active development should be marked as ALPHA.  When a widget is
     * considered functionally complete and free of major errors, it should
     * be marked as BETA.  Once QA has verified the widget functionality to
     * be stable, the widget can be marked as RELEASECANDIDATE.  When
     * a widget has full documentation (English) it can be changed to
     * a RELEASE level.  Any changes to a widget after marking as RELEASE
     * must be accompanied by a corresponding change in maturity level.  For
     * changes related to bug fixes, it is sufficient to drop one level to
     * RELEASECANDIDATE until the bug fix has been verified.  Any new
     * development will require that the widget be marked as BETA until
     * the new functionality has been tested.
     *
     * Code maturity level will be reflected in the widget documentation for
     * each widget.  In addition, it can affect whether the widget is available
     * in a given installation.  The Chameleon configuration file now contains
     * a code maturity level item that allows the administrator to define the
     * minimum maturity level available to all applications.  In addition,
     * individual applications can set a maturity level separately, but will be
     * restricted to the administratively configured minimum.  Only widgets at
     * or above the minimum maturity level will be available in an application.
     */
    var $mnMaturityLevel = MATURITY_UNKNOWN;

    /// Form index of the widget
    /**
     * Chameleon use can use a lot of HTML. In a page with multiple form,
     * the form index must be kept to know which form we need to edit to
     * add the Chameleon HTML variable.
     */
    var $mnFormIndex = 0;

    /// List of all shared resources between widgets
    var $maSharedResourceWidgets = array();

    /// Hold the Multi-Lingual Translator Module ( MLT class )
    /**
     * Chameleon as multi-lingual capabilities. All widgets have access to
     * a resource file that contains all error messages or text displayed by
     * the widget. This object is used to access those files and load the
     * text from the right language.
     */
    var $moMLT = "";

    /// Holder for map object ( use moMapObject->oMap instead )
    var $moMap;

    /// Hold the application object ( hold the Chameleon object )
    var $moApp;

    /// Widget name
    var $mszWidgetName = "Widget";

    /// Array of all error messages returned by the widget
    var $maszErrorMsg = array();

    /**
     * CWCWidget
     *
     * The parent constructor must always been called by the children classes.
     *
     * In the constructor, you set the Attribute avaible in the widget
     * declaration. A standard attribute creation will be like that:
     * \code
     * $this->maAttributes["FORMINDEX"] = new IntegerAttribute( "FORMINDEX", false, 0 );
     * \endcode
     *
     * Key of the array is the Attribute name in uppercase.
     * Type of the attribute is defined by the class used to create the object.
     * First argument will be the name of the attribute in uppercase.
     * Second argument is the mandatory status of the argument.
     *
     * A description of the widget can also be found in the constructor. This
     * is for help purpose.
     */
    function CWCWidget()
    {
        $this->moApp =& GetChameleonApplication();

        $this->maAttributes["TYPE"] = new StringAttribute( "TYPE", true, array( strtoupper(get_class($this))));
        $this->maAttributes["VISIBLE"] = new BooleanAttribute( "VISIBLE", false);
        $this->maAttributes["ENABLED"] = new BooleanAttribute( "ENABLED", false);
        $this->maAttributes["FORMINDEX"] = new IntegerAttribute( "FORMINDEX", false, 0 );

        // set the description for this widget
        $this->szWidgetDescription = "CWCWidget is the most basic type of ".
                        "widget. It provides the basic functionality of a ".
                        "widget and defines the standard API that widgets ".
                        "must implement in order to function correctly with ".
                        "the CWC service instance. All widgets represent ".
                        "themselves in the template using the CWCWidget tag ".
                        "and set the type and other attributes as necessary. ";

        // create a new MLT object
        if (!isset($_SESSION['gszCurrentLanguage']))
            $lang = "en-CA";
        else
            $lang = $_SESSION['gszCurrentLanguage'];


        // pass a copy of the application MLT object to the widget
        $this->moMLT = &$this->moApp->moGlobalMLT;


        // load the resourcre file into memory
/*      Move to WidgetManager.php (bug 961) because get_class() returns a lower
        case class name and resource files are case sensitive.  Resource files
        are not found on Unix.

        if ( is_object( $this->moMLT ) )
        {

            $this->moMLT->loadResource( get_class( $this ),
                str_replace("\\","/",dirname(__FILE__) ).'/'.
                get_class( $this ) );
        }
*/
    }

    /**
     * ValidateAttributes
     *
     * Valid all parameters passed in the widget declaration.
     *
     * Validation made:
     * - All attribute types respected
     * - All mandatory attribute present
     * - No attribute present that are not understand by the widget.
     *
     * @return boolean - Status of the widget, valid or not.
     */
    function ValidateAttributes( )
    {
        $bValid = true;
        $aszAllParams = array_keys( $this->maParams );
        foreach( $this->maAttributes as $szAttr => $oValidator )
        {
            if ($oValidator->mbMandatory && !isset($this->maParams[$szAttr]))
            {
                echo "<!-- Missing required attribute $szAttr in ".$this->maParams["TYPE"]." -->\n";
                $bValid = false;
                $_SESSION['gErrorManager']->setError(ERR_WARNING,
                "ERROR: missing required attribute $szAttr in ".$this->maParams["TYPE"]);
            }
            elseif (isset( $this->maParams[$szAttr] ) && !$oValidator->Validate( $this->maParams[$szAttr] ))
            {
                echo "<!-- Invalid attribute $szAttr in ".$this->maParams["TYPE"]." -->\n";
                $bValid = false;
                $_SESSION['gErrorManager']->setError(ERR_WARNING,
                "ERROR: Invalid attribute $szAttr in ".$this->maParams["TYPE"].", value is ".$this->maParams[$szAttr]);
            }
            $idx = array_search( $szAttr, $aszAllParams );
            if ($idx !== false)
            {
                unset( $aszAllParams[$idx] );
            }
        }

        if (count($aszAllParams) > 0)
        {
            $szMessage = "WARNING: following attributes are invalid for a ".
                          get_class($this)." widget: ".
                          (implode( ",", $aszAllParams)).".".
                          "  The widget tag was: ".htmlentities($this->mszCWCTag).".";
            $_SESSION['gErrorManager']->setError(ERR_WARNING, $szMessage );
            //$bValid = false;
        }

        return $bValid;
    }


    /**
     * SetMap($oMapObject)
     *
     * Set the map object from a MapSession object.
     *
     * @param oMapObject
     */
    function SetMap(&$oMapObject)
    {
        $this->moMapObject =& $oMapObject;
    }


    /**
     * SetSharedResourceWidgets
     *
     * Set maSharedResourceWidgets with a shared resource object.
     *
     * @param aSharedResourceWidgets By reference
     */
    function SetSharedResourceWidgets(&$aSharedResourceWidgets)
    {
        $this->maSharedResourceWidgets =& $aSharedResourceWidgets;
    }


    /**
     * SetURL($oURLObject)
     *
     * Set the URLObject from a HttpFormVars object.
     *
     * @param oFormVars By reference moHttpFormVars object
     * @param oURLArray By reference array of URL variables
     * @return boolean - return value of the ParseURL() member.
     */
    function SetURL(&$oFormVars, &$oURLArray)
    {

        if (!$this->mbInitDefaults)
        {
            $_SESSION['gErrorManager']->setError(ERR_WARNING,
            "ERROR: InitDefaults didn't call base class InitDefaults in widget ".get_class( $this ));

            return false;
        }

        $this->moURLArray =& $oURLArray;
        $this->moFormVars =& $oFormVars;

        return $this->ParseURL();
    }

    /**
     * Check if a URL variable is set
     * @return true if a var is set in the form, false otherwise
     */
    function isVarSet( $szVar )
    {
        // check if moFormVars is set first
        if ( !isset( $this->moFormVars ) )
        {
            return false;
        }

        // otherwise passthru to the function
        return $this->moFormVars->isVarSet( $szVar );
    }

    /**
     * Get the value of a URL variable.
     * @param szVar Name of the URL variable to return
     * @return value of a form variable or false if not set
     */
    function getVar( $szVar )
    {
        // check if moFormVars is set first
        if ( !isset( $this->moFormVars ) )
        {
            return false;
        }

        return $this->moFormVars->getVar( $szVar );
    }

    /**
     * Set a value in the form variables array
     * @return true if variable was set correctly.
     */
    function setVar( $szVar, $szValue )
    {
        return $this->moFormVars->setVar( $szVar, $szValue );
    }

    /**
     * ParseURL
     *
     * This method should be redefined by the widgets.
     * This is were all processing will be done.
     * It should return true on success, false otherwise
     */
    function ParseURL()
    {
        return true;
    }

    /**
     * SetUniqueId
     *
     * Unique id associated with the widget. Can be used to append
     * to function names/variables to guarantee uniqueness.
     *
     * @param $nId
     */
    function SetUniqueId($nId)
    {
        $this->mnId = $nId;
    }

    /**
     * InitUsingTag($szTag)
     *
     * Parse the widget tag (from the template) and extract all the parameters.
     *
     * @param $szTag string - Declaration tag of the widget from the template.
     * @return true if all attributes are valid.
     */
    function InitUsingTag(&$szTag)
    {
        $bResult = true;
        $this->mszCWCTag = $szTag;
        $this->ParseTag();
        //validate by default
        if (!isset($GLOBALS["gbValidateTemplate"]) || strcasecmp($GLOBALS["gbValidateTemplate"], "false") != 0)
            $bResult = $this->ValidateAttributes();
        return $bResult;
    }

    /**
     * InitDefaults
     *
     * per-widget initialization to initialize default values from
     * attributes etc.
     * Children must call the parent function.
     * This function is used to initialize all widget member variable with
     * parameters present in the widget tag.
     *
     * All InitDefault function are called in order of widget priority and
     * in the order they appear in the template. So a widget of high priority
     * will be processed first and two widget of the same priority will be
     * processed in the order they are in the template.
     *
     * @return void
     */
     function InitDefaults()
     {
        if (isset($this->maParams["VISIBLE"]))
            $this->mbVisible = ((strcasecmp($this->maParams["VISIBLE"], "TRUE")==0) ? true : false);

        if (isset($this->maParams["ENABLED"]))
            $this->mbEnabled = ((strcasecmp($this->maParams["ENABLED"], "TRUE")==0) ? true : false);

        if (isset($this->maParams["FORMINDEX"]))
        {
            $this->mnFormIndex = $this->maParams["FORMINDEX"];
        }
        else
        {
            $this->mnFormIndex = isset( $this->moApp->mnFormIndex )?
                $this->moApp->mnFormIndex : 0;
        }
        $this->mszHTMLForm = "document.forms[".$this->mnFormIndex."]";
        $this->mbInitDefaults = true;

        if(!isset($this->moMLT))
        {
            if(isset($this->moApp) && isset($this->moApp->moGlobalMLT))
            {
                $this->moMLT =& $this->moApp->moGlobalMLT;
            }
            else if(class_exists('MLTPHPIncludeDefault'))
            {
                $this->moMLT = new MLTPHPIncludeDefault();
            }
        }

        return;
     }

    /**
     * ParseAttributes( $szAttributes )
     *
     * parse a string of attributes name="attribute" into an array with
     * names as keys and attributes as values
     *
     * @param $szAttributes Attribute string
     * @return array with attributes name as key and values as value.
     */
    function ParseAttributes( $szAttributes )
    {
        $aszAttribs = array();
        $szRegExp = "/(.+)=\"(.*)\"/U";

        $nResults = preg_match_all( $szRegExp, $szAttributes, $aszParams);

        $aszKeys = array_values( $aszParams[1] );
        $aszValues = array_values( $aszParams[2] );

        for($i=0;$i<count($aszKeys);$i++)
        {
            $szName = trim($aszKeys[$i]);
            $szValue = $this->CWCHtmlEntities($aszValues[$i]);
            $aszAttribs[strtoupper($szName)] = $szValue;
        }

        return $aszAttribs;
    }


    /**
     * ParseTag()
     *
     * Parse a CWC tag and extract the parameters and the values. The tag used
     * is stored in mszCWCTag.
     */
    function ParseTag()
    {
        if (strlen($this->mszCWCTag) > 5 &&
            (strncasecmp("<CWC2", $this->mszCWCTag, 5) == 0))
        {
            $szTag = trim($this->mszCWCTag);

            //first look for a complex tag
            if( stristr( $szTag, "</CWC2>" ) === false )
            {
                $iEndTag = strpos($szTag, "/>");
            }
            else
            {
                $iEndTag = strpos( $szTag, ">" );
                $iStartContents = $iEndTag + 1;
                $iEndContents = strrpos( $szTag, "<" ) - 1;
                $this->mszContent = substr( $szTag, $iStartContents, $iEndContents - $iStartContents );
            }
            if ($iEndTag === false)
              return;

            $szParams = substr($szTag, 5, ($iEndTag -5));
            $szParams = trim($szParams);

            $this->maParams = array_merge($this->ParseAttributes( $szParams ), $this->maParams);
        }

        if (trim($this->mszContent) != "")
            $this->ParseContents();
    }

    /**
     * ParseContents()
     *
     * Parse the contents of a <CWC2 ...>contents</CWC2> tag
     * assuming that the contents conform to the following
     * formatting:
     * \code
     * <contenttype param1="value" param2="value"/>
     * <contenttype param1="value" param2="value"/>
     * \endcode
     * into a member variable array structured as follows:
     * \code
     * maszContent[contenttype][index][param] = value \endcode
     * different content types are grouped by contenttype and then
     * by index of appearance.  Each parameter is given a key in a
     * subarray with the value being parsed from the tag.
     *
     * @return true on success, void otherwise
     */
    function ParseContents()
    {

        $szContents = trim($this->mszCWCTag);
        if (strlen($szContents) <= 0)
            return;

        $oXML = new XMLObject( $szContents );
        if (!isset($oXML->children[0]))
        {
            //$oXML->dumpObject();
            //error parsing contents of tag as an XML structure.
        }
        else
        {
            $this->maszContents = $this->ProcessContentChildren( $oXML->children[0]->children );
        }
        return true;

    }

    /**
     * Process a XML object and make it into an array. This function is for the
     * strict use of ParseContents()
     * @param aXMLObj - XMLObject
     * @return array of key/value
     */
    function ProcessContentChildren( $aXMLObj  )
    {
        if (!is_array( $aXMLObj))
            return;

        $aContents = array();
        foreach($aXMLObj as $oXMLObj)
        {
            //$oXMLObj->dumpObject();
            $szIndex = strtoupper($oXMLObj->_type);
            if (!isset($aContents[$szIndex]))
            {
                $aContents[$szIndex] = array();
            }

            $idx = array_push( $aContents[$szIndex], array_change_key_case($oXMLObj->_attributes, CASE_UPPER) ) - 1;
            if (isset($oXMLObj->value))
            {
                $aContents[$szIndex][$idx]["_CONTENTS"] = $oXMLObj->value;
            }

            $nChildren = count($oXMLObj->children);
            if ($nChildren > 0)
            {
                $aContents[$szIndex][$idx]["CHILDREN"] = array();
                array_push( $aContents[$szIndex][$idx]["CHILDREN"], $this->ProcessContentChildren( $oXMLObj->children) );
            }
        }
        return $aContents;
    }

    /**
     * GetValue($szParameter,$szDefault)
     *
     * Get the value from the maParams array using the parameter name or
     * return the default value if not found.
     *
     * @param $szParameter Parameter name
     * @param $szDefault Optional default value. Default ""
     * @return string - Value from the maParams array.
     */
    function GetValue($szParameter, $szDefault = "")
    {
        $szResult = $szDefault;
        if (isset($this->maParams[strtoupper($szParameter)]))
            $szResult = $this->maParams[strtoupper($szParameter)];

        return $szResult;
    }

    /**
     * SetValue( $szParameter, $szValue )
     *
     * Sets the parameter (in maParams) to value and creates it if necessary
     *
     * @param szParameter - the value to set
     * @param szValue - the value to place in that parameter
     */
    function SetValue( $szParameter, $szValue )
    {
        $this->maParams[strtoupper($szParameter)] = $szValue;
    }

    /**
     * DrawPublish()
     *
     * Should be redefined for Widgets returning HTML code.
     *
     * All string returned by this function will be printed in the output
     * template result at the place the widget declaration was.
     *
     * @return string - What should be printed to screen
     */
    function DrawPublish()
    {
        $szReturn = "";

        foreach ($this->maszErrorMsg as $szErrorMsg)
            $szReturn .= $szErrorMsg."<br>\n";

        return $szReturn;
    }

    /**
     * GetJavascriptFunctions
     *
     * Should be redefined for Widgets returning Javascript code.
     *
     * The return value must be an array. The key is the function name and
     * the value the javascript function declaration.
     *
     * @return array - JavaScript functions that will be used by the widget
     */
    function GetJavascriptFunctions()
    {
        return "";
    }

    /**
     * GetJavascriptOnLoadFunctions
     *
     * Should be redefined for Widgets returning Javascript code.
     *
     * Return value is an array key and value are both the functions that
     * need to be called in the form onLoad event.
     * The function should be declared in GetJavaScriptFunction().
     * @return array - JavaScript functions that will be called on page load
     */
    function GetJavascriptOnLoadFunctions()
    {
        return "";
    }


    /**
     * GetJavascriptOnMouseMoveFunctions
     *
     * Should be redefined for Widgets returning Javascript code.
     *
     * Return value is an array key and value are both the functions that
     * need to be called by the OnMouseMove event.
     * The function should be declared in GetJavaScriptFunction().
     * @return array - JavaScript functions that will be called
     */
    function GetJavascriptOnMouseMoveFunctions()
    {
        return "";
    }

    /**
     * GetJavascriptOnMouseUpFunctions
     *
     * Should be redefined for Widgets returning Javascript code.
     *
     * Return value is an array key and value are both the functions that
     * need to be called by the OnMouseUp event.
     * The function should be declared in GetJavaScriptFunction().
     * @return array - JavaScript functions that will be called
     */
    function GetJavascriptOnMouseUpFunctions()
    {
        return "";
    }

    /**
     * GetJavascriptOnMouseDownFunctions
     *
     * Should be redefined for Widgets returning Javascript code.
     *
     * Return value is an array key and value are both the functions that
     * need to be called by the OnMouseDown event.
     * The function should be declared in GetJavaScriptFunction().
     * @return array - JavaScript functions that will be called
     */
    function GetJavascriptOnMouseDownFunctions()
    {
        return "";
    }

    /**
     * GetJavascriptInitFunctions
     *
     * Should be redefined for Widgets returning Javascript code.
     *
     * Returns an array that contains the initialization for the javascript
     * variables if necessary. Function calls can also be placed here. This
     * will be called after the onLoad event.
     * @return array - JavaScript variables and functions with their value
     */
    function GetJavascriptInitFunctions()
    {
        return "";
     }

    /**
     * GetJavascriptIncludeFunctions
     *
     * Should be redefined for Widgets returning Javascript code.
     *
     * Returns an array that contains the include statement for the javascript
     * includes if necessary. Key is the included file and value the include
     * statement.
     * The function should be declared in GetJavaScriptFunction().
     * @return array - JavaScript includes declaration with their value
     */
    function GetJavascriptIncludeFunctions()
    {
        return "";
    }

    /**
     * GetJavascriptVariables
     *
     * Should be redefined for Widgets returning Javascript code.
     *
     * Returns an array that contains the global JavaScript variables
     * used by this widget. Key is the variable name and value the variable
     * declaration JavaScript code.
     * @return array - JavaScript global variables declaration with their value
     */
    function GetJavascriptVariables()
    {
        return "";
    }

    /**
     * GetHTMLHiddenVariables
     *
     * Should be redefined for Widgets using HTML hidden variables.
     *
     * Return an array that contains all HTML hidden variables used by this
     * widget. Key is the variable name and value the HTML variable
     * declaration.
     * @return array - HTML hidden variables declaration with their value
     */
    function GetHTMLHiddenVariables()
    {
        //make sure that the session id makes it into the output page.
        $szVariable = "sid";
        $szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"".session_id()."\">";
        return array($szVariable => $szValue);
    }

    /**
     * Sets the HTML form that is used by Chameleon.
     * @param szForm - JavaScript code to access the form
     */
    function SetHTMLForm($szForm)
    {
        $this->mszHTMLForm = $szForm;
    }

    /**
     * Return a HTML encoded version of the widget tag.
     * @return string - HTML encoded widget tag
     */
    function GetSample()
    {
        $szResult = "&lt;CWC2 ";

        foreach( $this->maAttributes as $oAttr )
        {
            $szResult .= " ".$oAttr->getSample();
        }

        $szResult .= "/&gt;";
        return $szResult;
    }

    /**
     * ReprojectAuto
     *
     * Utility function to do the reprojection for AUTO
     * projections.
     *
     * @param szNewProj Optional. Default ""
     */
    function ReprojectAuto($szNewProj = "")
    {
        $oMap = $this->moMapObject->oMap;
        $szMapProj = $oMap->getProjection();

        if ($szNewProj == "")
          $szProj = $szMapProj;
        else
          $szProj = $szNewProj;

        if(strncasecmp(trim($szProj), "AUTO", 4) ==0)
        {
          //echo "ReprojectAuto() : initial= " . $szProj . "<br>\n";

            $nCenterX = ($oMap->extent->minx + $oMap->extent->maxx) / 2.0;
            $nCenterY = ($oMap->extent->miny + $oMap->extent->maxy) / 2.0;
            $oCenter = ms_newPointObj( );
            $oCenter->setXY( $nCenterX, $nCenterY );

            $oProj = ms_newProjectionObj($szMapProj);

            $oLatLonProj = ms_newProjectionObj("+proj=latlong +ellps=WGS84");
            $oCenter->project($oProj, $oLatLonProj);

            $aProjElem = split(",",$szProj);
            $nProjElem = count($aProjElem);
            if ($nProjElem == 1)
            {
                $szProj =
                  $szProj.",".AUTO_PROJECTION_UNIT.",".$oCenter->x.",".$oCenter->y;
            }
            else if ($nProjElem == 4) //valid AUTO projection
            {
                $szProj =
                  $aProjElem[0].",".AUTO_PROJECTION_UNIT.",".$oCenter->x.",".$oCenter->y;
            }
            else
              $szProj = "";

            $oProj = ms_newProjectionObj($szProj);

            if ($szProj != "")
            {
                //echo "ReprojectAuto() : final= " . $szProj . "<br>\n";
                $oMap->setProjection( $szProj, MS_TRUE );
            }
        }
    }

    /**
     * ProcessEvent($szEvent)
     *
     * Event handling is all done from here. The possible event behavior are
     * to redirect to a URL, redirect to another template or simply print text.
     * @param szEvent string - Name of the event to be processed
     * @return Error level of the event.
     */
    function ProcessEvent($szEvent)
    {
        if (isset($this->maszContents["ONEVENT"]))
        {
            foreach ($this->maszContents["ONEVENT"] as $aEvent)
            {
                $nErrorLevel = ONEVENT_WARNING;

                if (strcasecmp($aEvent['EVENT'], $szEvent)==0)
                {
                    if (isset($aEvent['ERRORLEVEL']))
                      $nErrorLevel = $aEvent['ERRORLEVEL'];

                    if (isset($aEvent['URL']))
                    {
                        // Then, replace all [something] to the correct url var value
                        $nPos = strpos($aEvent['URL'], "[$");
                        while($nPos !== false)
                        {
                            $nEndPos = strpos($aEvent['URL'], "$]", $nPos);
                            $szTag = substr($aEvent['URL'], $nPos, $nEndPos-$nPos+2);

                            if ($this->isVarSet( strtoupper(substr($szTag, 2, -2)) ))
                                $aEvent['URL'] =& str_replace($szTag,
                                        $this->getVar( strtoupper(substr($szTag, 2, -2)) ),
                                        $aEvent['URL']);
                            $nPos = strpos($aEvent['URL'], "[", $nPos+2);
                        }
                        header("Location: ".$aEvent['URL']);

                        return ONEVENT_FATAL;
                    }
                    else
                    if (isset($aEvent['TEMPLATE']))
                    {
                        $this->moApp->CWCSetTemplate($aEvent['TEMPLATE']);
                        echo "<!-- redirecting to (".$aEvent['TEMPLATE'].") template. -->\n\n";

                        return ONEVENT_FATAL;
                    }
                    else
                    if (isset($aEvent['TEXT']))
                    {
                        array_push($this->maszErrorMsg, $aEvent['TEXT']);

                        return $nErrorLevel;
                    }
                }
            }

        }
        return ONEVENT_UNDEFINED;
    }

    /**
     * GetEvent
     *
     * Used to get information on a specific event.
     * @param szEvent string - Name of the event to get
     * @return array - Array of event informations
     */
    function GetEvent($szEvent)
    {
        $aReturn = array();

        if(isset($this->maszContents["ONEVENT"]) &&
           is_array($this->maszContents["ONEVENT"]))

        {
            foreach($this->maszContents["ONEVENT"] as $oEvent)
            {
                if (isset($oEvent["EVENT"]) && $oEvent["EVENT"] == $szEvent)
                array_push($aReturn, $oEvent);
            }
        }

        return $aReturn;
    }

    /**
     * Replace specific html encoded tags by their real values.
     * @param szString The string that need to be decoded
     * @return string - The decoded string
     */
    function CWCHtmlEntities($szString)
    {
        $szString = str_replace(array("&amp;", "&#039;", "&gt;", "&lt;", "&quot;"),
                                array( "&",    "'",      ">",    "<",    "\""), $szString);

        return $szString;
    }
}

/*
 * Attribute
 *
 * Basic attribute validators. Class to add attributes available
 * in a widget.
 */
class Attribute
{
    var $mbMandatory;
    var $mszName;

    function Attribute( $szName, $bMandatory )
    {
        $this->mszName = $szName;
        $this->mbMandatory = $bMandatory;
    }
}

/*
 * StringAttribute
 *
 * String Attribute validators. Class to add string attributes available
 * in a widget declaration. This class validate the value of the attribute
 * and manage the value.
 */
class StringAttribute extends Attribute
{
    var $maValues = array();

    function StringAttribute( $szName, $bMandatory, $aValues = array() )
    {
        $this->Attribute( $szName, $bMandatory );
        array_walk( $aValues, create_function( '&$a', '$a = strtoupper($a);' ) );
        $this->maValues = $aValues;
    }

    function Validate( $szValue )
    {
        $bResult = true;
        if (count($this->maValues) > 0)
        {
            return (in_array( strtoupper($szValue), $this->maValues ));
        }
        else
            return true;
    }

    function GetSample()
    {
        if (count($this->maValues) > 0)
        {
            $szValue = implode( "|", $this->maValues );
        }
        else
            $szValue = "string";
        if ($this->mszName == "TYPE")
        {
            $szValue = "&quot;$szValue&quot;";
        }
        else
        {
            $szValue = "&quot;[$szValue]&quot;";
        }
        $szResult = "$this->mszName=$szValue";
        if ($this->mbMandatory)
            $szResult = "<font color=#FF3333>$szResult</font>";

        return $szResult;
    }
}

/*
 * BooleanAttribute
 *
 * Boolean Attribute validators. Class to add boolean attributes available
 * in a widget declaration. This class validate the value of the attribute
 * and manage the value.
 */
class BooleanAttribute extends StringAttribute
{
    function BooleanAttribute( $szName, $bMandatory )
    {
        $this->StringAttribute( $szName, $bMandatory, array( "TRUE", "FALSE" ) );
    }
}

define( "PHP_MINIMUM_INTEGER", -2147483647 );
define( "PHP_MAXIMUM_INTEGER", 2147483647 );
define( "PHP_MINIMUM_FLOAT", -1.8e308 );
define( "PHP_MAXIMUM_FLOAT", 1.8e308 );

/*
 * IntegerAttribute
 *
 * Integer Attribute validators. Class to add integer attributes available
 * in a widget declaration. This class validate the value of the attribute
 * and manage the value.
 */
class IntegerAttribute extends Attribute
{
    var $mnMin;
    var $mnMax;

    function IntegerAttribute( $szName, $bMandatory, $nMin = PHP_MINIMUM_INTEGER, $nMax = PHP_MAXIMUM_INTEGER )
    {
        $this->Attribute( $szName, $bMandatory );
        $this->mnMin = $nMin;
        $this->mnMax = $nMax;
    }

    function Validate( $nValue )
    {
        if (!is_numeric($nValue))
            return false;

        if (is_float($nValue))
            return false;

        return ($nValue >= $this->mnMin && $nValue <= $this->mnMax);
    }

    function GetSample()
    {
        $szValue = "&quot;[";
        if ($this->mnMin > PHP_MINIMUM_INTEGER)
        {
            $szValue .= $this->mnMin." < ";
        }
        $szValue .= "integer";
        if ($this->mnMax < PHP_MAXIMUM_INTEGER )
        {
            $szValue .= " < ".$this->mnMax;
        }
        $szValue .= "]&quot;";
        $szResult = "$this->mszName=$szValue";
        if ($this->mbMandatory)
            $szResult = "<font color=#FF3333>$szResult</font>";
        return $szResult;
    }
}

/*
 * FloatAttribute
 *
 * Float Attribute validators. Class to add float attributes available
 * in a widget declaration. This class validate the value of the attribute
 * and manage the value.
 */
class FloatAttribute extends Attribute
{
    var $mfMin;
    var $mfMax;

    function FloatAttribute( $szName, $bMandatory, $fMin = PHP_MINIMUM_FLOAT, $fMax = PHP_MAXIMUM_FLOAT)
    {
        $this->Attribute( $szName, $bMandatory );
        $this->mfMin = $fMin;
        $this->mfMax = $fMax;
    }

    function Validate( $fValue )
    {
        if (!is_numeric($fValue))
            return false;

        return ($fValue >= $this->mfMin && $fValue <= $this->mfMax);
    }

    function GetSample()
    {
        $szValue = "&quot;[";
        if ($this->mfMin > PHP_MINIMUM_FLOAT)
        {
            $szValue .= $this->mfMin." < ";
        }
        $szValue .= "integer";
        if ($this->mfMax < PHP_MAXIMUM_FLOAT )
        {
            $szValue .= " < ".$this->mfMax;
        }
        $szValue .= "]&quot;";
        $szResult = "$this->mszName=$szValue";
        if ($this->mbMandatory)
            $szResult = "<font color=#FF3333>$szResult</font>";
        return $szResult;
    }
}

$GLOBALS['gaHexValues'] = array( "0", "1", "2", "3", "4", "5", "6", "7",
                      "8", "9", "A", "B", "C", "D", "E", "F" );
/*
 * HexColorAttribute
 *
 * Color Attribute in hexadecimal format validators.
 * Class to add color attributes available in a widget declaration.
 * This class validate the value of the attribute and manage the value.
 */
class HexColorAttribute extends Attribute
{
    function HexColorAttribute( $szName, $bMandatory )
    {
        $this->Attribute( $szName, $bMandatory );
    }

    function Validate( $szValue )
    {
        if (strlen( $szValue ) < 6 || strlen( $szValue ) > 7)
        {
            return false;
        }
        if (strlen($szValue) == 7 && $szValue[0] != "#")
        {
            return false;
        }
        if ($szValue[0] == "#")
            $szValue = substr( $szValue, 1 );
        for( $i=0; $i<6; $i++)
        {
            if (!in_array( strtoupper($szValue[$i]), $GLOBALS["gaHexValues"] ))
                return false;
        }
        return true;
    }

    function GetSample()
    {
        $szValue = "&quot;[[#]RRGGBB]&quot;";
        $szResult = "$this->mszName=$szValue";
        if ($this->mbMandatory)
            $szResult = "<font color=#FF3333>$szResult</font>";
        return $szResult;
    }
}

/*
 * RGBColorAttribute
 *
 * Color Attribute in RGB format validators.
 * Class to add color attributes available in a widget declaration.
 * This class validate the value of the attribute and manage the value.
 */
class RGBColorAttribute extends Attribute
{
    function RGBColorAttribute( $szName, $bMandatory )
    {
        $this->Attribute( $szName, $bMandatory );
    }

    function Validate( $szValue )
    {
        $aszRGB = explode( " ", $szValue );

        if (count($aszRGB) != 3)
            return false;

        for ($i=0; $i<3; $i++)
            if (!is_numeric($aszRGB[$i]) || $aszRGB[$i] < -1 || $aszRGB[$i] > 255 )
                return false;

        return true;
    }


    function GetSample()
    {
        $szValue = "&quot;[RRR GGG BBB]&quot;";
        $szResult = "$this->mszName=$szValue";
        if ($this->mbMandatory)
            $szResult = "<font color=#FF3333>$szResult</font>";
        return $szResult;
    }
}
