<?php
/**
 * XYLocalisation Widget class
 *
 * @project     CWC2
 * @revision    $Id:
 * @purpose     XYLocalisation Popup Widget class
 * @author      DM Solutions Group (sfournier@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
include_once(CHAMELEON_PATH."/widgets/Widget.php");
include_once(CHAMELEON_PATH."/widgets/Button.php");
include_once(CHAMELEON_PATH."/widgets/geomath.php");
include_once( COMMON."wrapper/map_navigator.php" );

/**
 * a simple widget to link to another page
 */
class XYLocalisation extends CWCWidget
{
    var $moButton;
    var $maszProjections = array();
    var $maszScales = array();
    var $mbSubmit = false;
    var $mbHiddenVar = false;
    var $mszX = '';
    var $mszY = '';
    var $mszProj = '';
    var $mszScale = '';
    var $maszErrors = array();

    /**
     * construct a new link widget
     */
    function XYLocalisation()
    {
        // invoke constructor of parent
        parent::CWCWidget();

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The XYLocalisation widget allows the template to display a button that will open an
arbitrary URL such as a help page for an application.
EOT;

        $this->moButton = new CWCButton($this);
        $this->maAttributes["INCLUDESUBMIT"] = new BooleanAttribute("INCLUDESUBMIT", false );
        $this->maAttributes["INCLUDEHIDDEN"] = new BooleanAttribute("INCLUDEHIDDEN", false );
        $this->mnMaturityLevel = MATURITY_BETA;
    }

    /**
     * initialize default values
     */
    function InitDefaults()
    {
        parent::InitDefaults();
        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick( 'XYLocalisation' );

        if (isset($this->maParams['INCLUDESUBMIT']))
        {
            $this->mbSubmit = strcasecmp($this->maParams['INCLUDESUBMIT'], 'true') == 0 ? true : false;
        }
        if (isset($this->maParams['INCLUDEHIDDEN']))
        {
            $this->mbHiddenVar = strcasecmp($this->maParams['INCLUDEHIDDEN'], 'true') == 0 ? true : false;
        }

        // List of projection by default + proj specified in template
        $this->maszProjections[] = array('EPSG:4269', 
                                         'Latitude/Longitude (NAD 83)');
        $this->maszProjections[] = array('EPSG:32198', 
                                       'LCC du gouvernement du Qu&eacute;bec');
        $this->maszProjections[] = array('EPSG:32188', 'MTM NAD 83 fuseau 8');
        $this->maszProjections[] = array('EPSG:26918', 'UTM NAD 83 fuseau 18');

        // Add user specific projection
        if(isset($this->maszContents['PROJECTION']) && 
           is_array($this->maszContents['PROJECTION']))
        {
            foreach($this->maszContents['PROJECTION'] as $aszProj)
            {
                if(isset($aszProj['NAME']) && $aszProj['NAME'] != '' && 
                   isset($aszProj['CODE']) && $aszProj['CODE'] != '')
                {
                    $this->maszProjections[] = array($aszProj['CODE'], 
                                                     $aszProj['NAME']);
                }
            }
        }

        // List of scales by default + scales specified in template
        $this->maszScales[] = array('50000', '1:50000');
        $this->maszScales[] = array('25000', '1:25000');
        $this->maszScales[] = array('12500', '1:12500');
        $this->maszScales[] = array('5000', '1:5000');
        $this->maszScales[] = array('1000', '1:1000');

        // Add user specific projection
        if(isset($this->maszContents['SCALE']) && 
           is_array($this->maszContents['SCALE']))
        {
            foreach($this->maszContents['SCALE'] as $aszScale)
            {
                if(isset($aszScale['NAME']) && $aszScale['NAME'] != '' && 
                   isset($aszScale['CODE']) && $aszScale['CODE'] != '')
                {
                    $this->maszScales[] = array($aszScale['CODE'], 
                                                      $aszScale['NAME']);
                }
            }
        }
    }
    
    function ParseURL()
    {
        if ($this->mbSubmit && $this->mbVisible)
            $this->moButton->ParseURL();

        $this->mszX = $this->getVar('XYLOCALISATION_X');
        $this->mszY = $this->getVar('XYLOCALISATION_Y');
        $this->mszProj = $this->getVar('XYLOCALISATION_PROJ');
        $this->mszScale = $this->getVar('XYLOCALISATION_SCALE');

        if($this->getVar('XYLOCALISATION') != 1)
        {
            return true;
        }

        // calculate the center pixel co-ordinates of the map if necessary
        $nX = $this->moMapObject->oMap->width / 2;
        $nY = $this->moMapObject->oMap->height / 2;
        
        $oPixelPos = ms_newpointobj();
        
        //set the click position
        $oPixelPos->setxy($nX,$nY);

        // create new rectangle object
        $oRect = ms_newrectobj();
        
        // get current map extents

        // In case of decimal degree we can have a DMS
        if(strtolower($this->mszProj) == "epsg:4269" && 
           strpos($this->mszX, ' ') !== false || 
           strpos($this->mszX, '°') !== false ||
           strpos($this->mszX, 'd') !== false)
        {
            $aszX = explode(' ', $this->mszX);
            if(!isset($aszX[1]))
                $aszX[1] = 0;
            if(!isset($aszX[2]))
                $aszX[2] = 0;
            $this->mszX = dd(doubleval($aszX[0]), 
                             doubleval($aszX[1]), 
                             doubleval($aszX[2]));echo $this->mszX.'<br>';
            if($this->mszX < 0)
                $this->mszX *= -1.0;
        }
        if(strtolower($this->mszProj) == "epsg:4269" && 
           strpos($this->mszY, ' ') !== false || 
           strpos($this->mszY, '°') !== false ||
           strpos($this->mszY, 'd') !== false)
        {
            $aszY = explode(' ', $this->mszY);
            if(!isset($aszY[1]))
                $aszY[1] = 0;
            if(!isset($aszY[2]))
                $aszY[2] = 0;
            $this->mszY = dd(doubleval($aszY[0]), 
                             doubleval($aszY[1]), 
                             doubleval($aszY[2]));echo $this->mszY.'<br>';
        }

        $nXDiff = $this->mszX * 0.00000001;
        $nYDiff = $this->mszY * 0.00000001;
        $dMapMinX = $this->mszX-$nXDiff;
        $dMapMinY = $this->mszY-$nYDiff;
        $dMapMaxX = $this->mszX+$nXDiff;
        $dMapMaxY = $this->mszY+$nYDiff;
        // set the extents of the rectangle
        $oRect->setextent($dMapMinX, $dMapMinY, $dMapMaxX, $dMapMaxY);

        // set projection if needed
        $oIn = ms_newProjectionObj("init=".strtolower($this->mszProj));
        $oOut = ms_newProjectionObj($this->moMapObject->oMap->getProjection());
        $oRect->project($oIn, $oOut);
        print_r($oRect);         
        // call the zoomscale function
        $this->moMapObject->oMap->zoomscale($this->mszScale, $oPixelPos, 
                                             $this->moMapObject->oMap->width, 
                                             $this->moMapObject->oMap->height,
                                             $oRect);

        return true;
    }
    
    /**
     * return javascript functions
     */
    function GetJavascriptFunctions()
    {
        $aReturn = array();
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptFunctions();

        $szJsFunctionName = "XYLocalisation";
        $szFunction = <<<EOT
function $szJsFunctionName()
{
    {$this->mszHTMLForm}.XYLOCALISATION = 1;
    {$this->mszHTMLForm}.submit();
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "XYLocalisationGetArguments";
        $szFunction = <<<EOT
function $szJsFunctionName()
{
    var aszArgs = Array();
    aszArgs[0] = Array('XYLOCALISATION', 1);
    aszArgs[1] = Array('XYLOCALISATION_X', 
                       {$this->mszHTMLForm}.XYLOCALISATION_X.value);
    aszArgs[2] = Array('XYLOCALISATION_Y', 
                       {$this->mszHTMLForm}.XYLOCALISATION_Y.value);
    aszArgs[3] = Array('XYLOCALISATION_PROJ', 
                       {$this->mszHTMLForm}.XYLOCALISATION_PROJ.value);
    aszArgs[4] = Array('XYLOCALISATION_SCALE', 
                       {$this->mszHTMLForm}.XYLOCALISATION_SCALE.value);

    return aszArgs;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "XYLocalisationValid";
        $szFunction = <<<EOT
function $szJsFunctionName()
{
    var szX = {$this->mszHTMLForm}.XYLOCALISATION_X.value;
    var szY = {$this->mszHTMLForm}.XYLOCALISATION_Y.value;
    if(!szX)
    {
        alert('Longitude invalide');
        return false;
    }
    if(!szY)
    {
        alert('Latitude invalide');
        return false;
    }

    var postalPattern =/^(\+|-)?\d+\.?\d*$/;
    var postalRegExp = new RegExp(postalPattern);
    if (postalRegExp.test(szX) == false )
    {
        var DMSPattern =/^(\+|-)?\d+\.?\d*[\°d]*/;
        var DMSRegExp = new RegExp(DMSPattern);
        if (DMSRegExp.test(szX) == false )
        {
            alert('Longitude invalide');
            return false;
        }
    }
    if (postalRegExp.test(szY) == false )
    {
        if (DMSRegExp.test(szY) == false )
        {
            alert('Latitude invalide');
            return false;
        }
    }

    return true;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

    /**
     * return javascript functions
     */
    function GetJavascriptIncludeFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
            $aResult = $this->moButton->GetJavascriptIncludeFunctions();
        return $aResult;
    }

    /**
     * return javascript variables
     */
    function GetJavascriptVariables()
    {
        $aResult = array();
        if ($this->mbVisible)
            $aResult =  $this->moButton->GetJavascriptVariables();
        return $aResult;
    }

    /**
     * return javascript onload functions
     */
    function GetJavascriptOnLoadFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
            $aResult = $this->moButton->GetJavascriptOnLoadFunctions();
        return $aResult;
    }
    
    /**
     * return javascript initialization funcitons
     */
    function GetJavascriptInitFunctions()
    {
        $aResult = array();
        if ($this->mbVisible)
            $aResult =  $this->moButton->GetJavascriptInitFunctions();

        $szJSErrors = '';
        foreach($this->maszErrors as $szError)
        {
            $szJSErrors .= 'alert("'.$szError.'");';
        }
        if($szJSErrors)
            $aResult['CodePostalErrors'] = $szJSErrors;

        return $aResult;
    }

    function GetHTMLHiddenVariables()
    {
        $aReturn = array();
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetHTMLHiddenVariables();

        $szVariable = "XYLOCALISATION";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
        $aReturn[$szVariable] = $szValue;

        if($this->mbHiddenVar)
        {
            $szVariable = "XYLOCALISATION_X";
            $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
            $aReturn[$szVariable] = $szValue;
            $szVariable = "XYLOCALISATION_Y";
            $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
            $aReturn[$szVariable] = $szValue;
            $szVariable = "XYLOCALISATION_PROJ";
            $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
            $aReturn[$szVariable] = $szValue;
            $szVariable = "XYLOCALISATION_SCALE";
            $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"\">";
            $aReturn[$szVariable] = $szValue;
        }

        return $aReturn;
    }

    /**
     * draw the link widget
     */
    function DrawPublish()
    {
        if (!$this->mbVisible)
            return "<!-- XYLocalisationWidget is hidden -->";

        $szReturn = '';
        if($this->mbSubmit)
            $szReturn = $this->moButton->DrawPublish();

        $szX = $this->mszX;
        $szY = $this->mszY;
        $szProj = '<select name="XYLOCALISATION_PROJ">';
        foreach($this->maszProjections as $aszProj)
        {
            $szSelect = '';
            if($aszProj[0] == $this->mszProj)
                $szSelect = 'SELECTED';
            $szProj .= "<option value='".$aszProj[0]."' $szSelect>".
                $aszProj[1]."</option>\n";
        }
        $szProj .= '</select>';

        $szScale = '<select name="XYLOCALISATION_SCALE">';
        foreach($this->maszScales as $aszScale)
        {
            $szSelect = '';
            if($aszScale[0] == $this->mszScale)
                $szSelect = 'SELECTED';
            $szScale .= "<option value='".$aszScale[0]."'>".
                $aszScale[1]."</option>\n";
        }

        $szReturn = <<<EOT
        <table><tr>
          <td><img src="icon_find_ca_place.gif" width="22" height="22"> </td>
          <td><span class="label"> Coordonn&eacute;es </span></td></tr></table>
          <table><tr><td class="label"> Longitude: </td>
          <td><input class="inputBox" type="text" name ="XYLOCALISATION_X" value="$szX" size="10"></td>
          <td class="label"> Latitude: </td>
          <td><input class="inputBox" name ="XYLOCALISATION_Y" type="text" value="$szY" size="10"></td>
          </tr><tr><td class="label" colspan="8">Projection:</td></tr>
          <tr><td colspan="8" class="label">
              $szProj
          </td></tr></table>
          <table><tr><td><span class="label">&Eacute;chelle:</span><br>
          $szScale
          </td></tr></table>
EOT;

        return $szReturn;
    }
}
?>
