<?php
/**
 *
 * @project     Chameleon
 * @revision    $Id: ows.php,v 1.3 2006/04/03 19:28:46 jlacroix Exp $
 * @purpose     OWS services
 * @author      DM Solutions Group (pspencer@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/**
 * This module contains generic OWS services like loading contexts
 */
 
//access ms error messages 'cause stuff can go wrong loading contexts ...
include_once( 'ms_error.php' );

/**
 * load a context into a map object.  Optionally append the layers
 * to the context.
 *
 * @param szContextURL string the url to the context to load
 * @param oMap object the map object to load the context into
 * @param bAllowResize boolean if true, the map will be resized
 * @param bAppendLayers boolean if true, just append new layers.  If false,
 *        then remove existing layers and initialize MAP level information
 *        such as map size, projection etc and metadata
 * @return boolean true if the context loaded ok, false otherwise
 */
function loadContext( $szContextURL, $oMap, $bAllowResize = true, $bAppendLayers = false )
{
    $szLocalContext = retrieveContext( $szContextURL );
    
    //do all the work inside a temporary map object
    //to avoid screwing up stuff for chameleon ...
    $oTmpMap = ms_newMapObj( "" );
    $bResult = $oTmpMap->loadMapContext( $szLocalContext );
    if ($oTmpMap->numlayers == 0)
    {
        $szText = "ERROR: Invalid Context file";
        $_SESSION['gErrorManager']->setError(ERR_WARNING, $szText);
        return false;
    }
    else
    {
        //remove existing layers if not appending
        if (!$bAppendLayers)
        {
            for ($i = 0; $i < $oMap->numlayers; $i++)
            {
                $oLayer = $oMap->getLayer( $i );
                $oLayer->set( "status", MS_DELETE );
            }
        }

        //import the newly defined layers
        $nLayers = $oTmpMap->numlayers;
        //echo "adding ".$nLayers." layers<BR>";
        for ($i=0; $i<$nLayers; $i++)
        {
            ms_newLayerObj($oMap, $oTmpMap->getlayer($i));
        }

        //only modify map-level metadata and properties if not
        //appending ...
        if (!$bAppendLayers)
        {
            $oMap->set( "name", $oTmpMap->name );
            $oMap->setProjection( $oTmpMap->getprojection(), MS_TRUE );
            $oMap->extent->setExtent( $oTmpMap->extent->minx, $oTmpMap->extent->miny, $oTmpMap->extent->maxx, $oTmpMap->extent->maxy );
            $oMap->setMetadata( "original_extents", "");
            $oMap->setMetadata( "original_projection", "" );
            $oMap->web->set( "minscale", $oTmpMap->web->minscale );
            $oMap->web->set( "maxscale", $oTmpMap->web->maxscale );
            
            //TODO: check the list of metadata in each mapserver release ...
            $aMetaData = array( "wms_abstract", "wms_accesscontraints",
                                "wms_addresstype", "wms_address", 
                                "wms_city", "wms_stateorprovince", 
                                "wms_postcode", "wms_country",
                                "wms_contactelectronicmailaddress",
                                "wms_contactfacsimiletelephone",
                                "wms_contactperson", "wms_contactorganization", 
                                "wms_contactposition", "wms_contactvoicetelephone",
                                "wms_encoding","wms_fees", "wms_group_title",
                                "wms_keywordlist", "wms_onlineresource",
                                "wms_resx", "wms_resy", "wms_srs",
                                "wms_timeformat", "wms_title" );
            
            foreach($aMetaData as $szTag)
            {
                if( $oTmpMap->getmetadata( $szTag ) )
                    $oMap->setmetadata( $szTag, $oTmpMap->getmetadata($szTag));
            }
            
        }

        if ($bAllowResize)
        {
            $oMap->set( "width", $oTmpMap->width );
            $oMap->set( "height", $oTmpMap->height );
        }
        
        // Get mapserver error stack and display them as HTML comments.
        $oError = ms_GetErrorObj();
        while($oError && $oError->code > 0)
        {
            if (isset($this->moErrorManager))
            {
                $szMsg = getMapServerErrorMessage( $oError );
                $_SESSION['gErrorManager']->setError(ERR_WARNING, $szMsg);
            }
            echo "<!-- ".$szMsg." -->";

            $oError = $oError->next();
        }
    }
    return true;
}

/**
 * retrieve a remote context to a local file
 * @param szURL string the context to retrieve
 * @return string the local filename or false if it failed to download
 */
function retrieveContext( $szURL )
{
    if (strcasecmp( "http", substr($szURL, 0, 4 )) != 0)
    {
         return $szURL;
    }
    else
    {
        //at this point we need to fetch the file
        $aszFile = @file( $szURL );
        if ($aszFile === false)
        {
            $_SESSION["gErrorManager"]->setError(ERR_ERROR, "Error retrieving remote file ($szFilename), please ensure the URL is correct" );
            $szLocalFile = false;
        }
        else
        {
            $szLocalFile = tempnam($_SESSION['gszTmpPath'], "remote").".cml";
            $fh = @fopen($szLocalFile, "w");
            if ($fh === false)
            {
                $szLocalFile = false;
                $_SESSION["gErrorManager"]->setError(ERR_ERROR, "Error creating local copy ($szLocalFile) of remote file ($szFilename), please ensure permissions are correctly set." );
            }
            else
            {
                $nbLine = count($aszFile);

                for($i=0; $i<$nbLine; $i++)
                {
                    fwrite($fh, $aszFile[$i]);
                }
                fclose($fh);
            }
        }
    }
    return $szLocalFile;
}
?>
