<xsl:stylesheet version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
	xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
	xmlns:ca2="http://localhost/chameleon/utils/xml/schemas"
	xmlns:db="http://localhost/chameleon/utils/xml/schemas"
	xmlns:xs="http://www.w3.org/2001/XMLSchema" 
	exclude-result-prefixes="ca2 xs">

	<!-- ~~~~~~~~~~~~~~~~~~~~ -->
	<!-- DESCRIPTIONS SECTION (used by default index)  -->
	<!-- ~~~~~~~~~~~~~~~~~~~~ -->
	<xsl:template name="DescriptionsBlock">
		<xsl:for-each select="ca2:descriptions">
			<ca:descriptions>
						<xsl:call-template name="GenDescriptionBlock"/>	
				<!-- Instert by ca  -->
				<xsl:call-template name="TechRequireBlock"/>
				<xsl:call-template name="DependanciesBlock"/>
				<xsl:call-template name="SharedResourcesBlock"/>
				<xsl:call-template name="EmbeddedHelpBlock"/>
				<xsl:call-template name="GuiNotesBlock"/>
				<xsl:call-template name="QaNotesBlock"/>
			</ca:descriptions>
		</xsl:for-each>
	</xsl:template>
	
	
	<!-- ~~~~~~~~~~~~~~~~~~~~ -->
	<!-- DESCRIPTIONS SECTION  (used by manager index) -->
	<!-- ~~~~~~~~~~~~~~~~~~~~ -->
	<xsl:template name="DescriptionsUpdate">
		<xsl:for-each select="ca2:descriptions">
			<ca:descriptions>
				<!-- inserted by db -->
				<xsl:call-template name="GenDescriptionUpdate"/>	
				<!-- Instert by ca2  -->
				<xsl:copy-of select="ca2:techrequirements"/>
				<xsl:copy-of select="ca2:dependancies"/>
				<xsl:copy-of select="ca2:sharedresources"/>
				<xsl:copy-of select="ca2:embeddedhelp"/>
				<xsl:copy-of select="ca2:guinotes"/>	
				<xsl:copy-of select="ca2:qanotes"/>				
			</ca:descriptions>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="GenDescriptionUpdate">
	   <xsl:choose>
			<xsl:when test="normalize-space($dbdoc/db:widget/db:identification/db:description) = normalize-space(ca2:description/ca2:paras/ca2:para)" >
			
				<xsl:copy-of select="ca2:description"/>
				
			</xsl:when>

			<xsl:when test="normalize-space($dbdoc/db:widget/db:identification/db:description) != normalize-space(ca2:description/ca2:paras/ca2:para)" >
			
	<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Warning 1: The ca:description section has been modified by editors OR an update to the widget code has occured. Please identify which one it is. A developer may have to update the widget code description in order for this warning to be removed. The description found in the widget code is:

	"<xsl:value-of select="$dbdoc/db:widget/db:identification/db:description"/> "
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	</xsl:comment>
				<xsl:copy-of select="ca2:description"/>
			</xsl:when>
<!--
			<xsl:when test="normalize-space($dbdoc/db:widget/db:identification/db:description) != normalize-space($olddbdoc/db:widget/db:identification/db:description) and normalize-space($dbdoc/db:widget/db:identification/db:description) != normalize-space(ca2:description/ca2:paras/ca2:para)" >
			
	<xsl:comment>
		~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		ERROR 1: The ca:description and description found in the widget code have both been 
		modified since the last update. There is a conflit the the description, please fix.  
		~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	</xsl:comment>
				<xsl:copy-of select="ca2:description"/>
				
			</xsl:when>		

			<xsl:when test="normalize-space($dbdoc/db:widget/db:identification/db:description) != normalize-space($olddbdoc/db:widget/db:identification/db:description) and normalize-space($dbdoc/db:widget/db:identification/db:description) = normalize-space(ca2:description/ca2:paras/ca2:para)" >
			
				<xsl:comment>
					Update made from widget code.
				</xsl:comment>
				<xsl:call-template name="GenDescriptionBlock"/>	
				
			</xsl:when>	
-->	
			<xsl:otherwise>
				<xsl:comment>
					~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					ERROR 2: No ca:description Update Condition was met
					~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					<xsl:copy-of select="ca2:description"/>
				</xsl:comment>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

	<xsl:template name="GenDescriptionBlock">
		<xsl:for-each select="ca2:description">	
			<ca:description>
				<xsl:for-each select="ca2:paras">
					<ca:paras>
						<xsl:call-template name="AttTitleBlock"/>		
						<!-- Instert by db  -->		
						<ca:para>
							<xsl:value-of select="$dbdoc/db:widget/db:identification/db:description"/>
						</ca:para>
					</ca:paras>
				</xsl:for-each>
			</ca:description>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="TechRequireBlock">
		<xsl:for-each select="ca2:techrequirements">		
			<ca:techrequirements>
				<xsl:call-template name="ParasBlock"/>	
			</ca:techrequirements>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="ParaBlock">
		<xsl:for-each select="ca2:para">						
			<ca:para>
				<xsl:value-of select="."/>
			</ca:para>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="ParasBlock">
		<xsl:for-each select="ca2:paras">
			<ca:paras>	
                <!-- found in General index slt file -->
				<xsl:call-template name="AttTitleBlock"/>	

				<xsl:call-template name="ParaBlock"/>	
			</ca:paras>
		</xsl:for-each>	
	</xsl:template>
	
	<xsl:template name="DependanciesBlock">
		<xsl:for-each select="ca2:dependancies">
			<ca:dependancies>
				<xsl:for-each select="ca2:dependancy">
					<ca:dependancy>	
                       <!-- found in General index slt file -->
						<xsl:call-template name="AttTypeBlock"/>	
						<xsl:call-template name="AttTitleBlock"/>	
							
						<xsl:call-template name="ParaBlock"/>
					</ca:dependancy>
				</xsl:for-each>		
			</ca:dependancies>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="SharedResourcesBlock">
		<xsl:for-each select="ca2:sharedresources">		
			<ca:sharedresources>
				<xsl:call-template name="ParasBlock"/>			
			</ca:sharedresources>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="EmbeddedHelpBlock">
		<xsl:for-each select="ca2:embeddedhelp">		
			<ca:embeddedhelp>
				<xsl:call-template name="ParasBlock"/>		
			</ca:embeddedhelp>
		</xsl:for-each>
	</xsl:template>	

	<xsl:template name="GuiNotesBlock">
		<xsl:for-each select="ca2:guinotes">
			<ca:guinotes>
				<xsl:for-each select="ca2:guinote">
					<ca:guinote>	
                       <!-- found in General index slt file -->
						<xsl:call-template name="AttTypeBlock"/>	
						<xsl:call-template name="AttTitleBlock"/>	
							
						<xsl:call-template name="ParaBlock"/>
					</ca:guinote>
				</xsl:for-each>		
			</ca:guinotes>
		</xsl:for-each>
	</xsl:template>	

	<xsl:template name="QaNotesBlock">
		<xsl:for-each select="ca2:qanotes">		
			<ca:qanotes>
				<xsl:call-template name="ParasBlock"/>	
			</ca:qanotes>
		</xsl:for-each>
	</xsl:template>	

</xsl:stylesheet>
