<xsl:stylesheet version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
	xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
	xmlns:ca2="http://localhost/chameleon/utils/xml/schemas"
	xmlns:db="http://localhost/chameleon/utils/xml/schemas"
	xmlns:xs="http://www.w3.org/2001/XMLSchema" 
	exclude-result-prefixes="ca ca2 xs">
	<!-- ~~~~~~~~~~~~~~~~~ -->
	<!--FORMELEMENTS SECTION (used by manager index)-->
	<!-- ~~~~~~~~~~~~~~~~~ -->
  <xsl:template name="FormelementsUpdate"> 
	
	<xsl:variable name="dbnform" select="count($dbdoc/db:widget/db:formelements/db:formelement)"/>
   	<xsl:variable name="ca2nform" select="count(ca2:formelements/ca2:formelement)"/>

	<xsl:choose>

		<xsl:when test="$dbnform &lt; $ca2nform">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Form Elements Section ERROR 1 : 
THERE ARE LESS  DB:(IN WIDGET CODE) Form Elements THAN THERE ARE CURRENT CA:(WIDGET DOC)!!! 
(db:<xsl:value-of select=" $dbnform"/> &lt; current ca:<xsl:value-of select=" $ca2nform"/>)
Someone has either manually edited the ca: file and added an form element or 
there has been a change in the widget code. 
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
		</xsl:when>
		<xsl:when test="$dbnform &gt; $ca2nform">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Form Elements Section ERROR 2 : 
THERE ARE MORE DB:(IN WIDGET CODE) Form Elements THAN THERE ARE CURRENT CA:(WIDGET DOC)!!!
(db:<xsl:value-of select=" $dbnform"/> &gt; current ca:<xsl:value-of select=" $ca2nform"/>)
Someone has either manually edited the ca: file and removed form elements or 
there has been a change in the widget code. 
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
		</xsl:when>		
		
	</xsl:choose>
	<ca:formelements>
			<xsl:copy-of select="ca2:formelements/@title"/>
     		<xsl:call-template name="DB_FormNameSearchEval"/>
			<xsl:call-template name="CA_FormNameSearchEval"/>	
	</ca:formelements>
  </xsl:template>
  
  	<xsl:template name="DB_FormNameSearchEval"> 

		<xsl:for-each select="$dbdoc/db:widget/db:formelements">
			<xsl:sort select="db:formelement/db:name"/>
			<xsl:for-each select="db:formelement">
				<xsl:variable name="dbszform" select="db:name"/>
				<xsl:variable name="dbposform" select="position()"/>
				<xsl:variable name="ca2search" select="contains($glob_ca2formelements, $dbszform)"/>
				<xsl:choose>
					<xsl:when test=" string($ca2search) = 'false' ">
					
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Form Elements Section NOTE 1 :{db:formelement/ca:name[<xsl:value-of select="$dbposform"/>] =(<xsl:value-of select="$dbszform"/>)}
Current form element could not be found in the current widget document. This is probably a new formelement in the widget code 
</xsl:comment>
						
						<xsl:call-template name="DefaultFormelementBlock"/>
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>	
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>				
	
	<xsl:template name="CA_FormNameSearchEval">
		<xsl:for-each select="/ca2:widgetdoc/ca2:formelements">
			<xsl:sort select="ca2:formelement/ca2:name"/>
			<xsl:for-each select="ca2:formelement">
				<xsl:variable name="ca2szform" select="ca2:name"/>
				
				
				<xsl:variable name="ca2posform" select="position()"/>
				<xsl:variable name="dbsearch" select="contains($glob_dbformelements, $ca2szform)"/>
				<xsl:choose>
					<xsl:when test=" string($dbsearch) = 'false' ">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Form Element Section WARNING 1 : 
ca:formelement/ca:name[<xsl:value-of select="$ca2posform"/>] =(<xsl:value-of select="$ca2szform"/>)
The Current form element could not be found in widget code.	
The form element name could have been manually modified.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
					</xsl:when>
					<xsl:when test=" string($dbsearch) = 'true' ">
						<xsl:for-each select="$dbdoc/db:widget/db:formelements">	
							<xsl:sort select="db:formelement/db:name"/>
							<xsl:for-each select="db:formelement">
								
								<xsl:variable name="dbszform" select="db:name"/>
								<xsl:variable name="dbposform" select="position()"/>

								<xsl:if test="$ca2posform = $dbposform and $ca2szform != $dbszform">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Form Elements Section WARNING 2 : 
ca:formelement/ca:name[node position(<xsl:value-of select="$ca2posform"/>), value= <xsl:value-of select="$ca2szform"/>] 
		Does NOT Equal 
db:formelement/db:name[node position(<xsl:value-of select="$dbposform"/>), value=<xsl:value-of select="$dbszform"/>] 
This means that there maybe a manual edit done on the widget doc. 
Please make sure all form elements are presented correctly.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
								</xsl:if>
<!-- NO Form element values to be checked since they are manually added -->
							</xsl:for-each>
						</xsl:for-each>		
					</xsl:when>
				</xsl:choose>
				<xsl:copy-of select="/ca2:widgetdoc/ca2:formelements/ca2:formelement[$ca2posform]" />
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>

	<!-- ~~~~~~~~~~~~~~~~~~~~~ -->
	<!--FORMELEMENTS SECTION  (used by default index)-->
	<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~ -->
	<xsl:template name="FormelementsBlock">
		<xsl:for-each select="ca2:formelements">	
			<ca:formelements>
				<xsl:for-each select="$dbdoc/db:widget/db:formelements/db:formelement">		
					<xsl:call-template name="DefaultFormelementBlock"/>
				</xsl:for-each>
			</ca:formelements>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="DefaultFormelementBlock">
		<ca:formelement>
			<ca:name>
				<xsl:value-of select="db:name"/>
			</ca:name>
			<ca:type>
				<ca:value>unknown</ca:value>
			</ca:type>
			<ca:default>
				<ca:value>
				</ca:value>
			</ca:default>	
			<ca:possvalues type="list">
						<ca:minimum>
						</ca:minimum>
						<ca:maximum>
						</ca:maximum>
						<ca:value>
						</ca:value>
			</ca:possvalues>
			<ca:description>
			</ca:description>
		</ca:formelement>						
	</xsl:template>

</xsl:stylesheet>