<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
  xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
  xmlns:ca2="http://localhost/chameleon/utils/xml/schemas"
  xmlns:db="http://localhost/chameleon/utils/xml/schemas"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  exclude-result-prefixes="ca2 xs">
	<!-- ~~~~~~~~~~~~~~~~~ -->
	<!--JS FUNCTIONS SECTION  (used by manager index)-->
	<!-- ~~~~~~~~~~~~~~~~~ -->
	<xsl:template name="JSfunctionsUpdate"> 
  
		<xsl:variable name="dbnjsfunc" select="count($dbdoc/db:widget/db:jsfunctions/db:jsfunction)"/>
		<xsl:variable name="ca2njsfunc" select="count(ca2:jsfunctions/ca2:jsfunction)"/>
		
		<xsl:choose>
			<xsl:when test="$dbnjsfunc &lt; $ca2njsfunc">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Function Section ERROR 1 : 
THERE ARE LESS  DB:(IN WIDGET CODE) JS Function THAN THERE ARE CURRENT CA:(WIDGET DOC)!!! 
(db:<xsl:value-of select=" $dbnjsfunc"/> &lt; current ca:<xsl:value-of select=" $ca2njsfunc"/>)
Someone has either manually edited the ca: file and added an JS Function or 
there has been a change in the widget code. 
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
			</xsl:when>
			<xsl:when test="$dbnjsfunc &gt; $ca2njsfunc">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Function Section ERROR 2 : 
THERE ARE MORE DB:(IN WIDGET CODE) JS FunctionTHAN THERE ARE CURRENT CA:(WIDGET DOC)!!!
(db:<xsl:value-of select=" $dbnjsfunc"/> &gt; current ca:<xsl:value-of select=" $ca2njsfunc"/>)
Someone has either manually edited the ca: file and removed an JS Function or 
there has been a change in the widget code. 
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
			</xsl:when>     
		</xsl:choose>
 
	<ca:jsfunctions>
		<xsl:copy-of select="ca2:jsfunctions/@title"/>
		
		<xsl:call-template name="DB_JSFuncNameSearchEval"/>
		<xsl:call-template name="CA_JSFuncNameSearchEval"/> 
	</ca:jsfunctions>
  </xsl:template>
  
	<!-- ++++++++++++++++++++++++++++++++++++++++++ -->
	<!--             DB Test -->
	<!-- ++++++++++++++++++++++++++++++++++++++++++ -->
	<xsl:template name="DB_JSFuncNameSearchEval"> 
	
		<xsl:for-each select="$dbdoc/db:widget/db:jsfunctions">  
			<xsl:sort select="db:jsfunction/db:name"/>    
			<xsl:for-each select="db:jsfunction">
				<xsl:variable name="dbszjsfunc" select="db:name"/>
				<xsl:variable name="dbposjsfunc" select="position()"/>
				<xsl:variable name="ca2search" select="contains($glob_ca2jsfunctions, $dbszjsfunc)"/>
				<xsl:choose>
					<xsl:when test=" string($ca2search) = 'false' ">
					
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Function Section NOTE 1 :{db:jsfunction/ca:name[<xsl:value-of select="$dbposjsfunc"/>] =(<xsl:value-of select="$dbszjsfunc"/>)}
Current JS Function could not be found in the current widget document. This is probably a new JS Function in the widget code 
</xsl:comment>
<xsl:call-template name="DefaultJSFuncBlock"/>
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>  
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>
   
  <!-- ++++++++++++++++++++++++++++++++++++++++++ -->
  <!--            CA Test -->
  <!-- ++++++++++++++++++++++++++++++++++++++++++ -->  
	<xsl:template name="CA_JSFuncNameSearchEval">
		<xsl:for-each select="/ca2:widgetdoc/ca2:jsfunctions">
			<xsl:sort select="ca2:jsfunction/ca2:name"/>  
			<xsl:for-each select="ca2:jsfunction">
				<xsl:variable name="vca2szjsfunc" select="ca2:name"/>
				<xsl:variable name="ca2posjsfunc" select="position()"/>
				<xsl:variable name="dbsearch" select="contains($glob_dbjsfunctions, $vca2szjsfunc)"/>
				<xsl:choose>
					<xsl:when test=" string($dbsearch) = 'false' ">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Function Section WARNING 1 : 
ca:jsfunction/ca:name[<xsl:value-of select="$ca2posjsfunc"/>] =(<xsl:value-of select="$ca2szjsfunc"/>)
The Current JS Function could not be found in widget code. 
The JS Function name could have been manually modified.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
					</xsl:when>
					<xsl:when test=" string($dbsearch) = 'true' ">
						<xsl:for-each select="$dbdoc/db:widget/db:jsfunctions"> 
							<xsl:sort select="db:jsfunction/db:name"/>  
							<xsl:for-each select="db:jsfunction">
								<xsl:variable name="vdbszjsfunc" select="db:name"/>
								<xsl:variable name="dbposjsfunc" select="position()"/>
								<xsl:if test="$ca2posjsfunc = $dbposjsfunc and $vca2szjsfunc != $vdbszjsfunc">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Function Section WARNING 2 : 
ca:jsfunction/ca:name[node position(<xsl:value-of select="$ca2posjsfunc"/>), value= <xsl:value-of select="$vca2szjsfunc"/>] 
Does NOT Equal 
db:jsfunction/db:name[node position(<xsl:value-of select="$dbposjsfunc"/>), value=<xsl:value-of select="$vdbszjsfunc"/>] 
This means that there maybe a manual edit done on the widget doc. 
Please make sure all JS Function are presented correctly.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
								</xsl:if>
							</xsl:for-each>
						</xsl:for-each>   
					</xsl:when>
				</xsl:choose>
				<ca:jsfunction>
					<xsl:copy-of select="/ca2:widgetdoc/ca2:jsfunctions/ca2:jsfunction[$ca2posjsfunc]/@title"/>					
					<xsl:copy-of select="/ca2:widgetdoc/ca2:jsfunctions/ca2:jsfunction[$ca2posjsfunc]/ca2:name"/>
					<xsl:copy-of select="/ca2:widgetdoc/ca2:jsfunctions/ca2:jsfunction[$ca2posjsfunc]/ca2:scope"/>
					<xsl:copy-of select="/ca2:widgetdoc/ca2:jsfunctions/ca2:jsfunction[$ca2posjsfunc]/ca2:description"/>
					<xsl:call-template name="ParametersUpdate">
						<xsl:with-param name="curpositon" select="$ca2posjsfunc"/>
					</xsl:call-template>
					<xsl:copy-of select="/ca2:widgetdoc/ca2:jsfunctions/ca2:jsfunction[$ca2posjsfunc]/ca2:parameters"/>   
					<xsl:copy-of select="/ca2:widgetdoc/ca2:jsfunctions/ca2:jsfunction[$ca2posjsfunc]/ca2:returnvalue"/>
					<xsl:copy-of select="/ca2:widgetdoc/ca2:jsfunctions/ca2:jsfunction[$ca2posjsfunc]/ca2:triggeredjsapievents"/>
				</ca:jsfunction>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>
	<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~ -->
	<!-- JS FUNCTIONS SECTION  (used by default index)-->
	<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~ -->
	<xsl:template name="JSfunctionsBlock">
		<xsl:for-each select="ca2:jsfunctions"> 
			<ca:jsfunctions>
				<xsl:for-each select="$dbdoc/db:widget/db:jsfunctions/db:jsfunction">     
					<ca:jsfunction>
						<ca:name>
							<xsl:value-of select="db:name"/>
						</ca:name>
						<ca:scope>
							<ca:value>Private</ca:value>
						</ca:scope> 
						<ca:description>
						</ca:description>
						<xsl:call-template name="ParametersBlock"/>
						<xsl:call-template name="ReturnValueBlock"/>
						<xsl:call-template name="JSAPITriggeredEventBlock"/>
					</ca:jsfunction>          
				</xsl:for-each>
			</ca:jsfunctions>
		</xsl:for-each>
	</xsl:template>

  <xsl:template name="DefaultJSFuncBlock">
		<ca:jsfunction>
			<ca:name>
				<xsl:value-of select="db:name"/>
			</ca:name>
			<ca:scope>
				<ca:value>Private</ca:value>
			</ca:scope> 
			<ca:description>
			</ca:description>
			<xsl:call-template name="ParametersBlock"/>
			<xsl:call-template name="ReturnValueBlock"/>
			<xsl:call-template name="JSAPITriggeredEventBlock"/>
		</ca:jsfunction>    
  </xsl:template>
  
  <xsl:template name="ReturnValueBlock">
    <ca:returnvalues>
      <ca:returnvalue>
        <ca:type>
          <ca:value>unknown</ca:value>
        </ca:type>
        <ca:default>
          <ca:value></ca:value>
        </ca:default>
        <ca:possvalues type="list" >
          <ca:value></ca:value>
        </ca:possvalues>  
        <ca:description></ca:description>
      </ca:returnvalue>
      </ca:returnvalues>
    </xsl:template>
    
  <xsl:template name="JSAPITriggeredEventBlock">
    <ca:triggeredjsapievents>
      <ca:triggeredjsapievent>
        <ca:name></ca:name>
        <ca:description></ca:description>
      </ca:triggeredjsapievent>
    </ca:triggeredjsapievents>    
    </xsl:template>
  
</xsl:stylesheet>



