<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
  xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
  xmlns:ca2="http://localhost/chameleon/utils/xml/schemas"
  xmlns:db="http://localhost/chameleon/utils/xml/schemas"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  exclude-result-prefixes="ca2 xs">
  <!-- ~~~~~~~~~~~~~~~~~ -->
  <!--JS VARIABLESSECTION  (used by manager index)-->
  <!-- ~~~~~~~~~~~~~~~~~ -->
  <xsl:template name="JSvariablesUpdate"> 
  
  <xsl:variable name="dbnjsvar" select="count($dbdoc/db:widget/db:jsvariables/db:jsvariable)"/>
    <xsl:variable name="ca2njsvar" select="count(ca2:jsvariables/ca2:jsvariable)"/>

  <xsl:choose>

    <xsl:when test="$dbnjsvar &lt; $ca2njsvar">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Variables Section ERROR 1 : 
THERE ARE LESS  DB:(IN WIDGET CODE) JS Variables THAN THERE ARE CURRENT CA:(WIDGET DOC)!!! 
(db:<xsl:value-of select=" $dbnjsvar"/> &lt; current ca:<xsl:value-of select=" $ca2njsvar"/>)
Someone has either manually edited the ca: file and added an JS Variable or 
there has been a change in the widget code. 
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
    </xsl:when>
    <xsl:when test="$dbnjsvar &gt; $ca2njsvar">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Variables Section ERROR 2 : 
THERE ARE MORE DB:(IN WIDGET CODE) JS VariablesTHAN THERE ARE CURRENT CA:(WIDGET DOC)!!!
(db:<xsl:value-of select=" $dbnjsvar"/> &gt; current ca:<xsl:value-of select=" $ca2njsvar"/>)
Someone has either manually edited the ca: file and removed JS Variable or 
there has been a change in the widget code. 
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
    </xsl:when>   
    
  </xsl:choose>
  <ca:jsvariables>
      <xsl:copy-of select="ca2:jsvariables/@title"/>
        <xsl:call-template name="DB_JSVarNameSearchEval"/>
      <xsl:call-template name="CA_JSVarNameSearchEval"/>  
  </ca:jsvariables>
  </xsl:template>
  
    <xsl:template name="DB_JSVarNameSearchEval"> 

    <xsl:for-each select="$dbdoc/db:widget/db:jsvariables">
      <xsl:sort select="db:jsvariable/db:name"/>
      <xsl:for-each select="db:jsvariable">
        <xsl:variable name="dbszjsvar" select="db:name"/>
        <xsl:variable name="dbposjsvar" select="position()"/>
        <xsl:variable name="ca2search" select="contains($glob_ca2jsvariables, $dbszjsvar)"/>
        <xsl:choose>
          <xsl:when test=" string($ca2search) = 'false' ">
          
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Variables Section NOTE 1 :{db:jsvariable/ca:name[<xsl:value-of select="$dbposjsvar"/>] =(<xsl:value-of select="$dbszjsvar"/>)}
Current JS variables could not be found in the current widget document. This is probably a new JS Variables in the widget code 
</xsl:comment>
            
            <xsl:call-template name="DefaultJSvariablesBlock"/>
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>  
          </xsl:when>
        </xsl:choose>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:template>       
  
  <xsl:template name="CA_JSVarNameSearchEval">
    <xsl:for-each select="/ca2:widgetdoc/ca2:jsvariables">
      <xsl:sort select="ca2:jsvariable/ca2:name"/>
      <xsl:for-each select="ca2:jsvariable">
        <xsl:variable name="ca2szjsvar" select="ca2:name"/>
        
        
        <xsl:variable name="ca2posjsvar" select="position()"/>
        <xsl:variable name="dbsearch" select="contains($glob_dbjsvariables, $ca2szjsvar)"/>
        <xsl:choose>
          <xsl:when test=" string($dbsearch) = 'false' ">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Variables Section WARNING 1 : 
ca:jsvariable/ca:name[<xsl:value-of select="$ca2posjsvar"/>] =(<xsl:value-of select="$ca2szjsvar"/>)
The Current JS Variable could not be found in widget code.  
The JS Variable name could have been manually modified.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
          </xsl:when>
          <xsl:when test=" string($dbsearch) = 'true' ">
            <xsl:for-each select="$dbdoc/db:widget/db:jsvariables"> 
              <xsl:sort select="db:jsvariable/db:name"/>
              <xsl:for-each select="db:jsvariable">
                
                <xsl:variable name="dbszjsvar" select="db:name"/>
                <xsl:variable name="dbposjsvar" select="position()"/>

                <xsl:if test="$ca2posjsvar = $dbposjsvar and $ca2szjsvar != $dbszjsvar">
<xsl:comment>
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
JS Variables Section WARNING 2 : 
ca:jsvariable/ca:name[node position(<xsl:value-of select="$ca2posjsvar"/>), value= <xsl:value-of select="$ca2szjsvar"/>] 
    Does NOT Equal 
db:jsvariable/db:name[node position(<xsl:value-of select="$dbposjsvar"/>), value=<xsl:value-of select="$dbszjsvar"/>] 
This means that there maybe a manual edit done on the widget doc. 
Please make sure all JS Variable are presented correctly.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
</xsl:comment>
                </xsl:if>
<!-- NO JavaScript variables values to be checked since they are manually added -->
              </xsl:for-each>
            </xsl:for-each>   
          </xsl:when>
        </xsl:choose>
        <xsl:copy-of select="/ca2:widgetdoc/ca2:jsvariables/ca2:jsvariable[$ca2posjsvar]" />
      </xsl:for-each>
    </xsl:for-each>
  </xsl:template>

  <!-- ~~~~~~~~~~~~~~~~~~~~~~~~~ -->
  <!-- JS VARIABLES SECTION  (used by default index)-->
  <!-- ~~~~~~~~~~~~~~~~~~~~~~~~~ -->
  <xsl:template name="JSvariablesBlock">
    <xsl:for-each select="ca2:jsvariables"> 
      <ca:jsvariables>
        <xsl:for-each select="$dbdoc/db:widget/db:jsvariables/db:jsvariable">     
          <xsl:call-template name="DefaultJSvariablesBlock"/>
        </xsl:for-each>
      </ca:jsvariables>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="DefaultJSvariablesBlock">
    <ca:jsvariable>
      <ca:name>
        <xsl:value-of select="db:name"/>
      </ca:name>
      <ca:type>
        <ca:value>unknown</ca:value>
      </ca:type>
      <ca:scope>
        <ca:value>Private</ca:value>
      </ca:scope>           
      <ca:default>
        <ca:value>
        </ca:value>
      </ca:default> 
      <ca:possvalues type="list">
            <ca:minimum>
            </ca:minimum>
            <ca:maximum>
            </ca:maximum>
            <ca:value>
            </ca:value>
      </ca:possvalues>
      <ca:description>
      </ca:description>
    </ca:jsvariable>            
  </xsl:template>       

</xsl:stylesheet>