<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:fn="http://www.w3.org/2004/07/xpath-functions" 
  xmlns:xdt="http://www.w3.org/2004/07/xpath-datatypes" 
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <xsl:output version="1.0" encoding="utf-8" omit-xml-declaration="no" indent="no" media-type="text/html"/>

  <!-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx --> 
  <!-- Widget Attribute  Section --> 
  <xsl:template name="WidgetTagSection">
    <xsl:call-template name="AttributesTitle"/>
    <xsl:choose>
		<xsl:when test="ca:widgettag/ca:attribute/ca:name != &quot;&quot;">
			<xsl:call-template name="WidgetTagList"/>
		</xsl:when>
		<xsl:otherwise>
			<div class="Section">
				<div class="AttributeName">
					None Found.
				</div>
			</div>
		</xsl:otherwise>
	</xsl:choose>
  </xsl:template>   
  
  
  <!-- Main Title For Description Section -->
  <xsl:template name="AttributesTitle">
    <div class="MainHeading">         
      <a>
        <xsl:attribute name="name"><xsl:text disable-output-escaping="yes">#_attributes</xsl:text></xsl:attribute>
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgettag/@title" />
      </a>
    </div>  
  </xsl:template>
  <!-- CWC2 Widget tag Display Option with Attributes -->
  <xsl:template name="WidgetTagList">
    <xsl:for-each select="ca:widgettag" >
      <!-- ++++++++++++++++++++++++++++ -->
      <!--Choose if  Widget Tag name Existsl  -->
      <xsl:choose>
        <xsl:when test="ca:name = &quot;&lt;CWC2 /&gt;&quot;" >
          <div class="Section">
            <xsl:for-each select="ca:name">
              <div class="AttributeName">
                <xsl:apply-templates/>
              </div>
            </xsl:for-each>
            <xsl:call-template name="WidgetTagRequired"/>
            <xsl:for-each select="ca:description">
              <div class="SectionContent">
                <xsl:apply-templates/>
              </div>
            </xsl:for-each> 
            <div class="SubSection">
              <xsl:call-template name="WidgetAttributes"/>
            </div>
          </div>
        </xsl:when>
        <xsl:otherwise>
          <div class="Section">
            <xsl:call-template name="WidgetAttributes"/>
            <xsl:call-template name="WidgetSubTag"/>
          </div>
        </xsl:otherwise>
      </xsl:choose>     
    </xsl:for-each>
  </xsl:template>
  <!-- Displays the Widget Tag info as Required Tag -->
  <xsl:template name="WidgetTagRequired">
    <xsl:for-each select="ca:type">
      <div class="AttributeRequired">
        <xsl:call-template name="TitleValueDescription"/>
      </div>
    </xsl:for-each>
    <xsl:for-each select="ca:class">
      <div class="AttributeRequired">
        <xsl:call-template name="TitleValueDescription"/>
      </div>
    </xsl:for-each>
  </xsl:template>
   <!-- Display Widget Attributes -->
  <xsl:template name="WidgetAttributes">
    <xsl:for-each select="ca:attribute">
      <xsl:sort  select="ca:name "/>
      <xsl:call-template name="WidgetVariableDefinition"/>  
    </xsl:for-each>
  </xsl:template> 
  <!-- Widget SubTag -->
  <xsl:template name="WidgetSubTag">
    <xsl:for-each select="ca:subtag">
      <xsl:sort  select="ca:name "/>
      <xsl:call-template name="WidgetVariableDefinition"/>  
      <div class="SubSection">
        <xsl:call-template name="WidgetAttributes"/>      
      </div>  
    </xsl:for-each>       
  </xsl:template> 
   <!-- Outputs the Value and Description of that value -->
  <xsl:template name="TitleValueDescription">
    <xsl:for-each select="@title">          
      <xsl:value-of select="." />: </xsl:for-each>
    <xsl:for-each select="ca:value">          
      <xsl:apply-templates/>
    </xsl:for-each>
    <xsl:for-each select="ca:description">          
      - <xsl:apply-templates/>
    </xsl:for-each> 
  </xsl:template> 
   <!-- Widget Attribute, SubTag, or SubTag Attributes -->
  <xsl:template name="WidgetVariableDefinition">
      <xsl:for-each select="ca:name">
        <div class="AttributeName">
          <xsl:apply-templates/>        
        </div>
      </xsl:for-each>
          
      <xsl:call-template name="WidgetAttributesConditions"/>  
      <xsl:for-each select="ca:description">
        <div class="SectionContent">
          <xsl:apply-templates/>
        </div>
      </xsl:for-each> 
  </xsl:template>
  <!-- Attribute Value Display differences -->
  <xsl:template name="WidgetAttributesConditions">
     <!-- +++++++++++++++++++++++++++++++++++ -->
     <!-- if Class of Attribute is defined as "Required" -->
    <xsl:if test="ca:class/ca:value = &quot;Required&quot;">
      <div class="AttributeRequired">
        <xsl:call-template name="AttributeType"/>
      </div>
      <div class="AttributeRequired">
        <xsl:call-template name="AttributeClass"/>
      </div>
      <div class="AttributeRequired">
        <xsl:call-template name="AttributeDefault"/>
      </div>    
      <div class="AttributeRequired">
        <xsl:call-template name="AttributePossValue"/>
      </div>      
    </xsl:if>     
     <!-- ++++++++++++++++++++++++++++++++++++++ -->
     <!-- if Class of Attribute is defined as "Widget Only" -->
    <xsl:if test="ca:class/ca:value =&quot;Widget Only&quot;">
      <div class="AttributeWidgetOnly">
        <xsl:call-template name="AttributeType"/>
      </div>
      <div class="AttributeWidgetOnly">
        <xsl:call-template name="AttributeClass"/>
      </div>
      <div class="AttributeWidgetOnly">
        <xsl:call-template name="AttributeDefault"/>
      </div>    
      <div class="AttributeWidgetOnly">
        <xsl:call-template name="AttributePossValue"/>
      </div>      
    </xsl:if>     
     <!-- ++++++++++++++++++++++++++++++++++++++++++++++++++ -->
     <!-- if Class of Attribute is defined as "Widget & Shared Resource" -->
    <xsl:if test="ca:class/ca:value =&quot;Widget &amp; Shared Resource&quot;">
      <div class="AttributeShared">
        <xsl:call-template name="AttributeType"/>
      </div>
      <div class="AttributeShared">
        <xsl:call-template name="AttributeClass"/>
      </div>
      <div class="AttributeShared">
        <xsl:call-template name="AttributeDefault"/>
      </div>    
      <div class="AttributeShared">
        <xsl:call-template name="AttributePossValue"/>
      </div>      
    </xsl:if> 
  </xsl:template> 
   <!-- Attribute Type Display -->
  <xsl:template name="AttributeType">
    <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgettag/ca:attribute/ca:type/@title" />: 
    <xsl:for-each select="ca:type">
      <xsl:call-template name="ValueDescription"/>  
    </xsl:for-each>
  </xsl:template>
   <!-- Attribute Class Display -->
  <xsl:template name="AttributeClass">
    <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgettag/ca:attribute/ca:class/@title" />: 
    <xsl:for-each select="ca:class">  
      <xsl:call-template name="ValueDescription"/>  
    </xsl:for-each>
  </xsl:template>
   <!-- Attribute Default Display --> 
  <xsl:template name="AttributeDefault">
    <xsl:for-each select="ca:default">      
      <xsl:if test="ca:value!=&quot;&quot;">   
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgettag/ca:attribute/ca:default/@title" />: 
        <xsl:call-template name="ValueDescription"/>  
      </xsl:if>
    </xsl:for-each> 
  </xsl:template>
     <!-- Attribute Discription Display -->
  <xsl:template name="ValueDescription">
    <xsl:for-each select="ca:value">          
      <xsl:apply-templates/>
    </xsl:for-each>
    <xsl:if test="ca:description  != &quot;&quot;"> 
      <xsl:for-each select="ca:description">          
         - <xsl:apply-templates/>
      </xsl:for-each>     
    </xsl:if>
  </xsl:template> 
  <!-- Possible Values for Widget Feature -->
  <xsl:template name="AttributePossValue">
    <xsl:if test="ca:possvalues/@type != &quot;&quot;">  
      <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgettag/ca:attribute/ca:possvalues/@title" />:
      <xsl:call-template name="PossValueCondition"/>    
    </xsl:if> 
  </xsl:template> 
  <!-- List or Range Condition of Possible Values --> 
  <xsl:template name="PossValueCondition">
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ -->
    <!-- If PossValue Attribute Type = "Range" Define Value String As Such -->
    <xsl:if test="ca:possvalues/@type  = &quot;range&quot;"> 
      <xsl:choose >
        <xsl:when test="ca:possvalues/ca:minimum != &quot;&quot;">
            | <xsl:value-of select="ca:possvalues/ca:minimum" /> &lt; 
        </xsl:when>
        <xsl:otherwise>
          | # &lt; 
        </xsl:otherwise>    
      </xsl:choose>     
      <xsl:choose >
        <xsl:when test="ca:possvalues/ca:maximum != &quot;&quot;">
           <xsl:value-of select="ca:possvalues/ca:maximum" /> |
        </xsl:when>
        <xsl:otherwise>
          # |
        </xsl:otherwise>    
      </xsl:choose> 
    </xsl:if>
    <!-- If PossValue Attribute Type = "List" Define Value String As Such -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ -->
    <xsl:if test="ca:possvalues/@type  = &quot;list&quot;">
      <xsl:choose >
        <xsl:when test="ca:possvalues/ca:value != &quot;&quot;">
          <xsl:for-each select="ca:possvalues"> | <xsl:for-each select="ca:value">
            <xsl:apply-templates/> | </xsl:for-each>
          </xsl:for-each>
        </xsl:when>
        <xsl:otherwise>
          | - |
        </xsl:otherwise>    
      </xsl:choose>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet> 