<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:fn="http://www.w3.org/2004/07/xpath-functions" 
  xmlns:xdt="http://www.w3.org/2004/07/xpath-datatypes" 
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <xsl:output version="1.0" encoding="utf-8" omit-xml-declaration="no" indent="no" media-type="text/html"/>

  <!-- Main Title For Description Section -->
  <xsl:template name="DescriptionsTitle">
      <div class="MainHeading">       
        <a>
          <xsl:attribute name="name"><xsl:text disable-output-escaping="yes">#_descriptions</xsl:text></xsl:attribute>
            <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/@title" />
        </a>
      </div>  
  </xsl:template>
  <!-- Paragraph Description for General,  --> 
    <!-- Shared Resource, Tech Requirements, --> 
    <!-- QA Notes and  User Help Description Sections --> 
  <xsl:template name="TitlePara">     
    <xsl:for-each select="ca:paras">
      <div class="SubSection">  
        <xsl:for-each select="@title">
          <div class="SubSectionHeading2">   
            <xsl:value-of select="." />
          </div>    
        </xsl:for-each> 
        <xsl:for-each select="ca:para">
          <div class="SubSectionContent2">      
            <xsl:apply-templates />
          </div>    
        </xsl:for-each>
      </div>    
    </xsl:for-each>
  </xsl:template>
  <!-- Paragraph Description for Dependancy and GuiNotes --> 
  <xsl:template name="TypeTitlePara"> 
    <div class="SubSectionHeading2">
      <xsl:for-each select="@type">
        <xsl:value-of select="." />
      </xsl:for-each>: <xsl:for-each select="@title">
        <xsl:value-of select="." />
      </xsl:for-each>
    </div>     
    <xsl:for-each select="ca:para">
      <div class="SubSectionContent2">          
        <xsl:apply-templates />
      </div>    
    </xsl:for-each>
  </xsl:template>   
      
  <!-- General Widget Description Section -->    
  <xsl:template name="GeneralDescription"> 
    <xsl:if test="ca:description/ca:paras/ca:para != &quot;&quot;">   
      <div class="Section">    
        <div class="SectionHeading">  
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/ca:description/@title"/>
        </div>  
        <xsl:for-each select="ca:description">
          <xsl:call-template name="TitlePara"/> 
        </xsl:for-each> 
      </div>
    </xsl:if>   
  </xsl:template>
  <!-- Shared Resource Section -->    
  <xsl:template name="SharedResources">      
    <xsl:if test="ca:sharedresources/ca:paras/@title != &quot;&quot;" >     
      <div class="Section">       
        <div class="SectionHeading">            
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/ca:sharedresources/@title"/>
        </div>
        <xsl:for-each select="ca:sharedresources">
          <xsl:call-template name="TitlePara"/> 
        </xsl:for-each> 
      </div>    
    </xsl:if> 
  </xsl:template> 
  <!-- Embedded Help Section -->         
  <xsl:template name="UserHelp">    
    <xsl:if test="ca:embeddedhelp/ca:paras/ca:para != &quot;&quot;">  
      <div class="Section">       
        <div class="SectionHeading">            
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/ca:embeddedhelp/@title"/>
        </div>
        <div class="SectionIntro">            
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/ca:embeddedhelp/ca:intro"/>
        </div>        
        <xsl:for-each select="ca:embeddedhelp">
          <xsl:call-template name="TitlePara"/> 
        </xsl:for-each> 
      </div>     
    </xsl:if>
  </xsl:template>
  <!--  Technical Requirements Section -->              
  <xsl:template name="TechRequirements">   
    <xsl:if test="ca:techrequirements/ca:paras/@title != &quot;&quot;">                        
      <div class="Section">
        <div class="SectionHeading">            
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/ca:techrequirements/@title"/>
        </div>
        <xsl:for-each select="ca:techrequirements">
          <xsl:call-template name="TitlePara"/> 
        </xsl:for-each> 
      </div>          
    </xsl:if>
  </xsl:template>
  <!--  QA notes Section --> 
  <xsl:template name="QANotes">     
    <xsl:if test="ca:qanotes/ca:paras/@title != &quot;&quot;">         
      <div class="Section">     
        <div class="SectionHeading">            
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/ca:qanotes/@title"/>
        </div>
        <xsl:for-each select="ca:qanotes">
          <xsl:call-template name="TitlePara"/> 
        </xsl:for-each> 
      </div>          
    </xsl:if> 
  </xsl:template>   
  <!--  Interface Functionality Section -->        
  <xsl:template name="GUINotes">        
    <xsl:if test="ca:guinotes/ca:guinote/@title != &quot;&quot;">      
      <div class="Section"> 
        <div class="SectionHeading">            
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/ca:guinotes/@title"/>
        </div>
        <xsl:for-each select="ca:guinotes">
          <xsl:for-each select="ca:guinote">
             <div class="SubSection">  
              <xsl:call-template name="TypeTitlePara"/>   
            </div>  
          </xsl:for-each>
        </xsl:for-each>                   
      </div>    
    </xsl:if>
  </xsl:template> 
  <!--  Interface Functionality Section -->        
  <xsl:template name="Dependancy">           
    <xsl:if test="ca:dependancies/ca:dependancy/ca:para != &quot;&quot;">       
      <div class="Section">  
        <div class="SectionHeading">         
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:descriptions/ca:dependancies/@title"/>
        </div>
        <xsl:for-each select="ca:dependancies">
          <xsl:for-each select="ca:dependancy">
             <div class="SubSection">  
              <xsl:call-template name="TypeTitlePara"/>   
            </div>  
          </xsl:for-each>
        </xsl:for-each>                              
      </div>          
    </xsl:if> 
  </xsl:template>
</xsl:stylesheet> 