<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:fn="http://www.w3.org/2004/07/xpath-functions" 
  xmlns:xdt="http://www.w3.org/2004/07/xpath-datatypes" 
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <xsl:output version="1.0" encoding="utf-8" omit-xml-declaration="no" indent="no" media-type="text/html"/>

  <!-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx --> 
  <!-- Error Section -->
  <xsl:template name="ErrorsSection">
    <xsl:call-template name="ErrorsTitle"/>
    <xsl:choose>
		<xsl:when test="ca:errors/ca:error/ca:severity/ca:value != &quot;none&quot;">
			<xsl:call-template name="ErrorsList"/>
		</xsl:when>
		<xsl:otherwise>
			<div class="Section">
				<div class="ErrorName">
					None Recorded.
				</div>
			</div>
		</xsl:otherwise>
	</xsl:choose>
  </xsl:template>   
  
  <!-- Main Title For Description Section -->
  <xsl:template name="ErrorsTitle">
      <div class="MainHeading">       
        <a>
          <xsl:attribute name="name"><xsl:text disable-output-escaping="yes">#_errors</xsl:text></xsl:attribute>
            <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:errors/@title" />
        </a>
      </div>  
  </xsl:template>
      
  <!-- Errors List -->
  <xsl:template name="ErrorsList">
    <xsl:choose>
      <xsl:when test="ca:errors/ca:error/@title != &quot;&quot;">
        <xsl:for-each select="ca:errors"> 
          <div class="Section">
            <xsl:for-each select="ca:error">  
              <!-- Error Title -->
                <div class="ErrorName">
                  <xsl:value-of select="@title"/>
                </div>
              <!-- Error Definition -->
              <div class="SubSection">
                <xsl:call-template name="ErrorType"/> 
                <xsl:call-template name="ErrorSeverity"/>             
                <xsl:call-template name="ErrorMessage"/>
                <xsl:call-template name="ErrorMeaning"/>
                <xsl:call-template name="ErrorSolution"/> 
              </div>
            </xsl:for-each>       
          </div>
        </xsl:for-each>       
      </xsl:when>
      <xsl:otherwise>
        <div class="Section">
          <!-- Error Title -->
          <div class="SectionHeading">
            Not Available.
          </div>
        </div>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>     
      
    
  <!-- General Widget Description Section -->    
  <xsl:template name="ErrorType"> 
    <div class="SectionContent2">
      <span class="DefinitionTitle">  
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:errors/ca:error/ca:type/@title"/>:
      </span> 
      <xsl:for-each select="ca:type">
        <xsl:call-template name="ValueDescription"/>  
      </xsl:for-each> 
    </div>
  </xsl:template>

  <!--Error Severity  -->
  <xsl:template name="ErrorSeverity">
    <div class="SectionContent2">
      <span class="DefinitionTitle">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:errors/ca:error/ca:severity/@title"/>:
      </span>
      <xsl:for-each select="ca:severity">
        <xsl:call-template name="ValueDescription"/>  
      </xsl:for-each>
    </div>  
  </xsl:template> 
  
  <!-- Error Message  -->
  <xsl:template name="ErrorMessage">
    <div class="DefinitionTitle">
      <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:errors/ca:error/ca:message/@title"/>:
    </div>
    <xsl:for-each select="ca:message">
      <xsl:call-template name="TitlePara"/>     
    </xsl:for-each>
  </xsl:template> 
  
  <!-- Error Meaning  -->
  <xsl:template name="ErrorMeaning">
    <div class="DefinitionTitle">
      <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:errors/ca:error/ca:meaning/@title"/>:
    </div>
    <xsl:for-each select="ca:meaning">
      <xsl:call-template name="TitlePara"/> 
    </xsl:for-each>
  </xsl:template> 

  <!-- Error Solution  -->
  <xsl:template name="ErrorSolution">
    <div class="DefinitionTitle">
      <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:errors/ca:error/ca:solution/@title"/>:
    </div>  
    <xsl:for-each select="ca:solution">
      <xsl:call-template name="TitlePara"/>     
    </xsl:for-each>
  </xsl:template>
  
</xsl:stylesheet> 