<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:fn="http://www.w3.org/2004/07/xpath-functions" 
  xmlns:xdt="http://www.w3.org/2004/07/xpath-datatypes" 
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <xsl:output version="1.0" encoding="utf-8" omit-xml-declaration="no" indent="no" media-type="text/html"/>
  
  <!-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx --> 
  <!-- JS Function Section -->
	<xsl:template name="JSfunctionsSection">
		<xsl:call-template name="JSfunctionsTitle"/>
		<xsl:choose>
			<xsl:when test="ca:jsfunctions/ca:jsfunction/ca:name != &quot;&quot;">
				 <xsl:call-template name="JSfunctionsList"/>     
			</xsl:when>
			<xsl:otherwise>
				<div class="Section">
					<div class="JSfunctionName">
						None Found.
					</div>
				</div>
			</xsl:otherwise>
		</xsl:choose>      
	</xsl:template>    
  
  <!-- Main Title For Description Section -->
  <xsl:template name="JSfunctionsTitle">
    <div class="MainHeading">         
      <a>
        <xsl:attribute name="name"><xsl:text disable-output-escaping="yes">#_jsfunctions</xsl:text></xsl:attribute>
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/@title" />
      </a>
    </div>  
  </xsl:template>
  <!-- JS Function List -->
  <xsl:template name="JSfunctionsList">
    <xsl:for-each select="ca:jsfunctions">  
      <xsl:sort  select="ca:name "/>
      <div class="Section">
        <xsl:for-each select="ca:jsfunction"> 
          <!-- JS Function Name -->
          <xsl:for-each select="ca:name">
            <div class="JSfunctionName">
              <xsl:apply-templates/>
            </div>
          </xsl:for-each>
          <!-- JS Function  Definition -->
          <div class="SubSection">
            <xsl:call-template name="JSfunctionScope"/>
            <xsl:for-each select="ca:description">
              <div class="SectionContent2">
                <xsl:apply-templates/>
              </div>
            </xsl:for-each>                         
            <xsl:call-template name="FeatureParameters"/>
            <xsl:call-template name="FunctionReturnValues"/>  
            <xsl:call-template name="TriggeredEvents"/> 
          </div>
        </xsl:for-each>       
      </div>
    </xsl:for-each>       
  </xsl:template>
  <!--JS Function Scope -->
  <xsl:template name="JSfunctionScope">
    <div class="SectionContent2">
      <span class="DefinitionTitle">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:scope/@title" />: 
      </span>
      <xsl:for-each select="ca:scope">
        <xsl:call-template name="ValueDescription"/>  
      </xsl:for-each>
    </div>  
  </xsl:template> 
  
  <!-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx -->
  <!--Parameters Section -->
  <xsl:template name="FeatureParameters">
    <xsl:if test="ca:parameters/ca:parameter/ca:name != &quot;&quot;">  
      <xsl:for-each select="ca:parameters">
        <!-- Parameter Section Title -->
        <xsl:call-template name="ParametersTitle"/>
        <xsl:call-template name="ParametersList"/>
      </xsl:for-each>
    </xsl:if>
  </xsl:template> 
  
  <!-- Parameter Section Title -->
  <xsl:template name="ParametersTitle">
    <div class="SectionHeading">
      <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:parameters/@title" />: 
    </div>
  </xsl:template>

  <xsl:template name="ParametersList">
    <xsl:for-each select="ca:parameter">  
      <xsl:sort  select="ca:name "/>
      <!-- Formelement Definition -->
      <div class="SubSection">
      <!-- Parameter Name -->
        <xsl:for-each select="ca:name">
          <div class="ParameterName">
            <xsl:apply-templates/>
          </div>
        </xsl:for-each> 
        <div class="SubSection">    
          <xsl:call-template name="ParameterType"/> 
          <xsl:call-template name="ParameterRequired"/> 
          <xsl:call-template name="ParameterDefault"/>
          <xsl:call-template name="ParameterPossValues"/> 
          <xsl:for-each select="ca:description">
            <div class="SectionContent2">
              <xsl:apply-templates/>
            </div>
          </xsl:for-each>
        </div>
      </div>          
    </xsl:for-each>
  </xsl:template>
  
  <!-- Parameter Value Type -->
  <xsl:template name="ParameterType">
    <div class="SectionContent2">
      <span class="DefinitionTitle">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:parameters/ca:parameter/ca:type/@title" />: 
      </span>
      <xsl:for-each select="ca:type">
        <xsl:call-template name="ValueDescription"/>  
      </xsl:for-each>
    </div>  
  </xsl:template> 
  <!-- Parameter Value Required -->
  <xsl:template name="ParameterRequired">
    <div class="SectionContent2">
      <span class="DefinitionTitle">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:parameters/ca:parameter/ca:required/@title" />: 
      </span>
      <xsl:for-each select="ca:required">
        <xsl:call-template name="ValueDescription"/>  
      </xsl:for-each>
    </div>  
  </xsl:template>   
  <!-- Parameter Default Value -->
  <xsl:template name="ParameterDefault">
      <xsl:for-each select="ca:default">      
        <xsl:if test="ca:value!=&quot;&quot;">
          <div class="SectionContent2">       
            <span class="DefinitionTitle">   
            <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:parameters/ca:parameter/ca:default/@title" />: 
            </span>
            <xsl:call-template name="ValueDescription"/>  
          </div>
        </xsl:if>
      </xsl:for-each> 
  </xsl:template> 
  <!-- Possible Values for Parameter -->
  <xsl:template name="ParameterPossValues">
    <xsl:if test="ca:possvalues/@type != &quot;&quot;">  
      <div class="SectionContent2">   
        <span class="DefinitionTitle">      
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:parameters/ca:parameter/ca:possvalues/@title" />: 
        </span> 
        <xsl:call-template name="PossValueCondition"/>  
      </div>
    </xsl:if> 
  </xsl:template> 
  <!-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx -->
  <!--Function Return Values Section -->
  <xsl:template name="FunctionReturnValues">
    <xsl:if test="ca:returnvalues/ca:returnvalue/ca:type != &quot;&quot;">
      <xsl:for-each select="ca:returnvalues">
          <!-- Parameter Section Title -->
          <xsl:call-template name="ReturnValuesTitle"/>
          <xsl:call-template name="ReturnValuesList"/>
      </xsl:for-each>
    </xsl:if>
  </xsl:template> 
  <!-- Function ReturnValues Title-->
  <xsl:template name="ReturnValuesTitle">
    <div class="SectionHeading">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:returnvalues/@title" />: 
    </div>
  </xsl:template>
   <!-- Return Value List [1] -->
  <xsl:template name="ReturnValuesList">
    <xsl:for-each select="ca:returnvalue">
      <xsl:sort  select="ca:name "/>
      <!-- Formelement Definition -->
      <div class="SubSection">
          <xsl:call-template name="ReturnValueType"/> 
          <xsl:call-template name="ReturnValueDefault"/>
          <xsl:call-template name="ReturnPossValues"/>  
          <xsl:for-each select="ca:description">
            <div class="SectionContent2">
              <xsl:apply-templates/>
            </div>
          </xsl:for-each>
      </div>      
    </xsl:for-each>
  </xsl:template>   
  <!-- Return  Value Type -->
  <xsl:template name="ReturnValueType">
    <div class="SectionContent2">
      <span class="DefinitionTitle">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:returnvalues/ca:returnvalue/ca:type/@title" />: 
      </span>
      <xsl:for-each select="ca:type">
        <xsl:call-template name="ValueDescription"/>  
      </xsl:for-each>
    </div>  
  </xsl:template>   
  <!-- Parameter Default Value -->
  <xsl:template name="ReturnValueDefault">
      <xsl:for-each select="ca:default">      
        <xsl:if test="ca:value!=&quot;&quot;">
          <div class="SectionContent2">       
            <span class="DefinitionTitle">   
            <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:returnvalues/ca:returnvalue/ca:default/@title" />: 
            </span>
            <xsl:call-template name="ValueDescription"/>  
          </div>
        </xsl:if>
      </xsl:for-each> 
  </xsl:template> 
  <!-- Possible Values for Parameter -->
  <xsl:template name="ReturnPossValues">
    <xsl:if test="ca:possvalues/@type != &quot;&quot;">  
      <div class="SectionContent2">   
        <span class="DefinitionTitle">      
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:returnvalues/ca:returnvalue/ca:possvalues/@title" />: 
        </span> 
        <xsl:call-template name="PossValueCondition"/>  
      </div>
    </xsl:if> 
  </xsl:template> 
  
  <!-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx -->
  <!-- Api Triggered Events  Section -->  
  <xsl:template name="TriggeredEvents">
    <xsl:if test="ca:triggeredjsapievents/ca:triggeredjsapievent/ca:name != &quot;&quot;">
      <xsl:for-each select="ca:triggeredjsapievents">
          <!-- Parameter Section Title -->
          <xsl:call-template name="TriggeredEventsTitle"/>
          <xsl:call-template name="TriggeredEventsList"/>
      </xsl:for-each>
    </xsl:if>
  </xsl:template> 
  <!-- Api Triggered Events Title-->
  <xsl:template name="TriggeredEventsTitle">
    <div class="SectionHeading">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:jsfunctions/ca:jsfunction/ca:triggeredjsapievents/@title" />: 
    </div>  
  </xsl:template>
  <!-- Api Triggered Events Definitions --> 
  <xsl:template name="TriggeredEventsList">
    <xsl:for-each select="ca:triggeredjsapievent">
      <xsl:sort  select="ca:name "/>
      <div class="SubSection">
        <xsl:for-each select="ca:name">
          <div class="TriggeredEventName">
            <xsl:apply-templates/>
          </div>
        </xsl:for-each> 
        <div class="SubSection">    
          <xsl:for-each select="ca:description">
            <div class="SectionContent2">
              <xsl:apply-templates/>
            </div>
          </xsl:for-each>
        </div>
      </div>          
    </xsl:for-each>
  </xsl:template>
  
</xsl:stylesheet> 