<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:ca="http://localhost/chameleon/utils/xml/schemas" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:fn="http://www.w3.org/2004/07/xpath-functions" 
  xmlns:xdt="http://www.w3.org/2004/07/xpath-datatypes" 
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <xsl:output version="1.0" encoding="utf-8" omit-xml-declaration="no" indent="no" media-type="text/html"/>
  
	<!-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx --> 
	<!-- Widget Events Section -->
	<xsl:template name="WidgetEventsSection">
		<xsl:call-template name="WidgetEventsTitle"/>
		<xsl:choose>
			<xsl:when test="ca:widgetevents/ca:widgetevent/ca:name  != &quot;&quot;">
				 <xsl:call-template name="WidgetEventsList"/>     
			</xsl:when>
			<xsl:otherwise>
				<div class="Section">
					<div class="WidgetEventName">
						None Recorded.
					</div>
				</div>
			</xsl:otherwise>
		</xsl:choose>      
	</xsl:template>  
  <!-- Main Title For Description Section -->
  <xsl:template name="WidgetEventsTitle">
    <div class="MainHeading">         
      <a>
        <xsl:attribute name="name"><xsl:text disable-output-escaping="yes">#_widgetevents</xsl:text></xsl:attribute>
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgetevents/@title" />
      </a>
    </div>  
  </xsl:template>
  <!-- Widget Events List -->
  <xsl:template name="WidgetEventsList">
    <xsl:for-each select="ca:widgetevents"> 
      <div class="Section">
        <xsl:for-each select="ca:widgetevent">  
          <!-- Widget Event Name -->
          <xsl:for-each select="ca:name">
            <div class="WidgetEventName">
              <xsl:apply-templates/>
            </div>
          </xsl:for-each>
          <!--Widget Event Descirption -->
          <div class="SubSection">
            <xsl:for-each select="ca:description">
              <div class="SectionContent2">
                <xsl:apply-templates/>
              </div>
            </xsl:for-each>                         
            <xsl:call-template name="WEParameters"/>  
          </div>
        </xsl:for-each>       
      </div>
    </xsl:for-each>       
  </xsl:template> 
  <!-- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx -->
  <!--Parameters Section -->
  <xsl:template name="WEParameters">
    <xsl:if test="ca:parameters/ca:parameter/ca:name != &quot;&quot;">  
      <xsl:for-each select="ca:parameters">
        <!-- Parameter Section Title -->
        <xsl:call-template name="WEParametersTitle"/>
        <xsl:call-template name="WEParametersList"/>
      </xsl:for-each>
    </xsl:if>
  </xsl:template> 
  
  <!-- Parameter Section Title -->
  <xsl:template name="WEParametersTitle">
    <div class="SectionHeading">
      <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgetevents/ca:widgetevent/ca:parameters/@title" />: 
    </div>
  </xsl:template>

  <xsl:template name="WEParametersList">
    <xsl:for-each select="ca:parameter">  

      <!-- Formelement Definition -->
      <div class="SubSection">
      <!-- Parameter Name -->
        <xsl:for-each select="ca:name">
          <div class="ParameterName">
            <xsl:apply-templates/>
          </div>
        </xsl:for-each> 
        <div class="SubSection">    
          <xsl:call-template name="WEParameterType"/> 
          <xsl:call-template name="WEParameterRequired"/> 
          <xsl:call-template name="WEParameterDefault"/>
          <xsl:call-template name="WEParameterPossValues"/> 
          <xsl:for-each select="ca:description">
            <div class="SectionContent2">
              <xsl:apply-templates/>
            </div>
          </xsl:for-each>
        </div>
      </div>          
    </xsl:for-each>
  </xsl:template>
  
  <!-- Parameter Value Type -->
  <xsl:template name="WEParameterType">
    <div class="SectionContent2">
      <span class="DefinitionTitle">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgetevents/ca:widgetevent/ca:parameters/ca:parameter/ca:type/@title" />: 
      </span>
      <xsl:for-each select="ca:type">
        <xsl:call-template name="ValueDescription"/>  
      </xsl:for-each>
    </div>  
  </xsl:template> 
  <!-- Parameter Value Required -->
  <xsl:template name="WEParameterRequired">
    <div class="SectionContent2">
      <span class="DefinitionTitle">
        <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgetevents/ca:widgetevent/ca:parameters/ca:parameter/ca:required/@title" />: 
      </span>
      <xsl:for-each select="ca:required">
        <xsl:call-template name="ValueDescription"/>  
      </xsl:for-each>
    </div>  
  </xsl:template>   
  <!-- Parameter Default Value -->
  <xsl:template name="WEParameterDefault">
      <xsl:for-each select="ca:default">      
        <xsl:if test="ca:value!=&quot;&quot;">
          <div class="SectionContent2">       
            <span class="DefinitionTitle">   
            <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgetevents/ca:widgetevent/ca:parameters/ca:parameter/ca:default/@title" />: 
            </span>
            <xsl:call-template name="ValueDescription"/>  
          </div>
        </xsl:if>
      </xsl:for-each> 
  </xsl:template> 
  <!-- Possible Values for Parameter -->
  <xsl:template name="WEParameterPossValues">
    <xsl:if test="ca:possvalues/@type != &quot;&quot;">  
      <div class="SectionContent2">   
        <span class="DefinitionTitle">      
          <xsl:value-of select="document('../templates/widget-default.en-ca.doc.xml')//ca:widgetevents/ca:widgetevent/ca:parameters/ca:parameter/ca:possvalues/@title" />: 
        </span> 
        <xsl:call-template name="PossValueCondition"/>  
      </div>
    </xsl:if> 
  </xsl:template> 
  
</xsl:stylesheet> 