<?php
/**
 * handle a cache of CWC2 buttons and provide convenience functions for accessing them.
 */
include_once("./session.inc.php");
include_once( COMMON."buttonizer/buttonizer.php" );

/**
 * create a button based on a graphic, a label, and some parameters.  The
 * button is cached so it is only actually created once.
 */
function makeButton( $szJSCallback, $szJSParam, $szClassName, $szGraphic, $szLabel, $szAltText, $aExtraParams = array() )
{
    $aExtraParams['graphic'] = $szGraphic;
    $aExtraParams['label'] = $szLabel;
    $szResourceName = strtoupper( $szClassName.'PopupStyleResource' );
    
    $aResource = $_SESSION[$_SESSION[$szResourceName]];
    
    if ($szJSParam == "")
        $szJSParam = "null";
    elseif ($szJSParam == "true" || $szJSParam === true)
        $szJSParam = "true";
    elseif ($szJSParam == "false" || $szJSParam === false)
        $szJSParam = "false";
    elseif (!is_numeric( $szJSParam ))
        $szJSParam = '"'.$szJSParam.'"';
 
    $szImageID = md5($szJSCallback.$szClassName.$szGraphic.$szLabel);

    $szNormalImage = 'null';
    if (isset($aResource['normal']))
    {
         $aResource['normal'] = array_merge( $aResource['normal'], $aExtraParams );
         $szNormalImage = '"'.GetImage( $szImageID, $aResource['normal'] ).'"';
         $nNormalWidth = $aResource['normal']['width'];
         $nNormalHeight = $aResource['normal']['height'];
    }
    $szSelectedImage = 'null';
    if (isset($aResource['selected']))
    {
         $aResource['selected'] = array_merge( $aResource['selected'], $aExtraParams );
         $szSelectedImage = '"'.GetImage( $szImageID, $aResource['selected']).'"';
    }
    $szHoverImage = 'null';
    if (isset($aResource['hover']))
    {
         $aResource['hover'] = array_merge( $aResource['hover'], $aExtraParams );
         $szHoverImage = '"'.GetImage( $szImageID, $aResource['hover']).'"';
    }
    $szDisabledImage = 'null';
    if (isset($aResource['disabled']))
    {
         $aResource['disabled'] = array_merge( $aResource['disabled'], $aExtraParams );
        $szDisabledImage = '"'.GetImage( $szImageID, $aResource['disabled']).'"';
    }


    $szReturn = <<<EOT
<img name="{$szImageID}" id="{$szImageID}" src={$szNormalImage} width="{$nNormalWidth}" height="{$nNormalHeight}" ALT="{$szAltText}" TITLE="{$szAltText}" BORDER="0">
<script language="Javascript">
goCWCButtonManager.AddButton( new CWCButton("{$szImageID}", 0, "null", $szJSCallback, $szJSParam, $szNormalImage, $szHoverImage, $szSelectedImage, $szDisabledImage) );
</script>
EOT;
    return $szReturn;
}

/**
 * GetImage returns the name of an image to use for a particular resource and state
 */
function GetImage( $szImageID, $aParams )
{
        //TODO: figure out an effective caching mechanism for this?
        //using $szImageID
        
        //load platform specific behaviours
        $aParams["gd_module"] = $_SESSION["gszGDVersion"];
        $aParams["freetype"] = $_SESSION["gszFreeTypeVersion"];
        
        //generate the cached image.
        $szImageFile = md5( implode( "", $aParams ) ).".png";
        
        $szImagePath = $_SESSION['gszButtonCachePath'].$szImageFile;

        $szImageWebPath = $_SESSION['gszButtonCacheWebPath'].$szImageFile;
        
        //fix relative paths if necessary
        $aPaths = array( 'graphic', 
                         'border_tl_image',
                         'border_t_image',
                         'border_tr_image',
                         'border_r_image',
                         'border_br_image',
                         'border_b_image',
                         'border_bl_image',
                         'border_l_image',
                         'backgroundimage',
                         'labelfont'
                        );
        
        foreach ($aPaths as $aPath)
            if (isset($aParams[$aPath]))
                $aParams[$aPath] = FixRelPath( $aParams[$aPath] );
        //cache the URL and generate the image, if required.
        if (!isset($aParams['usecache']) || (isset($aParams['usecache']) && $aParams['usecache']))
        {
            //TODO: can we effectively cache this?
            //$_SESSION[$szCacheName] = $szImageWebPath;
            if (!is_file( $szImagePath ))
            {
                buttonize( $szImagePath, $aParams );
            }
        }
        else
        {
            buttonize( $szImagePath, $aParams );
        }
        
        return $szImageWebPath;
}

/**
 * fix relative paths to be absolute, and try to find a reasonable location
 * for the file by checking its existance both against the application and
 * the chameleon installation
 */
function FixRelPath( $szRelPath )
{
    //check for absolute file system or web paths
    if (substr($szRelPath, 0, 1) == "/" ||
        substr($szRelPath, 0, 1) == "\\"  ||
        substr($szRelPath, 1, 1) == ":" )
        return $szRelPath;

    if (strcasecmp(substr($szRelPath, 0, 4), "http") == 0)
        return $szRelPath;

    //assume it is relative, try to validate the location of the image
    //try local filesystem first
    if (file_exists( $_SESSION['gszAppPath'].'/'.$szRelPath ))
        return $_SESSION['gszAppPath'].'/'.$szRelPath;

    //try app web path next
    if (file_exists( $_SESSION['gszAppWebPath'].$szRelPath ))
        return $_SESSION['gszAppWebPath'].$szRelPath;

    //try chameleon file system
    if (file_exists( $_SESSION['gszCorePath'].$szRelPath ))
        return $_SESSION['gszCorePath'].$szRelPath;

    //assume chameleon web path
    return $_SESSION['gszCoreWebPath'].$szRelPath;
}

?>
