<?php
/**
 * Popup Class
 *
 * @project     CWC2
 * @revision    $Id:
 * @purpose     Helper class to manage Popup attributes
 * @author      DM Solutions Group (sfournier@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/**
 * helper class for widgets that want to popup another window through a
 * javascript call
 */
class CWCPopup
{
    var $moParent;
    var $miWidth = 200;
    var $miHeight = 200;
    var $mbToolbar = true;
    var $mbMenubar = true;
    var $mbStatus = true;
    var $mbScrollbars = true;
    var $mbResizable = true;
    var $mszPopupStyleResource = "";

    var $mszLink = "";
    var $mszParam = "";

    /**
     * construct a new popup widget
     */
    function CWCPopup( &$oParent )
    {
        $this->moParent =& $oParent;
        $oParent->maAttributes["POPUPWIDTH"] = new IntegerAttribute( "POPUPWIDTH", true, 0 );
        $oParent->maAttributes["POPUPHEIGHT"] = new IntegerAttribute( "POPUPHEIGHT", true, 0 );
        $oParent->maAttributes["TOOLBAR"] = new BooleanAttribute( "TOOLBAR", false );
        $oParent->maAttributes["MENUBAR"] = new BooleanAttribute( "MENUBAR", false );
        $oParent->maAttributes["STATUS"] = new BooleanAttribute( "STATUS", false );
        $oParent->maAttributes["SCROLLBARS"] = new BooleanAttribute( "SCROLLBARS", false );
        $oParent->maAttributes["RESIZABLE"] = new BooleanAttribute( "RESIZABLE", false );
        $oParent->maAttributes["POPUPSTYLERESOURCE"] = new StringAttribute( "POPUPSTYLERESOURCE", false );
    }

    /**
     * initialize default values
     */
    function InitDefaults()
    {
    }

    /**
     * construct a URL for the popup
     */
    function GetPopupURL()
    {
        if ((dirname($this->mszLink) == "widgets" ||
            dirname($this->mszLink) == "./widgets") &&
            isset($_SESSION['gszCoreWebPath']))
        {
            $this->mszLink = $_SESSION['gszCoreWebPath']."/widgets/".
                              basename($this->mszLink);
        }

        if (substr($this->mszLink, -1) != "?")
        {
            if (stristr($this->mszLink, "?") === false)
                $this->mszLink .= "?";
            else
                $this->mszLink .= "&";
        }
        return $this->mszLink.SID.$this->mszParam;
    }

    /**
     * get a unique name for this widget to refer to a window by
     */
    function GetPopupName()
    {
        $szName = "popup";
        if (isset($this->moParent->maParams["TYPE"]))
            $szName = $this->moParent->maParams["TYPE"];

        return  $szName."_".
                $this->moParent->mnId."_".session_id();
    }

    /**
     * get the options to apply to the popup window
     */
    function GetPopupOptions()
    {
        // Mandatory values
        if (isset($this->moParent->maParams["POPUPWIDTH"]))
            $this->miWidth = $this->moParent->maParams["POPUPWIDTH"];
        if (isset($this->moParent->maParams["POPUPHEIGHT"]))
            $this->miHeight = $this->moParent->maParams["POPUPHEIGHT"];

        // Optional Values
        if (isset($this->moParent->maParams["POPUPSTYLERESOURCE"]))
            $this->mszPopupStyleResource = $this->moParent->maParams["POPUPSTYLERESOURCE"];
        if (isset($this->moParent->maParams["TOOLBAR"]))
            $this->mbToolbar = $this->moParent->maParams["TOOLBAR"];
        if (isset($this->moParent->maParams["MENUBAR"]))
            $this->mbMenubar = $this->moParent->maParams["MENUBAR"];
        if (isset($this->moParent->maParams["STATUS"]))
            $this->mbStatus = $this->moParent->maParams["STATUS"];
        if (isset($this->moParent->maParams["SCROLLBARS"]))
            $this->mbScrollbars = $this->moParent->maParams["SCROLLBARS"];
        if (isset($this->moParent->maParams["RESIZABLE"]))
            $this->mbResizable = $this->moParent->maParams["RESIZABLE"];

        $szResult = "width=".$this->miWidth.",height=".$this->miHeight.",";

        $szResult .= (($this->mbToolbar == "true") ? "toolbar=yes," : "toolbar=no,");
        $szResult .= (($this->mbMenubar == "true") ? "menubar=yes," : "menubar=no,");
        $szResult .= (($this->mbStatus == "true") ? "status=yes," : "status=no,");
        $szResult .= (($this->mbScrollbars == "true") ? "scrollbars=yes," : "scrollbars=no,");
        $szResult .= (($this->mbResizable == "true") ? "resizable=yes" : "resizable=no");
        return $szResult;
    }

    /**
     * draw the popup javascript call
     */
    function DrawPublish( $szContent = "" )
    {
        //pass style resource to the popup
        $this->SetStyleResource();
        
        if (isset($this->moParent->maSharedResourceWidgets['PopupCSS']) && !isset($_SESSION['gszCSSFile']))
        {
            $szPopupCSS = $this->moParent->maSharedResourceWidgets['PopupCSS']->getValue("VALUE","popup.css");
            $_SESSION['gszCSSFile'] = $this->FixRelPath( $szPopupCSS );
        }

        $szResult  = "var wh = window.open('".$this->GetPopupURL().
                     "', '".$this->GetPopupName()."', '".
                     $this->GetPopupOptions()."');".
                     "wh.focus();";

        return $szResult;
    }
    
    function SetStyleResource()
    {
        $szSessionName = strtoupper(get_class( $this->moParent )).'POPUPSTYLERESOURCE';
        if (!isset($_SESSION[$szSessionName]))
        {
            if ( isset( $this->moParent->maParams['POPUPSTYLERESOURCE']) ||
                 isset( $this->moParent->maParams['STYLERESOURCE']) )
            {

                if (isset($this->moParent->maParams['POPUPSTYLERESOURCE']) &&
                    $this->moParent->maParams['POPUPSTYLERESOURCE'] != "" )
                {
                    $_SESSION[$szSessionName] = $this->moParent->maParams['POPUPSTYLERESOURCE'];
                }
                elseif (isset($this->moParent->maParams['STYLERESOURCE']))
                {
                    $_SESSION[$szSessionName] = $this->moParent->maParams['STYLERESOURCE'];
                }
                else
                {
                    $_SESSION[$szSessionName] = "";
                }
            }
        }
    }
    
    /**
     * fix relative paths to be absolute, and try to find a reasonable location
     * for the file by checking its existance both against the application and
     * the chameleon installation
     */
    function FixRelPath( $szRelPath )
    {
        //check for absolute file system or web paths
        if (substr($szRelPath, 0, 1) == "/" ||
            substr($szRelPath, 0, 1) == "\\"  ||
            substr($szRelPath, 1, 1) == ":" )
            return $szRelPath;
    
        if (strcasecmp(substr($szRelPath, 0, 4), "http") == 0)
            return $szRelPath;
    
        $hFile = fopen( $_SESSION['gszAppWebPath'].'/'.$szRelPath, 'r' );
        
        if ($hFile === FALSE)
            $szResult = $_SESSION['gszCoreWebPath'].'/'.$szRelPath;
        else
        {
            $szResult = $_SESSION['gszAppWebPath'].'/'.$szRelPath;
            fclose( $hFile );
        }
        return $szResult;
        
    }
}
?>
