<?php
/**
 * MapTips Widget class
 *
 * @project     CWC2
 * @revision    $Id: MapTips.widget.php,v 1.7 2004/01/22 18:41:49 pspencer Exp $
 * @purpose     MapTips Widget class
 * @author      Jason Fournier (dev@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
 
 include_once("Widget.php");
 include_once("Label.php");
 include_once("Button.php");
 include_once("SharedResource.widget.php");
 
 /**
  * MapTips Widget
  *
  * @desc A widget that draws dynamic MapTips on the MapImage.  Uses an
  *       imagemap to dynamically position a hidden layer with relevant
  *       information.  Information taken from WFS query SharedResource 
  */
 class MapTips extends CWCWidget
 {
     /* the name of the shared resource to dump the results into */
     var $mszSharedResourceName;
     
     var $moLabel;
     var $moButton;
     var $mszImagePath = "images/tool_zoombounds.gif";
     var $mszImageSelectedPath = "images/tool_zoombounds.gif";
     var $mszImageHoverPath = "images/tool_zoombounds.gif";
     var $mszImageTip = "Show/Hide MapTips";

     /* default settings for the widget params */
     var $mszName = "MapTips";                      // Name of the Widget  
     var $mszLayerType = "dynamic";                 // MapTips mode
     var $mszDefaultText = "No Location Selected";  // Default text for noon-active maptip
     var $mnWidth = 120;                            // Width of DIV
     var $mnHeight = 35;                            // Height of DIV
     var $mnXOffset = 10;                           // X pixel Offset from point location for dynamic DIV's
     var $mnYOffset = 5;                            // Y pixel Offset from point location for dynamic DIV's
     var $mnTolerance = 4;                          // Imagemap point tolerance for each AREA entry
     var $mszBGColor = "#E8E9D3";                   // DIV BGColor
     var $mszBorderColor = "#000000";               // DIV Border Color
     var $mnAlpha = 100;                            // DIV Alpha
     var $mbLink = false;                           // TRUE if there's a link onClick for each location
     var $mszLinkURL = '';                          // URL to go to
     var $mszLinkUniqueID = '';                     // unique location id to append to url
     var $mbStatus = true;                          // is it on?
     var $mbVisible = true;                         // is it visible?
     var $mszPixelX = 'pix_x';                      // the dump array x pixel index to use for the imagemap creation
     var $mszPixelY = 'pix_y';                      // the dump array y pixel index to use for the imagemap creation
     
     /* internal vars */
     var $mszImageMapName = 'maptips_imagemap';
     var $mszDynamicDIV = 'maptips_dynamic';
     var $mszTransparentMapDIV = 'maptips_container';
     var $mszTransparentMapImage = 'maptips_image';
     var $mszPixelGif = 'images/a_pixel.gif';
     var $maSR = null;                              //temporary storage for shared resource - deprecated?

 
     /**
      * MapTips
      *
      * @desc Constructor method for the MapTips widget.
      */
     function MapTips()
     {
         /* invoke constructor of parent */
         parent::CWCWidget();
 
         /* set the description for this widget */
         $this->szWidgetDescription = <<<EOT
             The MapTips widget allows users to show dynamic or static
             maptips (tooltips) containing information pertinent to 
             their application.  The widget relies on an array of 
             data stored as a Shared Resource.  It will create
             an imagemap from the query results and place js to handle the 
             dhmtl
EOT;
         
         $this->moLabel = new CWCLabel( $this );
         $this->moButton = new CWCButton( $this );

         /* declare the required OR allowable cwc2 parameters for this widget */
         $this->maAttributes["NAME"]           = new StringAttribute( "NAME", true );
         $this->maAttributes["LAYERTYPE"]      = new StringAttribute( "LAYERTYPE", true );
         $this->maAttributes["DEFAULTTEXT"]    = new StringAttribute( "DEFAULTTEXT", false );

         $this->maAttributes["WIDTH"]          = new IntegerAttribute( "WIDTH", false );
         $this->maAttributes["HEIGHT"]         = new IntegerAttribute( "HEIGHT", false );
         $this->maAttributes["XOFFSET"]        = new IntegerAttribute( "XOFFSET", false );
         $this->maAttributes["YOFFSET"]        = new IntegerAttribute( "YOFFSET", false );
         $this->maAttributes["TOLERANCE"]      = new IntegerAttribute( "TOLERANCE", false );

         $this->maAttributes["BGCOLOR"]        = new HexColorAttribute( "BGCOLOR", false );
         $this->maAttributes["BORDERCOLOR"]    = new HexColorAttribute( "BORDERCOLOR", false );
         $this->maAttributes["ALHPA"]          = new IntegerAttribute( "ALPHA", false );

         $this->maAttributes["LINK"]           = new BooleanAttribute( "LINK", false );
         $this->maAttributes["LINKURL"]        = new StringAttribute( "LINKURL", false );
         $this->maAttributes["LINKUNIQUEID"]   = new StringAttribute( "LINKUNIQUEID", false );

         $this->maAttributes["PIXELX"]         = new StringAttribute( "PIXELX", false );
         $this->maAttributes["PIXELY"]         = new StringAttribute( "PIXELY", false );

         $this->maAttributes["STATUS"]         = new StringAttribute( "STATUS", false );
         $this->maAttributes["VISIBLE"]        = new StringAttribute( "VISIBLE", false );

         $this->maAttributes["SHAREDRESOURCE"] = new StringAttribute( "SHAREDRESOURCE", true );
         
     }
     

     /**
      * InitDefaults
      *
      */
     function InitDefaults()
     {
         
         parent::InitDefaults();
         
         $this->moButton->mszImagePath = $this->mszImagePath;
         $this->moButton->mszImageSelectedPath = $this->mszImageSelectedPath;
         $this->moButton->mszImageHoverPath = $this->mszImageHoverPath;
         $this->moButton->mszImageTip = $this->mszImageTip;
         $this->moButton->InitDefaults();

         /* init the params if user set them */
         if (isset($this->maParams["LAYERTYPE"]))
            $this->mszLayerType = strtolower($this->maParams["LAYERTYPE"]);
         if (isset($this->maParams["DEFAULTTEXT"]))
            $this->mszDefaultText = $this->maParams["DEFAULTTEXT"];
            
         if (isset($this->maParams["WIDTH"]))
            $this->mnWidth = $this->maParams["WIDTH"];
         if (isset($this->maParams["HEIGHT"]))
            $this->mnHeight = $this->maParams["HEIGHT"];
         if (isset($this->maParams["XOFFSET"]))
            $this->mnXOffset = $this->maParams["XOFFSET"];
         if (isset($this->maParams["YOFFSET"]))
            $this->mnYOffset = $this->maParams["YOFFSET"];
         if (isset($this->maParams["TOLERANCE"]))
            $this->mnTolerance = $this->maParams["TOLERANCE"];
            
         if (isset($this->maParams["BGCOLOR"]))
            $this->mnBGColor = $this->maParams["BGCOLOR"];
         if (isset($this->maParams["BORDERCOLOR"]))
            $this->mnBorderColor = $this->maParams["BORDERCOLOR"];
         if (isset($this->maParams["ALPHA"]))
            $this->mnAlpha = $this->maParams["ALPHA"];

         if (isset($this->maParams["LINK"]))
         {
            $this->mszLink = $this->maParams["LINK"];
            if (isset($this->maParams["LINKURL"]))
               $this->mszLinkURL = $this->maParams["LINKURL"];
            if (isset($this->maParams["LINKUNIQUEID"]))
               $this->mszLinkUniqueID = $this->maParams["LINKUNIQUEID"];
         }
         
         if (isset($this->maParams["PIXELX"]))
            $this->mszPixelX = $this->maParams["PIXELX"];
         if (isset($this->maParams["PIXELY"]))
            $this->mszPixelY = $this->maParams["PIXELY"];

         if (isset($this->maParams["STATUS"]))
            $this->mbStatus = $this->maParams["STATUS"];
         if (isset($this->maParams["VISIBLE"]))
            $this->mbVisible = $this->maParams["VISIBLE"];

         if (isset($this->maParams["SHAREDRESOURCE"]))
            $this->mszSharedResourceName = $this->maParams["SHAREDRESOURCE"];
            
         $this->mszImageMapName .= $this->mnId;
         $this->mszDynamicDIV .= $this->mnId;
         $this->mszTransparentMapDIV .= $this->mnId;
         $this->mszTransparentMapImage .= $this->mnId;

     }

     
    /**
     * DrawPublish()
     *
     * @desc draw this widget on the page
     */
    function DrawPublish()
    {

        /* grab a handle on the map object and get the height & width */
        $poMap = $this->moMapObject->oMap;
        $nMapWidth = $poMap->width;
        $nMapHeight = $poMap->height;
        
        /* start top write out the DIV's and trans Img */
        $szResult = "";
        
        $szResult .= "<div id=\"".$this->mszDynamicDIV."\" 
                      class=\"maptipDynamic\">".$this->mszDefaultText."</div>
                      \n";
                      
        $szResult .= "<div id=\"".$this->mszTransparentMapDIV."\" 
                      style=\"position:absolute; 
                      width:".$nMapWidth."; 
                      height:".$nMapHeight."; 
                      top:-1000; left:-1000; 
                      z-index:500;\">\n";
                      
        $szResult .= "<img src=\"".$this->mszPixelGif."\" 
                      width=\"".$nMapWidth."\" 
                      height=\"".$nMapHeight."\" 
                      border=\"0\" 
                      usemap=\"#".$this->mszImageMapName."\" 
                      name=\"maptipsimage\">\n";
                      
        $szResult .= "</div>\n\n";

        $szResult .= $this->GenerateImageMap();
        $szResult .= "\n\n";
        
        /* return the result */
        if (!$this->mbVisible)
            return "<!-- MapTips activation widget -->";
        
        $this->moButton->SetOnClick("SetMapTipsStatus();");
        
        $szResult .= $this->moButton->DrawPublish();

        return $szResult;
        
    }
    

    
    /**
     * GenerateImageMap()
     *
     * @desc ImageMap generation
     */
    function GenerateImageMap()
    {

        /**
        * SAMPLE MAPTIP IMAGEMAP LINE OUTPUT
        * <map name="maptip_circle_1">
        *  <area shape="circle" coords="41,125,6" href="http://url-to-info" target="_blank" 
        *  onmouseover="MapTipInit(10,'maptips_static','static');"   
        *  onmouseout="MapTipClear( 'maptips_static', 'No Location Selected' );">
        * </map>
        *
        * This is a general client-side imagemap that attaches itself to the
        * transparent gif inside the maptipscontainer DIV.  pix coords from
        * GetQueryDump() populate the circle locations of the image map.
        */
        
        $szMap = "<map name=\"".$this->mszImageMapName."\">\n";
        
        $aszWFSR = $this->maSharedResourceWidgets[$this->mszSharedResourceName]->maszContents['LayerAttributes'];
        $nCount = count($aszWFSR);
        
        for($i=0; $i<$nCount; $i++)
        {
            /* create the coord string */
            $szCoord = $aszWFSR[$i][$this->mszPixelX] . ',' .
                       $aszWFSR[$i][$this->mszPixelY] . ',' .
                       $this->mnTolerance;
            
            /* create the onMouseOver */
            $szOnMouseOver = "onmouseover=\"MapTipInit('".
                              $aszWFSR[$i]['station_id']."','".
                              $this->mszDynamicDIV."','".
                              $this->mszLayerType."', '".
                              $this->mszSharedResourceName."', ".
                              $this->mnXOffset." , ".
                              $this->mnYOffset." );\"";
            
            /* create the onMouseOut */
            $szOnMouseOut = "onmouseout=\"CWCDHTML_SetLayerVis('".
                             $this->mszDynamicDIV."', false);
                             MapTipClear( '".$this->mszDynamicDIV."', '".
                             $this->mszDefaultText."', '".
                             $this->mszSharedResourceName."' );\"";
            
            /* create the URL */
            if($this->mbLink)
            {
                // TODO make this better!
                // TODO error checking required here
                $szURL = $this->mszLinkURL . $this->mszLinkUniqueID;
                $szTarget = "target=\"_blank\" ";
            }
            else
            {
                $szURL = 'javascript:void(0)';
                $szTarget = ' ';
            }

            /* Create the AREA entry and append to the string */
            $szMap .= "<area shape=\"circle\" coords=\"$szCoord\"
                      href=\"". $szURL ."\" $szTarget
                      $szOnMouseOver $szOnMouseOut >\n\n";
            
        }
        
        $szMap .= "</map>";
        $szMap .= "<map name=\"nullmap\"></map>\n";
        
        return $szMap;

    }


    /**
     * GetJavascriptIncludeFunctions
     *
     * javascript include files
     */
    function GetJavascriptIncludeFunctions()
    {
        $aReturn = array();
        
        $aReturn = parent::GetJavascriptIncludeFunctions();
        
        /* layer_fn.js contains generic layer functions */
        $szJsIncludeName = "cwc_dhtml.js";

        $szInclude = "<script language=\"JavaScript\" src=\"" .
                     $_SESSION["gszCoreWebPath"]."/widgets/js/cwc_dhtml.js\"></script>";
        $aReturn[$szJsIncludeName] = $szInclude;
        
        /* maptips.js contains all the maptip js functionality */
        $szJsIncludeName = $_SESSION["gszCoreWebPath"]."/widgets/js/maptips.js";
        $szInclude = "<script language=\"JavaScript\" src=\"" .
                     $_SESSION["gszCoreWebPath"]."/widgets/js/maptips.js\"></script>";
        $aReturn[$szJsIncludeName] = $szInclude;

        /* maptips.css styles the dynamic and static DIV's used with maptips */
        $szJsIncludeName = $_SESSION["gszCoreWebPath"]."/widgets/css/maptips.css";
        $szInclude = "<link rel=\"stylesheet\" href=\"" . 
                     $_SESSION["gszCoreWebPath"]."/widgets/css/maptips.css\" type=\"text/css\">";
        $aReturn[$szJsIncludeName] = $szInclude;

        return $aReturn;
    }


    /**
     * GetJavascriptInitFunctions
     *
     * @desc These get spit out at the bottom of the template after the page is processed
     */
    function GetJavascriptInitFunctions()
    {

        $aReturn = array();
        
        /* this positions the transparent gif over the map to be used in conjunction with the MapTips. */
        /* the gif has an image map associated with it */
        $szJsFunctionName = "CWCDHTML_PositionLayerByImage";
        $szFunction = "$szJsFunctionName('".$this->mszTransparentMapDIV."', 'mapimage', 'tl', 0, 0 );";
        $aReturn[$szJsFunctionName] = $szFunction;
        
        /* this hides the Dynamic DIV layer */
        $szJsFunctionName = "CWCDHTML_SetLayerVis";
        $szFunction = "$szJsFunctionName('".$this->mszDynamicDIV."', false);";
        $aReturn[$szJsFunctionName] = $szFunction;
        
        return $aReturn;
        
    }


    /**
     * GetHTMLHiddenVariables
     *
     * return the map size  variables.
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();
        
        $szVariable = "SHOW_MAPTIPS";
        $szValue = "<INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"1\">\n";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }
    

    /**
     * GetJavascriptFunctions
     *
     * Build and return the array of functions needed in the
     * widget.
     */
    function GetJavascriptFunctions()
    {
        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        /**
         * {fnname}
         * desc
         */
        // set the status for maptips to ON|OFF
        $szJsFunctionName = "SetMapTipsStatus";
        $szImageMapName = $this->mszImageMapName;
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * show/hide maptips
 */
function {$szJsFunctionName}()
{
    if( document.forms[0].SHOW_MAPTIPS.value == 1 )
    {
        document.forms[0].SHOW_MAPTIPS.value = 0;
        document['maptipsimage'].useMap = '#nullmap';
    }
    else
    {
        document.forms[0].SHOW_MAPTIPS.value = 1;
        document['maptipsimage'].useMap = '{$szImageMapName}';
    }
    
    // return
    return;
}
EOT;

        $aReturn[$szJsFunctionName] = $szFunction;


        // return the array of functions
        return $aReturn;

    }
}

?>


