<?php
/**
 * SLDUpload Widget class
 *
 * @project     CWC2
 * @revision    $Id: UploadSLD.widget.php,v 1.4 2004/01/22 18:41:49 pspencer Exp $
 * @purpose     SLDUpload Popup Widget class
 * @author      DM Solutions Group (zjames@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2002, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
include_once("Widget.php");
include_once("Button.php");
include_once("Popup.php");

/**
 * SLDUpload
 *
 * @desc SLDUpload widget class
 */
class UploadSLD extends CWCWidget
{
    var $mszTemplate;      // [string]

    var $moButton;         // Button object
    var $moPopup;          // Popup object

    var $mszLink = 'widgets/UploadSLD.phtml';
    var $mszParam = '&szCallbackFunc=OpenSLDCB';

    /**
     * SLDUpload
     *
     * Constctor method for the SLDUpload widget.
     */
    function UploadSLD()
    {
        // invoke constructor of parent
        parent::CWCWidget();

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The SLDUpload widget allows the user to upload a SLD file from their
local hard drive or from a URL to the current application.
EOT;

        $this->maAttributes["singlelayer"] = new BooleanAttribute( "singlelayer", false );

        $this->moButton = new CWCButton($this);

        $this->moPopup = new CWCPopup($this);
        $this->mnPriority = PRIORITY_MAXIMUM;
    }


    function InitDefaults()
    {
        $this->moButton->InitDefaults();
        $this->moButton->SetOnClick('clickSLDUpload');
        $this->moPopup->mszLink = $this->mszLink;
        $this->moPopup->mszParam = $this->mszParam;
        $this->moPopup->InitDefaults();
        if (isset($this->maParams["singlelayer"]))
            $this->mbSingleLayer =
                $this->maParams["singlelayer"];
    }

    function GetJavascriptVariables()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptVariables();
        else return array();
    }

    function GetJavascriptInitFunctions()
    {
        if ($this->mbVisible)
            return $this->moButton->GetJavascriptInitFunctions();
        else return array();
    }

    function GetJavascriptOnLoadFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptOnLoadFunctions();
        else
            $aReturn = array();
        return $aReturn;
    }

    function GetJavascriptIncludeFunctions()
    {
        if ($this->mbVisible)
            $aReturn = $this->moButton->GetJavascriptIncludeFunctions();
        else
            $aReturn = array();
        return $aReturn;
    }


    /**
     * GetJavascriptFunctions
     *
     * Build and return the array of functions needed in the
     * widget.
     */
    function GetJavascriptFunctions()
    {
        $aReturn = array();

        if (isset($this->maAttributes['singlelayer']) &&
            isset($this->moURLArray['nlayerindex']))
        {
            $this->moPopup->mszParam .= "&szlayerindex=".
                  $this->moURLArray['nlayerindex'];
        }

        $szJsFunctionName = "clickSLDUpload";
        $szFunction = "function $szJsFunctionName()\n" .
                      "{\n".
                       $this->moPopup->DrawPublish()."\n".
                      "return;\n" .
                      "}\n";
        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "OpenSLDCB";
        $szFunction = <<<EOF

function {$szJsFunctionName}(actionID, wh)
{
    if (actionID == 0 || actionID == 1) // ok
    {
        // The SLD has been loaded into the map session, simply reload
        // the page.  First we'll need to clear some previous state, though.
            if ({$this->mszHTMLForm}.MAP_EXTENTS_MINX !=null && {$this->mszHTMLForm}.MAP_EXTENTS_MINX.value != null)
                {$this->mszHTMLForm}.MAP_EXTENTS_MINX.value = "";
            if ({$this->mszHTMLForm}.MAP_EXTENTS_MINY !=null && {$this->mszHTMLForm}.MAP_EXTENTS_MINY.value != null)
               {$this->mszHTMLForm}.MAP_EXTENTS_MINY.value = "";
            if ({$this->mszHTMLForm}.MAP_EXTENTS_MAXX !=null && {$this->mszHTMLForm}.MAP_EXTENTS_MAXX.value != null)
                {$this->mszHTMLForm}.MAP_EXTENTS_MAXX.value = "";
            if ({$this->mszHTMLForm}.MAP_EXTENTS_MAXY !=null && {$this->mszHTMLForm}.MAP_EXTENTS_MAXY.value != null)
                {$this->mszHTMLForm}.MAP_EXTENTS_MAXY.value = "";
            if ({$this->mszHTMLForm}.MAP_WIDTH !=null && {$this->mszHTMLForm}.MAP_WIDTH.value != null)
                {$this->mszHTMLForm}.MAP_WIDTH.value = "";
            if ({$this->mszHTMLForm}.MAP_HEIGHT !=null && {$this->mszHTMLForm}.MAP_HEIGHT.value != null)
                {$this->mszHTMLForm}.MAP_HEIGHT.value = "";       
            if ({$this->mszHTMLForm}.LEGENDTEMPLATE_RESET !=null && {$this->mszHTMLForm}.LEGENDTEMPLATE_RESET.value != null)
                {$this->mszHTMLForm}.LEGENDTEMPLATE_RESET.value = "1";     
        wh.close();
        {$this->mszHTMLForm}.submit();
    }
    else if (actionID == 2) // cancel
    {
        wh.close();
    }

    return;
}

EOF;
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

    function GetHTMLHiddenVariables()
    {
        $aReturn = $this->moButton->GetHTMLHiddenVariables();
        
        return $aReturn;
    }

    /**
     * DrawPublish
     *
     * Return the HTML code to display the link to the legend popup
     *
     * @return HTML Code
     */
    function DrawPublish()
    {
        if (!$this->mbVisible)
            return '<!-- Upload SLD widget hidden -->';

        $szReturn = $this->moButton->DrawPublish();

        return $szReturn;
    }
}
?>
