<?php
/**
 * SLDCache Widget class
 *
 * @project     GeoInnovations 2002 - SECT
 * @revision    $Id: SLDCache.widget.php,v 1.20 2004/03/02 19:51:14 sfournier Exp $
 * @purpose     SLDCache Widget class puts the contents of a layer or sld table
 *              into a shared resource.
 * @author      DM Solutions Group (zjames@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2003, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once($GLOBALS['gszChameleonPath'].'/htdocs/widgets/Widget.php');
include_once($GLOBALS['gszChameleonPath'].'/htdocs/widgets/SharedResource.widget.php');

/**
 * SLDCache Widget
 *
 * @desc a widget that extracts the contents of a mapfile layer and inserts it
 *       into a SharedResource.
 */
class SLDCache extends CWCWidget
{
    /* the name of the shared resource to dump the results into */
    var $mszSharedResourceName;

    /* the index of the layer in the mapfile */
    var $mnSLDLayerIndex;

    /* the index of the class in the mapfile */
    var $mnSLDClassIndex;

    /* the number of styles in the class*/
    var $mnSLDNumStyles;
    
    /* the internal state of the SLDCache widget maintained separately
       from the main map state to facilitate management of classes
       using PHP MapScript functions without affecting the main
       state until the user accepts the changes.  Also facilitates
       an UNDO function :)
     */
     
    var $moSLDSession = null;
    
    var $mszSLDMapFile = "";
    
    var $mbUpdateOpener = false;
    
    var $mbDirty = false;

    /**
     * SLDCache
     *
     * Constructor method for the SLDCache widget.
     */
    function SLDCache()
    {
        // invoke constructor of parent
        parent::CWCWidget();

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The SLDCache widget gets the contents of a mapfile layer and adds an array to
a SharedResource widget in a format suitable for generating tables for SECT.
The layer to process must be passed in the url as nlayerindex. This uses and
requires a Table widget for output.
EOT;

        $this->maAttributes["SHAREDRESOURCENAME"] =
            new StringAttribute( "SHAREDRESOURCENAME", true );

        $this->maAttributes["LAYERNAMEINPUT"] =
            new StringAttribute( "LAYERNAMEINPUT", false );
        $this->maAttributes["CLASSITEMINPUT"] =
            new StringAttribute( "CLASSITEMINPUT", false );
        $this->maAttributes["LABELITEMINPUT"] =
            new StringAttribute( "LABELITEMINPUT", false );
        $this->maAttributes["ADDCLASSINPUT"] =
            new StringAttribute( "ADDCLASSINPUT", false );

    }

    /**
     * initialize respectable defaults
     */
    function InitDefaults()
    {
        parent::InitDefaults();

        if (isset($this->maParams["SHAREDRESOURCENAME"]))
            $this->mszSharedResourceName =
                $this->maParams["SHAREDRESOURCENAME"];
                
        $this->mszSLDMapFile = getSessionSavePath()."/SLDCache".$this->mnId.".map";
            
        $oMap = ms_newMapObj( "" );
        $oMap->set( 'width', 1 );
        $oMap->set( 'height', 1 );
        $oMap->extent->setextent( -1, -1, 1, 1 );
        //this shouldn't be needed, assefa to fix a bug related to this
        $oMap->setsymbolset( realpath($GLOBALS['gszAppPath']."/../etc/sectsym.sym"));

        $this->moSLDSession = new MapSession_RW();
        $this->moSLDSession->szMapFile = $this->mszSLDMapFile;
        $this->moSLDSession->szTempDir = getSessionSavePath();
        $this->moSLDSession->oMap = $oMap;

    }

    /**
     * return an array of HTML variables to include in the page
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = array();
        $szVariable = 'nlayerindex';

        $aReturn[$szVariable] = '<INPUT TYPE="HIDDEN" NAME="'.$szVariable.
            '" VALUE="'.$this->moURLArray['nlayerindex'].'">';

        $szVariable = 'nstyles';
        $aReturn[$szVariable] = '<INPUT TYPE="HIDDEN" NAME="'.$szVariable.
            '" VALUE="'.$this->mnSLDNumStyles.'">';

        $szVariable = 'SLD_TABLE_COMMAND';
        $aReturn[$szVariable] = '<INPUT TYPE="HIDDEN" NAME="'.$szVariable. '" VALUE="">';

        return $aReturn;
    }
    
    function GetJavascriptVariables()
    {
        $aReturn = array();
        
        $aReturn['gbDirty'] = 'var gbDirty = '.( $this->mbDirty ? 'true' : 'false' );
        
        return $aReturn;
    }
    
    function GetJavascriptOnLoadFunctions()
    {
        $aReturn = array();
        if ( $this->mbUpdateOpener )
        {
            //0 is OK, opener will close this window.
            //2 is apply, opener will refresh
            $aReturn['sldcacheupdateopener'] = "SECTCallbackFunc( 2 );\n";
        }
        
        return $aReturn;
    }
    
    function GetJavascriptInitFunctions()
    {
        $aReturn = array();
        
        $aReturn["SLDCacheInit".$this->mnId] = "SLDCacheInit".$this->mnId."();\n";
        
        return $aReturn;
    }


    /**
     * return javascript functions required for this widget
     */
    function GetJavascriptFunctions()
    {
        $aReturn = array();

        $szJsFunctionName = "SetSLDTableCommand";
        $szFunction = <<<EOT
/**
 * set the submit command
 */
function {$szJsFunctionName}(oButton, szCommand)
{
    //wicked hack until buttons are moved from widget into template
    if (szCommand == null)
        szCommand = oButton;
    {$this->mszHTMLForm}.SLD_TABLE_COMMAND.value = szCommand;
    {$this->mszHTMLForm}.submit();
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        $szJsFunctionName = "loadSymbolizer";
        $szDispatchURL = '\'./index.phtml?template=sect_symbolizer_template.html'.
                            '&'.SID.
                            '&nlayerindex='.$this->moURLArray['nlayerindex'].
                            '&szWebCommon='.WEBCOMMON.
                            '&szCallBack=symbolizerCallBack\'';
        //:TODO: see if this can be done with the link widget
        $szFunction = <<<EOT
/**
 * launch the symbolizer
 */
function {$szJsFunctionName}(nClassid, nstyles, szclasstype)
{
    selectClass(nClassid);
    szURL = {$szDispatchURL};
    szURL = szURL + '&initclassindex=' + nClassid;
    szURL += '&nstyles=' + nstyles;
    szURL += '&szclasstype=' + szclasstype;
    currentTool = window.open(szURL,'Style_Editor',
                                  'width=510,height=420,scrollbars=yes');
    currentTool.focus();
    return;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;


        $szJsFunctionName = "SECTCallbackFunc";
        //:TODO: make this work even when we're not in maplab
        $szFunction = <<<EOT

/**
 *  application callback
 */
function {$szJsFunctionName}(actionId)
{
    window.opener.SECTCallback(actionId, window);
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;
        
        //determine layer name
        $szLayerName = "unnamed";
        $oLayer = $this->moSLDSession->oMap->GetLayer( 0 );
        if ($oLayer->getMetaData( "WMS_TITLE" ) != "" )
            $szLayerName = $oLayer->getMetaData( "WMS_TITLE" );
        else
            $szLayerName = $oLayer->name;
            
        //determine layer type
        //TODO detect WMS connectiontype and try to determine 
        //the type of the layer on the other end using WFS if 
        //possible ... Could get attributes this way too!
        $nType = $oLayer->type;
        
        if ($nType > 3)
        {
            $nType = 3;
        }
       
        //TODO determine layer attributes if possible
        //classitem
        $szClassItem = $oLayer->classitem;
        $szLabelItem = $oLayer->labelitem;
        
        $szJsFunctionName = "SLDCacheInit".$this->mnId;
        $szJsFunction = <<<EOT
                
/**
 * initialize the SLDCache interface elements
 */
function {$szJsFunctionName}()
{
    if ({$this->mszHTMLForm}.layername != null)
    {
        document.title = "Layer: {$szLayerName}";
    }
    if ({$this->mszHTMLForm}.layertype != null)
    {
        {$this->mszHTMLForm}.layertype.selectedIndex = {$nType};
    }
    if ({$this->mszHTMLForm}.labelitem != null)
    {
        {$this->mszHTMLForm}.labelitem.value = "{$szLabelItem}";
    }
    if ({$this->mszHTMLForm}.classitem != null)
    {
        {$this->mszHTMLForm}.classitem.value = "{$szClassItem}";
    }
    
}
EOT;
        $aReturn[$szJsFunctionName] = $szJsFunction;
        
        $szCoreURL = WEBCOMMON;

        $szJsFunctionName = "GetCoreURL";
        $szFunction = <<<EOT
/**
 * set the submit command
 */
function {$szJsFunctionName}()
{
    
    return "{$szCoreURL}";
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }


    /**
     * return the current layer to the mapsession
     */
    function ReturnLayer()
    {
        $oSrcLayer = $this->moSLDSession->oMap->getLayer( 0 );
        if ($oSrcLayer == null)
        {
            echo "ERROR: source layer not set in ReturnLayer<BR>";
            return false;
        }
        $oDstLayer = $this->moMapObject->oMap->getLayer( $this->moURLArray['nlayerindex'] );
        if ($oDstLayer == null)
        {
            echo "ERROR: destination layer not set in ReturnLayer<BR>";
            return false;
        }
        
        //remove existing classes
        for( $i=0; $i<$oDstLayer->numclasses; $i++)
        {
            $oClass = $oDstLayer->getClass( $i );
            $oClass->set( "status",  MS_DELETE );
        }
        
        //copy over classes from the local mapsession layer
        for ($i=0; $i<$oSrcLayer->numclasses; $i++)
        {
            $oSrcClass = $oSrcLayer->getClass( $i );
            $oDstClass = ms_newClassObj( $oDstLayer, $oSrcClass );
        }
        
        //copy classitem, labelitem, layer type and mark to auto-generate SLD
        $oDstLayer->set( 'classitem', $oSrcLayer->classitem );
        $oDstLayer->set( 'labelitem', $oSrcLayer->labelitem );
        $oDstLayer->set( 'type', $oSrcLayer->type );
        $oDstLayer->setMetaData( 'wms_sld_body', 'auto' );
        
        $this->mbUpdateOpener = true;
        $this->mbDirty = false;
        
        return true;
    }

    /**
     * Parse the layer and create the necessary entries in the
     * shared resource.
     */
    function ParseURL()
    {
        $srName = $this->mszSharedResourceName;

        $sr = array();

        if (isset($this->moURLArray['initlayername']))
        {
            //this is coming from the main interface
            //convert a named layer into a layer index
            $oLayer = $this->moMapObject->oMap->getLayerByName( $this->moURLArray['initlayername'] );
            if ($oLayer != null)
            {
                $this->moURLArray['nlayerindex'] = $oLayer->index;
                ms_newLayerObj( $this->moSLDSession->oMap, $oLayer );
                
                //todo: create inline symbols for sld
            }
        }
        elseif(isset($this->moURLArray['initlayerindex']))
        {
            $oLayer = $this->moMapObject->oMap->getLayer( $this->moURLArray['initlayerindex'] );
            if ($oLayer != null)
            {
                ms_newLayerObj( $this->moSLDSession->oMap, $oLayer );
                $this->moURLArray['nlayerindex'] = $this->moURLArray['initlayerindex'];
            }
        }
        else
        {
            if (isset($_SESSION['SLDCACHESTATE']) && $_SESSION['SLDCACHESTATE'] != "" )
            {
                $this->moSLDSession->restoreState( $_SESSION['SLDCACHESTATE'], $this->mszSLDMapFile );
            }
        }
        
        if (!isset($this->moURLArray['nlayerindex']))
        {
            //todo add error
            echo "ERROR: No Layer";
            return false;
        }

        if ($this->moSLDSession->oMap->numlayers != 1)
        {
            echo "ERROR: moSLDSession has ".$this->moSLDSession->oMap->numlayers." layers!<BR>";
            return false;
        }
        $oLayer = $this->moSLDSession->oMap->getLayer( 0 );
        $nSelectedClass = null;
        
        if (isset($this->moURLArray['classitem']))
        {
            if ($oLayer->classitem != $this->moURLArray['classitem'])
            {
                $this->mbDirty = true;
                $oLayer->set( 'classitem', $this->moURLArray['classitem'] );
            }
        }
        
         if (isset($this->moURLArray['labelitem']))
        {
            if ($oLayer->labelitem != $this->moURLArray['labelitem'])
            {
                $this->mbDirty = true;
                $oLayer->set( 'labelitem', $this->moURLArray['labelitem'] );
            }
        }

       if (isset($this->moURLArray['layertype']))
        {
            if ($oLayer->type != $this->moURLArray['layertype'])
            {
                $this->mbDirty = true;
                $oLayer->set( "type", intval( $this->moURLArray['layertype'] ) );
            }
        }

        /* first we check for incoming changes */
        if (isset($this->moURLArray['input_0_1']))
        {
            //apply changes
            for( $i=0; $i<$oLayer->numclasses; $i++)
            {
                if (isset($this->moURLArray['input_'.$i.'_1']))
                {
                    $oClass = $oLayer->getClass( $i );
                    if ($oClass->name != $this->moURLArray['input_'.$i.'_1'])
                    {
                        $this->mbDirty = true;
                        $oClass->set( 'name', $this->moURLArray['input_'.$i.'_1'] );
                    }
                    if ($oClass->getexpression() != stripslashes($this->moURLArray['input_'.$i.'_2'] ))
                    {
                        $this->mbDirty = true;
                        $oClass->setexpression( stripslashes($this->moURLArray['input_'.$i.'_2'] ));
                    }
                    if ($oClass->minscale != $this->moURLArray['input_'.$i.'_5'] )
                    {
                        $this->mbDirty = true;
                        $oClass->set( 'minscale', $this->moURLArray['input_'.$i.'_5'] );
                    }
                    if ($oClass->maxscale != $this->moURLArray['input_'.$i.'_5'] )
                    {
                        $this->mbDirty = true;
                        $oClass->set( 'maxscale', $this->moURLArray['input_'.$i.'_6'] );
                    }
                    if ($oClass->type != $this->moURLArray['input_'.$i.'_8'] )
                    {
                        $this->mbDirty = true;
                        $oClass->set( 'type', $this->moURLArray['input_'.$i.'_8'] );
                    }
                    
                    if ($oLayer->type == MS_LAYER_RASTER)
                    {
                        $red = $this->moURLArray['input_'.$i.'_10'] ;
                        $green = $this->moURLArray['input_'.$i.'_11'] ;
                        $blue = $this->moURLArray['input_'.$i.'_12'] ;
                        
                        if ($oClass->numstyles == 0)
                        {
                            ms_newStyleObj( $oClass );
                        }
                        $oClass = $oLayer->getClass( $i );
                        $oStyle = $oClass->getStyle ( 0 );
                        $oStyle->color->setRGB( $red, $green, $blue );
                    }                
                    
                    //preserve selection state
                    if (isset($this->moURLArray['radiobutton']))
                    {
                        if (($this->moURLArray['radiobutton'] == $i))
                        {
                            $nSelectedClass = $i;
                        }
                    }
                }
            }
        }
        
        /* now handle actions */
        if (isset($this->moURLArray['SLD_TABLE_COMMAND']))
        {
             $szTableCommand = $this->moURLArray['SLD_TABLE_COMMAND'];
             $nResult = 0;
             switch ($szTableCommand)
             {
             case 'ADD':
                $this->mbDirty = true;
                
                if (isset($this->moURLArray['nClasses']))
                {
                    $nClasses = $this->moURLArray['nClasses'];
                }
                else
                    $nClasses = 1;
                    
                $nReselect = $nSelectedClass;
                for ($i=0; $i<$nClasses; $i++)
                {
                    $nClass = $oLayer->numclasses;
                    $oClass = ms_newClassObj( $oLayer );
                    $nSelectedClass ++;
                    while($nClass > $nSelectedClass)
                    {
                        $oLayer->moveclassup( $nClass-- );
                    }
                }
                if ($nReselect != $nSelectedClass)
                    $nSelectedClass = $nReselect + 1;
                
                break;
             case 'DUPLICATE':
                if ($nSelectedClass !== null && $nSelectedClass >= 0 && $nSelectedClass < $oLayer->numclasses)
                {
                    $this->mbDirty = true;
                    $oSrcClass = $oLayer->GetClass( $nSelectedClass );
                    $nSelectedClass = $oLayer->numclasses;
                    $oClass = ms_newClassObj( $oLayer, $oSrcClass );
                }
                break;
             case 'DELETE':
                
                if ($nSelectedClass !== null && $nSelectedClass >= 0 && $nSelectedClass < $oLayer->numclasses)
                {
                    $this->mbDirty = true;
                    $oClass = $oLayer->getClass( $nSelectedClass );
                    $oClass->set( "status", MS_DELETE );
                    //oLayer->numclasses doesn't change when setting status to MS_DELETE
                    if ($nSelectedClass == $oLayer->numclasses - 1)
                    {
                        $nSelectedClass --;
                    }
                    if ($nSelectedClass < 0)
                        $nSelectedClass = null;
                }
                break;
             case 'PROMOTE':
                if ($nSelectedClass !== null && $nSelectedClass > 0 && $nSelectedClass < $oLayer->numclasses)
                {
                    $this->mbDirty = true;
                    $oLayer->moveClassUp( $nSelectedClass );
                    $nSelectedClass --;
                }
                break;
             case 'DEMOTE':
                if ($nSelectedClass !== null && $nSelectedClass >= 0 && $nSelectedClass < $oLayer->numclasses - 1)
                {
                    $this->mbDirty = true;
                    $oLayer->moveClassDown( $nSelectedClass );
                    $nSelectedClass ++;
                }    
                break;
             case 'UNDO':
                //undo previous command by going back in the state array
                if (isset($_SESSION['SLDCACHESTATEARRAY']))
                {
                    $this->mbDirty = true;
                    $szState = "";
                    if (count( $_SESSION['SLDCACHESTATEARRAY'] ) > 0)
                    {
                        $szState = array_pop( $_SESSION['SLDCACHESTATEARRAY'] );
                        $szState = array_pop( $_SESSION['SLDCACHESTATEARRAY'] );
                    }
                    $this->moSLDSession->restoreState( $szState, $this->mszSLDMapFile );
                    $oLayer = $this->moSLDSession->oMap->getLayer( 0 );
                    if ($nSelectedClass >= $oLayer->numclasses)
                    {
                        $nSelectedClass = null;
                    }
                }
                break;
             case 'RETURN':
                $this->ReturnLayer();
                break;
            }
        }
             
         //now turn the classes into an SR for the table widget
         $i = 0;
         for($k=0;$k < $oLayer->numclasses; $k++)
         {
            $oClass = $oLayer->getClass($k);
            if ($oClass->status == MS_DELETE)
                continue;

            //temp styles holder
            $aszStyles = array();

            $sr[$i]['class_name'] = $oClass->name;
            $sr[$i]['expression'] = stripslashes($oClass->getExpression());
            $sr[$i]['class_minscale'] = $oClass->minscale;
            $sr[$i]['class_maxscale'] = $oClass->maxscale;
            $sr[$i]['class_isselected'] = ($nSelectedClass == $i) ? "CHECKED" : "";
            $sr[$i]['class_layerindex'] = $this->mnSLDLayerIndex;
            //index has to be in the array for table's template parsing
            $sr[$i]['class_rowindex'] = $i;

            //iterate over styles
            $nNumStyles = $oClass->numstyles;
            $nClassType = $oLayer->type;
            $this->mnSLDNumStyles = $nNumStyles;
            $sr[$i]['class_numstyles'] = $nNumStyles;
            $sr[$i]['class_type'] = $nClassType;
            
            if ($nNumStyles > 0)
            {
                for($j=0; $j < $nNumStyles; $j++)
                {
                    $oStyle = &$oClass->getStyle($j);
                    if (!($aszStyles['name'][$j] = $oStyle->symbolname))
                    {
                        $aszStyles['name'][$j] = $oStyle->symbol;
                    }
    
                    //$aszStyles['symbol'][$j] = $oStyle->symbol;
                    $aszStyles['size'][$j]= $oStyle->size;
                    $aszStyles['color'][$j]= $oStyle->color->red .
                                             '_' . $oStyle->color->green .
                                             '_' . $oStyle->color->blue;
                    $aszStyles['outlinecolor'][$j]= $oStyle->outlinecolor->red .
                                             '_' . $oStyle->outlinecolor->green .
                                             '_' . $oStyle->outlinecolor->blue;
                    $aszStyles['backgroundcolor'][$j]= $oStyle->backgroundcolor->red .
                                             '_' . $oStyle->backgroundcolor->green .
                                             '_' . $oStyle->backgroundcolor->blue;
                    //offsets?
                }
                $szStyleNames = implode(',', $aszStyles['name']);
                $szStyleSizes = implode(',', $aszStyles['size']);
                $szStyleColors = implode(',', $aszStyles['color']);
                $szStyleOutlineColors = implode(',', $aszStyles['outlinecolor']);
                $szStyleBackgroundColors = implode(',', $aszStyles['backgroundcolor']);
            }
            else
            {
                $szStyleNames = "";
                $szStyleSizes = "";
                $szStyleColors = "";
                $szStyleOutlineColors = "";
                $szStyleBackgroundColors = "";
                
            }
            
            //the symbolizer to use for this class.  For RASTER layers, use
            //a color picker.  For POINT, LINE and POLYGON layers, use
            //the symbolizer.  What to do for other layers???
            if ($nClassType < MS_LAYER_RASTER)
            {
                $sym  = "<a href=javascript:loadSymbolizer('$i','$nNumStyles','$nClassType')>";
                $sym .= "<img src='./symbolizer.php?".SID."&width=43&height=26";
                $sym .= "&nSymbolName=$szStyleNames";
                $sym .= "&nSymbolSize=$szStyleSizes";
                $sym .= "&nSymbolColor=$szStyleColors";
                $sym .= "&nSymbolOutlineColor=$szStyleOutlineColors";
                $sym .= "&nSymbolBackgroundColor=$szStyleBackgroundColors";
                $sym .= "&nstyles=$nNumStyles&szclasstype=$nClassType' ";
                $sym .= "name='symbol_preview_$i' id='symbol_preview_$i' BORDER='0' width='43' height='26'></a>";
                $sr[$i]['class_symbolizer'] = $sym;
            }
            elseif ($nClassType == MS_LAYER_RASTER )
            {
                if ($oClass->numstyles > 0)
                {
                    $oStyle = $oClass->getStyle( 0 );
                    $red = $oStyle->color->red;
                    $green = $oStyle->color->green;
                    $blue = $oStyle->color->blue;
                }
                else
                {
                    $red = -1;
                    $green = -1;
                    $blue = -1;
                }
                $sr[$i]['class_color_red'] = $red;
                $sr[$i]['class_color_blue'] = $blue;
                $sr[$i]['class_color_green'] = $green;
                $sr[$i]['class_symbolizer'] = '<a href="javascript:SelectColor(' . $i . ')"><img name="color_'.$i.'" id="color_'.$i.'" src="./color.php?red='.$red.'&green='.$green.'&blue='.$blue.'&x=43&y=26" border="0" width="43" height="26"></a>';
            }
            else
            {
                $sr[$i]['class_symbolizer'] = '<img src="images/a_pixel.gif" BORDER="0" width="43" height="26" >';
            }
            
            //for label, only collect params needed for preview
            $oLabel = $oClass->label;
            
            if ($oLayer->labelitem != "" && $oLabel->type == MS_BITMAP && $oLabel->font == "")
            {
                $oLabel->set( "font", "Bitmap" );
            }
            
            $sr[$i]['class_label_font_name'] = $oLabel->font;
            $sr[$i]['class_label_font_type'] = $oLabel->type;
            $sr[$i]['class_label_color'] = $oLabel->color->red .
                                     '_' . $oLabel->color->green .
                                     '_' . $oLabel->color->blue;
            $sr[$i]['class_label_outlinecolor'] = $oLabel->outlinecolor->red .
                                            '_' . $oLabel->outlinecolor->green .
                                            '_' . $oLabel->outlinecolor->blue;
            $sr[$i]['class_label_size'] = $oLabel->size;
            $sr[$i]['class_label_angle'] = $oLabel->angle;
            if ($oLabel->autoangle)
            {
                $sr[$i]['class_label_angle'] = 20;
            }
            
            //the label preview is only displayed if a labelitem is set
            //:TODO:
            //the onblur for labelitem checks if there is an item picked
            //and updates the preview.
            //like symbols, this doesn't apply to rasters
            if (($nClassType < MS_LAYER_RASTER) &&
                (strcasecmp($oLayer->labelitem, "") != 0))
            {
                $label  = "<a href=javascript:selectLabel('$i')>";
                $label .= "<img src='./sect_label_preview.php?".SID."&width=83&height=26";
                $label .= "&nLabelType=" . $sr[$i]['class_label_font_type'];
                $label .= "&szFontName=" . $sr[$i]['class_label_font_name'];
                $label .= "&nSize=". $sr[$i]['class_label_size'];
                $label .= "&nFontColor=".$sr[$i]['class_label_color'];
                $label .= "&nFontOutlineColor=".$sr[$i]['class_label_outlinecolor'];
                $label .= "&nAngle=".$sr[$i]['class_label_angle'];
                $label .= "' name='font_preview_$i' id='font_preview_$i' BORDER='0' width='83' height='26'></a>";
                $sr[$i]['class_labelizer'] = $label;
            }
            else
            {
                $sr[$i]['class_labelizer'] = '<img src="images/a_pixel.gif" BORDER="0" width="83" height="26" alt="Please select a label attribute to enable labelling." title="Please select a label attribute to enable labelling.">';
            }

            $i++;
        }    

        $this->maSharedResourceWidgets[$srName] = $sr;

        $_SESSION['SLDCACHESTATE'] = $this->moSLDSession->saveState( );
        if (!isset( $_SESSION['SLDCACHESTATEARRAY'] ))
        {
            $_SESSION['SLDCACHESTATEARRAY'] = array();
        }
        array_push( $_SESSION['SLDCACHESTATEARRAY'], $_SESSION['SLDCACHESTATE']);
        
        return true;
    }
}
?>