<?php
/**
 * Style Manager Widget class
 *
 * @project     GeoInnovations 2002 - SECT
 * @revision    $Id: StyleManager.widget.php,v 1.7 2004/02/19 03:47:14 pspencer Exp $
 * @purpose     Style manager Widget class puts the contents of a class
 *              into a shared resource.
 * @author      DM Solutions Group (zjames@dmsolutions.ca)
 * @copyright
 * <b>Copyright (c) 2003, DM Solutions Group Inc.</b>
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


include_once($GLOBALS['gszChameleonPath']."/htdocs/widgets/Widget.php");
include_once($GLOBALS['gszChameleonPath']."/htdocs/widgets/SharedResource.widget.php");

/**
 * SLDCache Widget
 *
 * @desc a widget that extracts the contents of a mapfile class and inserts it
 *       into a SharedResource.
 */
class StyleManager extends CWCWidget
{
    /* the name of the shared resource to dump the results into */
    var $mszSharedResourceName;

    /* the index of the layer in the mapfile */
    var $mnSLDLayerIndex;

    /* the index of the class in the mapfile */
    var $mnSLDClassIndex;

    /* the number of styles in the class */
    var $mnSLDNumStyles;
    
    /* the internal state of the StyleManager widget maintained separately
       from the main map state to facilitate management of classes
       using PHP MapScript functions without affecting the main
       state until the user accepts the changes.  Also facilitates
       an UNDO function :)
     */
     
    var $moStyleSession = null;
    
    var $mszStyleMapFile = "";
    
    var $moSLDSession = null;
    
    var $mbCloseWindow = false;

    /**
     * SLDCache
     *
     * Constructor method for the SLDCache widget.
     */
    function StyleManager()
    {
        // invoke constructor of parent
        parent::CWCWidget();

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
The Style manager widget gets the contents of a mapfile class and adds an array to
a SharedResource widget in a format suitable for generating tables for SECT.
The class to process must be passed in the url as nclassindex. This uses and
requires a Table widget for output.
EOT;

        $this->maAttributes["SHAREDRESOURCENAME"] =
            new StringAttribute( "SHAREDRESOURCENAME", true );
            
        $this->mszStyleMapFile = getSessionSavePath()."/SLDCache".$this->mnId.".map";
            
        $oMap = ms_newMapObj( "" );
        $oMap->set( 'width', 1 );
        $oMap->set( 'height', 1 );
        $oMap->extent->setextent( -1, -1, 1, 1 );
        $oMap->setsymbolset( realpath( $GLOBALS['gszAppPath']."/../etc/sectsym.sym") );
        
        $this->moStyleSession = new MapSession_RW();
        $this->moStyleSession->szMapFile = $this->mszStyleMapFile;
        $this->moStyleSession->szTempDir = getSessionSavePath();
        $this->moStyleSession->oMap = $oMap;
   }

    /**
     * initialize respectable defaults
     */
    function InitDefaults()
    {
        parent::InitDefaults();

        if (isset($this->maParams["SHAREDRESOURCENAME"]))
            $this->mszSharedResourceName =
                $this->maParams["SHAREDRESOURCENAME"];
                
    }

    /**
     * return an array javascript variables required by the manager
     */
    function GetJavascriptVariables()
    {
        $aReturn = array();

        $szVariable = "bDirty".$this->mnId;
        $szValue = "false";
        $aReturn[$szVariable] = $szValue;

        return $aReturn;
    }

    /**
     * return an array of HTML variables to include in the page
     */
    function GetHTMLHiddenVariables()
    {
        $aReturn = array();

        $szVariable = 'NLAYERINDEX';
        $aReturn[$szVariable] = '<input TYPE="HIDDEN" NAME="'.$szVariable.
            '" VALUE="'.$this->moURLArray['NLAYERINDEX'].'">';

        $oLayer = $this->moStyleSession->oMap->getLayer( 0 );
        $szVariable = 'NLAYERTYPE';
        $aReturn[$szVariable] = '<input TYPE="HIDDEN" NAME="'.$szVariable.
            '" VALUE="'.$oLayer->type.'">';

        
        $szVariable = 'nclassindex';
        $aReturn[$szVariable] = '<input TYPE="HIDDEN" NAME="'.$szVariable.
            '" VALUE="'.$this->moURLArray['nclassindex'].'">';

        $szVariable = 'NSTYLES';
        $aReturn[$szVariable] = '<input TYPE="HIDDEN" NAME="'.$szVariable.
            '" VALUE="'.$this->mnSLDNumStyles.'">';

        $szVariable = 'SLD_TABLE_COMMAND';
        $aReturn[$szVariable] = '<input TYPE="HIDDEN" NAME="'.$szVariable. '" VALUE="">';

        $szVariable = 'TEMPLATE';
        $aReturn[$szVariable] = '<input TYPE="HIDDEN" NAME="'.$szVariable.
            '" VALUE="'.$this->moURLArray['TEMPLATE'].'">';

        return $aReturn;
    }

    function GetJavascriptOnLoadFunctions()
    {
        $aReturn = array();
        if ($this->mbCloseWindow)
        {
            $aReturn['closewindow'] = "SymbolizerCallback();\n";
        }
        $oLayer = $this->moStyleSession->oMap->getLayer( 0 );
        $aReturn['feature_type'] = "ToggleFeatureType( ".$oLayer->type." );\n";
        $aReturn['styles'] = "ToggleTabs( 'styles' );\n";
    
        return $aReturn;
    }

    /**
     * return javascript functions required for this widget
     */
    function GetJavascriptFunctions()
    {
        $aReturn = array();
        
        $szCoreURL = $_SESSION['gszCoreWebPath'];

        $szJsFunctionName = "GetCoreURL";
        $szFunction = <<<EOT
/**
 * set the submit command
 */
function {$szJsFunctionName}()
{
    
    return "{$szCoreURL}";
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;


        $szJsFunctionName = "SetSLDTableCommand";
        $szFunction = <<<EOT
/**
 * set the submit command
 */
function {$szJsFunctionName}(oButton, szCommand)
{
    
    if ({$this->mszHTMLForm}.SLD_TABLE_COMMAND != null &&
        {$this->mszHTMLForm}.SLD_TABLE_COMMAND.value != null)
    {
        {$this->mszHTMLForm}.SLD_TABLE_COMMAND.value = szCommand;
    }
    {$this->mszHTMLForm}.submit();
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;


        $szJsFunctionName = "SymbolizerCallback";
        $nStyles = $this->mnSLDNumStyles;
        $nclassindex= $this->moURLArray['nclassindex'];
        $szFunction = <<<EOT
/**
 * write styles back to the Classifier panel (see SLDCache widget)
 */
function {$szJsFunctionName}()
{
    //trigger a write of the layer to the mapfile.
    window.opener.symbolCallBack( 1, window );
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;

        return $aReturn;
    }

/**
 *  copy the styles from the StyleManager's class to the right class
 * in the SLDCache's map
 */
    function ReturnClass( $oSLDSession )
    {
        $oSLDLayer = $oSLDSession->oMap->getLayer( 0 );
        $oSLDClass = $oSLDLayer->getClass( $this->moURLArray['nclassindex'] );
        
        $oStyleLayer = $this->moStyleSession->oMap->getLayer( 0 );
        $oStyleClass = $oStyleLayer->getClass( 0 );
        
        //remove old styles
        /*for ($i=0; $i<$oSLDClass->numstyles; $i++)
        {
            $oSLDClass->deleteStyle( $i );
        }*/
        while( $oSLDClass->numstyles > 0)
        {
            $oSLDClass->deleteStyle( $oSLDClass->numstyles - 1 );
            $oSLDClass = $oSLDLayer->getClass( $this->moURLArray['nclassindex'] );
        }
        //refetch the class
        $oSLDClass = $oSLDLayer->getClass( $this->moURLArray['nclassindex'] );
        //copy over new styles
        for ($i=0; $i<$oStyleClass->numstyles; $i++)
        {
            $oStyle = $oStyleClass->getStyle( $i );
            ms_newStyleObj( $oSLDClass, $oStyle );
        }
        $_SESSION['SLDCACHESTATE'] = $oSLDSession->saveState();
        array_push( $_SESSION['SLDCACHESTATEARRAY'], $_SESSION['SLDCACHESTATE'] );
        
        $this->mbCloseWindow = true;
    }

    /**
     * Parse the class and create the necessary entries in the
     * shared resource.
     */
    function ParseURL()
    {
        //sr is the SharedResource array that we use for the Table widget to
        //display the contents of the styles.
        $srName = $this->mszSharedResourceName;
        $sr = array();
        
        //used to grab layers from the SLDCache session, and to commit changes
        //back to it
        $oSLDSession = new MapSession_RW();
        $oSLDSession->szMapFile = $this->mszStyleMapFile;
        $oSLDSession->szTempDir = getSessionSavePath();
        $oSLDSession->restoreState( $_SESSION['SLDCACHESTATE'], $this->mszStyleMapFile );
            
        //test to see if we should reset.  initclassindex is passed via url to indicate
        //which class from the SLD session we should be working on.
        if (isset($this->moURLArray['initclassindex']) )
        {
            $this->moURLArray['nclassindex'] = $this->moURLArray['initclassindex'];
            $oSLDLayer = $oSLDSession->oMap->getLayer( 0 );
            $oSLDClass = $oSLDLayer->getClass( $this->moURLArray['nclassindex'] );
                      
            //create a new layer in this class and copy the selected class into it
            //copy the layer type the SLD session as well 'cause mapscript complains
            $oLayer = ms_newLayerObj( $this->moStyleSession->oMap );
            $oLayer->set( 'type', $oSLDLayer->type );
            $oClass = ms_newClassObj( $oLayer, $oSLDClass );
        }
        else
        {
            if (isset($_SESSION['STYLEMANAGERSTATE']) && $_SESSION['STYLEMANAGERSTATE'] != "" )
            {
                $this->moStyleSession->restoreState( $_SESSION['STYLEMANAGERSTATE'], $this->mszStyleMapFile );
            }
        }
        
        //validate that the class index is set at this point
        if (!isset($this->moURLArray['nclassindex']))
        {
            //TODO: convert to error manager error
            echo "ERROR: no class";
            return false;
        }

        //at this point, the StyleManager's mapsession should have one layer with one class
        $oLayer = $this->moStyleSession->oMap->getLayer( 0 );
        $oClass = $oLayer->getClass( 0 );
        
        $nSelectedStyle = null;

        /*
         * first check for incoming changes
         */
        if (isset($this->moURLArray['input_0_1']))
        {
            //TODO: the template parser can't handle newlines because of the
            //regular expression parser used in Widget.php. Also, this will
            //break if there is a single quote in any field; double quotes
            //should be used in the template but cause problems when there is
            //a value in the field. Table.widget may need to be modified

            for($i=0;$i < $oClass->numstyles; $i++)
            {
                if (isset($this->moURLArray['input_'.$i.'_1']))
                {
                    $oStyle = $oClass->getStyle( $i );
                    $xSymbol = $this->moURLArray['input_'.$i.'_1'];
                    if (is_numeric($xSymbol))
                        $oStyle->set( 'symbol', $xSymbol);
                    else
                        $oStyle->set( 'symbolname', $xSymbol);
                    $oStyle->set( 'size', $this->moURLArray['input_'.$i.'_2']);
                    $aColor = explode( "_", $this->moURLArray['input_'.$i.'_3']);
                    $oStyle->color->setRGB( $aColor[0], $aColor[1], $aColor[2] );
                    $aColor = explode( "_", $this->moURLArray['input_'.$i.'_4']);
                    $oStyle->outlinecolor->setRGB(  $aColor[0], $aColor[1], $aColor[2] );
                    
                    
                    //preserve selection state
                    if (isset($this->moURLArray['radiobutton']))
                    {
                        if (($this->moURLArray['radiobutton'] == $i))
                        {
                            $nSelectedStyle = $i;
                        }
                    }
                }
            }
        }
        
        /* now handle actions */
        if (isset($this->moURLArray['SLD_TABLE_COMMAND']))
        {
             $szTableCommand = $this->moURLArray['SLD_TABLE_COMMAND'];
             $nResult = 0;
             switch ($szTableCommand)
             {
             case 'ADD':
                //nStyle is the index at which the style will be added
                $nStyle = $oClass->numstyles;
                $oStyle = ms_newStyleObj( $oClass );
                if ($nSelectedStyle === null)
                    $nSelectedStyle = 0;
                else
                    $nSelectedStyle ++;
                       
                while($nStyle > $nSelectedStyle)
                {
                    $oClass->moveStyleUp( $nStyle-- );
                }
                break;
             case 'DUPLICATE':
                if ($nSelectedStyle !== null && $nSelectedStyle >= 0 && $nSelectedStyle < $oClass->numstyles)
                {
                    //TODO: should styles be duplicated right after the selected one?
                    $oSrcStyle = $oClass->GetStyle( $nSelectedStyle );
                    $nSelectedStyle = $oClass->numstyles;
                    $oStyle = ms_newStyleObj( $oClass, $oSrcStyle );
                }
                $nSelectedStyle ++;
                while($nStyle > $nSelectedStyle)
                {
                    $oClass->movestyleup( $nStyle-- );
                }
                //change the selection to the new style
                //$nSelectedStyle++;
                break;
                break;
             case 'DELETE':
                if ($nSelectedStyle !== null && $nSelectedStyle >= 0 && $nSelectedStyle < $oClass->numstyles)
                {
                    $oClass->deleteStyle( $nSelectedStyle );
                    if ($nSelectedStyle == $oClass->numstyles - 1)
                    {
                        $nSelectedStyle --;
                    }
                    if ($nSelectedStyle < 0)
                        $nSelectedStyle = null;
                }
                break;
             case 'PROMOTE':
                if ($nSelectedStyle != null && $nSelectedStyle > 0 && $nSelectedStyle < $oClass->numstyles)
                {
                    $oClass->moveStyleUp( $nSelectedStyle );
                    $nSelectedStyle --;
                }
                break;
             case 'DEMOTE':
                if ($nSelectedStyle !== null && $nSelectedStyle >= 0 && $nSelectedStyle < $oClass->numstyles - 1)
                {
                    $oClass->moveStyleDown( $nSelectedStyle );
                    $nSelectedStyle ++;
                }    
                break;
            case 'UNDO':
                //undo previous command by going back in the state array
                //TODO this isn't tested at ALL!
                if (isset($_SESSION['STYLEMANAGERSTATEARRAY']))
                {
                    $szState = "";
                    if (count( $_SESSION['STYLEMANAGERSTATEARRAY'] ) > 0)
                    {
                        $szState = array_pop( $_SESSION['STYLEMANAGERSTATEARRAY'] );
                        $szState = array_pop( $_SESSION['STYLEMANAGERSTATEARRAY'] );
                    }
                    $this->moStyleSession->restoreState( $szState, $this->mszStyleMapFile );
                    $oLayer = $this->moStyleSession->oMap->getLayer( 0 );
                    $oClass = $oLayer->getClass( 0 );
                    
                    if ($nSelectedStyle >= $oClass->numstyles)
                    {
                        $nSelectedStyle = null;
                    }
                }
                break;
             case 'RETURN':
                $this->ReturnClass( $oSLDSession );
                break;
            }
        }
        
        //make sure we have an updated reference to the class to catch changes
        //to the number of styles after a deletesytle() call
        $oClass = $oLayer->getClass( 0 );
        
        if ($nSelectedStyle === null)
            $nSelectedStyle = 0;
                    
        //now turn the styles into an SR for the table widget
        for ($i=0; $i < $oClass->numstyles; $i++)
        {
            
            $oStyle = $oClass->getStyle($i);
                
            $szSymbolName = $oStyle->symbolname;
            if ( strcasecmp( substr($szSymbolName, 0, 16), 'sld_mark_symbol_' ) == 0)
            {
                $szSymbolName = substr( $szSymbolName, 16 );
            }
            $index = strrpos($szSymbolName, '_filled');
            if ( $index !== false)
            {
                $szSymbolName = substr( $szSymbolName, 0, $index );
            }
            
            $sr[$i]['style_symbol_name'] = $szSymbolName;
            if (!strlen($oStyle->symbolname))
            {
                $sr[$i]['style_symbol_name'] = $oStyle->symbol;
            }
            if(!($sr[$i]['style_symbol_name']))
            {
                $sr[$i]['style_symbol_name'] = 0;
            }
            $sr[$i]['style_symbol_size'] = $oStyle->size;
            
            $sr[$i]['style_color'] = $oStyle->color->red . '_' .
                                     $oStyle->color->green . '_' .
                                     $oStyle->color->blue;
            $sr[$i]['style_outline_color'] = $oStyle->outlinecolor->red . '_' .
                                     $oStyle->outlinecolor->green . '_' .
                                     $oStyle->outlinecolor->blue;
            $sr[$i]['style_isselected'] = ($nSelectedStyle == $i) ? "CHECKED" : "";
            $sr[$i]['style_classindex'] = $this->mnSLDClassIndex;
            
            if ( $oStyle->outlinecolor->red == -1 )
                $szPolyType = 'polygons_fill';
            else
                $szPolyType = 'polygons_stroke';
            
            /*if (isset( $this->moURLArray['input_'.$i.'_7'] ))
            {
                $sr[$i]['style_polygon_type'] = $this->moURLArray['input_'.$i.'_7'];
            }
            else
            {
                $sr[$i]['style_polygon_type'] = 'polygons_stroke';
            }*/
            $sr[$i]['style_polygon_type'] = $szPolyType;
            //index has to be in the array for table's template parsing
            $sr[$i]['style_rowindex'] = $i;
        }
        $this->mnSLDNumStyles = $oClass->numstyles;
        
        $this->maSharedResourceWidgets[$srName] = $sr;
        $_SESSION['STYLEMANAGERSTATE'] = $this->moStyleSession->saveState( );
        
        return true;
    }
}
?>